@@ -28,6 +28,8 @@
 #include "my_dbug.h"
 #include "my_sys.h"
 
+struct CHARSET_INFO;
+
 /**
   @brief Parses concatenated patterns and adds them to internal pattern list
 
@@ -39,7 +41,7 @@
 size_t Pattern_matcher::add_patterns(const std::string &patterns,
                                      char delimiter) {
   DBUG_TRACE;
-  size_t length = patterns.length();
+  const size_t length = patterns.length();
   size_t pattern_count = 0;
 
   // we don't parse empty patterns
