@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,9 +21,9 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+#include <assert.h>
 #include <cstring>
 
-#include "my_dbug.h"
 #include "pfs_example_country.h"
 
 PFS_engine_table_share_proxy country_st_share;
@@ -35,11 +36,11 @@ unsigned int country_rows_in_table = 0;
 unsigned int country_next_available_index = 0;
 
 Country_record country_records_array[COUNTRY_MAX_ROWS] = {
-    {"", 0, "", 0, {0, true}, {0, true}, {0, true}, false}};
+    {"", 0, "", 0, "", 0, {0, true}, {0, true}, {0, true}, false}};
 
 /**
   Check for duplicate value of Primary/Unique Key column(s).
-  A sequential search is being used here, but its upto plugin writer to
+  A sequential search is being used here, but it is up to the plugin writer to
   implement his/her own search to make sure duplicate values are not
   inserted/updated for Primary/Unique Key Column(s).
 
@@ -71,6 +72,7 @@ PSI_table_handle *country_open_table(PSI_pos **pos) {
   Country_Table_Handle *temp = new Country_Table_Handle();
   temp->current_row.name_length = 0;
   temp->current_row.continent_name_length = 0;
+  temp->current_row.country_code_length = 0;
   temp->current_row.year.is_null = true;
   temp->current_row.population.is_null = true;
   temp->current_row.growth_factor.is_null = true;
@@ -99,6 +101,8 @@ static void copy_record(Country_record *dest, Country_record *source) {
   dest->population = source->population;
   dest->growth_factor = source->growth_factor;
   dest->m_exist = source->m_exist;
+  dest->country_code_length = source->country_code_length;
+  strncpy(dest->country_code, source->country_code, dest->country_code_length);
 }
 
 /* Define implementation of PFS_engine_table_proxy. */
@@ -160,7 +164,7 @@ int country_index_init(PSI_table_handle *handle, unsigned int idx, bool,
       *index = (PSI_index_handle *)i;
     } break;
     default:
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
 
@@ -174,13 +178,15 @@ int country_index_read(PSI_index_handle *index, PSI_key_reader *reader,
     case 0: {
       Country_index_by_name *i = (Country_index_by_name *)index;
       /* Read all keys on index one by one */
-      mysql_service_pfs_plugin_table->read_key_string(
-          reader, &i->m_country_name, find_flag);
-      mysql_service_pfs_plugin_table->read_key_string(
-          reader, &i->m_continent_name, find_flag);
-    } break;
+      pc_string_srv->read_key_string(reader, &i->m_country_name, find_flag);
+      pc_string_srv->read_key_string(reader, &i->m_continent_name, find_flag);
+
+      /* Remember the number of key parts found. */
+      i->m_fields = pt_srv->get_parts_found(reader);
+      break;
+    }
     default:
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
 
@@ -197,7 +203,7 @@ int country_index_next(PSI_table_handle *handle) {
       i = (Country_index_by_name *)&h->m_index;
       break;
     default:
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
 
@@ -231,28 +237,28 @@ int country_read_column_value(PSI_table_handle *handle, PSI_field *field,
 
   switch (index) {
     case 0: /* COUNTRY_NAME */
-      mysql_service_pfs_plugin_table->set_field_char_utf8(
-          field, h->current_row.name, h->current_row.name_length);
+      pc_string_srv->set_char_utf8mb4(field, h->current_row.name,
+                                      h->current_row.name_length);
       break;
     case 1: /* CONTINENT_NAME */
-      mysql_service_pfs_plugin_table->set_field_char_utf8(
-          field, h->current_row.continent_name,
-          h->current_row.continent_name_length);
+      pc_string_srv->set_char_utf8mb4(field, h->current_row.continent_name,
+                                      h->current_row.continent_name_length);
       break;
     case 2: /* YEAR */
-      mysql_service_pfs_plugin_table->set_field_year(field,
-                                                     h->current_row.year);
+      pc_year_srv->set(field, h->current_row.year);
       break;
     case 3: /* POPULATION */
-      mysql_service_pfs_plugin_table->set_field_bigint(
-          field, h->current_row.population);
+      pc_bigint_srv->set(field, h->current_row.population);
       break;
     case 4: /* GROWTH_FACTOR */
-      mysql_service_pfs_plugin_table->set_field_double(
-          field, h->current_row.growth_factor);
+      pc_double_srv->set(field, h->current_row.growth_factor);
+      break;
+    case 5: /* COUNTRY_CODE */
+      pc_text_srv->set(field, h->current_row.country_code,
+                       h->current_row.country_code_length);
       break;
     default: /* We should never reach here */
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
 
@@ -310,30 +316,31 @@ int country_write_column_value(PSI_table_handle *handle, PSI_field *field,
   unsigned int *name_length = &h->current_row.name_length;
   char *continent_name = (char *)h->current_row.continent_name;
   unsigned int *continent_name_length = &h->current_row.continent_name_length;
+  char *country_code = (char *)h->current_row.country_code;
+  unsigned int *country_code_length = &h->current_row.country_code_length;
 
   switch (index) {
     case 0: /* COUNTRY_NAME */
-      mysql_service_pfs_plugin_table->get_field_char_utf8(field, name,
-                                                          name_length);
+      pc_string_srv->get_char_utf8mb4(field, name, name_length);
       break;
     case 1: /* CONTINENT_NAME */
-      mysql_service_pfs_plugin_table->get_field_char_utf8(
-          field, continent_name, continent_name_length);
+      pc_string_srv->get_char_utf8mb4(field, continent_name,
+                                      continent_name_length);
       break;
     case 2: /* YEAR */
-      mysql_service_pfs_plugin_table->get_field_year(field,
-                                                     &h->current_row.year);
+      pc_year_srv->get(field, &h->current_row.year);
       break;
     case 3: /* POPULATION */
-      mysql_service_pfs_plugin_table->get_field_bigint(
-          field, &h->current_row.population);
+      pc_bigint_srv->get(field, &h->current_row.population);
       break;
     case 4: /* GROWTH_FACTOR */
-      mysql_service_pfs_plugin_table->get_field_double(
-          field, &h->current_row.growth_factor);
+      pc_double_srv->get(field, &h->current_row.growth_factor);
+      break;
+    case 5: /* COUNTRY_CODE */
+      pc_text_srv->get(field, country_code, country_code_length);
       break;
     default: /* We should never reach here */
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
 
@@ -347,7 +354,7 @@ int country_update_row_values(PSI_table_handle *handle) {
 
   Country_record *cur = &country_records_array[h->m_pos.get_index()];
 
-  DBUG_ASSERT(cur->m_exist == true);
+  assert(cur->m_exist == true);
 
   native_mutex_lock(&LOCK_country_records_array);
   if (is_duplicate(&h->current_row, h->m_pos.get_index()))
@@ -367,30 +374,31 @@ int country_update_column_value(PSI_table_handle *handle, PSI_field *field,
   unsigned int *name_length = &h->current_row.name_length;
   char *continent_name = (char *)h->current_row.continent_name;
   unsigned int *continent_name_length = &h->current_row.continent_name_length;
+  char *country_code = (char *)h->current_row.country_code;
+  unsigned int *country_code_length = &h->current_row.country_code_length;
 
   switch (index) {
     case 0: /* COUNTRY_NAME */
-      mysql_service_pfs_plugin_table->get_field_char_utf8(field, name,
-                                                          name_length);
+      pc_string_srv->get_char_utf8mb4(field, name, name_length);
       break;
     case 1: /* CONTINENT_NAME */
-      mysql_service_pfs_plugin_table->get_field_char_utf8(
-          field, continent_name, continent_name_length);
+      pc_string_srv->get_char_utf8mb4(field, continent_name,
+                                      continent_name_length);
       break;
     case 2: /* YEAR */
-      mysql_service_pfs_plugin_table->get_field_year(field,
-                                                     &h->current_row.year);
+      pc_year_srv->get(field, &h->current_row.year);
       break;
     case 3: /* POPULATION */
-      mysql_service_pfs_plugin_table->get_field_bigint(
-          field, &h->current_row.population);
+      pc_bigint_srv->get(field, &h->current_row.population);
       break;
     case 4: /* GROWTH_FACTOR */
-      mysql_service_pfs_plugin_table->get_field_double(
-          field, &h->current_row.growth_factor);
+      pc_double_srv->get(field, &h->current_row.growth_factor);
+      break;
+    case 5: /* COUNTRY_CODE */
+      pc_text_srv->get(field, country_code, country_code_length);
       break;
     default: /* We should never reach here */
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
   return 0;
@@ -402,7 +410,7 @@ int country_delete_row_values(PSI_table_handle *handle) {
 
   Country_record *cur = &country_records_array[h->m_pos.get_index()];
 
-  DBUG_ASSERT(cur->m_exist == true);
+  assert(cur->m_exist == true);
 
   native_mutex_lock(&LOCK_country_records_array);
   cur->m_exist = false;
@@ -431,6 +439,7 @@ void init_country_share(PFS_engine_table_share_proxy *share) {
   share->m_table_definition =
       "NAME char(20) not null, CONTINENT char(20),"
       " YEAR year, POPULATION bigint, GROWTH_FACTOR double(10,2),"
+      " COUNTRY_CODE text,"
       " UNIQUE KEY(NAME, CONTINENT)";
   share->m_ref_length = sizeof(Country_POS);
   share->m_acl = EDITABLE;
