@@ -1,18 +1,19 @@
 #ifndef MY_AES_INCLUDED
 #define MY_AES_INCLUDED
 
-/* Copyright (c) 2000, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is also distributed with certain software (including
+ This program is designed to work with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
+ separately licensed software that they have either included with
+ the program or referenced in the documentation.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -81,7 +82,7 @@ extern const char *my_aes_opmode_names[];
   @param [in] source           Pointer to data for encryption
   @param [in] source_length    Size of encryption data
   @param [out] dest            Buffer to place encrypted data (must be large
-  enough)
+  enough and not overlap with source)
   @param [in] key              Key to be used for encryption
   @param [in] key_length       Length of the key. Will handle keys of any length
   @param [in] mode             encryption mode
