@@ -1,15 +1,16 @@
-/* Copyright (c) 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2021, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,25 +24,10 @@
 #include "my_config.h"
 
 /*
-  In case of kerberos authentication we need to fill user name as kerberos user
-  name if it is empty. We need to fill user name inside mysql->user, clients
-  uses my_strdup directly to create new string. To use MySQL alloc functions we
-  need to include "/mysql/service_mysql_alloc.h". Inside service_mysql_alloc.h
-  there is #define which forces all dynamic plugins to use MySQL malloc function
-  via services. Client side plugin cannot use any services as of now.   Code
-  check in service_mysql_alloc.h #ifdef MYSQL_DYNAMIC_PLUGIN #define my_strdup
-  mysql_malloc_service->my_strdup #else extern char *my_strdup(PSI_memory_key
-  key, const char *from, myf_t flags); #endif Client authentication plugin
-  defines MYSQL_DYNAMIC_PLUGIN. And this forces to use always my_strdup via
-  services. To use native direct my_strdup, we need to undefine
-  MYSQL_DYNAMIC_PLUGIN. And again define MYSQL_DYNAMIC_PLUGIN once correct
-  my_strdup are declared. service_mysql_alloc.h should provide proper fix like
-  Instead of #ifdef MYSQL_DYNAMIC_PLUGIN
-  #ifdef  MYSQL_DYNAMIC_PLUGIN  &&   ! MYSQL_CLIENT_PLUGIN
+  This is a CLIENT_ONLY plugin, so allocation functions are my_malloc,
+  my_free etc.
 */
-#undef MYSQL_DYNAMIC_PLUGIN
 #include <mysql/service_mysql_alloc.h>
-#define MYSQL_DYNAMIC_PLUGIN
 
 #include "auth_kerberos_client_plugin.h"
 
@@ -53,14 +39,22 @@
 #include <mysql/client_plugin.h>
 #include <sql_common.h>
 #include "auth_kerberos_client_io.h"
-#include "gssapi_utility.h"
 #include "log_client.h"
 
+static authentication_mode g_authentication_mode {
+#if defined(_WIN32)
+  authentication_mode::MODE_SSPI
+#else
+  authentication_mode::MODE_GSSAPI
+#endif /* _WIN32 */
+};
+
 Logger_client *g_logger_client{nullptr};
 
 Kerberos_plugin_client::Kerberos_plugin_client(MYSQL_PLUGIN_VIO *vio,
-                                               MYSQL *mysql)
-    : m_vio{vio}, m_mysql{mysql} {}
+                                               MYSQL *mysql,
+                                               authentication_mode mode)
+    : m_vio{vio}, m_mysql{mysql}, m_mode{mode} {}
 
 /*
   This method try to get kerberos TGT if user name and password are not
@@ -69,14 +63,16 @@ Kerberos_plugin_client::Kerberos_plugin_client(MYSQL_PLUGIN_VIO *vio,
   authentication process will start.
 */
 bool Kerberos_plugin_client::obtain_store_credentials() {
+  log_client_dbg("Obtaining TGT TGS tickets from kerberos server.");
   if (!m_kerberos_client) {
-    m_kerberos_client = std::unique_ptr<Kerberos_client>(
-        new Kerberos_client(m_user_principal_name.c_str(), m_password.c_str()));
+    m_kerberos_client =
+        std::unique_ptr<I_Kerberos_client>(Kerberos_client_create_factory(
+            m_mode == authentication_mode::MODE_GSSAPI, m_service_principal,
+            m_vio, m_user_principal_name, m_password, m_as_user_relam));
   }
-  log_client_dbg("Obtaining TGT TGS tickets from kerberos.");
   if (!m_kerberos_client->obtain_store_credentials()) {
     log_client_error(
-        "Plug-in has failed to obtained kerberos TGT, authentication process "
+        "Plug-in has failed to obtain kerberos TGT, authentication process "
         "will be aborted. Please provide valid configuration, user name and "
         "password.");
     return false;
@@ -85,26 +81,44 @@ bool Kerberos_plugin_client::obtain_store_credentials() {
   }
 }
 
-Kerberos_plugin_client::~Kerberos_plugin_client() {}
-
 void Kerberos_plugin_client::set_mysql_account_name(
     std::string mysql_account_name) {
   std::string cc_user_name;
   std::stringstream log_client_stream;
 
   if (mysql_account_name.empty()) {
-    Kerberos_client kerberos_client{m_user_principal_name.c_str(),
-                                    m_password.c_str()};
-    log_client_dbg("Getting user name from Kerberos credential cache.");
-    kerberos_client.get_upn(cc_user_name);
+    /*
+      Kerberos authentication client plug-in called twice if default plug-in is
+      set as authentication_kerberos_client in the client.
+
+      First time, If MySQL account name is empty, we get account name from
+      credential cache (Linux) or windows session (windows) and set as MySQL
+      account name.
+
+      Second time, It gets called for actual authentication with MySQL account
+      name and password.
+    */
+    if (!m_kerberos_client) {
+      m_kerberos_client =
+          std::unique_ptr<I_Kerberos_client>(Kerberos_client_create_factory(
+              m_mode == authentication_mode::MODE_GSSAPI, m_service_principal,
+              m_vio, m_user_principal_name, m_password, m_as_user_relam));
+    }
+    cc_user_name = m_kerberos_client->get_user_name();
+    log_client_stream << "Cached/ OS session user name is: ";
+    log_client_stream << cc_user_name;
+    log_client_dbg(log_client_stream.str());
+    log_client_stream.str("");
   } else {
-    log_client_dbg("MySQL user account name is not empty.");
+    log_client_stream << "Provided MySQL user account name in client is: ";
+    log_client_stream << mysql_account_name;
+    log_client_dbg(log_client_stream.str());
     return;
   }
   if (!cc_user_name.empty()) {
     log_client_dbg(
         "Setting MySQL account name using Kerberos credential cache default "
-        "UPN.");
+        "(Linux )or logged-in account (Windows).");
     /*
       MySQL clients/lib uses my_free, my_strdup my_* string function for string
       management. We also need to use same methods as these are used/free inside
@@ -114,12 +128,7 @@ void Kerberos_plugin_client::set_mysql_account_name(
       my_free(m_mysql->user);
       m_mysql->user = nullptr;
     }
-    size_t pos = std::string::npos;
-    /* Remove realm */
-    if ((pos = cc_user_name.find("@")) != std::string::npos) {
-      log_client_dbg("Trimming realm from upn.");
-      cc_user_name.erase(pos, cc_user_name.length() - pos + 1);
-    }
+
     m_mysql->user =
         my_strdup(PSI_NOT_INSTRUMENTED, cc_user_name.c_str(), MYF(MY_WME));
     log_client_stream.str("");
@@ -135,9 +144,7 @@ void Kerberos_plugin_client::set_mysql_account_name(
 
 void Kerberos_plugin_client::set_upn_info(std::string name, std::string pwd) {
   /*
-    For some cases MySQL user name will be empty as user name is supposed to
-    come from kerberos TGT. Get kerberos user name from TGT and assign this as
-    MySQL user name.
+    Setting UPN using MySQL account name + user realm.
   */
   m_password = pwd;
   if (!name.empty()) {
@@ -152,11 +159,10 @@ void Kerberos_plugin_client::create_upn(std::string account_name) {
 }
 
 bool Kerberos_plugin_client::authenticate() {
-  Gssapi_client gssapi_client{m_service_principal, m_vio};
-  if (gssapi_client.authenticate()) {
-    return CR_OK;
+  if (m_kerberos_client->authenticate()) {
+    return true;
   } else {
-    return CR_ERROR;
+    return false;
   }
 }
 
@@ -176,7 +182,9 @@ static int deinitialize_plugin() SUPPRESS_UBSAN;
 
 static int kerberos_authenticate(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
   std::stringstream log_client_stream;
-  Kerberos_plugin_client client{vio, mysql};
+  Kerberos_plugin_client client{vio, mysql, g_authentication_mode};
+
+  log_client_info("*** Kerberos authentication starting. ***");
   client.set_mysql_account_name(mysql->user);
   if (!client.read_spn_realm_from_server()) {
     log_client_info(
@@ -209,7 +217,7 @@ static int kerberos_authenticate(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
 static int initialize_plugin(char *, size_t, int, va_list) {
   g_logger_client = new Logger_client();
   const char *opt = getenv("AUTHENTICATION_KERBEROS_CLIENT_LOG");
-  int opt_val = opt ? atoi(opt) : 0;
+  const int opt_val = opt ? atoi(opt) : 0;
   if (opt && opt_val > 0 && opt_val < 6) {
     g_logger_client->set_log_level(static_cast<log_client_level>(opt_val));
   }
@@ -222,7 +230,28 @@ static int deinitialize_plugin() {
   return 0;
 }
 
+#if defined(_WIN32)
+static int authentication_kerberos_client_option(const char *option,
+                                                 const void *val) {
+  // Supported options
+  if (strcmp(option, "plugin_authentication_kerberos_client_mode") == 0 &&
+      val != nullptr) {
+    g_authentication_mode =
+        strcmp(static_cast<const char *>(val), "GSSAPI") == 0
+            ? authentication_mode::MODE_GSSAPI
+            : authentication_mode::MODE_SSPI;
+    return 0;
+  }
+  return 1;
+}
+#endif /* _WIN32 */
+
 mysql_declare_client_plugin(AUTHENTICATION) "authentication_kerberos_client",
     MYSQL_CLIENT_PLUGIN_AUTHOR_ORACLE, "Kerberos Client Authentication Plugin",
     {0, 1, 0}, "PROPRIETARY", nullptr, initialize_plugin, deinitialize_plugin,
+#if defined(_WIN32)
+    authentication_kerberos_client_option,
+#else
+    nullptr,
+#endif
     nullptr, kerberos_authenticate, nullptr mysql_end_client_plugin;
