@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -37,6 +38,8 @@
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_sys.h"
+#include "nulls.h"
+#include "strxnmov.h"
 
 /**
   Returns full load-path for a file. to may be = path.
@@ -54,7 +57,7 @@
   @param path Pointer to buffer containing the supplied path.
   @param own_path_prefix  Prefix to be appended to path.
 
-  @retval to  Pointer to the supplied destination buffer
+  @returns to Pointer to the supplied destination buffer
               that will hold the full load-path.
 */
 
@@ -84,7 +87,7 @@ char *my_load_path(char *to, const char *path, const char *own_path_prefix) {
     and path is not a hard-path.
     If path is hard path or home dir, return the path.
   */
-  else if (own_path_prefix != NULL &&
+  else if (own_path_prefix != nullptr &&
            !(path[0] == FN_HOMELIB && path[1] == FN_LIBCHAR) &&
            !test_if_hard_path(path)) {
     (void)strxnmov(buff, sizeof(buff) - 1, own_path_prefix, path, NullS);
