@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2001, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -36,7 +37,6 @@
     * the internal size is a set of 32 bit words
     * the number of bits specified in creation can be any number > 0
       a bitmap with zero bits can be created and initialized, but not used.
-    * there are THREAD safe versions of most calls called bitmap_lock_*
 
   TODO:
   Make assembler THREAD safe versions of these using test-and-set instructions
@@ -47,21 +47,19 @@
   Kindahl.
 */
 
+#include "my_bitmap.h"
+
 #include <string.h>
 #include <sys/types.h>
+#include <algorithm>
+#include <bit>
 
-#include "my_bit.h"
-#include "my_bitmap.h"
 #include "my_byteorder.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
-#include "my_macros.h"
-#include "my_pointer_arithmetic.h"
 #include "my_sys.h"
-#include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysys/mysys_priv.h"
-#include "thr_mutex.h"
 
 void create_last_word_mask(MY_BITMAP *map) {
   /* Get the number of used bits (1..8) in the last byte */
@@ -107,14 +105,6 @@ void create_last_word_mask(MY_BITMAP *map) {
   }
 }
 
-static inline void bitmap_lock(MY_BITMAP *map) {
-  if (map->mutex) mysql_mutex_lock(map->mutex);
-}
-
-static inline void bitmap_unlock(MY_BITMAP *map) {
-  if (map->mutex) mysql_mutex_unlock(map->mutex);
-}
-
 static inline uint get_first_set(uint32 value, uint word_pos) {
   uchar *byte_ptr = (uchar *)&value;
   uchar byte_value;
@@ -147,57 +137,37 @@ static inline uint get_first_not_set(uint32 value, uint word_pos) {
   return MY_BIT_NONE;
 }
 
-bool bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
-                 bool thread_safe) {
+bool bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits) {
   DBUG_TRACE;
   if (!buf) {
-    uint size_in_bytes = bitmap_buffer_size(n_bits);
-    uint extra = 0;
-
-    if (thread_safe) {
-      size_in_bytes = ALIGN_SIZE(size_in_bytes);
-      extra = sizeof(mysql_mutex_t);
-    }
-    map->mutex = 0;
+    const uint size_in_bytes = bitmap_buffer_size(n_bits);
+    const uint extra = 0;
 
     if (!(buf = (my_bitmap_map *)my_malloc(key_memory_MY_BITMAP_bitmap,
                                            size_in_bytes + extra, MYF(MY_WME))))
-      return 1;
-
-    if (thread_safe) {
-      map->mutex = (mysql_mutex_t *)((char *)buf + size_in_bytes);
-      mysql_mutex_init(key_BITMAP_mutex, map->mutex, MY_MUTEX_INIT_FAST);
-    }
-
-  }
-
-  else {
-    DBUG_ASSERT(thread_safe == 0);
-    map->mutex = NULL;
+      return true;
   }
 
   map->bitmap = buf;
   map->n_bits = n_bits;
   create_last_word_mask(map);
   bitmap_clear_all(map);
-  return 0;
+  return false;
 }
 
 void bitmap_free(MY_BITMAP *map) {
   DBUG_TRACE;
   if (map->bitmap) {
-    if (map->mutex) mysql_mutex_destroy(map->mutex);
-
     my_free(map->bitmap);
-    map->bitmap = 0;
+    map->bitmap = nullptr;
   }
 }
 
 /*
-  test if bit already set and set it if it was not (thread unsafe method)
+  test if bit already set and set it if it was not
 
   SYNOPSIS
-    bitmap_fast_test_and_set()
+    bitmap_test_and_set()
     MAP   bit map struct
     BIT   bit number
 
@@ -206,69 +176,17 @@ void bitmap_free(MY_BITMAP *map) {
     !=0  bit was set
 */
 
-bool bitmap_fast_test_and_set(MY_BITMAP *map, uint bitmap_bit) {
-  uchar *value = ((uchar *)map->bitmap) + (bitmap_bit / 8);
-  uchar bit = 1 << ((bitmap_bit)&7);
-  uchar res = (*value) & bit;
-  *value |= bit;
-  return res;
-}
-
-/*
-  test if bit already set and set it if it was not (thread safe method)
-
-  SYNOPSIS
-    bitmap_fast_test_and_set()
-    map          bit map struct
-    bitmap_bit   bit number
-
-  RETURN
-    0    bit was not set
-    !=0  bit was set
-*/
-
 bool bitmap_test_and_set(MY_BITMAP *map, uint bitmap_bit) {
-  bool res;
-  DBUG_ASSERT(map->bitmap && bitmap_bit < map->n_bits);
-  bitmap_lock(map);
-  res = bitmap_fast_test_and_set(map, bitmap_bit);
-  bitmap_unlock(map);
-  return res;
-}
-
-/*
-  test if bit already set and clear it if it was set(thread unsafe method)
-
-  SYNOPSIS
-    bitmap_fast_test_and_set()
-    MAP   bit map struct
-    BIT   bit number
-
-  RETURN
-    0    bit was not set
-    !=0  bit was set
-*/
-
-static bool bitmap_fast_test_and_clear(MY_BITMAP *map, uint bitmap_bit) {
-  uchar *byte = (uchar *)map->bitmap + (bitmap_bit / 8);
-  uchar bit = 1 << ((bitmap_bit)&7);
-  uchar res = (*byte) & bit;
-  *byte &= ~bit;
-  return res;
-}
-
-bool bitmap_test_and_clear(MY_BITMAP *map, uint bitmap_bit) {
-  bool res;
-  DBUG_ASSERT(map->bitmap && bitmap_bit < map->n_bits);
-  bitmap_lock(map);
-  res = bitmap_fast_test_and_clear(map, bitmap_bit);
-  bitmap_unlock(map);
+  uchar *value = ((uchar *)map->bitmap) + (bitmap_bit / 8);
+  const uchar bit = 1 << ((bitmap_bit)&7);
+  const uchar res = (*value) & bit;
+  *value |= bit;
   return res;
 }
 
 uint bitmap_set_next(MY_BITMAP *map) {
   uint bit_found;
-  DBUG_ASSERT(map->bitmap);
+  assert(map->bitmap);
   if ((bit_found = bitmap_get_first(map)) != MY_BIT_NONE)
     bitmap_set_bit(map, bit_found);
   return bit_found;
@@ -284,9 +202,9 @@ void bitmap_set_prefix(MY_BITMAP *map, uint prefix_size) {
   uint prefix_bytes, prefix_bits, d;
   uchar *m = (uchar *)map->bitmap;
 
-  DBUG_ASSERT(map->bitmap &&
-              (prefix_size <= map->n_bits || prefix_size == (uint)~0));
-  set_if_smaller(prefix_size, map->n_bits);
+  assert(map->bitmap &&
+         (prefix_size <= map->n_bits || prefix_size == (uint)~0));
+  prefix_size = std::min(prefix_size, map->n_bits);
   if ((prefix_bytes = prefix_size / 8)) memset(m, 0xff, prefix_bytes);
   m += prefix_bytes;
   if ((prefix_bits = prefix_size & 7)) {
@@ -298,16 +216,16 @@ void bitmap_set_prefix(MY_BITMAP *map, uint prefix_size) {
 }
 
 bool bitmap_is_prefix(const MY_BITMAP *map, uint prefix_size) {
-  uint prefix_bits = prefix_size % 32;
+  const uint prefix_bits = prefix_size % 32;
   my_bitmap_map *word_ptr = map->bitmap, last_word;
   my_bitmap_map *end_prefix = word_ptr + prefix_size / 32;
-  DBUG_ASSERT(word_ptr && prefix_size <= map->n_bits);
+  assert(word_ptr && prefix_size <= map->n_bits);
 
   /* 1: Words that should be filled with 1 */
   for (; word_ptr < end_prefix; word_ptr++)
     if (*word_ptr != 0xFFFFFFFF) return false;
 
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->n_bits > 0);
   last_word = *map->last_word_ptr & ~map->last_word_mask;
 
   /* 2: Word which contains the end of the prefix (if any) */
@@ -340,7 +258,7 @@ bool bitmap_is_set_all(const MY_BITMAP *map) {
   my_bitmap_map *data_ptr = map->bitmap;
   my_bitmap_map *end = map->last_word_ptr;
 
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->n_bits > 0);
   for (; data_ptr < end; data_ptr++)
     if (*data_ptr != 0xFFFFFFFF) return false;
   if ((*map->last_word_ptr | map->last_word_mask) != 0xFFFFFFFF) return false;
@@ -351,7 +269,7 @@ bool bitmap_is_clear_all(const MY_BITMAP *map) {
   my_bitmap_map *data_ptr = map->bitmap;
   my_bitmap_map *end = map->last_word_ptr;
 
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->n_bits > 0);
   for (; data_ptr < end; data_ptr++)
     if (*data_ptr) return false;
   if (*map->last_word_ptr & ~map->last_word_mask) return false;
@@ -363,14 +281,14 @@ bool bitmap_is_clear_all(const MY_BITMAP *map) {
 bool bitmap_is_subset(const MY_BITMAP *map1, const MY_BITMAP *map2) {
   my_bitmap_map *m1 = map1->bitmap, *m2 = map2->bitmap, *end;
 
-  DBUG_ASSERT(map1->bitmap && map2->bitmap && map1->n_bits == map2->n_bits);
+  assert(map1->bitmap && map2->bitmap && map1->n_bits == map2->n_bits);
 
   end = map1->last_word_ptr;
   for (; m1 < end; m1++, m2++)
     if (*m1 & ~(*m2)) return false;
 
-  DBUG_ASSERT(map1->n_bits > 0);
-  DBUG_ASSERT(map2->n_bits > 0);
+  assert(map1->n_bits > 0);
+  assert(map2->n_bits > 0);
 
   if ((*map1->last_word_ptr & ~map1->last_word_mask) &
       ~(*map2->last_word_ptr & ~map2->last_word_mask))
@@ -383,10 +301,10 @@ bool bitmap_is_subset(const MY_BITMAP *map1, const MY_BITMAP *map2) {
 bool bitmap_is_overlapping(const MY_BITMAP *map1, const MY_BITMAP *map2) {
   my_bitmap_map *m1 = map1->bitmap, *m2 = map2->bitmap, *end;
 
-  DBUG_ASSERT(map1->bitmap && map2->bitmap && map1->n_bits == map2->n_bits);
+  assert(map1->bitmap && map2->bitmap && map1->n_bits == map2->n_bits);
 
-  DBUG_ASSERT(map1->n_bits > 0);
-  DBUG_ASSERT(map2->n_bits > 0);
+  assert(map1->n_bits > 0);
+  assert(map2->n_bits > 0);
 
   end = map1->last_word_ptr;
   for (; m1 < end; m1++, m2++)
@@ -398,21 +316,37 @@ bool bitmap_is_overlapping(const MY_BITMAP *map1, const MY_BITMAP *map2) {
   return false;
 }
 
-void bitmap_intersect(MY_BITMAP *map, const MY_BITMAP *map2) {
-  my_bitmap_map *to = map->bitmap, *from = map2->bitmap, *end;
-  uint len = no_words_in_map(map), len2 = no_words_in_map(map2);
-
-  DBUG_ASSERT(map->bitmap && map2->bitmap);
-
-  end = to + MY_MIN(len, len2);
-  for (; to < end; to++, from++) *to &= *from;
-
-  if (len >= len2) map->bitmap[len2 - 1] &= ~map2->last_word_mask;
-
-  if (len2 < len) {
-    end += len - len2;
-    for (; to < end; to++) *to = 0;
+/**
+   Check if 'map' is valid.
+   @param map The map that we wish to verify.
+   @returns 'true' if 'map' passes a consistency check.
+*/
+bool bitmap_is_valid(const MY_BITMAP *map) {
+  if (map->bitmap == nullptr) {
+    return false;
   }
+
+  // Check that last_word_mask is set correctly.
+  MY_BITMAP copy = *map;
+  create_last_word_mask(&copy);
+  return map->last_word_mask == copy.last_word_mask;
+}
+
+void bitmap_intersect(MY_BITMAP *to, const MY_BITMAP *from) {
+  assert(to->bitmap && from->bitmap);
+
+  const uint to_length = no_words_in_map(to);
+  const uint from_length = no_words_in_map(from);
+  uint min_length = std::min(to_length, from_length);
+
+  // Clear bits in 'to' not set in 'from'
+  for (uint i = 0; i < min_length; i++) to->bitmap[i] &= from->bitmap[i];
+
+  if (to_length >= from_length)
+    to->bitmap[from_length - 1] &= ~from->last_word_mask;
+
+  // Clear bits in 'to' where no corresponding bits exist in 'from'
+  for (uint i = min_length; i < to_length; i++) to->bitmap[i] = 0;
 }
 
 /*
@@ -427,7 +361,7 @@ void bitmap_intersect(MY_BITMAP *map, const MY_BITMAP *map2) {
   NOTE
     You can only set/clear full bytes.
     The function is meant for the situation that you copy a smaller bitmap
-    to a bigger bitmap. Bitmap lengths are always multiple of eigth (the
+    to a bigger bitmap. Bitmap lengths are always multiple of eight (the
     size of a byte). Using 'from_byte' saves multiplication and division
     by eight during parameter passing.
 
@@ -435,8 +369,8 @@ void bitmap_intersect(MY_BITMAP *map, const MY_BITMAP *map2) {
     void
 */
 
-void bitmap_set_above(MY_BITMAP *map, uint from_byte, uint use_bit) {
-  uchar use_byte = use_bit ? 0xff : 0;
+void bitmap_set_above(MY_BITMAP *map, uint from_byte, bool use_bit) {
+  const uchar use_byte = use_bit ? 0xff : 0;
   uchar *to = (uchar *)map->bitmap + from_byte;
   uchar *end = (uchar *)map->bitmap + (map->n_bits + 7) / 8;
 
@@ -445,8 +379,8 @@ void bitmap_set_above(MY_BITMAP *map, uint from_byte, uint use_bit) {
 
 void bitmap_subtract(MY_BITMAP *map, const MY_BITMAP *map2) {
   my_bitmap_map *to = map->bitmap, *from = map2->bitmap, *end;
-  DBUG_ASSERT(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
+  assert(map->n_bits > 0);
   end = map->last_word_ptr;
 
   for (; to <= end; to++, from++) *to &= ~(*from);
@@ -454,8 +388,8 @@ void bitmap_subtract(MY_BITMAP *map, const MY_BITMAP *map2) {
 
 void bitmap_union(MY_BITMAP *map, const MY_BITMAP *map2) {
   my_bitmap_map *to = map->bitmap, *from = map2->bitmap, *end;
-  DBUG_ASSERT(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
+  assert(map->n_bits > 0);
   end = map->last_word_ptr;
 
   for (; to <= end; to++, from++) *to |= *from;
@@ -463,8 +397,8 @@ void bitmap_union(MY_BITMAP *map, const MY_BITMAP *map2) {
 
 void bitmap_xor(MY_BITMAP *map, const MY_BITMAP *map2) {
   my_bitmap_map *to = map->bitmap, *from = map2->bitmap, *end;
-  DBUG_ASSERT(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
+  assert(map->n_bits > 0);
   end = map->last_word_ptr;
 
   for (; to <= end; to++, from++) *to ^= *from;
@@ -472,8 +406,8 @@ void bitmap_xor(MY_BITMAP *map, const MY_BITMAP *map2) {
 
 void bitmap_invert(MY_BITMAP *map) {
   my_bitmap_map *to = map->bitmap, *end;
-  DBUG_ASSERT(map->bitmap);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap);
+  assert(map->n_bits > 0);
   end = map->last_word_ptr;
 
   for (; to <= end; to++) *to ^= 0xFFFFFFFF;
@@ -483,20 +417,20 @@ uint bitmap_bits_set(const MY_BITMAP *map) {
   my_bitmap_map *data_ptr = map->bitmap;
   my_bitmap_map *end = map->last_word_ptr;
   uint res = 0;
-  DBUG_ASSERT(map->bitmap);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap);
+  assert(map->n_bits > 0);
 
-  for (; data_ptr < end; data_ptr++) res += my_count_bits_uint32(*data_ptr);
+  for (; data_ptr < end; data_ptr++) res += std::popcount(*data_ptr);
 
   /*Reset last bits to zero*/
-  res += my_count_bits_uint32(*map->last_word_ptr & ~map->last_word_mask);
+  res += std::popcount(*map->last_word_ptr & ~map->last_word_mask);
   return res;
 }
 
 void bitmap_copy(MY_BITMAP *map, const MY_BITMAP *map2) {
   my_bitmap_map *to = map->bitmap, *from = map2->bitmap, *end;
-  DBUG_ASSERT(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap && map2->bitmap && map->n_bits == map2->n_bits);
+  assert(map->n_bits > 0);
   end = map->last_word_ptr;
 
   for (; to <= end; to++, from++) *to = *from;
@@ -506,8 +440,8 @@ uint bitmap_get_first_set(const MY_BITMAP *map) {
   uint word_pos;
   my_bitmap_map *data_ptr, *end = map->last_word_ptr;
 
-  DBUG_ASSERT(map->bitmap);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap);
+  assert(map->n_bits > 0);
   data_ptr = map->bitmap;
 
   for (word_pos = 0; data_ptr < end; data_ptr++, word_pos++)
@@ -531,8 +465,8 @@ uint bitmap_get_next_set(const MY_BITMAP *map, uint bitmap_bit) {
   unsigned char *ptr = (unsigned char *)&first_word;
   my_bitmap_map *data_ptr, *end = map->last_word_ptr;
 
-  DBUG_ASSERT(map->bitmap);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap);
+  assert(map->n_bits > 0);
 
   /* Look for the next bit */
   bitmap_bit++;
@@ -561,8 +495,8 @@ uint bitmap_get_first(const MY_BITMAP *map) {
   uint word_pos;
   my_bitmap_map *data_ptr, *end = map->last_word_ptr;
 
-  DBUG_ASSERT(map->bitmap);
-  DBUG_ASSERT(map->n_bits > 0);
+  assert(map->bitmap);
+  assert(map->n_bits > 0);
   data_ptr = map->bitmap;
 
   for (word_pos = 0; data_ptr < end; data_ptr++, word_pos++)
@@ -571,17 +505,42 @@ uint bitmap_get_first(const MY_BITMAP *map) {
   return get_first_not_set(*map->last_word_ptr | map->last_word_mask, word_pos);
 }
 
-uint bitmap_lock_set_next(MY_BITMAP *map) {
-  uint bit_found;
-  bitmap_lock(map);
-  bit_found = bitmap_set_next(map);
-  bitmap_unlock(map);
-  return bit_found;
-}
+/**
+    Copy as many bits as 'dst' can hold from 'src', but no more than
+    max_bits_to_copy bits. 'src' and 'dst' should not overlap. If 'dst'
+    and 'src' have the same size (in bits), and this is less or equal than
+    max_bits_to_copy, this function behaves identical to bitmap_copy().
 
-void bitmap_lock_clear_bit(MY_BITMAP *map, uint bitmap_bit) {
-  bitmap_lock(map);
-  DBUG_ASSERT(map->bitmap && bitmap_bit < map->n_bits);
-  bitmap_clear_bit(map, bitmap_bit);
-  bitmap_unlock(map);
+    @param dst The destination bitmap.
+    @param src The source bitmap.
+    @param max_bits_to_copy The maximal number of bits to copy.
+    @return The number of bits copied.
+*/
+uint bitmap_n_copy(MY_BITMAP *dst, const MY_BITMAP *src,
+                   uint max_bits_to_copy) {
+  assert(bitmap_is_valid(dst));
+  assert(bitmap_is_valid(src));
+  // Since bitmap_copy() also does this.
+  assert(dst->n_bits > 0);
+  assert(src->n_bits > 0);
+  const uint input_bits = std::min(src->n_bits, max_bits_to_copy);
+  if (input_bits >= dst->n_bits) {
+    memcpy(dst->bitmap, src->bitmap, bitmap_buffer_size(dst->n_bits));
+    return dst->n_bits;
+  } else {
+    const uint full_words = input_bits / 32;
+    // Number of bits in the last (incomplete) word.
+    const uint tail_bits = input_bits % 32;
+    memcpy(dst->bitmap, src->bitmap, full_words * 4);
+    /*
+       We must not overwrite bits in the range [input_bits..dst->n_bits-1]
+       in 'dst'.
+    */
+    if (tail_bits > 0) {
+      dst->bitmap[full_words] =
+          (dst->bitmap[full_words] & (~0U << tail_bits)) |
+          (src->bitmap[full_words] & (~0U >> (32 - tail_bits)));
+    }
+    return input_bits;
+  }
 }
