@@ -28,9 +28,9 @@
 #ifdef _WIN32
 
 #include <assert.h>
-#include "m_ctype.h"
 
 #include "my_sys.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysys_priv.h"
 
 extern CHARSET_INFO my_charset_utf16le_bin;
@@ -56,7 +56,6 @@ extern CHARSET_INFO my_charset_utf16le_bin;
 
   @param file Input stream
 
-  @return
   @retval  0 if file is not Windows console
   @retval  1 if file is Windows console
 */
@@ -97,9 +96,10 @@ char *my_win_console_readline(const CHARSET_INFO *cs, char *mbbuf,
   SetConsoleMode(
       console, ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT | ENABLE_ECHO_INPUT);
 
-  if (!ReadConsoleW(console, u16buf, MAX_NUM_OF_CHARS_TO_READ, &nchars, NULL)) {
+  if (!ReadConsoleW(console, u16buf, MAX_NUM_OF_CHARS_TO_READ, &nchars,
+                    nullptr)) {
     SetConsoleMode(console, console_mode);
-    return NULL;
+    return nullptr;
   }
 
   *nread = nchars;
@@ -198,11 +198,11 @@ static size_t my_mbstou16s(const CHARSET_INFO *cs, const uchar *from,
 void my_win_console_write(const CHARSET_INFO *cs, const char *data,
                           size_t datalen) {
   static wchar_t u16buf[MAX_CONSOLE_LINE_SIZE + 1];
-  size_t nchars = my_mbstou16s(cs, (const uchar *)data, datalen, u16buf,
-                               sizeof(u16buf) / sizeof(u16buf[0]));
+  const size_t nchars = my_mbstou16s(cs, (const uchar *)data, datalen, u16buf,
+                                     sizeof(u16buf) / sizeof(u16buf[0]));
   DWORD nwritten;
   WriteConsoleW(GetStdHandle(STD_OUTPUT_HANDLE), u16buf, (DWORD)nchars,
-                &nwritten, NULL);
+                &nwritten, nullptr);
 }
 
 /**
@@ -215,7 +215,7 @@ void my_win_console_write(const CHARSET_INFO *cs, const char *data,
   @param c   Character (single byte)
 */
 void my_win_console_putc(const CHARSET_INFO *cs, int c) {
-  char ch = (char)c;
+  const char ch = (char)c;
   my_win_console_write(cs, &ch, 1);
 }
 
@@ -235,7 +235,7 @@ void my_win_console_fputs(const CHARSET_INFO *cs, const char *data) {
 void my_win_console_vfprintf(const CHARSET_INFO *cs, const char *fmt,
                              va_list args) {
   static char buff[MAX_CONSOLE_LINE_SIZE + 1];
-  size_t len = vsnprintf(buff, sizeof(buff) - 1, fmt, args);
+  const size_t len = vsnprintf(buff, sizeof(buff) - 1, fmt, args);
   my_win_console_write(cs, buff, len);
 }
 
@@ -246,7 +246,7 @@ void my_win_console_vfprintf(const CHARSET_INFO *cs, const char *fmt,
   (Typically to utf8mb4).
   Translated parameters are allocated using my_once_alloc().
 
-  @param      tocs    Character set to convert parameters to.
+  @param      cs      Character set to convert parameters to.
   @param[out] argc    Write number of parameters here
   @param[out] argv    Write pointer to allocated parameters here.
 */
@@ -256,15 +256,16 @@ int my_win_translate_command_line_args(const CHARSET_INFO *cs, int *argc,
   char **av;
   wchar_t *command_line = GetCommandLineW();
   wchar_t **wargs = CommandLineToArgvW(command_line, &ac);
-  size_t nbytes = (ac + 1) * sizeof(char *);
+  const size_t nbytes = (ac + 1) * sizeof(char *);
 
   /* Allocate new command line parameter */
   av = (char **)my_once_alloc(nbytes, MYF(MY_ZEROFILL));
 
   for (i = 0; i < ac; i++) {
     uint dummy_errors;
-    size_t arg_len = wcslen(wargs[i]);
-    size_t len, alloced_len = arg_len * cs->mbmaxlen + 1;
+    const size_t arg_len = wcslen(wargs[i]);
+    size_t len;
+    const size_t alloced_len = arg_len * cs->mbmaxlen + 1;
     av[i] = (char *)my_once_alloc(alloced_len, MYF(0));
     len = my_convert(av[i], alloced_len, cs, (const char *)wargs[i],
                      arg_len * sizeof(wchar_t), &my_charset_utf16le_bin,
