@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -34,13 +35,14 @@
 #include <stdlib.h>
 #include <sys/types.h>
 
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_sys.h"
+#include "mysql/strings/m_ctype.h"
+#include "template_utils.h"
 #include "typelib.h"
 
 #define is_field_separator(X) ((X) == ',' || (X) == '=')
@@ -73,7 +75,7 @@ int find_type_or_exit(const char *x, TYPELIB *typelib, const char *option) {
                         FIND_TYPE_ALLOW_NUMBER
                         FIND_TYPE_COMMA_TERM.
                         FIND_TYPE_NO_OVERWRITE can be passed but is
-                        superfluous (is always implicitely on).
+                        redundant (is always implicitly on).
 
   @retval
     -1  Too many matching values
@@ -91,8 +93,8 @@ int find_type(const char *x, const TYPELIB *typelib, uint flags) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("x: '%s'  lib: %p", x, typelib));
 
-  DBUG_ASSERT(!(flags & ~(FIND_TYPE_NO_PREFIX | FIND_TYPE_ALLOW_NUMBER |
-                          FIND_TYPE_NO_OVERWRITE | FIND_TYPE_COMMA_TERM)));
+  assert(!(flags & ~(FIND_TYPE_NO_PREFIX | FIND_TYPE_ALLOW_NUMBER |
+                     FIND_TYPE_NO_OVERWRITE | FIND_TYPE_COMMA_TERM)));
   if (!typelib->count) {
     DBUG_PRINT("exit", ("no count"));
     return 0;
@@ -144,7 +146,7 @@ const char *get_type(TYPELIB *typelib, uint nr) {
 }
 
 /**
-  Create an integer value to represent the supplied comma-seperated
+  Create an integer value to represent the supplied comma-separated
   string where each string in the TYPELIB denotes a bit position.
 
   @param x      string to decompose
@@ -252,7 +254,7 @@ static TYPELIB on_off_default_typelib = {
 static int parse_name(const TYPELIB *lib, const char **strpos,
                       const char *end) {
   const char *pos = *strpos;
-  int find = find_type(pos, lib, FIND_TYPE_COMMA_TERM);
+  const int find = find_type(pos, lib, FIND_TYPE_COMMA_TERM);
   for (; pos != end && *pos != '=' && *pos != ','; pos++)
     ;
   *strpos = pos;
@@ -309,7 +311,7 @@ uint64_t find_set_from_flags(const TYPELIB *lib, int default_name,
       const char *pos = start;
       uint value;
 
-      int flag_no = parse_name(lib, &pos, end);
+      const int flag_no = parse_name(lib, &pos, end);
       if (flag_no <= 0) goto err;
 
       if (flag_no == default_name) {
@@ -317,7 +319,7 @@ uint64_t find_set_from_flags(const TYPELIB *lib, int default_name,
         if (set_defaults) goto err;
         set_defaults = true;
       } else {
-        uint64_t bit = (1ULL << (flag_no - 1));
+        const uint64_t bit = (1ULL << (flag_no - 1));
         /* parse the '=on|off|default' */
         if ((flags_to_clear | flags_to_set) & bit || pos >= end ||
             *pos++ != '=' ||
