@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,6 +34,7 @@
 #include "mysqld_error.h"
 #include "plugin/connection_control/connection_control_coordinator.h" /* g_connection_event_coordinator */
 #include "plugin/connection_control/connection_delay_api.h" /* connection_delay apis */
+#include "template_utils.h"
 
 static SERVICE_TYPE(registry) *reg_srv = nullptr;
 SERVICE_TYPE(log_builtins) *log_bi = nullptr;
@@ -41,7 +43,7 @@ SERVICE_TYPE(log_builtins_string) *log_bs = nullptr;
 namespace connection_control {
 class Connection_control_error_handler : public Error_handler {
  public:
-  void handle_error(longlong errcode, ...) {
+  void handle_error(longlong errcode, ...) override {
     va_list vl;
     va_start(vl, errcode);
     LogPluginErrV(ERROR_LEVEL, errcode, vl);
@@ -60,8 +62,51 @@ using connection_control::Error_handler;
 Connection_control_statistics g_statistics;
 Connection_control_variables g_variables;
 
-Connection_event_coordinator *g_connection_event_coordinator = 0;
-MYSQL_PLUGIN connection_control_plugin_info = 0;
+Connection_event_coordinator *g_connection_event_coordinator = nullptr;
+MYSQL_PLUGIN connection_control_plugin_info = nullptr;
+
+/* Performance Schema instrumentation */
+
+PSI_mutex_key key_connection_delay_mutex = PSI_NOT_INSTRUMENTED;
+
+static PSI_mutex_info all_connection_delay_mutex_info[] = {
+    {&key_connection_delay_mutex, "connection_delay_mutex", 0, 0,
+     PSI_DOCUMENT_ME}};
+
+PSI_rwlock_key key_connection_event_delay_lock;
+
+static PSI_rwlock_info all_connection_delay_rwlock_info[] = {
+    {&key_connection_event_delay_lock, "connection_event_delay_lock",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
+
+PSI_cond_key key_connection_delay_wait = PSI_NOT_INSTRUMENTED;
+
+static PSI_cond_info all_connection_delay_cond_info[] = {
+    {&key_connection_delay_wait, "connection_delay_wait_condition", 0, 0,
+     PSI_DOCUMENT_ME}};
+
+PSI_stage_info stage_waiting_in_connection_control_plugin = {
+    0, "Waiting in connection_control plugin", 0, PSI_DOCUMENT_ME};
+
+static PSI_stage_info *all_connection_delay_stage_info[] = {
+    &stage_waiting_in_connection_control_plugin};
+
+static void init_performance_schema() {
+  const char *category = "conn_delay";
+
+  int count_mutex = array_elements(all_connection_delay_mutex_info);
+  mysql_mutex_register(category, all_connection_delay_mutex_info, count_mutex);
+
+  int count_rwlock = array_elements(all_connection_delay_rwlock_info);
+  mysql_rwlock_register(category, all_connection_delay_rwlock_info,
+                        count_rwlock);
+
+  int count_cond = array_elements(all_connection_delay_cond_info);
+  mysql_cond_register(category, all_connection_delay_cond_info, count_cond);
+
+  int count_stage = array_elements(all_connection_delay_stage_info);
+  mysql_stage_register(category, all_connection_delay_stage_info, count_stage);
+}
 
 /**
   event_notify() implementation for connection_control
@@ -106,6 +151,12 @@ static int connection_control_notify(MYSQL_THD thd,
 */
 
 static int connection_control_init(MYSQL_PLUGIN plugin_info) {
+  /*
+    Declare all performance schema instrumentation up front,
+    so it is discoverable.
+  */
+  init_performance_schema();
+
   // Initialize error logging service.
   if (init_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs)) return 1;
 
@@ -137,11 +188,11 @@ static int connection_control_init(MYSQL_PLUGIN plugin_info) {
   @returns success
 */
 
-static int connection_control_deinit(void *arg MY_ATTRIBUTE((unused))) {
+static int connection_control_deinit(void *arg [[maybe_unused]]) {
   delete g_connection_event_coordinator;
-  g_connection_event_coordinator = 0;
+  g_connection_event_coordinator = nullptr;
   connection_control::deinit_connection_delay_event();
-  connection_control_plugin_info = 0;
+  connection_control_plugin_info = nullptr;
 
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
   return 0;
@@ -150,7 +201,7 @@ static int connection_control_deinit(void *arg MY_ATTRIBUTE((unused))) {
 /** Connection_control plugin descriptor */
 static struct st_mysql_audit connection_control_descriptor = {
     MYSQL_AUDIT_INTERFACE_VERSION, /* interface version */
-    NULL,                          /* release_thd() */
+    nullptr,                       /* release_thd() */
     connection_control_notify,     /* event_notify() */
     {
         0, /* MYSQL_AUDIT_GENERAL_CLASS */
@@ -184,9 +235,10 @@ static struct st_mysql_audit connection_control_descriptor = {
     @retval 1 Value is not within valid bounds
 */
 
-static int check_failed_connections_threshold(
-    MYSQL_THD thd MY_ATTRIBUTE((unused)), SYS_VAR *var MY_ATTRIBUTE((unused)),
-    void *save MY_ATTRIBUTE((unused)), struct st_mysql_value *value) {
+static int check_failed_connections_threshold(MYSQL_THD thd [[maybe_unused]],
+                                              SYS_VAR *var [[maybe_unused]],
+                                              void *save [[maybe_unused]],
+                                              struct st_mysql_value *value) {
   longlong new_value;
   if (value->val_int(value, &new_value)) return 1; /* NULL value */
 
@@ -212,9 +264,10 @@ static int check_failed_connections_threshold(
   connection_control_failed_connections_threshold
 */
 
-static void update_failed_connections_threshold(
-    MYSQL_THD thd MY_ATTRIBUTE((unused)), SYS_VAR *var MY_ATTRIBUTE((unused)),
-    void *var_ptr MY_ATTRIBUTE((unused)), const void *save) {
+static void update_failed_connections_threshold(MYSQL_THD thd [[maybe_unused]],
+                                                SYS_VAR *var [[maybe_unused]],
+                                                void *var_ptr [[maybe_unused]],
+                                                const void *save) {
   /*
     This won't result in overflow because we have already checked that this is
     within valid bounds.
@@ -251,12 +304,12 @@ static MYSQL_SYSVAR_LONGLONG(
     @retval 1 Value is not within valid bounds
 */
 
-static int check_min_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
-                                      SYS_VAR *var MY_ATTRIBUTE((unused)),
-                                      void *save MY_ATTRIBUTE((unused)),
+static int check_min_connection_delay(MYSQL_THD thd [[maybe_unused]],
+                                      SYS_VAR *var [[maybe_unused]],
+                                      void *save [[maybe_unused]],
                                       struct st_mysql_value *value) {
   long long new_value;
-  int64 existing_value = g_variables.max_connection_delay;
+  const int64 existing_value = g_variables.max_connection_delay;
   if (value->val_int(value, &new_value)) return 1; /* NULL value */
 
   if (new_value >= connection_control::MIN_DELAY &&
@@ -280,9 +333,9 @@ static int check_min_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
   @param save       New value for connection_control_min_connection_delay
 */
 
-static void update_min_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
-                                        SYS_VAR *var MY_ATTRIBUTE((unused)),
-                                        void *var_ptr MY_ATTRIBUTE((unused)),
+static void update_min_connection_delay(MYSQL_THD thd [[maybe_unused]],
+                                        SYS_VAR *var [[maybe_unused]],
+                                        void *var_ptr [[maybe_unused]],
                                         const void *save) {
   longlong new_value = *(reinterpret_cast<const longlong *>(save));
   g_variables.min_connection_delay = (int64)new_value;
@@ -315,12 +368,12 @@ static MYSQL_SYSVAR_LONGLONG(
     @retval 1 Value is not within valid bounds
 */
 
-static int check_max_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
-                                      SYS_VAR *var MY_ATTRIBUTE((unused)),
-                                      void *save MY_ATTRIBUTE((unused)),
+static int check_max_connection_delay(MYSQL_THD thd [[maybe_unused]],
+                                      SYS_VAR *var [[maybe_unused]],
+                                      void *save [[maybe_unused]],
                                       struct st_mysql_value *value) {
   long long new_value;
-  int64 existing_value = g_variables.min_connection_delay;
+  const int64 existing_value = g_variables.min_connection_delay;
   if (value->val_int(value, &new_value)) return 1; /* NULL value */
 
   if (new_value >= connection_control::MIN_DELAY &&
@@ -344,9 +397,9 @@ static int check_max_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
   @param save       New value for connection_control_max_connection_delay
 */
 
-static void update_max_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
-                                        SYS_VAR *var MY_ATTRIBUTE((unused)),
-                                        void *var_ptr MY_ATTRIBUTE((unused)),
+static void update_max_connection_delay(MYSQL_THD thd [[maybe_unused]],
+                                        SYS_VAR *var [[maybe_unused]],
+                                        void *var_ptr [[maybe_unused]],
                                         const void *save) {
   longlong new_value = *(reinterpret_cast<const longlong *>(save));
   g_variables.max_connection_delay = (int64)new_value;
@@ -364,11 +417,58 @@ static MYSQL_SYSVAR_LONGLONG(
     connection_control::DEFAULT_MAX_DELAY, connection_control::MIN_DELAY,
     connection_control::MAX_DELAY, 1);
 
+/**
+  update() function for exempt_unknown_users
+
+  Updates g_connection_event_coordinator with new value.
+  Also notifies observers about the update.
+
+  @param thd        Not used.
+  @param var        Not used.
+  @param var_ptr    Variable information
+  @param save       New value for exempt_unknown_users
+*/
+
+static void update_exempt_unknown_users(MYSQL_THD thd [[maybe_unused]],
+                                        SYS_VAR *var [[maybe_unused]],
+                                        void *var_ptr [[maybe_unused]],
+                                        const void *save) {
+  bool new_value = *(reinterpret_cast<const bool *>(save));
+  g_variables.exempt_unknown_users = new_value;
+  Connection_control_error_handler error_handler;
+  g_connection_event_coordinator->notify_sys_var(
+      &error_handler, OPT_EXEMPT_UNKNOWN_USERS, &new_value);
+}
+
+/** Declaration of connection_control_max_connection_delay */
+static MYSQL_SYSVAR_BOOL(
+    exempt_unknown_users, g_variables.exempt_unknown_users, PLUGIN_VAR_RQCMDARG,
+    "Skip delay for unknown users (like load balancer checking service "
+    "availability). Default is OFF.",
+    nullptr, update_exempt_unknown_users, false);
+
 /** Array of system variables. Used in plugin declaration. */
 SYS_VAR *connection_control_system_variables[OPT_LAST + 1] = {
     MYSQL_SYSVAR(failed_connections_threshold),
     MYSQL_SYSVAR(min_connection_delay), MYSQL_SYSVAR(max_connection_delay),
-    NULL};
+    MYSQL_SYSVAR(exempt_unknown_users), nullptr};
+
+/**
+  Helper function to display value for status variable defined by its index
+  within the array of all status variables.
+  @param var_index  Status variable index.
+  @param var  Status variable structure
+  @param buff Value buffer.
+  @returns Always returns success.
+*/
+static int show_status_variable(int var_index, SHOW_VAR *var, char *buff) {
+  var->type = SHOW_LONGLONG;
+  var->value = buff;
+  auto *value = reinterpret_cast<longlong *>(buff);
+  const int64 current_val = g_statistics.stats_array[var_index].load();
+  *value = static_cast<longlong>(current_val);
+  return 0;
+}
 
 /**
   Function to display value for status variable :
@@ -381,14 +481,22 @@ SYS_VAR *connection_control_system_variables[OPT_LAST + 1] = {
   @returns Always returns success.
 */
 
-static int show_delay_generated(MYSQL_THD, SHOW_VAR *var, char *buff) {
-  var->type = SHOW_LONGLONG;
-  var->value = buff;
-  longlong *value = reinterpret_cast<longlong *>(buff);
-  int64 current_val =
-      g_statistics.stats_array[STAT_CONNECTION_DELAY_TRIGGERED].load();
-  *value = static_cast<longlong>(current_val);
-  return 0;
+static int show_delay_generated(MYSQL_THD thd [[maybe_unused]], SHOW_VAR *var,
+                                char *buff) {
+  return show_status_variable(STAT_CONNECTION_DELAY_TRIGGERED, var, buff);
+}
+
+/**
+  Function to display value for status variable :
+  Component_connection_control_exempted_unknown_users
+  @param thd  MYSQL_THD handle. Unused.
+  @param var  Status variable structure
+  @param buff Value buffer.
+  @returns Always returns success.
+*/
+static int show_exempted_users(MYSQL_THD thd [[maybe_unused]], SHOW_VAR *var,
+                               char *buff) {
+  return show_status_variable(STAT_CONNECTION_EXEMPTED_USERS, var, buff);
 }
 
 /** Array of status variables. Used in plugin declaration. */
@@ -396,35 +504,37 @@ SHOW_VAR
 connection_control_status_variables[STAT_LAST + 1] = {
     {"Connection_control_delay_generated", (char *)&show_delay_generated,
      SHOW_FUNC, SHOW_SCOPE_GLOBAL},
-    {0, 0, enum_mysql_show_type(0), enum_mysql_show_scope(0)}};
+    {"Connection_control_exempted_unknown_users", (char *)&show_exempted_users,
+     SHOW_FUNC, SHOW_SCOPE_GLOBAL},
+    {nullptr, nullptr, enum_mysql_show_type(0), enum_mysql_show_scope(0)}};
 
 mysql_declare_plugin(audit_log){
     MYSQL_AUDIT_PLUGIN,                  /* plugin type                   */
     &connection_control_descriptor,      /* type specific descriptor      */
     "CONNECTION_CONTROL",                /* plugin name                   */
-    "Oracle Inc",                        /* author                        */
+    PLUGIN_AUTHOR_ORACLE,                /* author                        */
     "Connection event processing",       /* description                   */
     PLUGIN_LICENSE_GPL,                  /* license                       */
     connection_control_init,             /* plugin initializer            */
-    NULL,                                /* plugin check uninstall        */
+    nullptr,                             /* plugin check uninstall        */
     connection_control_deinit,           /* plugin deinitializer          */
     0x0100,                              /* version                       */
     connection_control_status_variables, /* status variables              */
     connection_control_system_variables, /* system variables              */
-    NULL,                                /* reserverd                     */
+    nullptr,                             /* reserved                      */
     0                                    /* flags                         */
 },
     {MYSQL_INFORMATION_SCHEMA_PLUGIN,
      &connection_control_failed_attempts_view,
      "CONNECTION_CONTROL_FAILED_LOGIN_ATTEMPTS",
-     "Oracle Inc",
+     PLUGIN_AUTHOR_ORACLE,
      "I_S table providing a view into failed attempts statistics",
      PLUGIN_LICENSE_GPL,
      connection_control_failed_attempts_view_init,
-     NULL,
-     NULL,
+     nullptr,
+     nullptr,
      0x0100,
-     NULL,
-     NULL,
-     NULL,
+     nullptr,
+     nullptr,
+     nullptr,
      0} mysql_declare_plugin_end;
