@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,10 +26,10 @@
 #include <mysql/psi/mysql_thread.h>
 #include <time.h>
 
-#include "m_ctype.h" /* my_charset_bin */
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_systime.h"
+#include "mysql/strings/m_ctype.h" /* my_charset_bin */
 #include "mysqld_error.h"
 #include "plugin/connection_control/connection_control.h"
 #include "plugin/connection_control/security_context_wrapper.h"
@@ -54,9 +55,9 @@ struct st_mysql_information_schema connection_control_failed_attempts_view = {
 /** Fields of information_schema.connection_control_failed_attempts */
 static ST_FIELD_INFO failed_attempts_view_fields[] = {
     {"USERHOST", USERNAME_LENGTH + HOSTNAME_LENGTH + 6, MYSQL_TYPE_STRING, 0,
-     MY_I_S_UNSIGNED, 0, 0},
-    {"FAILED_ATTEMPTS", 16, MYSQL_TYPE_LONG, 0, MY_I_S_UNSIGNED, 0, 0},
-    {0, 0, MYSQL_TYPE_NULL, 0, 0, 0, 0}};
+     MY_I_S_UNSIGNED, nullptr, 0},
+    {"FAILED_ATTEMPTS", 16, MYSQL_TYPE_LONG, 0, MY_I_S_UNSIGNED, nullptr, 0},
+    {nullptr, 0, MYSQL_TYPE_NULL, 0, 0, nullptr, 0}};
 
 namespace connection_control {
 
@@ -74,21 +75,17 @@ int64 MAX_DELAY = INT_MAX32;
 
 /** variables used by connection_delay.cc */
 static mysql_rwlock_t connection_event_delay_lock;
-static PSI_rwlock_key key_connection_event_delay_lock;
-static PSI_rwlock_info all_rwlocks[] = {{&key_connection_event_delay_lock,
-                                         "connection_event_delay_lock", 0, 0,
-                                         PSI_DOCUMENT_ME}};
 
-static opt_connection_control opt_enums[] = {OPT_FAILED_CONNECTIONS_THRESHOLD,
-                                             OPT_MIN_CONNECTION_DELAY,
-                                             OPT_MAX_CONNECTION_DELAY};
-size_t opt_enums_size = 3;
+static opt_connection_control opt_enums[] = {
+    OPT_FAILED_CONNECTIONS_THRESHOLD, OPT_MIN_CONNECTION_DELAY,
+    OPT_MAX_CONNECTION_DELAY, OPT_EXEMPT_UNKNOWN_USERS};
+size_t opt_enums_size = std::size(opt_enums);
 
 static stats_connection_control status_vars_enums[] = {
-    STAT_CONNECTION_DELAY_TRIGGERED};
-size_t status_vars_enums_size = 1;
+    STAT_CONNECTION_DELAY_TRIGGERED, STAT_CONNECTION_EXEMPTED_USERS};
+size_t status_vars_enums_size = std::size(status_vars_enums);
 
-static Connection_delay_action *g_max_failed_connection_handler = 0;
+static Connection_delay_action *g_max_failed_connection_handler = nullptr;
 
 Sql_string I_S_CONNECTION_CONTROL_FAILED_ATTEMPTS_USERHOST(
     "information_schema.connection_control_failed_login_attempts.userhost");
@@ -98,7 +95,7 @@ Sql_string I_S_CONNECTION_CONTROL_FAILED_ATTEMPTS_USERHOST(
 
   @returns 1 to indicate that entry is a match
 */
-int match_all_entries(const uchar *) { return 1; }
+int match_all_entries(const uchar *, void *) { return 1; }
 
 /**
   Callback function for LF hash to get key information
@@ -119,7 +116,7 @@ const uchar *connection_delay_event_hash_key(const uchar *el, size_t *length) {
   const Connection_event_record *const *entry;
   const Connection_event_record *entry_info;
   entry = reinterpret_cast<const Connection_event_record *const *>(el);
-  DBUG_ASSERT(entry != NULL);
+  assert(entry != nullptr);
   entry_info = *entry;
   *length = entry_info->get_length();
   return (const_cast<uchar *>(entry_info->get_userhost()));
@@ -149,14 +146,14 @@ Connection_delay_event::Connection_delay_event() {
 */
 
 bool Connection_delay_event::create_or_update_entry(const Sql_string &s) {
-  Connection_event_record **searched_entry = NULL;
-  Connection_event_record *searched_entry_info = NULL;
-  Connection_event_record *new_entry = NULL;
+  Connection_event_record **searched_entry = nullptr;
+  Connection_event_record *searched_entry_info = nullptr;
+  Connection_event_record *new_entry = nullptr;
   int insert_status;
-  DBUG_ENTER("Connection_delay_event::create_or_update_entry");
+  DBUG_TRACE;
 
   LF_PINS *pins = lf_hash_get_pins(&m_entries);
-  if (unlikely(pins == NULL)) DBUG_RETURN(true);
+  if (unlikely(pins == nullptr)) return true;
 
   searched_entry = reinterpret_cast<Connection_event_record **>(
       lf_hash_search(&m_entries, pins, s.c_str(), s.length()));
@@ -164,11 +161,11 @@ bool Connection_delay_event::create_or_update_entry(const Sql_string &s) {
   if (searched_entry && (searched_entry != MY_LF_ERRPTR)) {
     /* We found an entry, so increment the count */
     searched_entry_info = *searched_entry;
-    DBUG_ASSERT(searched_entry_info != NULL);
+    assert(searched_entry_info != nullptr);
     searched_entry_info->inc_count();
     lf_hash_search_unpin(pins);
     lf_hash_put_pins(pins);
-    DBUG_RETURN(false);
+    return false;
   } else {
     /* No entry found, so try to add new entry */
     lf_hash_search_unpin(pins);
@@ -178,7 +175,7 @@ bool Connection_delay_event::create_or_update_entry(const Sql_string &s) {
 
     if (likely(insert_status == 0)) {
       lf_hash_put_pins(pins);
-      DBUG_RETURN(false);
+      return false;
     } else {
       /*
         OOM. We are likely in bigger trouble than just
@@ -186,8 +183,8 @@ bool Connection_delay_event::create_or_update_entry(const Sql_string &s) {
       */
       lf_hash_put_pins(pins);
       delete new_entry;
-      new_entry = NULL;
-      DBUG_RETURN(true);
+      new_entry = nullptr;
+      return true;
     }
   }
 }
@@ -203,9 +200,9 @@ bool Connection_delay_event::create_or_update_entry(const Sql_string &s) {
 */
 
 bool Connection_delay_event::remove_entry(const Sql_string &s) {
-  Connection_event_record **searched_entry = NULL;
-  Connection_event_record *searched_entry_info = NULL;
-  DBUG_ENTER("Connection_delay_event::reset_entry");
+  Connection_event_record **searched_entry = nullptr;
+  Connection_event_record *searched_entry_info = nullptr;
+  DBUG_TRACE;
 
   LF_PINS *pins = lf_hash_get_pins(&m_entries);
 
@@ -214,20 +211,20 @@ bool Connection_delay_event::remove_entry(const Sql_string &s) {
 
   if (searched_entry && searched_entry != MY_LF_ERRPTR) {
     searched_entry_info = *searched_entry;
-    DBUG_ASSERT(searched_entry_info != NULL);
-    int rc = lf_hash_delete(&m_entries, pins, s.c_str(), s.length());
+    assert(searched_entry_info != nullptr);
+    const int rc = lf_hash_delete(&m_entries, pins, s.c_str(), s.length());
     lf_hash_search_unpin(pins);
     lf_hash_put_pins(pins);
     if (rc == 0) {
       /* free memory upon successful deletion */
       delete searched_entry_info;
     }
-    DBUG_RETURN(rc != 0);
+    return rc != 0;
   } else {
     /* No entry found. */
     lf_hash_search_unpin(pins);
     lf_hash_put_pins(pins);
-    DBUG_RETURN(true);
+    return true;
   }
 }
 
@@ -243,11 +240,11 @@ bool Connection_delay_event::remove_entry(const Sql_string &s) {
 */
 
 bool Connection_delay_event::match_entry(const Sql_string &s, void *value) {
-  Connection_event_record **searched_entry = NULL;
-  Connection_event_record *searched_entry_info = NULL;
+  Connection_event_record **searched_entry = nullptr;
+  Connection_event_record *searched_entry_info = nullptr;
   int64 count = DISABLE_THRESHOLD;
   bool error = true;
-  DBUG_ENTER("Connection_delay_event::match_entry");
+  DBUG_TRACE;
 
   LF_PINS *pins = lf_hash_get_pins(&m_entries);
 
@@ -264,41 +261,36 @@ bool Connection_delay_event::match_entry(const Sql_string &s, void *value) {
   lf_hash_put_pins(pins);
   *(reinterpret_cast<int64 *>(value)) = count;
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
   Delete all entries from hash and free memory
-
-  @returns Reset was successful or not
-    @retval true All entries removed.
-    @retval false Failed to remove some entries
 */
 
 void Connection_delay_event::reset_all() {
-  Connection_event_record **searched_entry = NULL;
-  DBUG_ENTER("Connection_delay_event::reset_all");
+  Connection_event_record **searched_entry = nullptr;
+  DBUG_TRACE;
   LF_PINS *pins = lf_hash_get_pins(&m_entries);
 
   do {
     /* match anything */
     searched_entry = reinterpret_cast<Connection_event_record **>(
-        lf_hash_random_match(&m_entries, pins, match_all_entries, 0));
+        lf_hash_random_match(&m_entries, pins, match_all_entries, 0, nullptr));
 
-    if (searched_entry != NULL && searched_entry != MY_LF_ERRPTR &&
+    if (searched_entry != nullptr && searched_entry != MY_LF_ERRPTR &&
         (*searched_entry) &&
         !lf_hash_delete(&m_entries, pins, (*searched_entry)->get_userhost(),
                         (*searched_entry)->get_length())) {
       delete (*searched_entry);
-      *searched_entry = NULL;
+      *searched_entry = nullptr;
     } else {
       /* Failed to delete didn't remove any pins */
       lf_hash_search_unpin(pins);
     }
-  } while (searched_entry != 0);
+  } while (searched_entry != nullptr);
 
   lf_hash_put_pins(pins);
-  DBUG_VOID_RETURN;
 }
 
 /** This works because connction_delay_IS_table is protected by wrlock */
@@ -315,7 +307,8 @@ void set_connection_delay_IS_table(TABLE *t) { connection_delay_IS_table = t; }
     @retval 1 Error
 */
 
-int connection_delay_IS_table_writer(const uchar *ptr) {
+int connection_delay_IS_table_writer(const uchar *ptr,
+                                     void *arg [[maybe_unused]]) {
   /* Always return "no match" so that we go through all entries */
   THD *thd = current_thd;
   const Connection_event_record *const *entry;
@@ -338,24 +331,23 @@ int connection_delay_IS_table_writer(const uchar *ptr) {
                      information_schema.connection_control_failed_attempts
 */
 
-void Connection_delay_event::fill_IS_table(TABLE_LIST *tables) {
-  DBUG_ENTER("Connection_delay_event::fill_IS_table");
+void Connection_delay_event::fill_IS_table(Table_ref *tables) {
+  DBUG_TRACE;
   TABLE *table = tables->table;
   set_connection_delay_IS_table(table);
   LF_PINS *pins = lf_hash_get_pins(&m_entries);
-  void *key = 0;
+  void *key = nullptr;
 
   do {
     key =
         lf_hash_random_match(&m_entries, pins,
                              /* Functor: match anything and store the fields */
-                             connection_delay_IS_table_writer, 0);
+                             connection_delay_IS_table_writer, 0, nullptr);
     /* Always unpin after lf_hash_random_match() */
     lf_hash_search_unpin(pins);
-  } while (key != 0);
+  } while (key != nullptr);
 
   lf_hash_put_pins(pins);
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -365,6 +357,7 @@ void Connection_delay_event::fill_IS_table(TABLE_LIST *tables) {
   triggered
   @param [in] min_delay         Lower cap on wait
   @param [in] max_delay         Upper cap on wait
+  @param [in] exempt_unknown_users Exempt unauthenticated sessions
   @param [in] sys_vars          System variables
   @param [in] sys_vars_size     Size of sys_vars array
   @param [in] status_vars       Status variables
@@ -374,12 +367,13 @@ void Connection_delay_event::fill_IS_table(TABLE_LIST *tables) {
 
 Connection_delay_action::Connection_delay_action(
     int64 threshold, int64 min_delay, int64 max_delay,
-    opt_connection_control *sys_vars, size_t sys_vars_size,
-    stats_connection_control *status_vars, size_t status_vars_size,
-    mysql_rwlock_t *lock)
+    bool exempt_unknown_users, opt_connection_control *sys_vars,
+    size_t sys_vars_size, stats_connection_control *status_vars,
+    size_t status_vars_size, mysql_rwlock_t *lock)
     : m_threshold(threshold),
       m_min_delay(min_delay),
       m_max_delay(max_delay),
+      m_exempt_unknown_users(exempt_unknown_users),
       m_lock(lock) {
   if (sys_vars_size) {
     for (uint i = 0; i < sys_vars_size; ++i) m_sys_vars.push_back(sys_vars[i]);
@@ -454,7 +448,7 @@ void Connection_delay_action::make_hash_key(MYSQL_THD thd, Sql_string &s) {
 
 void Connection_delay_action::conditional_wait(MYSQL_THD thd,
                                                ulonglong wait_time) {
-  DBUG_ENTER("Connection_delay_action::conditional_wait");
+  DBUG_TRACE;
 
   /** mysql_cond_timedwait requires wait time in timespec format */
   struct timespec abstime;
@@ -463,29 +457,14 @@ void Connection_delay_action::conditional_wait(MYSQL_THD thd,
 
   /** PSI_stage_info for thd_enter_cond/thd_exit_cond */
   PSI_stage_info old_stage;
-  PSI_stage_info stage_waiting_in_connection_control_plugin = {
-      0, "Waiting in connection_control plugin", 0, PSI_DOCUMENT_ME};
 
   /** Initialize mutex required for mysql_cond_timedwait */
   mysql_mutex_t connection_delay_mutex;
-  const char *category = "conn_delay";
-  PSI_mutex_key key_connection_delay_mutex;
-  PSI_mutex_info connection_delay_mutex_info[] = {
-      {&key_connection_delay_mutex, "connection_delay_mutex",
-       PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
-  int count_mutex = array_elements(connection_delay_mutex_info);
-  mysql_mutex_register(category, connection_delay_mutex_info, count_mutex);
   mysql_mutex_init(key_connection_delay_mutex, &connection_delay_mutex,
                    MY_MUTEX_INIT_FAST);
 
   /* Initialize condition to wait for */
   mysql_cond_t connection_delay_wait_condition;
-  PSI_cond_key key_connection_delay_wait;
-  PSI_cond_info connection_delay_wait_info[] = {
-      {&key_connection_delay_wait, "connection_delay_wait_condition", 0, 0,
-       PSI_DOCUMENT_ME}};
-  int count_cond = array_elements(connection_delay_wait_info);
-  mysql_cond_register(category, connection_delay_wait_info, count_cond);
   mysql_cond_init(key_connection_delay_wait, &connection_delay_wait_condition);
 
   /** Register wait condition with THD */
@@ -507,17 +486,33 @@ void Connection_delay_action::conditional_wait(MYSQL_THD thd,
 
   /* Finish waiting and deregister wait condition */
   mysql_mutex_unlock(&connection_delay_mutex);
-  thd_exit_cond(thd, &stage_waiting_in_connection_control_plugin, __func__,
-                __FILE__, __LINE__);
+  thd_exit_cond(thd, &old_stage, __func__, __FILE__, __LINE__);
 
   /* Cleanup */
   mysql_mutex_destroy(&connection_delay_mutex);
   mysql_cond_destroy(&connection_delay_wait_condition);
-  DBUG_VOID_RETURN;
+}
+
+static bool is_unauthenticated_connection(MYSQL_THD thd) {
+  Security_context_wrapper sctx_wrapper(thd);
+  const char *proxy_user = sctx_wrapper.get_proxy_user();
+  if (nullptr != proxy_user && *proxy_user != '\0') {
+    return false;
+  }
+  const char *priv_user = sctx_wrapper.get_priv_user();
+  if (nullptr != priv_user && *priv_user != '\0') {
+    return false;
+  }
+  const char *user = sctx_wrapper.get_user();
+  if (nullptr != user && *user != '\0') {
+    return false;
+  }
+  // no user supplied, unauthenticated
+  return true;
 }
 
 /**
-  @brief  Handle a connection event and if requried,
+  @brief  Handle a connection event and, if required,
   wait for random amount of time before returning.
 
   We only care about CONNECT and CHANGE_USER sub events.
@@ -537,21 +532,36 @@ bool Connection_delay_action::notify_event(
     MYSQL_THD thd, Connection_event_coordinator_services *coordinator,
     const mysql_event_connection *connection_event,
     Error_handler *error_handler) {
-  DBUG_ENTER("Connection_delay_action::notify");
+  DBUG_TRACE;
   bool error = false;
-  unsigned int subclass = connection_event->event_subclass;
+  const unsigned int subclass = connection_event->event_subclass;
   Connection_event_observer *self = this;
 
   if (subclass != MYSQL_AUDIT_CONNECTION_CONNECT &&
       subclass != MYSQL_AUDIT_CONNECTION_CHANGE_USER)
-    DBUG_RETURN(error);
+    return error;
 
   RD_lock rd_lock(m_lock);
 
-  int64 threshold = this->get_threshold();
+  const int64 threshold = this->get_threshold();
 
   /* If feature was disabled, return */
-  if (threshold <= DISABLE_THRESHOLD) DBUG_RETURN(error);
+  if (threshold <= DISABLE_THRESHOLD) return error;
+
+  /* Ignore failed connections from unauthenticated users if exemption was
+   * enabled */
+  if (connection_event->status != 0 && get_exempt_unknown_users() &&
+      is_unauthenticated_connection(thd)) {
+    // increment statistics
+    error = coordinator->notify_status_var(
+        &self, STAT_CONNECTION_EXEMPTED_USERS, ACTION_INC);
+    if (error) {
+      error_handler->handle_error(
+          ER_LOG_PRINTF_MSG,
+          "Failed to update connection delay user exempt stats");
+    }
+    return error;
+  }
 
   int64 current_count = 0;
   bool user_present = false;
@@ -575,7 +585,7 @@ bool Connection_delay_action::notify_event(
       have to consider current connection as well - Hence the usage
       of current_count + 1.
     */
-    ulonglong wait_time = get_wait_time((current_count + 1) - threshold);
+    const ulonglong wait_time = get_wait_time((current_count + 1) - threshold);
 
     if ((error = coordinator->notify_status_var(
              &self, STAT_CONNECTION_DELAY_TRIGGERED, ACTION_INC))) {
@@ -589,6 +599,10 @@ bool Connection_delay_action::notify_event(
     rd_lock.unlock();
     conditional_wait(thd, wait_time);
     rd_lock.lock();
+
+    /* Introduce a delay to check that connection delay status doesn't last
+     * longer than configured */
+    DBUG_EXECUTE_IF("delay_after_connection_delay", sleep(2););
   }
 
   if (connection_event->status) {
@@ -612,7 +626,7 @@ bool Connection_delay_action::notify_event(
     }
   }
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
@@ -621,8 +635,7 @@ bool Connection_delay_action::notify_event(
   @param [in] coordinator        Handle to coordinator
   @param [in] variable           Enum of variable
   @param [in] new_value          New value for variable
-  @param [out] error_buffer      Buffer to log error message if any
-  @param [in] error_buffer_size  Size of error buffer
+  @param [in] error_handler      Error handler object
 
   @returns processing status
     @retval false  Change in variable value processed successfully
@@ -634,16 +647,16 @@ bool Connection_delay_action::notify_sys_var(
     Connection_event_coordinator_services *coordinator,
     opt_connection_control variable, void *new_value,
     Error_handler *error_handler) {
-  DBUG_ENTER("Connection_delay_action::notify_sys_var");
+  DBUG_TRACE;
   bool error = true;
   Connection_event_observer *self = this;
 
-  WR_lock wr_lock(m_lock);
+  const WR_lock wr_lock(m_lock);
 
   switch (variable) {
     case OPT_FAILED_CONNECTIONS_THRESHOLD: {
-      int64 new_threshold = *(static_cast<int64 *>(new_value));
-      DBUG_ASSERT(new_threshold >= DISABLE_THRESHOLD);
+      const int64 new_threshold = *(static_cast<int64 *>(new_value));
+      assert(new_threshold >= DISABLE_THRESHOLD);
       set_threshold(new_threshold);
 
       if ((error = coordinator->notify_status_var(
@@ -655,7 +668,7 @@ bool Connection_delay_action::notify_sys_var(
     }
     case OPT_MIN_CONNECTION_DELAY:
     case OPT_MAX_CONNECTION_DELAY: {
-      int64 new_delay = *(static_cast<int64 *>(new_value));
+      const int64 new_delay = *(static_cast<int64 *>(new_value));
       if ((error =
                set_delay(new_delay, (variable == OPT_MIN_CONNECTION_DELAY)))) {
         error_handler->handle_error(
@@ -664,12 +677,17 @@ bool Connection_delay_action::notify_sys_var(
       }
       break;
     }
+    case OPT_EXEMPT_UNKNOWN_USERS: {
+      const bool new_flag = *(static_cast<bool *>(new_value));
+      set_exempt_unknown_users(new_flag);
+      break;
+    }
     default:
       /* Should never reach here. */
-      DBUG_ASSERT(false);
+      assert(false);
       error_handler->handle_error(ER_CONN_CONTROL_INVALID_CONN_DELAY_TYPE);
   };
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
@@ -680,16 +698,15 @@ bool Connection_delay_action::notify_sys_var(
 */
 void Connection_delay_action::init(
     Connection_event_coordinator_services *coordinator) {
-  DBUG_ENTER("Connection_delay_action::init");
-  DBUG_ASSERT(coordinator);
+  DBUG_TRACE;
+  assert(coordinator);
   bool retval;
   Connection_event_observer *subscriber = this;
-  WR_lock wr_lock(m_lock);
+  const WR_lock wr_lock(m_lock);
   retval = coordinator->register_event_subscriber(&subscriber, &m_sys_vars,
                                                   &m_stats_vars);
-  DBUG_ASSERT(!retval);
+  assert(!retval);
   if (retval) retval = false; /* Make compiler happy */
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -703,7 +720,7 @@ void Connection_delay_action::deinit() {
   m_stats_vars.clear();
   m_threshold = DISABLE_THRESHOLD;
   mysql_rwlock_unlock(m_lock);
-  m_lock = 0;
+  m_lock = nullptr;
 }
 
 /**
@@ -720,9 +737,9 @@ void Connection_delay_action::deinit() {
 
 static bool get_equal_condition_argument(Item *cond, Sql_string *eq_arg,
                                          const Sql_string &field_name) {
-  if (cond != 0 && cond->type() == Item::FUNC_ITEM) {
+  if (cond != nullptr && cond->type() == Item::FUNC_ITEM) {
     Item_func *func = static_cast<Item_func *>(cond);
-    if (func != NULL && func->functype() == Item_func::EQ_FUNC) {
+    if (func != nullptr && func->functype() == Item_func::EQ_FUNC) {
       Item_func_eq *eq_func = static_cast<Item_func_eq *>(func);
       if (eq_func->arguments()[0]->type() == Item::FIELD_ITEM &&
           my_strcasecmp(system_charset_info,
@@ -731,7 +748,7 @@ static bool get_equal_condition_argument(Item *cond, Sql_string *eq_arg,
         char buff[1024];
         String *res;
         String filter(buff, sizeof(buff), system_charset_info);
-        if (eq_func->arguments()[1] != NULL &&
+        if (eq_func->arguments()[1] != nullptr &&
             (res = eq_func->arguments()[1]->val_str(&filter))) {
           eq_arg->append(res->c_ptr_safe(), res->length());
           return false;
@@ -756,21 +773,21 @@ static bool get_equal_condition_argument(Item *cond, Sql_string *eq_arg,
   @param [in] cond    Condition if any.
 */
 
-void Connection_delay_action::fill_IS_table(THD *thd, TABLE_LIST *tables,
+void Connection_delay_action::fill_IS_table(THD *thd, Table_ref *tables,
                                             Item *cond) {
-  DBUG_ENTER("Connection_delay_action::fill_IS_table");
+  DBUG_TRACE;
   Security_context_wrapper sctx_wrapper(thd);
   if (!(sctx_wrapper.is_super_user() || sctx_wrapper.is_connection_admin()))
-    DBUG_VOID_RETURN;
-  WR_lock wr_lock(m_lock);
+    return;
+  const WR_lock wr_lock(m_lock);
   Sql_string eq_arg;
-  if (cond != 0 &&
+  if (cond != nullptr &&
       !get_equal_condition_argument(
           cond, &eq_arg, I_S_CONNECTION_CONTROL_FAILED_ATTEMPTS_USERHOST)) {
     int64 current_count = 0;
     if (m_userhost_hash.match_entry(eq_arg, (void *)&current_count)) {
       /* There are no matches given the condition */
-      DBUG_VOID_RETURN;
+      return;
     } else {
       /* There is exactly one matching userhost entry */
       TABLE *table = tables->table;
@@ -781,8 +798,6 @@ void Connection_delay_action::fill_IS_table(THD *thd, TABLE_LIST *tables,
     }
   } else
     m_userhost_hash.fill_IS_table(tables);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -797,16 +812,13 @@ bool init_connection_delay_event(
   /*
     1. Initialize lock(s)
   */
-  const char *category = "conn_control";
-  int count = array_elements(all_rwlocks);
-  mysql_rwlock_register(category, all_rwlocks, count);
   mysql_rwlock_init(key_connection_event_delay_lock,
                     &connection_event_delay_lock);
   g_max_failed_connection_handler = new Connection_delay_action(
       g_variables.failed_connections_threshold,
       g_variables.min_connection_delay, g_variables.max_connection_delay,
-      opt_enums, opt_enums_size, status_vars_enums, status_vars_enums_size,
-      &connection_event_delay_lock);
+      g_variables.exempt_unknown_users, opt_enums, opt_enums_size,
+      status_vars_enums, status_vars_enums_size, &connection_event_delay_lock);
   if (!g_max_failed_connection_handler) {
     error_handler->handle_error(ER_CONN_CONTROL_DELAY_ACTION_INIT_FAILED);
     return true;
@@ -822,7 +834,7 @@ bool init_connection_delay_event(
 
 void deinit_connection_delay_event() {
   if (g_max_failed_connection_handler) delete g_max_failed_connection_handler;
-  g_max_failed_connection_handler = 0;
+  g_max_failed_connection_handler = nullptr;
   mysql_rwlock_destroy(&connection_event_delay_lock);
   return;
 }
@@ -839,7 +851,7 @@ void deinit_connection_delay_event() {
   @returns Always returns false.
 */
 
-int fill_failed_attempts_view(THD *thd, TABLE_LIST *tables, Item *cond) {
+int fill_failed_attempts_view(THD *thd, Table_ref *tables, Item *cond) {
   if (connection_control::g_max_failed_connection_handler)
     connection_control::g_max_failed_connection_handler->fill_IS_table(
         thd, tables, cond);
@@ -860,7 +872,5 @@ int connection_control_failed_attempts_view_init(void *ptr) {
 
   schema_table->fields_info = failed_attempts_view_fields;
   schema_table->fill_table = fill_failed_attempts_view;
-  schema_table->idx_field1 = 0;
-  schema_table->idx_field2 = 1;
   return 0;
 }
