@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -86,8 +87,11 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
     // Length of the payload item: 1 byte
     PIT_TRANSACTION_GTIDS_PRESENT = 12,
 
+    // Length of the payload item: 8 bytes
+    PIT_SENT_TIMESTAMP = 13,
+
     // No valid type codes can appear after this one.
-    PIT_MAX = 13
+    PIT_MAX = 14
   };
 
   /**
@@ -141,7 +145,7 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
   /**
     Message destructor
    */
-  virtual ~Pipeline_stats_member_message();
+  ~Pipeline_stats_member_message() override;
 
   /**
     Get transactions waiting certification counter value.
@@ -172,14 +176,14 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
   int64 get_transactions_applied();
 
   /**
-    Get local transactions that member tried to commmit.
+    Get local transactions that member tried to commit.
 
     @return the counter value
   */
   int64 get_transactions_local();
 
   /**
-    Get negatively certfied transaction by member.
+    Get negatively certified transaction by member.
 
     @return the counter value
   */
@@ -228,13 +232,25 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
   */
   Flow_control_mode get_flow_control_mode();
 
+  /**
+    Return the time at which the message contained in the buffer was sent.
+    @see Metrics_handler::get_current_time()
+
+    @param[in] buffer            the buffer to decode from.
+    @param[in] length            the buffer length
+
+    @return the time on which the message was sent.
+  */
+  static uint64_t get_sent_timestamp(const unsigned char *buffer,
+                                     size_t length);
+
  protected:
   /**
     Encodes the message contents for transmission.
 
     @param[out] buffer   the message buffer to be written
   */
-  void encode_payload(std::vector<unsigned char> *buffer) const;
+  void encode_payload(std::vector<unsigned char> *buffer) const override;
 
   /**
     Message decoding method
@@ -242,7 +258,8 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
     @param[in] buffer the received data
     @param[in] end    the end of the buffer
   */
-  void decode_payload(const unsigned char *buffer, const unsigned char *end);
+  void decode_payload(const unsigned char *buffer,
+                      const unsigned char *end) override;
 
  private:
   int32 m_transactions_waiting_certification;
@@ -286,6 +303,11 @@ class Pipeline_stats_member_collector {
   */
   void decrement_transactions_waiting_apply();
 
+  /**
+    Set transactions waiting apply counter to 0.
+  */
+  void clear_transactions_waiting_apply();
+
   /**
     Increment transactions certified counter value.
   */
@@ -528,7 +550,7 @@ class Pipeline_member_stats {
   int64 get_delta_transactions_applied();
 
   /**
-    Get local transactions that member tried to commmit
+    Get local transactions that member tried to commit
     since last stats message.
 
     @return the counter value
@@ -549,7 +571,7 @@ class Pipeline_member_stats {
   */
   uint64 get_stamp();
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   void debug(const char *member, int64 quota_size, int64 quota_used);
 #endif
 
