@@ -98,7 +98,7 @@ void Key::store_in_buffer(uchar *buffer, size_t *buffer_position) const {
   store_field(buffer, buffer_position, reinterpret_cast<char *>(key.get()),
               key_len);
 
-  size_t padding =
+  const size_t padding =
       (sizeof(size_t) - (*buffer_position % sizeof(size_t))) % sizeof(size_t);
 
   *buffer_position += padding;
@@ -163,7 +163,7 @@ bool Key::load_from_buffer(uchar *buffer,
   memcpy(this->key.get(), buffer + buffer_position, key_len);
   buffer_position += key_len;
 
-  size_t padding =
+  const size_t padding =
       (sizeof(size_t) - (buffer_position % sizeof(size_t))) % sizeof(size_t);
   buffer_position += padding;
   assert(buffer_position % sizeof(size_t) == 0);
@@ -178,14 +178,14 @@ bool Key::load_from_buffer(uchar *buffer,
   The memory is aligned to sizeof(size_t)
 */
 size_t Key::get_key_pod_size() const {
-  size_t key_pod_size = 4 * sizeof(size_t) + key_id.length() +
-                        key_type.length() + user_id.length() + sizeof(key_len) +
-                        key_len;
+  const size_t key_pod_size = 4 * sizeof(size_t) + key_id.length() +
+                              key_type.length() + user_id.length() +
+                              sizeof(key_len) + key_len;
 
-  size_t padding =
+  const size_t padding =
       (sizeof(size_t) - (key_pod_size % sizeof(size_t))) % sizeof(size_t);
 
-  size_t key_pod_size_aligned = key_pod_size + padding;
+  const size_t key_pod_size_aligned = key_pod_size + padding;
   assert(key_pod_size_aligned % sizeof(size_t) == 0);
   return key_pod_size_aligned;
 }
