@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +31,10 @@
 #include <vector>
 
 /* Service handle */
-extern SERVICE_TYPE(pfs_plugin_table) * table_svc;
+extern SERVICE_TYPE(pfs_plugin_column_integer_v1) * col_int_svc;
+extern SERVICE_TYPE(pfs_plugin_column_bigint_v1) * col_bigint_svc;
+extern SERVICE_TYPE(pfs_plugin_column_date_v1) * col_date_svc;
+extern SERVICE_TYPE(pfs_plugin_column_time_v1) * col_time_svc;
 
 /* Global share pointer for pfs_example_employee_salary table */
 extern PFS_engine_table_share_proxy esalary_st_share;
@@ -42,7 +46,7 @@ extern PFS_engine_table_share_proxy esalary_st_share;
 extern mysql_mutex_t LOCK_esalary_records_array;
 
 /* A structure to denote a single row of the table. */
-struct {
+struct Esalary_Record {
  public:
   PSI_int e_number;
   PSI_bigint e_salary;
@@ -53,7 +57,7 @@ struct {
 
   /* If there is a value in this row */
   bool m_exist;
-} typedef Esalary_Record;
+};
 
 /**
  * An array to keep rows of the tables.
@@ -68,7 +72,7 @@ class Esalary_POS {
   unsigned int m_index;
 
  public:
-  ~Esalary_POS() {}
+  ~Esalary_POS() = default;
   Esalary_POS() { m_index = 0; }
 
   bool has_more() {
@@ -89,7 +93,7 @@ class Esalary_POS {
 };
 
 /* A structure to define a handle for table in plugin/component code. */
-typedef struct {
+struct Esalary_Table_Handle {
   /* Current position instance */
   Esalary_POS m_pos;
   /* Next position instance */
@@ -97,7 +101,7 @@ typedef struct {
 
   /* Current row for the table */
   Esalary_Record current_row;
-} Esalary_Table_Handle;
+};
 
 PSI_table_handle *esalary_open_table(PSI_pos **pos);
 void esalary_close_table(PSI_table_handle *handle);
