@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +42,8 @@
 #include "sql_string.h" /* STRING_PSI_MEMORY_KEY */
 #include "template_utils.h"
 
+struct CHARSET_INFO;
+
 static const char *log_filename = "test_sql_views_triggers";
 
 struct st_test_statement {
@@ -51,10 +54,10 @@ struct st_test_statement {
 
 static struct st_test_statement test_query_plan[] = {
     /* DB   RESULT   QUERY */
-    {NULL, true, "SELECT 'first complex command' as a"},
+    {nullptr, true, "SELECT 'first complex command' as a"},
     /* View */
     {"test", true, "SELECT * FROM v1"},
-    {NULL, true,
+    {nullptr, true,
      "SELECT VIEW_DEFINITION FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_SCHEMA "
      "= 'test' AND TABLE_NAME = 'v1'"},
     {"test", false, "DROP VIEW v1"},
@@ -63,7 +66,7 @@ static struct st_test_statement test_query_plan[] = {
     /* Trigger */
     {"test", false, "UPDATE t1 SET c2 = 'abcde' WHERE c1 = 1"},
     {"test", true, "SELECT * FROM t2"},
-    {NULL, true,
+    {nullptr, true,
      "SELECT TRIGGER_NAME FROM INFORMATION_SCHEMA.TRIGGERS WHERE "
      "TRIGGER_SCHEMA = 'test' AND TRIGGER_NAME = 'trig'"},
     {"test", false, "DROP TRIGGER trig"},
@@ -162,7 +165,7 @@ struct st_plugin_ctx {
   st_plugin_ctx() { reset(); }
 
   void reset() {
-    resultcs = NULL;
+    resultcs = nullptr;
     meta_server_status = 0;
     meta_warn_count = 0;
     server_status = 0;
@@ -194,7 +197,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
+  DBUG_PRINT("info", ("resultcs->m_coll_name: %s", resultcs->m_coll_name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -267,8 +270,8 @@ static ulong sql_get_client_capabilities(void *) {
 static int sql_get_null(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
   memcpy(pctx->sql_str_value[row][col], "[NULL]", sizeof("[NULL]"));
@@ -280,12 +283,13 @@ static int sql_get_null(void *ctx) {
 static int sql_get_integer(void *ctx, longlong value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
-  size_t len = snprintf(pctx->sql_str_value[row][col],
-                        sizeof(pctx->sql_str_value[row][col]), "%lld", value);
+  const size_t len =
+      snprintf(pctx->sql_str_value[row][col],
+               sizeof(pctx->sql_str_value[row][col]), "%lld", value);
   pctx->sql_str_len[row][col] = len;
 
   return false;
@@ -294,13 +298,13 @@ static int sql_get_integer(void *ctx, longlong value) {
 static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
-  size_t len = snprintf(pctx->sql_str_value[row][col],
-                        sizeof(pctx->sql_str_value[row][col]),
-                        is_unsigned ? "%llu" : "%lld", value);
+  const size_t len = snprintf(pctx->sql_str_value[row][col],
+                              sizeof(pctx->sql_str_value[row][col]),
+                              is_unsigned ? "%llu" : "%lld", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -310,15 +314,15 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 static const char *test_decimal_as_string(char *buff, const decimal_t *val,
                                           int *length) {
   if (!val) return "NULL";
-  (void)decimal2string(val, buff, length, 0, 0, 0);
+  (void)decimal2string(val, buff, length);
   return buff;
 }
 
 static int sql_get_decimal(void *ctx, const decimal_t *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
   int len = SIZEOF_SQL_STR_VALUE;
@@ -331,12 +335,13 @@ static int sql_get_decimal(void *ctx, const decimal_t *value) {
 static int sql_get_double(void *ctx, double value, uint32) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
-  size_t len = snprintf(pctx->sql_str_value[row][col],
-                        sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
+  const size_t len =
+      snprintf(pctx->sql_str_value[row][col],
+               sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -346,11 +351,11 @@ static int sql_get_double(void *ctx, double value, uint32) {
 static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
-  size_t len =
+  const size_t len =
       snprintf(pctx->sql_str_value[row][col],
                sizeof(pctx->sql_str_value[row][col]), "%s%4d-%02d-%02d",
                value->neg ? "-" : "", value->year, value->month, value->day);
@@ -362,11 +367,11 @@ static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
 static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
-  size_t len = snprintf(
+  const size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%02d:%02d:%02d", value->neg ? "-" : "",
       value->day ? (value->day * 24 + value->hour) : value->hour, value->minute,
@@ -379,11 +384,11 @@ static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
 static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
-  size_t len = snprintf(
+  const size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%4d-%02d-%02d %02d:%02d:%02d", value->neg ? "-" : "", value->year,
       value->month, value->day, value->hour, value->minute, value->second);
@@ -397,8 +402,8 @@ static int sql_get_string(void *ctx, const char *const value, size_t length,
                           const CHARSET_INFO *const) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  uint row = pctx->num_rows;
-  uint col = pctx->current_col;
+  const uint row = pctx->num_rows;
+  const uint col = pctx->current_col;
   pctx->current_col++;
 
   strncpy(pctx->sql_str_value[row][col], value, length);
@@ -455,6 +460,7 @@ const struct st_command_service_cbs protocol_callbacks = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
+    nullptr,
 };
 
 #define WRITE_DASHED_LINE() \
@@ -585,7 +591,7 @@ static void dump_cs_info(const CHARSET_INFO *cs) {
 
   WRITE_VAL("\t\t[meta][charset result] number: %d\n", cs->number);
   WRITE_VAL("\t\t[meta][charset result] name: %s\n", cs->csname);
-  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->name);
+  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->m_coll_name);
   WRITE_VAL("\t\t[meta][charset result] sort order: %s\n", cs->sort_order);
 }
 
@@ -595,7 +601,7 @@ static void dump_decoded_server_status(const char *prefix, uint server_status) {
   WRITE_VAL("%u\n", server_status);
   WRITE_STR(prefix);
   for (int i = 0; i < 30; i++) {
-    uint flag = 1 << i;
+    const uint flag = 1 << i;
     if (server_status & flag) {
 #define FLAG_DELIMITER " "
       switch (flag) {
@@ -703,6 +709,7 @@ static void dump_closing_ok(struct st_plugin_ctx *ctx) {
 static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
   char buffer[STRING_BUFFER_SIZE];
 
+  memset(cmd, 0, sizeof(union COM_DATA));
   cmd->com_query.query = query;
   cmd->com_query.length = strlen(query);
   WRITE_VAL2("EXECUTING:[%u][%s]\n", cmd->com_query.length, query);
@@ -710,7 +717,7 @@ static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
 
 static void run_statement(MYSQL_SESSION session, const char *query,
                           struct st_plugin_ctx *ctx, bool generates_result_set,
-                          void *p MY_ATTRIBUTE((unused))) {
+                          void *p [[maybe_unused]]) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
 
@@ -722,9 +729,9 @@ static void run_statement(MYSQL_SESSION session, const char *query,
   WRITE_STR("[CS_TEXT_REPRESENTATION]\n");
 again:
   ctx->reset();
-  int fail = command_service_run_command(session, COM_QUERY, &cmd,
-                                         &my_charset_utf8_general_ci,
-                                         &protocol_callbacks, txt_or_bin, ctx);
+  const int fail = command_service_run_command(
+      session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci,
+      &protocol_callbacks, txt_or_bin, ctx);
   if (fail) {
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "run_statement code: %d\n",
                     fail);
@@ -748,7 +755,7 @@ again:
 
 void static change_current_db(MYSQL_SESSION session, const char *db,
                               struct st_plugin_ctx *ctx,
-                              void *p MY_ATTRIBUTE((unused))) {
+                              void *p [[maybe_unused]]) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
   cmd.com_init_db.db_name = db;
@@ -757,8 +764,8 @@ void static change_current_db(MYSQL_SESSION session, const char *db,
   WRITE_DASHED_LINE();
   WRITE_VAL("EXECUTING:[COM_INIT_DB][%s]\n", db);
   ctx->reset();
-  int fail = command_service_run_command(
-      session, COM_INIT_DB, &cmd, &my_charset_utf8_general_ci,
+  const int fail = command_service_run_command(
+      session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci,
       &protocol_callbacks, CS_TEXT_REPRESENTATION, ctx);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "change db code: %d\n",
@@ -771,8 +778,9 @@ static void test_selects(MYSQL_SESSION session, void *p) {
 
   struct st_plugin_ctx *plugin_ctx = new st_plugin_ctx();
 
-  const char *last_db = NULL;
-  size_t stmt_count = sizeof(test_query_plan) / sizeof(test_query_plan[0]);
+  const char *last_db = nullptr;
+  const size_t stmt_count =
+      sizeof(test_query_plan) / sizeof(test_query_plan[0]);
   for (size_t i = 0; i < stmt_count; i++) {
     /* Change current DB if needed */
     if (last_db != test_query_plan[i].db) {
@@ -783,7 +791,7 @@ static void test_selects(MYSQL_SESSION session, void *p) {
     run_statement(session, test_query_plan[i].query, plugin_ctx,
                   test_query_plan[i].generates_result_set, p);
 
-    last_db = NULL;
+    last_db = nullptr;
   }
 
   WRITE_DASHED_LINE();
@@ -796,7 +804,7 @@ static void test_sql(void *p) {
   DBUG_TRACE;
 
   WRITE_STR("[srv_session_open]\n");
-  MYSQL_SESSION session = srv_session_open(NULL, NULL);
+  MYSQL_SESSION session = srv_session_open(nullptr, nullptr);
   if (!session) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "srv_session_open failed.");
     return;
@@ -835,7 +843,7 @@ static void *test_sql_threaded_wrapper(void *param) {
   srv_session_deinit_thread();
 
   context->thread_finished = true;
-  return NULL;
+  return nullptr;
 }
 
 static void create_log_file(const char *log_name) {
@@ -864,7 +872,7 @@ static void test_in_spawned_thread(void *p, void (*test_function)(void *)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                  "Could not create test session thread");
   else
-    my_thread_join(&context.thread, NULL);
+    my_thread_join(&context.thread, nullptr);
 }
 
 static int test_sql_service_plugin_init(void *p) {
@@ -888,7 +896,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
+static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
@@ -906,15 +914,15 @@ mysql_declare_plugin(test_daemon){
     MYSQL_DAEMON_PLUGIN,
     &test_sql_service_plugin,
     "test_sql_views_triggers",
-    "Pavan Naik, Andrey Hristov",
+    PLUGIN_AUTHOR_ORACLE,
     "Test sql views triggers",
     PLUGIN_LICENSE_GPL,
     test_sql_service_plugin_init,   /* Plugin Init      */
-    NULL,                           /* Plugin Check uninstall    */
+    nullptr,                        /* Plugin Check uninstall    */
     test_sql_service_plugin_deinit, /* Plugin Deinit    */
     0x0100,                         /* 1.0              */
-    NULL,                           /* status variables */
-    NULL,                           /* system variables */
-    NULL,                           /* config options   */
+    nullptr,                        /* status variables */
+    nullptr,                        /* system variables */
+    nullptr,                        /* config options   */
     0,                              /* flags            */
 } mysql_declare_plugin_end;
