@@ -36,8 +36,10 @@
 // Harness interface include files
 #include "mysql/harness/config_option.h"
 #include "mysql/harness/config_parser.h"
+#include "mysql/harness/dynamic_config.h"
 #include "mysql/harness/plugin.h"
 #include "mysql/harness/plugin_config.h"
+#include "mysql/harness/section_config_exposer.h"
 
 #include "mysqlrouter/destination_status_component.h"
 #include "mysqlrouter/destination_status_plugin_export.h"
@@ -46,7 +48,11 @@
 template <class T>
 using IntOption = mysql_harness::IntOption<T>;
 
-static constexpr const std::string_view kSectionName{"destination_status"};
+static const std::string kSectionName{"destination_status"};
+static const std::string kOptionThreshold{"error_quarantine_threshold"};
+static const std::string kOptionInterval{"error_quarantine_interval"};
+static constexpr uint32_t kDefaultErrorQuarantineThreshold{1};
+static constexpr uint32_t kDefaultErrorQuarantineInterval{1};
 
 class DestinationStatusPluginConfig : public mysql_harness::BasePluginConfig {
  public:
@@ -56,26 +62,23 @@ class DestinationStatusPluginConfig : public mysql_harness::BasePluginConfig {
   explicit DestinationStatusPluginConfig(
       const mysql_harness::ConfigSection *section)
       : mysql_harness::BasePluginConfig(section),
-        error_quarantine_threshold(get_option(section,
-                                              "error_quarantine_threshold",
+        error_quarantine_threshold(get_option(section, kOptionThreshold,
                                               IntOption<uint32_t>{1, 65535})),
-        error_quarantine_interval(get_option(section,
-                                             "error_quarantine_interval",
+        error_quarantine_interval(get_option(section, kOptionInterval,
                                              IntOption<uint32_t>{1, 3600})) {}
 
-  std::string get_default(const std::string &option) const override {
+  std::string get_default(std::string_view option) const override {
     const std::map<std::string_view, std::string> defaults{
-        {"error_quarantine_threshold", "1"},
-        {"error_quarantine_interval", "1"},  // in seconds
+        {kOptionThreshold, std::to_string(kDefaultErrorQuarantineThreshold)},
+        {kOptionInterval,
+         std::to_string(kDefaultErrorQuarantineInterval)},  // in seconds
     };
 
     auto it = defaults.find(option);
-
     return it == defaults.end() ? std::string() : it->second;
   }
 
-  [[nodiscard]] bool is_required(
-      const std::string & /* option */) const override {
+  [[nodiscard]] bool is_required(std::string_view /* option */) const override {
     return false;
   }
 };
@@ -123,6 +126,50 @@ const static std::array<const char *, 2> required = {{
     "io",
 }};
 
+namespace {
+
+class DestinationStatusConfigExposer
+    : public mysql_harness::SectionConfigExposer {
+ public:
+  using DC = mysql_harness::DynamicConfig;
+  DestinationStatusConfigExposer(
+      const bool initial, const DestinationStatusPluginConfig &plugin_config,
+      const mysql_harness::ConfigSection &default_section)
+      : mysql_harness::SectionConfigExposer(initial, default_section,
+                                            DC::SectionId{kSectionName, ""}),
+        plugin_config_(plugin_config) {}
+
+  void expose() override {
+    expose_option(kOptionThreshold, plugin_config_.error_quarantine_threshold,
+                  kDefaultErrorQuarantineThreshold);
+    expose_option(kOptionInterval,
+                  plugin_config_.error_quarantine_interval.count(),
+                  kDefaultErrorQuarantineInterval);
+  }
+
+ private:
+  const DestinationStatusPluginConfig &plugin_config_;
+};
+
+}  // namespace
+
+static void expose_configuration(mysql_harness::PluginFuncEnv *env,
+                                 const char * /*key*/, bool initial) {
+  const mysql_harness::AppInfo *info = get_app_info(env);
+
+  if (!info->config) return;
+
+  for (const mysql_harness::ConfigSection *section : info->config->sections()) {
+    if (section->name == kSectionName) {
+      DestinationStatusPluginConfig config{section};
+
+      DestinationStatusConfigExposer(initial, config,
+                                     info->config->get_default_section())
+          .expose();
+    }
+  }
+}
+
 extern "C" {
 mysql_harness::Plugin DESTINATION_STATUS_PLUGIN_EXPORT
     harness_plugin_destination_status = {
@@ -143,5 +190,6 @@ mysql_harness::Plugin DESTINATION_STATUS_PLUGIN_EXPORT
         false,    // declares_readiness
         destination_status_supported_options.size(),
         destination_status_supported_options.data(),
+        expose_configuration,
 };
 }
