@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,15 +29,19 @@
 #include "harness_export.h"
 
 #include <memory>
+#include <ostream>
 #include <stdexcept>
 #include <string>
+#include <string_view>
+#include <system_error>
+#include <vector>
+
 #ifndef _WIN32
 #include <fcntl.h>
 #endif
 
-#ifdef _WIN32
-#include <aclapi.h>
-#endif
+#include "mysql/harness/access_rights.h"
+#include "mysql/harness/stdx/expected.h"
 
 namespace mysql_harness {
 
@@ -113,12 +118,12 @@ class HARNESS_EXPORT Path {
    * Construct a path
    *
    * @param path Non-empty string denoting the path.
+   * @throws std::invalid_argument
    */
-  /** @overload */                // throws std::invalid_argument
-  Path(const std::string &path);  // NOLINT(runtime/explicit)
+  Path(std::string path);
 
-  /** @overload */         // throws std::invalid_argument
-  Path(const char *path);  // NOLINT(runtime/explicit)
+  Path(std::string_view path) : Path(std::string(path)) {}
+  Path(const char *path) : Path(std::string(path)) {}
 
   /**
    * Create a path from directory, basename, and extension.
@@ -160,6 +165,17 @@ class HARNESS_EXPORT Path {
    */
   bool is_regular() const;
 
+  /**
+   * Check if the path is absolute or not
+   *
+   * The path is considered absolute if it starts with one of:
+   *   Unix:    '/'
+   *   Windows: '/' or '\' or '.:' (where . is any character)
+   * else:
+   *   it's considered relative (empty path is also relative in such respect)
+   */
+  bool is_absolute() const;
+
   /**
    * Check if path exists
    */
@@ -207,7 +223,7 @@ class HARNESS_EXPORT Path {
    * Append a path component to the current path.
    *
    * This function will append a path component to the path using the
-   * apropriate directory separator.
+   * appropriate directory separator.
    *
    * @param other Path component to append to the path.
    */
@@ -226,9 +242,6 @@ class HARNESS_EXPORT Path {
    */
   Path join(const Path &other) const;
 
-  /** @overload */
-  Path join(const char *other) const { return join(Path(other)); }
-
   /**
    * Returns the canonical form of the path, resolving relative paths.
    */
@@ -262,7 +275,7 @@ class HARNESS_EXPORT Path {
   /**
    * Directory separator string.
    *
-   * @note This is platform-dependent and defined in the apropriate
+   * @note This is platform-dependent and defined in the appropriate
    * source file.
    */
   static const char *const directory_separator;
@@ -270,7 +283,7 @@ class HARNESS_EXPORT Path {
   /**
    * Root directory string.
    *
-   * @note This is platform-dependent and defined in the apropriate
+   * @note This is platform-dependent and defined in the appropriate
    * source file.
    */
   static const char *const root_directory;
@@ -306,12 +319,16 @@ class HARNESS_EXPORT Directory : public Path {
    *
    * A directory iterator is an input iterator.
    */
-  using DirectoryIteratorBase = std::iterator<std::input_iterator_tag, Path>;
-
-  class HARNESS_EXPORT DirectoryIterator : public DirectoryIteratorBase {
+  class HARNESS_EXPORT DirectoryIterator {
     friend class Directory;
 
    public:
+    using value_type = Path;
+    using iterator_category = std::input_iterator_tag;
+    using difference_type = std::ptrdiff_t;
+    using pointer = value_type *;
+    using reference = value_type &;
+
     DirectoryIterator(const Path &path,
                       const std::string &pattern = std::string());
 
@@ -401,6 +418,15 @@ class HARNESS_EXPORT Directory : public Path {
    */
   DirectoryIterator begin();
 
+  DirectoryIterator begin() const { return cbegin(); }
+
+  /**
+   * Constant iterator to first entry.
+   *
+   * @return Returns a constant iterator pointing to the first entry.
+   */
+  DirectoryIterator cbegin() const;
+
   /**
    * Iterator past-the-end of entries.
    *
@@ -408,6 +434,33 @@ class HARNESS_EXPORT Directory : public Path {
    */
   DirectoryIterator end();
 
+  DirectoryIterator end() const { return cend(); }
+
+  /**
+   * Constant iterator past-the-end of entries.
+   *
+   * @return Returns a constant iterator pointing *past-the-end* of the entries.
+   */
+  DirectoryIterator cend() const;
+
+  /**
+   * Check if the directory is empty.
+   *
+   * @retval true Directory is empty.
+   * @retval false Directory is no empty.
+   */
+  bool is_empty() const;
+
+  /**
+   * Recursively list all paths in a directory.
+   *
+   * Recursively create a list of relative paths from a directory. Path will
+   * be relative to the given directory. Empty directories are also listed.
+   *
+   * @return Recursive list of paths from a directory.
+   */
+  std::vector<Path> list_recursive() const;
+
   /**
    * Iterate over entries matching a glob.
    */
@@ -426,10 +479,11 @@ class HARNESS_EXPORT Directory : public Path {
  *
  * @param dir path of the directory to be removed; this directory must be empty
  *
- * @return 0 on success, -1 on error and sets errno
+ * @return void on success, error_code on failure
  */
 HARNESS_EXPORT
-int delete_dir(const std::string &dir) noexcept;
+stdx::expected<void, std::error_code> delete_dir(
+    const std::string &dir) noexcept;
 
 /** @brief Removes a file.
  *
@@ -437,10 +491,11 @@ int delete_dir(const std::string &dir) noexcept;
  *
  * @param path of the file to be removed
  *
- * @return 0 on success, -1 on error
+ * @return void on success, error_code on failure
  */
 HARNESS_EXPORT
-int delete_file(const std::string &path) noexcept;
+stdx::expected<void, std::error_code> delete_file(
+    const std::string &path) noexcept;
 
 /** @brief Removes directory and all its contents.
  *
@@ -448,10 +503,11 @@ int delete_file(const std::string &path) noexcept;
  *
  * @param dir path of the directory to be removed
  *
- * @return 0 on success, -1 on error
+ * @return void on success, error_code on failure
  */
 HARNESS_EXPORT
-int delete_dir_recursive(const std::string &dir) noexcept;
+stdx::expected<void, std::error_code> delete_dir_recursive(
+    const std::string &dir) noexcept;
 
 /** @brief Creates a temporary directory with partially-random name and returns
  * its path.
@@ -482,35 +538,13 @@ std::string get_plugin_dir(const std::string &runtime_dir);
 HARNESS_EXPORT
 std::string get_tests_data_dir(const std::string &runtime_dir);
 
-#ifdef _WIN32
-/**
- * Deleter for smart pointers pointing to objects allocated with `std::malloc`.
- */
-template <typename T>
-class StdFreeDeleter {
- public:
-  void operator()(T *ptr) { std::free(ptr); }
-};
-
-/**
- * Retrieves file's DACL security descriptor.
- *
- * @param[in] file_name File name.
- *
- * @return File's DACL security descriptor.
- *
- * @throw std::exception Failed to retrieve security descriptor.
- */
-HARNESS_EXPORT std::unique_ptr<SECURITY_DESCRIPTOR, decltype(&free)>
-get_security_descriptor(const std::string &file_name);
-
-#endif
-
 #ifndef _WIN32
 using perm_mode = mode_t;
+HARNESS_EXPORT
 extern const perm_mode kStrictDirectoryPerm;
 #else
 using perm_mode = int;
+HARNESS_EXPORT
 extern const perm_mode kStrictDirectoryPerm;
 #endif
 
@@ -518,7 +552,7 @@ extern const perm_mode kStrictDirectoryPerm;
  * *
  * @param dir       name (or path) of the directory to create
  * @param mode      permission mode for the created directory
- * @param recursive if true then immitated unix `mkdir -p` recursively
+ * @param recursive if true then imitate unix `mkdir -p` recursively
  *                  creating parent directories if needed
  * @retval 0 operation succeeded
  * @retval -1 operation failed because of wrong parameters
@@ -539,6 +573,19 @@ int mkdir(const std::string &dir, perm_mode mode, bool recursive = false);
  */
 void HARNESS_EXPORT make_file_public(const std::string &file_name);
 
+#ifdef _WIN32
+/**
+ * Changes file access permissions to be readable by all users.
+ *
+ * On Windows, Everyone group is granted read access to the file.
+ *
+ * @param[in] file_name File name.
+ *
+ * @throw std::exception Failed to change file permissions.
+ */
+void make_file_readable_for_everyone(const std::string &file_name);
+#endif
+
 /**
  * Changes file access permissions to be accessible only by a limited set of
  * users.
@@ -558,6 +605,19 @@ void HARNESS_EXPORT
 make_file_private(const std::string &file_name,
                   const bool read_only_for_local_service = true);
 
+/**
+ * Changes file access permissions to be read only.
+ *
+ * On Unix, the function sets file permission mask to 555.
+ * On Windows, all permissions to this file are read access only for Everyone
+ * group, LocalService account gets read access.
+ *
+ * @param[in] file_name File name.
+ *
+ * @throw std::exception Failed to change file permissions.
+ */
+void HARNESS_EXPORT make_file_readonly(const std::string &file_name);
+
 /**
  * Verifies access permissions of a file.
  *
@@ -574,6 +634,25 @@ make_file_private(const std::string &file_name,
 
 void HARNESS_EXPORT check_file_access_rights(const std::string &file_name);
 
+/** @brief Copy contents of one file to another.
+ *
+ * Exception thrown if open, create read or write operation fails.
+ */
+void HARNESS_EXPORT copy_file(const std::string &from, const std::string &to);
+
+/**
+ * renames file.
+ *
+ * The function will overwrite the 'to' file if already exists.
+ *
+ * @param from old filename
+ * @param to   new filename
+ *
+ * @returns stdx::expected<void, std::error_code>
+ */
+stdx::expected<void, std::error_code> HARNESS_EXPORT
+rename_file(const std::string &from, const std::string &to);
+
 }  // namespace mysql_harness
 
 #endif /* MYSQL_HARNESS_FILESYSTEM_INCLUDED */
