@@ -53,7 +53,7 @@ stdx::expected<Allocated<SID>, std::error_code> create_well_known_sid(
 
   if (CreateWellKnownSid(well_known_sid, nullptr, sid.get(), &sid_size) ==
       FALSE) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   return {std::move(sid)};
@@ -110,7 +110,7 @@ stdx::expected<Handle, std::error_code> open_process_token(
 
   // Gets security token of the current process
   if (!OpenProcessToken(process_handle, TOKEN_READ | TOKEN_QUERY, &h_token)) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   return {h_token};
@@ -124,12 +124,12 @@ stdx::expected<Allocated<TOKEN_USER>, std::error_code> token_user(
   HANDLE h_token = handle.native_handle();
   // Gets the user token from the security token (this one only finds out the
   // buffer size required)
-  if (!GetTokenInformation(h_token, token_class, NULL, 0, &token_size)) {
+  if (!GetTokenInformation(h_token, token_class, nullptr, 0, &token_size)) {
     auto ec = last_error_code();
 
     if (ec !=
         std::error_code{ERROR_INSUFFICIENT_BUFFER, std::system_category()}) {
-      return stdx::make_unexpected(ec);
+      return stdx::unexpected(ec);
     }
   }
 
@@ -139,7 +139,7 @@ stdx::expected<Allocated<TOKEN_USER>, std::error_code> token_user(
   // user token)
   if (!GetTokenInformation(h_token, token_class, user.get(), user.size(),
                            &token_size)) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   return {std::move(user)};
@@ -149,13 +149,13 @@ stdx::expected<Allocated<SID>, std::error_code> current_user_sid() {
   auto process_token_res = open_process_token(GetCurrentProcess());
   if (!process_token_res) {
     auto ec = process_token_res.error();
-    return stdx::make_unexpected(ec);
+    return stdx::unexpected(ec);
   }
 
   auto token_user_res = token_user(process_token_res.value());
   if (!token_user_res) {
     auto ec = token_user_res.error();
-    return stdx::make_unexpected(ec);
+    return stdx::unexpected(ec);
   }
 
   auto token_user = std::move(token_user_res.value());
@@ -241,7 +241,7 @@ std::string Acl::to_string() const {
 stdx::expected<void, std::error_code> SecurityDescriptor::initialize(
     DWORD revision) {
   if (!InitializeSecurityDescriptor(desc_, revision)) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   return {};
@@ -254,14 +254,14 @@ stdx::expected<OptionalDacl, std::error_code> SecurityDescriptor::dacl() const {
 
   if (0 ==
       GetSecurityDescriptorDacl(desc_, &dacl_present, &dacl, &dacl_defaulted)) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   if (!dacl_present) {
     return {};
   }
 
-  return {std::in_place, dacl};
+  return stdx::expected<OptionalDacl, std::error_code>{std::in_place, dacl};
 }
 
 stdx::expected<void, std::error_code> SecurityDescriptor::dacl(
@@ -275,7 +275,7 @@ stdx::expected<void, std::error_code> SecurityDescriptor::dacl(
 
   if (0 ==
       SetSecurityDescriptorDacl(desc_, dacl_present, dacl, dacl_defaulted)) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   return {};
@@ -286,8 +286,8 @@ std::string SecurityDescriptor::to_string() const {
 
   LPSTR out_s{nullptr};
 
-  ConvertSecurityDescriptorToStringSecurityDescriptorA(desc_, SDDL_REVISION_1,
-                                                       sec_info, &out_s, NULL);
+  ConvertSecurityDescriptorToStringSecurityDescriptorA(
+      desc_, SDDL_REVISION_1, sec_info, &out_s, nullptr);
 
   // LocalFree(out_s) on exit.
   std::unique_ptr<char, decltype(&LocalFree)> out_s_guard(out_s, &LocalFree);
@@ -301,7 +301,7 @@ SecurityDescriptor::control() const {
   DWORD revision;
 
   if (0 == GetSecurityDescriptorControl(desc_, &control, &revision)) {
-    return stdx::make_unexpected(last_error_code());
+    return stdx::unexpected(last_error_code());
   }
 
   return control;
@@ -316,7 +316,7 @@ SecurityDescriptor::make_self_relative() {
     const auto ec = last_error_code();
 
     if (ec.value() != ERROR_INSUFFICIENT_BUFFER) {
-      return stdx::make_unexpected(ec);
+      return stdx::unexpected(ec);
     }
     // fall through
   }
@@ -325,7 +325,7 @@ SecurityDescriptor::make_self_relative() {
 
   if (0 == MakeSelfRelativeSD(desc_, self_rel.get(), &self_rel_size)) {
     const auto ec = last_error_code();
-    return stdx::make_unexpected(ec);
+    return stdx::unexpected(ec);
   }
 
   return self_rel;
@@ -471,7 +471,7 @@ stdx::expected<security_descriptor_type, std::error_code> AclBuilder::build() {
   using namespace mysql_harness::win32::access_rights;
 
   // if add/set/revoke failed, return the error.
-  if (ec_) return stdx::make_unexpected(ec_);
+  if (ec_) return stdx::unexpected(ec_);
 
   if (auto desc = old_desc_.get()) {
     // BuildSecurityDescriptorW needs the old security descriptor in
@@ -480,7 +480,7 @@ stdx::expected<security_descriptor_type, std::error_code> AclBuilder::build() {
 
     if (!s.is_self_relative()) {
       auto self_rel_res = s.make_self_relative();
-      if (!self_rel_res) return self_rel_res.get_unexpected();
+      if (!self_rel_res) return stdx::unexpected(self_rel_res.error());
 
       old_desc_ = std::move(self_rel_res.value());
     }
@@ -500,10 +500,11 @@ stdx::expected<security_descriptor_type, std::error_code> AclBuilder::build() {
   if (err != ERROR_SUCCESS) {
     std::error_code ec{static_cast<int>(err), std::system_category()};
 
-    return stdx::make_unexpected(ec);
+    return stdx::unexpected(ec);
   }
 
-  return {std::in_place, reinterpret_cast<SECURITY_DESCRIPTOR *>(new_sd)};
+  return stdx::expected<security_descriptor_type, std::error_code>{
+      std::in_place, reinterpret_cast<SECURITY_DESCRIPTOR *>(new_sd)};
 }
 
 /*
@@ -570,13 +571,13 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
   SecurityDescriptor sec_desc{desc.get()};
 
   auto dacl_res = sec_desc.dacl();
-  if (!dacl_res) return dacl_res.get_unexpected();
+  if (!dacl_res) return stdx::unexpected(dacl_res.error());
 
   auto optional_dacl = std::move(dacl_res.value());
 
   if (!optional_dacl) {
     // No DACL means: all access allow.
-    return stdx::make_unexpected(make_error_code(std::errc::permission_denied));
+    return stdx::unexpected(make_error_code(std::errc::permission_denied));
   }
 
   if (optional_dacl.value() == nullptr) {
@@ -586,7 +587,9 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
 
   // get the current user sid
   auto current_user_sid_res = current_user_sid();
-  if (!current_user_sid_res) return current_user_sid_res.get_unexpected();
+  if (!current_user_sid_res) {
+    return stdx::unexpected(current_user_sid_res.error());
+  }
 
   auto allocated_current_user_sid = std::move(current_user_sid_res.value());
 
@@ -594,7 +597,9 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
 
   // local system
   auto local_system_sid_res = create_well_known_sid(WinLocalSystemSid);
-  if (!local_system_sid_res) return local_system_sid_res.get_unexpected();
+  if (!local_system_sid_res) {
+    return stdx::unexpected(local_system_sid_res.error());
+  }
 
   auto allocated_local_system_sid = std::move(local_system_sid_res.value());
 
@@ -602,7 +607,9 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
 
   // local service
   auto local_service_sid_res = create_well_known_sid(WinLocalServiceSid);
-  if (!local_service_sid_res) return local_service_sid_res.get_unexpected();
+  if (!local_service_sid_res) {
+    return stdx::unexpected(local_service_sid_res.error());
+  }
 
   auto allocated_local_service_sid = std::move(local_service_sid_res.value());
 
@@ -629,8 +636,7 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
 
       // the current user
       if ((allowed_access.mask() & file_access_mask) != expected) {
-        return stdx::make_unexpected(
-            make_error_code(std::errc::permission_denied));
+        return stdx::unexpected(make_error_code(std::errc::permission_denied));
       }
     } else if (running_under_wine() &&
                allowed_access.sid() == local_system_sid) {
@@ -639,8 +645,7 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
       // make_file_public() also allows LocalService to
     } else {
       // everyone else.
-      return stdx::make_unexpected(
-          make_error_code(std::errc::permission_denied));
+      return stdx::unexpected(make_error_code(std::errc::permission_denied));
     }
   }
 
@@ -650,13 +655,13 @@ stdx::expected<void, std::error_code> AllowUserReadWritableVerifier::operator()(
 stdx::expected<void, std::error_code> DenyOtherReadWritableVerifier::operator()(
     const security_descriptor_type &desc) {
   auto dacl_res = SecurityDescriptor(desc.get()).dacl();
-  if (!dacl_res) return dacl_res.get_unexpected();
+  if (!dacl_res) return stdx::unexpected(dacl_res.error());
 
   auto optional_dacl = std::move(dacl_res.value());
 
   if (!optional_dacl) {
     // No DACL means: all access allow.
-    return stdx::make_unexpected(make_error_code(std::errc::permission_denied));
+    return stdx::unexpected(make_error_code(std::errc::permission_denied));
   }
 
   if (optional_dacl.value() == nullptr) {
@@ -666,7 +671,7 @@ stdx::expected<void, std::error_code> DenyOtherReadWritableVerifier::operator()(
 
   // everyone
   auto sid_res = create_well_known_sid(WinWorldSid);
-  if (!sid_res) return sid_res.get_unexpected();
+  if (!sid_res) return stdx::unexpected(sid_res.error());
 
   auto allocated_everyone_sid = std::move(sid_res.value());
 
@@ -685,8 +690,7 @@ stdx::expected<void, std::error_code> DenyOtherReadWritableVerifier::operator()(
           (FILE_EXECUTE |
            (FILE_WRITE_DATA | FILE_WRITE_EA | FILE_WRITE_ATTRIBUTES) |
            (FILE_READ_DATA | FILE_READ_EA | FILE_READ_ATTRIBUTES))) {
-        return stdx::make_unexpected(
-            make_error_code(std::errc::permission_denied));
+        return stdx::unexpected(make_error_code(std::errc::permission_denied));
       }
     }
   }
@@ -708,7 +712,7 @@ stdx::expected<security_descriptor_type, std::error_code> access_rights_get(
     // We expect to receive `ERROR_INSUFFICIENT_BUFFER`.
     if (ec !=
         std::error_code{ERROR_INSUFFICIENT_BUFFER, std::system_category()}) {
-      return stdx::make_unexpected(ec);
+      return stdx::unexpected(ec);
     }
   }
 
@@ -718,7 +722,7 @@ stdx::expected<security_descriptor_type, std::error_code> access_rights_get(
                        &sec_desc_size) == FALSE) {
     const auto ec = last_error_code();
 
-    return stdx::make_unexpected(ec);
+    return stdx::unexpected(ec);
   }
 
   return {std::move(desc)};
@@ -729,7 +733,7 @@ stdx::expected<void, std::error_code> access_rights_set(
   if (0 == SetFileSecurityA(file_name.c_str(), DACL_SECURITY_INFORMATION,
                             desc.get())) {
     std::error_code ec{last_error_code()};
-    return stdx::make_unexpected(ec);
+    return stdx::unexpected(ec);
   }
 
   return {};
@@ -748,8 +752,7 @@ stdx::expected<security_descriptor_type, std::error_code> access_rights_get(
   struct stat st;
 
   if (-1 == ::stat(filename.c_str(), &st)) {
-    return stdx::make_unexpected(
-        std::error_code{errno, std::generic_category()});
+    return stdx::unexpected(std::error_code{errno, std::generic_category()});
   }
 
   return {st.st_mode};
@@ -762,8 +765,7 @@ stdx::expected<void, std::error_code> access_rights_set(
   return win32::access_rights::access_rights_set(filename, rights);
 #else
   if (-1 == ::chmod(filename.c_str(), rights)) {
-    return stdx::make_unexpected(
-        std::error_code{errno, std::generic_category()});
+    return stdx::unexpected(std::error_code{errno, std::generic_category()});
   }
 
   return {};
