@@ -36,7 +36,7 @@
 #include <cassert>
 #include <cerrno>
 #include <climits>
-#include <cstring>
+#include <cstring>  // strlen, strcmp
 #include <fstream>
 #include <sstream>
 #include <stdexcept>
@@ -53,7 +53,6 @@
 #include "mysql/harness/stdx/expected.h"
 
 namespace {
-const std::string dirsep("/");
 const std::string extsep(".");
 }  // namespace
 
@@ -309,8 +308,7 @@ Path Path::real_path() const {
 stdx::expected<void, std::error_code> delete_dir(
     const std::string &dir) noexcept {
   if (::rmdir(dir.c_str()) != 0) {
-    return stdx::make_unexpected(
-        std::error_code(errno, std::generic_category()));
+    return stdx::unexpected(std::error_code(errno, std::generic_category()));
   }
 
   return {};
@@ -319,8 +317,7 @@ stdx::expected<void, std::error_code> delete_dir(
 stdx::expected<void, std::error_code> delete_file(
     const std::string &path) noexcept {
   if (::unlink(path.c_str()) != 0) {
-    return stdx::make_unexpected(
-        std::error_code(errno, std::generic_category()));
+    return stdx::unexpected(std::error_code(errno, std::generic_category()));
   }
 
   return {};
@@ -381,4 +378,12 @@ void make_file_readonly(const std::string &file_name) {
   }
 }
 
+stdx::expected<void, std::error_code> rename_file(const std::string &from,
+                                                  const std::string &to) {
+  if (0 != rename(from.c_str(), to.c_str())) {
+    return stdx::unexpected(std::error_code{errno, std::generic_category()});
+  }
+  return {};
+}
+
 }  // namespace mysql_harness
