@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,16 +28,15 @@
 
 #include <string>
 
-#include "mysqlrouter/http_server_component.h"
+#include "http/base/request.h"
+#include "mysqlrouter/component/http_server_component.h"
 
-class HttpStaticFolderHandler : public BaseRequestHandler {
+class HttpStaticFolderHandler : public ::http::base::RequestHandler {
  public:
   explicit HttpStaticFolderHandler(std::string static_basedir,
-                                   std::string require_realm)
-      : static_basedir_(std::move(static_basedir)),
-        require_realm_{std::move(require_realm)} {}
+                                   std::string require_realm);
 
-  void handle_request(HttpRequest &req) override;
+  void handle_request(http::base::Request &req) override;
 
  private:
   std::string static_basedir_;
