@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,8 +34,8 @@
  * runs = 3
  */
 
+#include <array>
 #include <chrono>
-#include <iostream>
 #include <thread>
 
 // Harness interface include files
@@ -42,11 +43,6 @@
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/plugin.h"
 
-using mysql_harness::ARCHITECTURE_DESCRIPTOR;
-using mysql_harness::ConfigSection;
-using mysql_harness::PLUGIN_ABI_VERSION;
-using mysql_harness::Plugin;
-using mysql_harness::PluginFuncEnv;
 using mysql_harness::logging::log_info;
 
 // Keep symbols with external linkage away from global scope so that
@@ -58,10 +54,10 @@ const int kRuns = 0;       // 0 means for ever
 
 }  // namespace
 
-static void init(PluginFuncEnv *) {}
+static void init(mysql_harness::PluginFuncEnv *) {}
 
-static void start(PluginFuncEnv *env) {
-  const ConfigSection *section = get_config_section(env);
+static void start(mysql_harness::PluginFuncEnv *env) {
+  const auto *section = get_config_section(env);
   int interval = kInterval;
   try {
     interval = std::stoi(section->get("interval"));
@@ -99,19 +95,23 @@ static void start(PluginFuncEnv *env) {
 #define DLLEXPORT
 #endif
 
+static std::array<const char *, 3> supported_options{"interval", "runs"};
+
 extern "C" {
-Plugin DLLEXPORT harness_plugin_keepalive = {
-    PLUGIN_ABI_VERSION,
-    ARCHITECTURE_DESCRIPTOR,
-    "Keepalive Plugin",
-    VERSION_NUMBER(0, 0, 1),
-    0,
-    nullptr,
-    0,
-    nullptr,  // conflicts
+mysql_harness::Plugin DLLEXPORT harness_plugin_keepalive = {
+    mysql_harness::PLUGIN_ABI_VERSION, mysql_harness::ARCHITECTURE_DESCRIPTOR,
+    "Keepalive Plugin", VERSION_NUMBER(0, 0, 1),
+    // requires
+    0, nullptr,
+    // conflicts
+    0, nullptr,
     init,     // init
     nullptr,  // deinit
     start,    // start
     nullptr,  // stop
+    // TODO: should it be service maybe?
+    false,  // declares_readiness
+    supported_options.size(), supported_options.data(),
+    nullptr,  // expose_configuration
 };
 }
