@@ -39,7 +39,7 @@ class RestRouterStatus : public RestApiHandler {
         running_since_(std::chrono::system_clock::now()),
         last_modified_(time(nullptr)) {}
 
-  bool on_handle_request(HttpRequest &req, const std::string &base_path,
+  bool on_handle_request(http::base::Request &req, const std::string &base_path,
                          const std::vector<std::string> &path_matches) override;
 
  private:
