@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,9 +32,9 @@
 #include <cerrno>
 #include <cstdio>
 #include <cstdlib>
-#include <cstring>
 #include <fstream>
 #include <initializer_list>
+#include <memory>  // unique_ptr
 #include <sstream>
 #include <stdexcept>
 #include <string>
@@ -46,20 +47,33 @@
 #include "harness_assert.h"
 #include "hostname_validator.h"
 #include "keyring/keyring_manager.h"
+#include "mysql/harness/arg_handler.h"
+#include "mysql/harness/config_option.h"
 #include "mysql/harness/config_parser.h"
+#include "mysql/harness/dynamic_config.h"
 #include "mysql/harness/dynamic_state.h"
 #include "mysql/harness/filesystem.h"
+#include "mysql/harness/log_reopen_component.h"
 #include "mysql/harness/logging/logger_plugin.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/logging/registry.h"
+#include "mysql/harness/process_state_component.h"
+#include "mysql/harness/section_config_exposer.h"
+#include "mysql/harness/signal_handler.h"
+#include "mysql/harness/supported_config_options.h"
 #include "mysql/harness/utility/string.h"  // string_format
 #include "mysql/harness/vt100.h"
+#include "mysql_router_thread.h"  // kDefaultStackSizeInKiloByte
 #include "mysqlrouter/config_files.h"
+#include "mysqlrouter/connection_pool.h"
 #include "mysqlrouter/default_paths.h"
 #include "mysqlrouter/mysql_session.h"
+#include "mysqlrouter/routing.h"
+#include "mysqlrouter/supported_router_options.h"
 #include "mysqlrouter/utils.h"  // substitute_envvar
 #include "print_version.h"
 #include "router_config.h"  // MYSQL_ROUTER_VERSION
+#include "scope_guard.h"
 #include "welcome_copyright_notice.h"
 
 #ifndef _WIN32
@@ -67,7 +81,6 @@
 #include <unistd.h>
 #include <csignal>
 const char dir_sep = '/';
-const std::string path_sep = ":";
 #else
 #include <process.h>
 #include <windows.h>
@@ -76,9 +89,7 @@ const std::string path_sep = ":";
 #include <string.h>
 #include "mysqlrouter/windows/password_vault.h"
 #include "mysqlrouter/windows/service_operations.h"
-#define strtok_r strtok_s
 const char dir_sep = '\\';
-const std::string path_sep = ";";
 #endif
 
 IMPORT_LOG_FUNCTIONS()
@@ -88,8 +99,6 @@ using mysql_harness::DIM;
 using mysql_harness::truncate_string;
 using mysql_harness::utility::string_format;
 using mysql_harness::utility::wrap_string;
-using mysqlrouter::substitute_envvar;
-using mysqlrouter::SysUserOperations;
 using mysqlrouter::SysUserOperationsBase;
 
 static const char *kDefaultKeyringFileName = "keyring";
@@ -97,16 +106,6 @@ static const char kProgramName[] = "mysqlrouter";
 
 namespace {
 
-inline void set_signal_handlers() {
-#ifndef _WIN32
-  // until we have proper signal handling we need at least
-  // mask out broken pipe to prevent terminating the router
-  // if the receiving end closes the socket while the router
-  // writes to it
-  signal(SIGPIPE, SIG_IGN);
-#endif
-}
-
 // Check if the value is valid regular filename and if it is add to the vector,
 // if it is not throw an exception
 void check_and_add_conf(std::vector<std::string> &configs,
@@ -121,7 +120,8 @@ void check_and_add_conf(std::vector<std::string> &configs,
 
   if (cfg_file_path.is_regular()) {
     configs.push_back(cfg_file_path.real_path().str());
-  } else if (!cfg_file_path.exists()) {
+  } else if (cfg_file_path.type() ==
+             mysql_harness::Path::FileType::FILE_NOT_FOUND) {
     throw std::runtime_error(string_format(
         "The configuration file '%s' does not exist.", value.c_str()));
   } else {
@@ -146,7 +146,7 @@ void check_config_overwrites(const CmdArgHandler::ConfigOverwrites &overwrites,
     // only --logger.level config overwrite is allowed currently for bootstrap
     for (const auto &option : overwrite.second) {
       const std::string name = section + "." + option.first;
-      if (name != "logger.level") {
+      if (name != "logger.level" && name != "DEFAULT.plugin_folder") {
         throw std::runtime_error(
             "Invalid argument '--" + name +
             "'. Only '--logger.level' configuration option can be "
@@ -156,6 +156,19 @@ void check_config_overwrites(const CmdArgHandler::ConfigOverwrites &overwrites,
   }
 }
 
+std::string get_plugin_folder_overwrite(
+    const CmdArgHandler::ConfigOverwrites &overwrites) {
+  const auto default_key = std::make_pair("DEFAULT"s, ""s);
+  if (overwrites.count(default_key) != 0) {
+    const auto &default_overwrites = overwrites.at(default_key);
+    if (default_overwrites.count("plugin_folder") != 0) {
+      return default_overwrites.at("plugin_folder");
+    }
+  }
+
+  return "";
+}
+
 }  // namespace
 
 // throws MySQLSession::Error, std::runtime_error, std::out_of_range,
@@ -183,8 +196,7 @@ MySQLRouter::MySQLRouter(const std::string &program_name,
       sys_user_operations_(sys_user_operations)
 #endif
 {
-  set_log_reopen_complete_callback(default_log_reopen_complete_cb);
-  set_signal_handlers();
+  signal_handler_.register_ignored_signals_handler();  // SIGPIPE
 
   init(program_name,
        arguments);  // throws MySQLSession::Error, std::runtime_error,
@@ -233,8 +245,41 @@ void MySQLRouter::init(const std::string &program_name,
     return;
   }
 
+  // signal-handler
+  signal_handler_.add_sig_handler(
+      SIGTERM, [](int /* sig */, const std::string &signal_info) {
+        mysql_harness::ProcessStateComponent::get_instance()
+            .request_application_shutdown(
+                mysql_harness::ShutdownPending::Reason::REQUESTED, signal_info);
+      });
+
+  signal_handler_.add_sig_handler(
+      SIGINT, [](int /* sig */, const std::string &signal_info) {
+        mysql_harness::ProcessStateComponent::get_instance()
+            .request_application_shutdown(
+                mysql_harness::ShutdownPending::Reason::REQUESTED, signal_info);
+      });
+
+  // block non-fatal signal handling for all threads
+  //
+  // - no other thread than the signal-handler thread should receive signals
+  // - syscalls should not get interrupted by signals either
+  //
+  // on windows, this is a no-op
+  signal_handler_.block_all_nonfatal_signals();
+
+  // for the fatal signals we want to have a handler that prints the stack-trace
+  // if possible
+  signal_handler_.register_fatal_signal_handler(core_file_);
+  signal_handler_.spawn_signal_handler_thread();
+
+#ifdef _WIN32
+  signal_handler_.register_ctrl_c_handler();
+#endif
+
   const bool is_bootstrap = !bootstrap_uri_.empty();
-  check_config_overwrites(arg_handler_.get_config_overwrites(),
+  const auto config_overwrites = arg_handler_.get_config_overwrites();
+  check_config_overwrites(config_overwrites,
                           is_bootstrap);  // throws std::runtime_error
 
   if (is_bootstrap) {
@@ -251,7 +296,7 @@ void MySQLRouter::init(const std::string &program_name,
 
     if (superuser && !user_option) {
       std::string msg(
-          "You are bootstraping as a superuser.\n"
+          "You are bootstrapping as a superuser.\n"
           "This will make all the result files (config etc.) privately owned "
           "by the superuser.\n"
           "Please use --user=username option to specify the user that will be "
@@ -262,7 +307,7 @@ void MySQLRouter::init(const std::string &program_name,
     }
 #endif
 
-    // default configuration for boostrap is not supported
+    // default configuration for bootstrap is not supported
     // extra configuration for bootstrap is not supported
     auto config_files_res =
         ConfigFilePathValidator({}, config_files_, {}).validate();
@@ -271,20 +316,23 @@ void MySQLRouter::init(const std::string &program_name,
       config_files = std::move(config_files_res.value());
     }
 
-    DIM::instance().reset_Config();  // simplifies unit tests
-    DIM::instance().set_Config(
-        [this, &config_files]() { return make_config({}, config_files); },
-        std::default_delete<mysql_harness::LoaderConfig>());
-    mysql_harness::LoaderConfig &config = DIM::instance().get_Config();
+    auto &dim = DIM::instance();
+
+    dim.set_Config(make_config({}, config_files),
+                   std::default_delete<mysql_harness::LoaderConfig>());
+
+    mysql_harness::LoaderConfig &config = dim.get_Config();
 
     // reinit logger (right now the logger is configured to log to STDERR,
     // here we re-configure it with settings from config file)
     init_main_logger(config, true);  // true = raw logging mode
 
-    bootstrap(
-        program_name,
-        bootstrap_uri_);  // throws MySQLSession::Error, std::runtime_error,
-                          // std::out_of_range, std::logic_error, ...?
+    bootstrap(program_name, bootstrap_uri_,
+              get_plugin_folder_overwrite(
+                  config_overwrites));  // throws MySQLSession::Error,
+                                        // std::runtime_error,
+                                        // std::out_of_range,
+                                        // std::logic_error, ...?
     return;
   }
 
@@ -335,15 +383,12 @@ void MySQLRouter::init_keyring(mysql_harness::Config &config) {
 }
 
 void MySQLRouter::init_dynamic_state(mysql_harness::Config &config) {
-  if (config.has_default("dynamic_state")) {
+  if (config.has_default(router::options::kDynamicState)) {
     using mysql_harness::DynamicState;
 
-    const std::string dynamic_state_file = config.get_default("dynamic_state");
     DIM::instance().set_DynamicState(
-        [=]() { return new DynamicState(dynamic_state_file); },
+        new DynamicState(config.get_default(router::options::kDynamicState)),
         std::default_delete<mysql_harness::DynamicState>());
-    // force object creation, the further code relies on it's existence
-    DIM::instance().get_DynamicState();
   }
 }
 
@@ -407,10 +452,11 @@ void MySQLRouter::init_main_logger(mysql_harness::LoaderConfig &config,
   harness_assert(use_os_log == false);
 #endif
 
-  if (!config.has_default("logging_folder"))
-    config.set_default("logging_folder", "");
+  if (!config.has_default(mysql_harness::loader::options::kLoggingFolder))
+    config.set_default(mysql_harness::loader::options::kLoggingFolder, "");
 
-  const std::string logging_folder = config.get_default("logging_folder");
+  const std::string logging_folder =
+      config.get_default(mysql_harness::loader::options::kLoggingFolder);
 
   // setup logging
   {
@@ -423,8 +469,7 @@ void MySQLRouter::init_main_logger(mysql_harness::LoaderConfig &config,
     //           with the new one at the very end.
 
     // our new logger registry, it will replace the current one if all goes well
-    std::unique_ptr<mysql_harness::logging::Registry> registry(
-        new mysql_harness::logging::Registry());
+    auto registry = std::make_unique<mysql_harness::logging::Registry>();
 
     const auto level = mysql_harness::logging::get_default_log_level(
         config, raw_mode);  // throws std::invalid_argument
@@ -444,9 +489,8 @@ void MySQLRouter::init_main_logger(mysql_harness::LoaderConfig &config,
     // nothing threw - we're good. Now let's replace the new registry with the
     // old one
     DIM::instance().set_LoggingRegistry(
-        [&registry]() { return registry.release(); },
+        registry.release(),
         std::default_delete<mysql_harness::logging::Registry>());
-    DIM::instance().reset_LoggingRegistry();
 
     // flag that the new loggers are ready for use
     DIM::instance().get_LoggingRegistry().set_ready();
@@ -499,6 +543,9 @@ void MySQLRouter::init_loader(mysql_harness::LoaderConfig &config) {
   } catch (const std::runtime_error &err) {
     throw std::runtime_error(string_format(err_msg.c_str(), err.what()));
   }
+
+  loader_->register_supported_app_options(router_supported_options);
+  loader_->register_expose_app_config_callback(expose_router_configuration);
 }
 
 void MySQLRouter::start() {
@@ -519,20 +566,22 @@ void MySQLRouter::start() {
   // throws system_error() in case of failure
   const auto config_files = check_config_files();
 
+  auto &dim = DIM::instance();
+
   // read config, and also make this config globally-available via DIM
-  DIM::instance().reset_Config();  // simplifies unit tests
-  DIM::instance().set_Config(
-      [this, &config_files]() {
-        return make_config(get_default_paths(), config_files);
-      },
-      std::default_delete<mysql_harness::LoaderConfig>());
+  dim.set_Config(make_config(get_default_paths(), config_files),
+                 std::default_delete<mysql_harness::LoaderConfig>());
+
   mysql_harness::LoaderConfig &config = DIM::instance().get_Config();
 
 #ifndef _WIN32
   // --user param given on the command line has a priority over
   // the user in the configuration
-  if (user_cmd_line_.empty() && config.has_default("user")) {
-    set_user(config.get_default("user"), true, this->sys_user_operations_);
+  {
+    if (user_cmd_line_.empty() && config.has_default(router::options::kUser)) {
+      set_user(config.get_default(router::options::kUser), true,
+               this->sys_user_operations_);
+    }
   }
 #endif
 
@@ -543,8 +592,8 @@ void MySQLRouter::start() {
   // Setup pidfile path for the application.
   // Order of significance: commandline > config file > ROUTER_PID envvar
   if (pid_file_path_.empty()) {
-    if (config.has_default("pid_file")) {
-      const std::string pidfile = config.get_default("pid_file");
+    if (config.has_default(router::options::kPidFile)) {
+      const std::string pidfile = config.get_default(router::options::kPidFile);
       if (!pidfile.empty()) {
         pid_file_path_ = pidfile;
       } else {
@@ -601,6 +650,25 @@ void MySQLRouter::start() {
     }
   }
 
+  register_on_switch_to_configured_loggers_callback([]() {
+    // once we switched to the configured logger(s) log the Router version
+    log_system("Starting '%s', version: %s (%s)", MYSQL_ROUTER_PACKAGE_NAME,
+               MYSQL_ROUTER_VERSION, MYSQL_ROUTER_VERSION_EDITION);
+  });
+
+  mysql_harness::ProcessStateComponent::get_instance()
+      .register_on_shutdown_request_callback(
+          [](mysql_harness::ShutdownPending::Reason reason,
+             const std::string &msg) {
+            // once we received the shutdown request, we want to log that along
+            // with the Router version
+            log_system(
+                "Stopping '%s', version: %s (%s), reason: %s%s",
+                MYSQL_ROUTER_PACKAGE_NAME, MYSQL_ROUTER_VERSION,
+                MYSQL_ROUTER_VERSION_EDITION, to_string(reason).c_str(),
+                msg.empty() ? "" : std::string(" ("s + msg + ")").c_str());
+          });
+
   init_loader(config);  // throws std::runtime_error
 
   if (!pid_file_path_.empty()) {
@@ -668,6 +736,63 @@ void MySQLRouter::start() {
   init_keyring(config);
   init_dynamic_state(config);
 
+#if !defined(_WIN32)
+  //
+  // reopen the logfile on SIGHUP.
+  // shutdown at SIGTERM|SIGINT
+  //
+
+  auto &log_reopener = mysql_harness::LogReopenComponent::get_instance();
+
+  static const char kLogReopenServiceName[]{"log_reopen"};
+
+  // report readiness of all services only after the log-reopen handlers is
+  // installed ... after all plugins are started.
+  loader_->waitable_services().emplace_back(kLogReopenServiceName);
+#endif
+
+  static const char kSignalHandlerServiceName[]{"signal_handler"};
+
+  loader_->waitable_services().emplace_back(kSignalHandlerServiceName);
+
+  loader_->after_all_started([&]() {
+#if !defined(_WIN32)
+    // as the LogReopener depends on the loggers being started, it must be
+    // initialized after Loader::start_all() has been called.
+    log_reopener.init();
+
+    log_reopener->set_complete_callback([](const auto &errmsg) {
+      if (errmsg.empty()) return;
+
+      mysql_harness::ProcessStateComponent::get_instance()
+          .request_application_shutdown(
+              mysql_harness::ShutdownPending::Reason::FATAL_ERROR, errmsg);
+    });
+
+    signal_handler_.add_sig_handler(
+        SIGHUP,
+        [&log_reopener](int /* sig */, const std::string & /*signal_info*/) {
+          // is run by the signal-thread.
+          log_reopener->request_reopen();
+        });
+
+    mysql_harness::on_service_ready(kLogReopenServiceName);
+#endif
+
+    mysql_harness::on_service_ready(kSignalHandlerServiceName);
+  });
+
+  // after the first plugin finished, stop the log-reopener and signal-handler
+  loader_->after_first_finished([&]() {
+#if !defined(_WIN32)
+    signal_handler_.remove_sig_handler(SIGTERM);
+    signal_handler_.remove_sig_handler(SIGINT);
+
+    signal_handler_.remove_sig_handler(SIGHUP);
+    log_reopener.reset();
+#endif
+  });
+
   loader_->start();
 }
 
@@ -715,7 +840,7 @@ std::vector<std::string> MySQLRouter::check_config_files() {
                        .validate();
 
   if (!res) {
-    const auto err = std::move(res.error());
+    const auto err = res.error();
     if (err.ec == make_error_code(ConfigFilePathValidatorErrc::kDuplicate)) {
       throw std::runtime_error(string_format(
           "The configuration file '%s' is provided multiple "
@@ -1111,6 +1236,20 @@ void MySQLRouter::prepare_command_options() noexcept {
                             check_and_add_conf(config_files_, value);
                           });
 
+  arg_handler_.add_option(
+      OptionNames({"--core-file"}), "Write a core file if mysqlrouter dies.",
+      CmdOptionValueReq::optional, "", [this](const std::string &value) {
+        if (value.empty() || value == "1") {
+          this->core_file_ = true;
+        } else if (value == "0") {
+          this->core_file_ = false;
+        } else {
+          throw std::runtime_error(
+              "Value for parameter '--core-file' needs to be "
+              "one of: ['0', '1']");
+        }
+      });
+
   arg_handler_.add_option(
       OptionNames({"--connect-timeout"}),
       "The time in seconds after which trying to connect to metadata server "
@@ -1227,11 +1366,14 @@ void MySQLRouter::prepare_command_options() noexcept {
 
   arg_handler_.add_option(
       OptionNames({"--force-password-validation"}),
-      "When autocreating database account do not use HASHED password. "
+      "When autocreating database account do not use HASHED password - this is "
+      "the default behavior now, this parameter is not needed, it is kept for "
+      "backward compatibility."
       "(bootstrap)",
       CmdOptionValueReq::none, "",
-      [this](const std::string &) {
-        this->bootstrap_options_["force-password-validation"] = "1";
+      [](const std::string &) {
+        // this is now always assumed, so this parameter is ignored, kept for
+        // backward compatibility
       },
       [this](const std::string &) {
         this->assert_bootstrap_mode("--force-password-validation");
@@ -1547,6 +1689,17 @@ void MySQLRouter::prepare_command_options() noexcept {
         this->assert_bootstrap_mode("--ssl-key");
       });
 
+  arg_handler_.add_option(
+      OptionNames({"--disable-rw-split"}),
+      "Do not generate routing section for RW Split endpoint",
+      CmdOptionValueReq::none, "",
+      [this](const std::string &) {
+        this->bootstrap_options_["disable-rw-split"] = "1";
+      },
+      [this](const std::string &) {
+        this->assert_bootstrap_mode("--disable-rw-split");
+      });
+
   arg_handler_.add_option(
       OptionNames({"--disable-rest"}),
       "Disable REST web service for Router monitoring", CmdOptionValueReq::none,
@@ -1683,8 +1836,7 @@ void MySQLRouter::prepare_command_options() noexcept {
 
   // in this context we only want the service-related options to be known and
   // displayed with --help; they are handled elsewhere (main-windows.cc)
-  ServiceConfOptions unused;
-  add_service_options(arg_handler_, unused);
+  add_service_options(arg_handler_);
 
   arg_handler_.add_option(
       CmdOption::OptionNames({"--remove-credentials-section"}),
@@ -1718,7 +1870,8 @@ void MySQLRouter::prepare_command_options() noexcept {
 // throws MySQLSession::Error, std::runtime_error, std::out_of_range,
 // std::logic_error, ... ?
 void MySQLRouter::bootstrap(const std::string &program_name,
-                            const std::string &server_url) {
+                            const std::string &server_url,
+                            const std::string &plugin_folder) {
   mysqlrouter::ConfigGenerator config_gen(out_stream_, err_stream_
 #ifndef _WIN32
                                           ,
@@ -1730,9 +1883,10 @@ void MySQLRouter::bootstrap(const std::string &program_name,
       bootstrap_options_);  // throws MySQLSession::Error, std::runtime_error,
                             // std::out_of_range, std::logic_error
   config_gen.warn_on_no_ssl(bootstrap_options_);  // throws std::runtime_error
+  config_gen.set_plugin_folder(plugin_folder);
 
 #ifdef _WIN32
-  // Cannot run boostrap mode as windows service since it requires console
+  // Cannot run bootstrap mode as windows service since it requires console
   // interaction.
   if (mysqlrouter::is_running_as_service()) {
     std::string msg = "Cannot run router in boostrap mode as Windows service.";
@@ -1818,7 +1972,7 @@ void MySQLRouter::show_help() {
 
     // fallback to .ini for each .conf file
     const std::string conf_ext(".conf");
-    if (mysql_harness::utility::ends_with(file, conf_ext)) {
+    if (file.ends_with(conf_ext)) {
       // replace .conf by .ini
       std::string ini_filename =
           file.substr(0, file.size() - conf_ext.size()) + ".ini";
@@ -1891,6 +2045,7 @@ void MySQLRouter::show_usage(bool include_options) noexcept {
         "--client-ssl-curves",
         "--client-ssl-key",
         "--client-ssl-mode",
+        "--core-file",
         "--directory",
         "--force",
         "--force-password-validation",
@@ -1918,10 +2073,20 @@ void MySQLRouter::show_usage(bool include_options) noexcept {
         "--tls-version",
         "--user"}},
       {"run",
-       {"--user", "--config", "--extra-config", "--clear-all-credentials",
-        "--service", "--remove-service", "--install-service",
-        "--install-service-manual", "--pid-file",
-        "--remove-credentials-section", "--update-credentials-section"}}};
+       {
+           "--user",
+           "--config",
+           "--extra-config",
+           "--clear-all-credentials",
+           "--service",
+           "--remove-service",
+           "--install-service",
+           "--install-service-manual",
+           "--pid-file",
+           "--remove-credentials-section",
+           "--update-credentials-section",
+           "--core-file",
+       }}};
 
   for (const auto &section : usage_sections) {
     for (auto line : arg_handler_.usage_lines_if(
@@ -1977,3 +2142,65 @@ void MySQLRouter::show_usage(bool include_options) noexcept {
 }
 
 void MySQLRouter::show_usage() noexcept { show_usage(true); }
+
+namespace {
+
+class RouterAppConfigExposer : public mysql_harness::SectionConfigExposer {
+ public:
+  using DC = mysql_harness::DynamicConfig;
+  RouterAppConfigExposer(const bool initial,
+                         const mysql_harness::ConfigSection &default_section)
+      : mysql_harness::SectionConfigExposer(initial, default_section,
+                                            DC::SectionId{"common", ""}) {}
+
+  void expose() override {
+    // set "global" router options from DEFAULT section
+    auto expose_int_option = [&](std::string_view option,
+                                 const int64_t default_val) {
+      auto value = default_val;
+      if (default_section_.has(option)) {
+        try {
+          value = mysql_harness::option_as_int<int64_t>(
+              default_section_.get(option), "");
+        } catch (...) {
+          // if it failed fallback to setting default
+        }
+      }
+
+      expose_option(option, value, default_val);
+    };
+
+    auto expose_str_option = [&](std::string_view option,
+                                 const std::string &default_val,
+                                 bool skip_if_empty = false) {
+      auto value = default_val;
+      if (default_section_.has(option)) {
+        value = default_section_.get(option);
+      }
+
+      const OptionValue op_val = (skip_if_empty && value.empty())
+                                     ? OptionValue(std::monostate{})
+                                     : value;
+      const OptionValue op_default_val = (skip_if_empty && default_val.empty())
+                                             ? OptionValue(std::monostate{})
+                                             : default_val;
+
+      expose_option(option, op_val, op_default_val);
+    };
+
+    expose_int_option(
+        router::options::kMaxTotalConnections,
+        static_cast<int64_t>(routing::kDefaultMaxTotalConnections));
+    expose_str_option(router::options::kName, kSystemRouterName);
+    // only share a user if it is not empty
+    expose_str_option(router::options::kUser, kDefaultSystemUserName, true);
+    expose_str_option("unknown_config_option", "error");
+  }
+};
+
+}  // namespace
+
+void expose_router_configuration(const bool initial,
+                                 const mysql_harness::ConfigSection &section) {
+  RouterAppConfigExposer(initial, section).expose();
+}
