@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,6 +34,7 @@
 #include <set>
 #include <stdexcept>
 #include <string>
+#include <string_view>
 #include <system_error>
 
 #include "mysql/harness/logging/logging.h"
@@ -42,6 +44,7 @@
 #include "tcp_address.h"
 
 using namespace std::chrono_literals;
+using namespace std::string_view_literals;
 
 IMPORT_LOG_FUNCTIONS()
 
@@ -50,45 +53,57 @@ IMPORT_LOG_FUNCTIONS()
 // TODO: possibly this should be made into a configurable option
 static const auto kPrimaryFailoverTimeout = 10s;
 
-static const std::set<std::string> supported_params{
+// we keep the allow_primary_reads on this list even though we no longer support
+// it, so that we give more specific error message for it
+static constexpr std::array supported_params{
     "role", "allow_primary_reads", "disconnect_on_promoted_to_primary",
     "disconnect_on_metadata_unavailable"};
 
 namespace {
 
-DestMetadataCacheGroup::ServerRole get_server_role_from_uri(
-    const mysqlrouter::URIQuery &uri) {
-  if (uri.find("role") == uri.end())
-    throw std::runtime_error(
-        "Missing 'role' in routing destination specification");
-
-  const std::string name = uri.at("role");
-  std::string name_uc = name;
-  std::transform(name.begin(), name.end(), name_uc.begin(), ::toupper);
-
-  if (name_uc == "PRIMARY")
-    return DestMetadataCacheGroup::ServerRole::Primary;
-  else if (name_uc == "SECONDARY")
-    return DestMetadataCacheGroup::ServerRole::Secondary;
-  else if (name_uc == "PRIMARY_AND_SECONDARY")
-    return DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary;
-
-  throw std::runtime_error("Invalid server role in metadata cache routing '" +
-                           name + "'");
+const constexpr std::array<
+    std::pair<std::string_view, DestMetadataCacheGroup::ServerRole>, 3>
+    known_roles{{
+        {"PRIMARY", DestMetadataCacheGroup::ServerRole::Primary},
+        {"SECONDARY", DestMetadataCacheGroup::ServerRole::Secondary},
+        {"PRIMARY_AND_SECONDARY",
+         DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary},
+    }};
 }
 
-std::string get_server_role_name(
-    const DestMetadataCacheGroup::ServerRole role) {
-  switch (role) {
-    case DestMetadataCacheGroup::ServerRole::Primary:
-      return "PRIMARY";
-    case DestMetadataCacheGroup::ServerRole::Secondary:
-      return "SECONDARY";
-    case DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary:
-      return "PRIMARY_AND_SECONDARY";
+DestMetadataCacheGroup::ServerRole get_server_role_from_uri(
+    const mysqlrouter::URIQuery &uri) {
+  const auto it = uri.find("role");
+  if (it == uri.end()) {
+    throw std::runtime_error(
+        "Missing 'role' in routing destination specification");
   }
 
-  return "unknown";
+  const std::string name = it->second;
+  std::string name_uc;
+  name_uc.resize(name.size());
+  std::transform(name.begin(), name.end(), name_uc.begin(), ::toupper);
+
+  auto role_it =
+      std::find_if(known_roles.begin(), known_roles.end(),
+                   [name = name_uc](const auto &p) { return p.first == name; });
+
+  if (role_it == known_roles.end()) {
+    std::string valid_names;
+    for (auto role : known_roles) {
+      if (!valid_names.empty()) {
+        valid_names += ", ";
+      }
+
+      valid_names += role.first;
+    }
+
+    throw std::runtime_error(
+        "The role in '?role=" + name +
+        "' does not contain one of the valid role names: " + valid_names);
+  }
+
+  return role_it->second;
 }
 
 routing::RoutingStrategy get_default_routing_strategy(
@@ -103,27 +118,6 @@ routing::RoutingStrategy get_default_routing_strategy(
   return routing::RoutingStrategy::kUndefined;
 }
 
-/** @brief check that mode (if present) is correct for the role */
-bool mode_is_valid(const routing::AccessMode mode,
-                   const DestMetadataCacheGroup::ServerRole role) {
-  // no mode given, that's ok, nothing to check
-  if (mode == routing::AccessMode::kUndefined) {
-    return true;
-  }
-
-  switch (role) {
-    case DestMetadataCacheGroup::ServerRole::Primary:
-      return mode == routing::AccessMode::kReadWrite;
-    case DestMetadataCacheGroup::ServerRole::Secondary:
-    case DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary:
-      return mode == routing::AccessMode::kReadOnly;
-    default:;  //
-               /* fall-through, no acces mode is valid for that role */
-  }
-
-  return false;
-}
-
 // throws:
 // - runtime_error if invalid value for the option was discovered
 // - check_option_allowed() throws std::runtime_error (it is expected to throw
@@ -159,7 +153,7 @@ bool get_disconnect_on_promoted_to_primary(
   auto check_option_allowed = [&]() {
     if (role != DestMetadataCacheGroup::ServerRole::Secondary) {
       throw std::runtime_error("Option '" + kOptionName +
-                               "' is valid only for mode=SECONDARY");
+                               "' is valid only for role=SECONDARY");
     }
   };
 
@@ -177,24 +171,18 @@ bool get_disconnect_on_metadata_unavailable(const mysqlrouter::URIQuery &uri) {
                            check_option_allowed);
 }
 
-}  // namespace
-
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 // doxygen confuses 'const mysqlrouter::URIQuery &query' with
 // 'std::map<std::string, std::string>'
 DestMetadataCacheGroup::DestMetadataCacheGroup(
     net::io_context &io_ctx, const std::string &metadata_cache,
-    const std::string &replicaset,
     const routing::RoutingStrategy routing_strategy,
     const mysqlrouter::URIQuery &query, const Protocol::Type protocol,
-    const routing::AccessMode access_mode,
     metadata_cache::MetadataCacheAPIBase *cache_api)
     : RouteDestination(io_ctx, protocol),
       cache_name_(metadata_cache),
-      ha_replicaset_(replicaset),
       uri_query_(query),
       routing_strategy_(routing_strategy),
-      access_mode_(access_mode),
       server_role_(get_server_role_from_uri(query)),
       cache_api_(cache_api),
       disconnect_on_promoted_to_primary_(
@@ -205,23 +193,31 @@ DestMetadataCacheGroup::DestMetadataCacheGroup(
 }
 #endif
 
-std::pair<DestMetadataCacheGroup::AvailableDestinations, bool>
+std::pair<metadata_cache::cluster_nodes_list_t, bool>
 DestMetadataCacheGroup::get_available(
-    const metadata_cache::LookupResult &managed_servers,
+    const metadata_cache::cluster_nodes_list_t &instances,
     bool for_new_connections) const {
-  DestMetadataCacheGroup::AvailableDestinations result;
+  metadata_cache::cluster_nodes_list_t result;
 
   bool primary_fallback{false};
-  const auto &managed_servers_vec = managed_servers.instance_vector;
   if (routing_strategy_ == routing::RoutingStrategy::kRoundRobinWithFallback) {
     // if there are no secondaries available we fall-back to primaries
+    std::lock_guard<std::mutex> lock(
+        query_quarantined_destinations_callback_mtx_);
     auto secondary = std::find_if(
-        managed_servers_vec.begin(), managed_servers_vec.end(),
-        [](const metadata_cache::ManagedInstance &i) {
-          return i.mode == metadata_cache::ServerMode::ReadOnly && !i.hidden;
+        instances.begin(), instances.end(),
+        [&](const metadata_cache::ManagedInstance &i) {
+          if (for_new_connections && query_quarantined_destinations_callback_) {
+            return i.mode == metadata_cache::ServerMode::ReadOnly &&
+                   !i.hidden && !i.ignore &&
+                   !query_quarantined_destinations_callback_(i);
+          } else {
+            return i.mode == metadata_cache::ServerMode::ReadOnly &&
+                   !i.hidden && !i.ignore;
+          }
         });
 
-    primary_fallback = secondary == managed_servers_vec.end();
+    primary_fallback = secondary == instances.end();
   }
   // if we are gathering the nodes for the decision about keeping existing
   // connections we look also at the disconnect_on_promoted_to_primary_ setting
@@ -231,7 +227,8 @@ DestMetadataCacheGroup::get_available(
     primary_fallback = true;
   }
 
-  for (const auto &it : managed_servers_vec) {
+  for (const auto &it : instances) {
+    if (it.ignore) continue;
     if (for_new_connections) {
       // for new connections skip (do not include) the node if it is hidden - it
       // is not allowed
@@ -242,30 +239,25 @@ DestMetadataCacheGroup::get_available(
       if (it.hidden && it.disconnect_existing_sessions_when_hidden) continue;
     }
 
-    auto port = (protocol_ == Protocol::Type::kXProtocol) ? it.xport : it.port;
-
     // role=PRIMARY_AND_SECONDARY
     if ((server_role_ == ServerRole::PrimaryAndSecondary) &&
         (it.mode == metadata_cache::ServerMode::ReadWrite ||
          it.mode == metadata_cache::ServerMode::ReadOnly)) {
-      result.emplace_back(mysql_harness::TCPAddress(it.host, port),
-                          it.mysql_server_uuid);
+      result.emplace_back(it);
       continue;
     }
 
     // role=SECONDARY
     if (server_role_ == ServerRole::Secondary &&
         it.mode == metadata_cache::ServerMode::ReadOnly) {
-      result.emplace_back(mysql_harness::TCPAddress(it.host, port),
-                          it.mysql_server_uuid);
+      result.emplace_back(it);
       continue;
     }
 
     // role=PRIMARY
     if ((server_role_ == ServerRole::Primary || primary_fallback) &&
         it.mode == metadata_cache::ServerMode::ReadWrite) {
-      result.emplace_back(mysql_harness::TCPAddress(it.host, port),
-                          it.mysql_server_uuid);
+      result.emplace_back(it);
       continue;
     }
   }
@@ -273,20 +265,16 @@ DestMetadataCacheGroup::get_available(
   return {result, primary_fallback};
 }
 
-DestMetadataCacheGroup::AvailableDestinations
+metadata_cache::cluster_nodes_list_t
 DestMetadataCacheGroup::get_available_primaries(
-    const metadata_cache::LookupResult &managed_servers) const {
-  DestMetadataCacheGroup::AvailableDestinations result;
-  const auto &managed_servers_vec = managed_servers.instance_vector;
+    const metadata_cache::cluster_nodes_list_t &managed_servers) const {
+  metadata_cache::cluster_nodes_list_t result;
 
-  for (const auto &it : managed_servers_vec) {
-    if (it.hidden) continue;
-
-    auto port = (protocol_ == Protocol::Type::kXProtocol) ? it.xport : it.port;
+  for (const auto &it : managed_servers) {
+    if (it.hidden || it.ignore) continue;
 
     if (it.mode == metadata_cache::ServerMode::ReadWrite) {
-      result.emplace_back(mysql_harness::TCPAddress(it.host, port),
-                          it.mysql_server_uuid);
+      result.emplace_back(it);
     }
   }
 
@@ -296,62 +284,24 @@ DestMetadataCacheGroup::get_available_primaries(
 void DestMetadataCacheGroup::init() {
   // check if URI does not contain parameters that we don't understand
   for (const auto &uri_param : uri_query_) {
-    if (supported_params.count(uri_param.first) == 0) {
+    if (std::find(supported_params.begin(), supported_params.end(),
+                  uri_param.first) == supported_params.end()) {
       throw std::runtime_error(
           "Unsupported 'metadata-cache' parameter in URI: '" + uri_param.first +
           "'");
     }
   }
 
-  // if the routing strategy is set we don't allow mode to be set
-  if (routing_strategy_ != routing::RoutingStrategy::kUndefined &&
-      access_mode_ != routing::AccessMode::kUndefined) {
-    throw std::runtime_error(
-        "option 'mode' is not allowed together with 'routing_strategy' option");
-  }
-
-  bool routing_strategy_default{false};
   // if the routing_strategy is not set we go with the default based on the role
   if (routing_strategy_ == routing::RoutingStrategy::kUndefined) {
     routing_strategy_ = get_default_routing_strategy(server_role_);
-    routing_strategy_default = true;
   }
 
-  // check that mode (if present) is correct for the role
-  // we don't actually use it but support it for backward compatibility
-  // and parity with STANDALONE routing destinations
-  if (!mode_is_valid(access_mode_, server_role_)) {
-    throw std::runtime_error(
-        "mode '" + routing::get_access_mode_name(access_mode_) +
-        "' is not valid for 'role=" + get_server_role_name(server_role_) + "'");
-  }
-
-  // this is for backward compatibility
-  // old(allow_primary_reads + role=SECONDARY) = new
-  // (role=PRIMARY_AND_SECONDARY)
   auto query_part = uri_query_.find("allow_primary_reads");
   if (query_part != uri_query_.end()) {
-    if (server_role_ != ServerRole::Secondary) {
-      throw std::runtime_error(
-          "allow_primary_reads is supported only for SECONDARY routing");
-    }
-    if (!routing_strategy_default) {
-      throw std::runtime_error(
-          "allow_primary_reads is only supported for backward compatibility: "
-          "without routing_strategy but with mode defined, use "
-          "role=PRIMARY_AND_SECONDARY instead");
-    }
-    auto value = query_part->second;
-    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-    if (value == "yes") {
-      server_role_ = ServerRole::PrimaryAndSecondary;
-    } else if (value == "no") {
-      // it's a default but we allow it for consistency
-    } else {
-      throw std::runtime_error(
-          "Invalid value for allow_primary_reads option: '" +
-          query_part->second + "'");
-    }
+    throw std::runtime_error(
+        "allow_primary_reads is no longer supported, use "
+        "role=PRIMARY_AND_SECONDARY instead");
   }
 
   // validate routing strategy:
@@ -374,18 +324,23 @@ void DestMetadataCacheGroup::init() {
 }
 
 void DestMetadataCacheGroup::subscribe_for_metadata_cache_changes() {
-  cache_api_->add_state_listener(ha_replicaset_, this);
+  cache_api_->add_state_listener(this);
   subscribed_for_metadata_cache_changes_ = true;
 }
 
 void DestMetadataCacheGroup::subscribe_for_acceptor_handler() {
-  cache_api_->add_acceptor_handler_listener(ha_replicaset_, this);
+  cache_api_->add_acceptor_handler_listener(this);
+}
+
+void DestMetadataCacheGroup::subscribe_for_md_refresh_handler() {
+  cache_api_->add_md_refresh_listener(this);
 }
 
 DestMetadataCacheGroup::~DestMetadataCacheGroup() {
   if (subscribed_for_metadata_cache_changes_) {
-    cache_api_->remove_state_listener(ha_replicaset_, this);
-    cache_api_->remove_acceptor_handler_listener(ha_replicaset_, this);
+    cache_api_->remove_state_listener(this);
+    cache_api_->remove_acceptor_handler_listener(this);
+    cache_api_->remove_md_refresh_listener(this);
   }
 }
 
@@ -393,18 +348,17 @@ class MetadataCacheDestination : public Destination {
  public:
   MetadataCacheDestination(std::string id, std::string addr, uint16_t port,
                            DestMetadataCacheGroup *balancer,
-                           std::string server_uuid)
+                           std::string server_uuid,
+                           mysqlrouter::ServerMode server_mode)
       : Destination(std::move(id), std::move(addr), port),
         balancer_{balancer},
-        server_uuid_{std::move(server_uuid)} {}
+        server_uuid_{std::move(server_uuid)},
+        server_mode_{server_mode} {}
 
   void connect_status(std::error_code ec) override {
     last_ec_ = ec;
 
     if (ec != std::error_code{}) {
-      balancer_->cache_api()->mark_instance_reachability(
-          server_uuid_, metadata_cache::InstanceStatus::Unreachable);
-
       // the tests
       //
       // - NodeUnavailable/NodeUnavailableTest.NodeUnavailable/1, where
@@ -422,18 +376,22 @@ class MetadataCacheDestination : public Destination {
 
   std::error_code last_error_code() const { return last_ec_; }
 
+  mysqlrouter::ServerMode server_mode() const override { return server_mode_; }
+
  private:
   DestMetadataCacheGroup *balancer_;
 
   std::string server_uuid_;
 
+  mysqlrouter::ServerMode server_mode_;
+
   std::error_code last_ec_;
 };
 
 // the first round of destinations didn't succeed.
 //
 // try to fallback.
-stdx::expected<Destinations, void> DestMetadataCacheGroup::refresh_destinations(
+std::optional<Destinations> DestMetadataCacheGroup::refresh_destinations(
     const Destinations &previous_dests) {
   if (cache_api_->cluster_type() == mysqlrouter::ClusterType::RS_V2) {
     // ReplicaSet
@@ -454,11 +412,11 @@ stdx::expected<Destinations, void> DestMetadataCacheGroup::refresh_destinations(
              "previous destinations MUST be primary destinations");
 
       if (previous_dests.empty()) {
-        return stdx::make_unexpected();
+        return std::nullopt;
       }
 
       if (!previous_dests.is_primary_destination()) {
-        return stdx::make_unexpected();
+        return std::nullopt;
       }
 
       // if connecting to the primary failed differentiate between:
@@ -475,20 +433,23 @@ stdx::expected<Destinations, void> DestMetadataCacheGroup::refresh_destinations(
       auto const *primary_member = dynamic_cast<MetadataCacheDestination *>(
           previous_dests.begin()->get());
 
-      if (primary_member->last_error_code() ==
-          make_error_condition(std::errc::timed_out)) {
-        return stdx::make_unexpected();
+      const auto err = primary_member->last_error_code();
+      log_debug("refresh_destinations(): %s:%s", err.category().name(),
+                err.message().c_str());
+
+      if (err == make_error_condition(std::errc::timed_out) ||
+          err.category() == net::ip::resolver_category()) {
+        return std::nullopt;
       }
 
-      if (cache_api_->wait_primary_failover(ha_replicaset_,
-                                            primary_member->server_uuid(),
+      if (cache_api_->wait_primary_failover(primary_member->server_uuid(),
                                             kPrimaryFailoverTimeout)) {
         return primary_destinations();
       }
     }
   }
 
-  return stdx::make_unexpected();
+  return std::nullopt;
 }
 
 void DestMetadataCacheGroup::advance(size_t n) {
@@ -498,61 +459,180 @@ void DestMetadataCacheGroup::advance(size_t n) {
 }
 
 Destinations DestMetadataCacheGroup::balance(
-    const AvailableDestinations &available, bool primary_fallback) {
+    const metadata_cache::cluster_nodes_list_t &available,
+    bool primary_fallback) {
   Destinations dests;
 
+  auto from_instance = [this](const auto &dest) {
+    mysql_harness::TCPAddress addr(
+        dest.host,
+        (protocol_ == Protocol::Type::kXProtocol) ? dest.xport : dest.port);
+
+    return std::make_unique<MetadataCacheDestination>(
+        addr.str(), addr.address(), addr.port(), this, dest.mysql_server_uuid,
+        dest.mode);
+  };
+
   std::lock_guard<std::mutex> lk(mutex_update_);
 
   switch (routing_strategy_) {
     case routing::RoutingStrategy::kFirstAvailable: {
       for (auto const &dest : available) {
-        dests.push_back(std::make_unique<MetadataCacheDestination>(
-            dest.address.str(), dest.address.address(), dest.address.port(),
-            this, dest.id));
+        dests.push_back(from_instance(dest));
       }
 
       break;
     }
     case routing::RoutingStrategy::kRoundRobinWithFallback:
     case routing::RoutingStrategy::kRoundRobin: {
+      if (available.empty()) break;
+
       const auto sz = available.size();
       const auto end = available.end();
       const auto begin = available.begin();
 
-      auto cur = begin;
+      // start pos moves forward with each call to balance().
+      //
+      // ensure it wraps around.
 
       if (start_pos_ >= sz) start_pos_ = 0;
+      if (rw_start_pos_ >= sz) rw_start_pos_ = 0;
+      if (ro_start_pos_ >= sz) ro_start_pos_ = 0;
 
-      // move iterator forward and remember the position as 'last'
-      std::advance(cur, start_pos_);
-      auto last = cur;
-
-      // for start_pos == 2:
+      // Goal:
       //
-      // 0 1 2 3 4 x
-      // ^   ^     ^
-      // |   |     `- end
-      // |   `- last|cur
-      // `- begin
-
-      // from last to end;
+      // - writes round-robins over read-write-servers
+      // - reads  round-robins over read-only servers
       //
-      // dests = [2 3 4]
-      for (; cur != end; ++cur) {
-        dests.push_back(std::make_unique<MetadataCacheDestination>(
-            cur->address.str(), cur->address.address(), cur->address.port(),
-            this, cur->id));
+      // Example:
+      //
+      // available  = [ W1, R1, R2, W2, R3 ]
+      // writers    = [ W1, W2 ]
+      // readers    = [ R1, R2, R3 ]
+      //
+      // Each group is rotated independently and then appened
+      // depending on the server-mode of the current "start_pos".
+      //
+      // input:         -> output         (auth, write, read)
+      //
+      // W1 R1 R2 W2 R3 -> W1 W2 R1 R2 R3 (a: W1, w: W1, r: R1)
+      // R1 R2 W2 R3 W1 -> R2 R3 R1 W2 W1 (a: R2: w: W2, r: R2)
+      // R2 W2 R3 W1 R1 -> R3 R1 R2 W1 W2 (a: R3: w: W1, r: R3)
+      // W2 R3 W1 R1 R2 -> W2 W1 R1 R2 R3 (a: W2: w: W2, r: R1)
+      // R3 W1 R1 R2 W2 -> R2 R3 R1 W1 W2 (a: R2: w: W1, r: R2)
+      // W1 R1 R2 W2 R3 -> W2 W1 R3 R1 R2 (a: W2: w: W2, r: R3)
+      //
+      // ^^- start-pos -> initial-server-mode.
+
+      //
+      const auto initial_server_mode = available[start_pos_].mode;
+
+      auto &initial_server_mode_start_pos =
+          initial_server_mode == mysqlrouter::ServerMode::ReadOnly
+              ? ro_start_pos_
+              : rw_start_pos_;
+
+      auto &other_server_mode_start_pos =
+          initial_server_mode == mysqlrouter::ServerMode::ReadOnly
+              ? rw_start_pos_
+              : ro_start_pos_;
+
+      {
+        if (initial_server_mode_start_pos >= sz) {
+          initial_server_mode_start_pos = 0;
+        }
+
+        // move iterator forward and remember the position as 'last'
+        auto cur = begin;
+        std::advance(cur, initial_server_mode_start_pos);
+        auto last = cur;
+
+        auto pos = initial_server_mode_start_pos;
+        bool is_first{true};
+
+        // for start_pos == 2:
+        //
+        // 0 1 2 3 4 x
+        // ^   ^     ^
+        // |   |     `- end
+        // |   `- last|cur
+        // `- begin
+
+        // from last to end;
+        //
+        // dests = [2 3 4]
+        for (; cur != end; ++cur, ++pos) {
+          if (cur->mode == initial_server_mode) {
+            if (is_first) {
+              initial_server_mode_start_pos = pos;
+              is_first = false;
+            }
+            dests.push_back(from_instance(*cur));
+          }
+        }
+
+        // from begin to before-last
+        //
+        // dests = [2 3 4] + [0 1]
+        for (cur = begin, pos = 0; cur != last; ++cur, ++pos) {
+          if (cur->mode == initial_server_mode) {
+            if (is_first) {
+              initial_server_mode_start_pos = pos;
+              is_first = false;
+            }
+            dests.push_back(from_instance(*cur));
+          }
+        }
+
+        if (++initial_server_mode_start_pos >= sz) {
+          initial_server_mode_start_pos = 0;
+        }
       }
 
-      // from begin to before-last
-      //
-      // dests = [2 3 4] + [0 1]
-      for (cur = begin; cur != last; ++cur) {
-        dests.push_back(std::make_unique<MetadataCacheDestination>(
-            cur->address.str(), cur->address.address(), cur->address.port(),
-            this, cur->id));
-      }
+      // ... and now the other server-mode set:
+      {
+        if (other_server_mode_start_pos >= sz) {
+          other_server_mode_start_pos = 0;
+        }
 
+        // move iterator forward and remember the position as 'last'
+        auto cur = begin;
+        std::advance(cur, other_server_mode_start_pos);
+        auto last = cur;
+
+        auto pos = other_server_mode_start_pos;
+        bool is_first{true};
+
+        // from last to end;
+        //
+        // dests = [2 3 4]
+        for (; cur != end; ++cur, ++pos) {
+          if (cur->mode != initial_server_mode) {
+            if (is_first) {
+              other_server_mode_start_pos = pos;
+              is_first = false;
+            }
+            dests.push_back(from_instance(*cur));
+          }
+        }
+
+        // from begin to before-last
+        //
+        // dests = [2 3 4] + [0 1]
+        for (cur = begin, pos = 0; cur != last; ++cur, ++pos) {
+          if (cur->mode != initial_server_mode) {
+            if (is_first) {
+              other_server_mode_start_pos = pos;
+              is_first = false;
+            }
+            dests.push_back(from_instance(*cur));
+          }
+        }
+
+        if (++other_server_mode_start_pos >= sz) {
+          other_server_mode_start_pos = 0;
+        }
+      }
       // NOTE: AsyncReplicasetTest.SecondaryAdded from
       // routertest_component_async_replicaset depends on the start_pos_ is
       // capped here.
@@ -574,8 +654,7 @@ Destinations DestMetadataCacheGroup::balance(
   }
 
   if (dests.empty()) {
-    log_warning("No available servers found for '%s' %s routing",
-                ha_replicaset_.c_str(),
+    log_warning("No available servers found for %s routing",
                 server_role_ == ServerRole::Primary ? "PRIMARY" : "SECONDARY");
 
     // return an empty list
@@ -597,12 +676,9 @@ Destinations DestMetadataCacheGroup::balance(
 Destinations DestMetadataCacheGroup::destinations() {
   if (!cache_api_->is_initialized()) return {};
 
-  AvailableDestinations available;
-  bool primary_failover;
-  const auto &all_replicaset_nodes =
-      cache_api_->lookup_replicaset(ha_replicaset_).instance_vector;
+  const auto &all_replicaset_nodes = cache_api_->get_cluster_nodes();
 
-  std::tie(available, primary_failover) = get_available(all_replicaset_nodes);
+  auto [available, primary_failover] = get_available(all_replicaset_nodes);
 
   return balance(available, primary_failover);
 }
@@ -610,8 +686,7 @@ Destinations DestMetadataCacheGroup::destinations() {
 Destinations DestMetadataCacheGroup::primary_destinations() {
   if (!cache_api_->is_initialized()) return {};
 
-  const auto &all_replicaset_nodes =
-      cache_api_->lookup_replicaset(ha_replicaset_).instance_vector;
+  const auto &all_replicaset_nodes = cache_api_->get_cluster_nodes();
 
   auto available = get_available_primaries(all_replicaset_nodes);
 
@@ -623,21 +698,20 @@ DestMetadataCacheGroup::AddrVector DestMetadataCacheGroup::get_destinations()
   // don't call lookup if the cache-api is not ready yet.
   if (!cache_api_->is_initialized()) return {};
 
-  auto available =
-      get_available(
-          cache_api_->lookup_replicaset(ha_replicaset_).instance_vector)
-          .first;
+  auto available = get_available(cache_api_->get_cluster_nodes()).first;
 
   AddrVector addresses;
   for (const auto &dest : available) {
-    addresses.emplace_back(dest.address);
+    addresses.emplace_back(dest.host, protocol_ == Protocol::Type::kXProtocol
+                                          ? dest.xport
+                                          : dest.port);
   }
 
   return addresses;
 }
 
 void DestMetadataCacheGroup::on_instances_change(
-    const metadata_cache::LookupResult &instances,
+    const metadata_cache::ClusterTopology &cluster_topology,
     const bool md_servers_reachable) {
   // we got notified that the metadata has changed.
   // If instances is empty then (most like is empty)
@@ -648,57 +722,61 @@ void DestMetadataCacheGroup::on_instances_change(
   const bool disconnect =
       md_servers_reachable || disconnect_on_metadata_unavailable_;
 
+  const auto instances = cluster_topology.get_all_members();
   const std::string reason =
       md_servers_reachable ? "metadata change" : "metadata unavailable";
 
-  const auto &nodes_for_new_connections =
-      get_available(instances, /*for_new_connections=*/true).first;
+  auto from_instances = [this](const auto &dests) {
+    std::vector<AvailableDestination> result;
 
-  AllowedNodes new_addresses;
-  for (const auto &dest : nodes_for_new_connections) {
-    new_addresses.emplace_back(dest.address.str());
-  }
+    for (const auto &dest : dests) {
+      mysql_harness::TCPAddress addr(
+          dest.host,
+          (protocol_ == Protocol::Type::kXProtocol) ? dest.xport : dest.port);
 
-  const auto &nodes_for_existing_connections =
-      get_available(instances, /*for_new_connections=*/false).first;
-  AllowedNodes addresses;
-  for (const auto &dest : nodes_for_existing_connections) {
-    addresses.emplace_back(dest.address.str());
-  }
+      result.emplace_back(addr, dest.mysql_server_uuid, dest.mode);
+    }
+
+    return result;
+  };
+
+  auto nodes_for_new_connections = from_instances(
+      get_available(instances, /*for_new_connections=*/true).first);
+
+  auto nodes_for_existing_connections = from_instances(
+      get_available(instances, /*for_new_connections=*/false).first);
 
   std::lock_guard<std::mutex> lock(allowed_nodes_change_callbacks_mtx_);
 
   // notify all the registered listeners about the list of available nodes
   // change
   for (auto &clb : allowed_nodes_change_callbacks_) {
-    clb(addresses, new_addresses, disconnect, reason);
+    clb(nodes_for_existing_connections, nodes_for_new_connections, disconnect,
+        reason);
   }
 }
 
 void DestMetadataCacheGroup::notify_instances_changed(
-    const metadata_cache::LookupResult &instances,
-    const bool md_servers_reachable, const unsigned /*view_id*/) noexcept {
-  on_instances_change(instances, md_servers_reachable);
+    const metadata_cache::ClusterTopology &cluster_topology,
+    const bool md_servers_reachable, const uint64_t /*view_id*/) noexcept {
+  on_instances_change(cluster_topology, md_servers_reachable);
 }
 
 bool DestMetadataCacheGroup::update_socket_acceptor_state(
-    const metadata_cache::LookupResult &instances) noexcept {
+    const metadata_cache::cluster_nodes_list_t &instances) noexcept {
   const auto &nodes_for_new_connections =
       get_available(instances, /*for_new_connections=*/true).first;
 
-  AllowedNodes new_addresses;
-  for (const auto &dest : nodes_for_new_connections) {
-    new_addresses.emplace_back(dest.address.str());
-  }
-
   {
     std::lock_guard<std::mutex> lock(socket_acceptor_handle_callbacks_mtx);
-    if (!new_addresses.empty() && start_router_socket_acceptor_callback_) {
+    if (!nodes_for_new_connections.empty() &&
+        start_router_socket_acceptor_callback_) {
       const auto &start_acceptor_res = start_router_socket_acceptor_callback_();
       return start_acceptor_res ? true : false;
     }
 
-    if (new_addresses.empty() && stop_router_socket_acceptor_callback_) {
+    if (nodes_for_new_connections.empty() &&
+        stop_router_socket_acceptor_callback_) {
       stop_router_socket_acceptor_callback_();
       return true;
     }
@@ -707,6 +785,34 @@ bool DestMetadataCacheGroup::update_socket_acceptor_state(
   return true;
 }
 
+void DestMetadataCacheGroup::on_md_refresh(
+    const bool nodes_changed,
+    const metadata_cache::ClusterTopology &cluster_topology) {
+  const auto instances = cluster_topology.get_all_members();
+
+  auto from_instances = [this](const auto &dests) {
+    std::vector<AvailableDestination> result;
+
+    for (const auto &dest : dests) {
+      mysql_harness::TCPAddress addr(
+          dest.host,
+          (protocol_ == Protocol::Type::kXProtocol) ? dest.xport : dest.port);
+
+      result.emplace_back(addr, dest.mysql_server_uuid, dest.mode);
+    }
+
+    return result;
+  };
+
+  const auto available_nodes = from_instances(
+      get_available(instances, /*for_new_connections=*/true).first);
+
+  std::lock_guard<std::mutex> lock(md_refresh_callback_mtx_);
+  if (md_refresh_callback_) {
+    md_refresh_callback_(nodes_changed, available_nodes);
+  }
+}
+
 void DestMetadataCacheGroup::start(const mysql_harness::PluginFuncEnv *env) {
   // before using metadata-cache we need to wait for it to be initialized
   while (!cache_api_->is_initialized() && (!env || is_running(env))) {
@@ -716,5 +822,6 @@ void DestMetadataCacheGroup::start(const mysql_harness::PluginFuncEnv *env) {
   if (!env || is_running(env)) {
     subscribe_for_metadata_cache_changes();
     subscribe_for_acceptor_handler();
+    subscribe_for_md_refresh_handler();
   }
 }
