@@ -59,7 +59,6 @@ var common_responses = common_stmts.prepare_statement_responses(
       "router_clusterset_view_id",
       "router_clusterset_all_nodes_by_clusterset_id",
       "router_clusterset_present",
-      "router_bootstrap_target_type",
       "router_clusterset_select_cluster_info_by_primary_role",
       "router_clusterset_select_cluster_info_by_gr_uuid",
       "router_clusterset_select_gr_members_status",
@@ -86,6 +85,10 @@ var router_update_last_check_in =
     common_stmts.get("router_update_last_check_in_v2", options);
 
 
+var router_bootstrap_target_type =
+    common_stmts.get("router_bootstrap_target_type", options);
+
+
 ({
   handshake: {greeting: {server_version: mysqld.global.server_version}},
   stmts: function(stmt) {
@@ -102,6 +105,14 @@ var router_update_last_check_in =
         (res = common_stmts.handle_regex_stmt(stmt, common_responses_regex)) !==
         undefined) {
       return res;
+    } else if (stmt === router_bootstrap_target_type.stmt) {
+      if (mysqld.global.simulate_router_options_no_rows === 1) {
+        return {
+          result: {columns: [{name: "NONE", type: "STRING"}], rows: []}
+        }
+      } else {
+        return router_bootstrap_target_type;
+      }
     } else if (stmt.match(router_update_attributes.stmt_regex)) {
       mysqld.global.update_attributes_count++;
       return router_update_attributes;
