@@ -15,8 +15,8 @@ if (mysqld.global.rest_user_credentials === undefined) {
   mysqld.global.rest_user_credentials = [];
 }
 
-if (mysqld.global.metadata_version === undefined) {
-  mysqld.global.metadata_version = [0, 0, 0];
+if (mysqld.global.metadata_schema_version === undefined) {
+  mysqld.global.metadata_schema_version = [0, 0, 0];
 }
 
 if (mysqld.global.rest_auth_query_count === undefined) {
@@ -27,35 +27,23 @@ if (mysqld.global.error_on_md_query === undefined) {
   mysqld.global.error_on_md_query = 0;
 }
 
-var nodes = function(host, port_and_state) {
-  return port_and_state.map(function(current_value) {
-    return [
-      current_value[0], host, current_value[1], current_value[2],
-      current_value[3]
-    ];
-  });
-};
 
 
-var group_replication_members_online =
-    nodes(gr_node_host, mysqld.global.gr_nodes, mysqld.global.gr_id);
+var members = gr_memberships.gr_members(
+    mysqld.global.gr_node_host, mysqld.global.gr_nodes);
 
 var options = {
-  metadata_schema_version: mysqld.global.metadata_version,
-  group_replication_members: group_replication_members_online,
+  metadata_schema_version: mysqld.global.metadata_schema_version,
+  group_replication_members: members,
   innodb_cluster_instances: gr_memberships.cluster_nodes(
       mysqld.global.gr_node_host, mysqld.global.cluster_nodes),
   gr_id: mysqld.global.gr_id,
   view_id: mysqld.global.view_id,
-  primary_port: group_replication_members_online[mysqld.global.primary_id][2],
   cluster_type: "gr",
   innodb_cluster_name: "test",
   rest_user_credentials: mysqld.global.rest_user_credentials
 };
 
-options.group_replication_primary_member =
-    options.group_replication_members[mysqld.global.primary_id][0];
-
 // prepare the responses for common statements
 var common_responses = common_stmts.prepare_statement_responses(
     [
@@ -65,15 +53,14 @@ var common_responses = common_stmts.prepare_statement_responses(
       "select_port",
       "router_commit",
       "router_rollback",
-      "router_select_metadata",
       "router_select_schema_version",
       "router_select_cluster_type_v2",
       "router_check_member_state",
       "router_select_members_count",
-      "router_select_group_replication_primary_member",
-      "router_select_group_membership_with_primary_mode",
+      "router_select_group_membership",
       "router_select_metadata_v2_gr",
       "router_update_last_check_in_v2",
+      "router_select_router_options_view",
     ],
     options);
 
