@@ -50,6 +50,8 @@ class SharedServer {
 
   stdx::expected<void, MysqlError> shutdown();
 
+  static stdx::expected<void, MysqlError> shutdown(MysqlClient &cli);
+
   [[nodiscard]] std::string mysqld_init_once_dir_name() const;
 
   [[nodiscard]] std::string mysqld_dir_name() const;
@@ -78,38 +80,72 @@ class SharedServer {
 
   stdx::expected<MysqlClient, MysqlError> admin_cli();
 
-  void create_schema(MysqlClient &cli, const std::string &schema);
+  static void create_schema(MysqlClient &cli, const std::string &schema);
 
-  void grant_access(MysqlClient &cli, const Account &account,
-                    const std::string &rights);
+  static void grant_access(MysqlClient &cli, const Account &account,
+                           const std::string &rights);
 
-  void grant_access(MysqlClient &cli, const Account &account,
-                    const std::string &rights, const std::string &schema);
+  static void grant_access(MysqlClient &cli, const Account &account,
+                           const std::string &rights,
+                           const std::string &schema);
 
-  void create_account(MysqlClient &cli, Account account);
+  static void create_account(MysqlClient &cli, Account account);
 
-  void drop_account(MysqlClient &cli, Account account);
+  static void drop_account(MysqlClient &cli, Account account);
 
   void setup_mysqld_accounts();
 
+  void install_plugins();
+
+  static void install_plugins(MysqlClient &cli);
+
   void flush_privileges();
 
-  void flush_privileges(MysqlClient &cli);
+  static void flush_privileges(MysqlClient &cli);
 
   // get all connections, but ignore internal connections and this
   // connection.
   static stdx::expected<std::vector<uint64_t>, MysqlError> user_connection_ids(
-      MysqlClient &cli);
+      MysqlClient &cli) {
+    return user_connection_ids(cli, default_usernames());
+  }
+
+  static stdx::expected<std::vector<uint64_t>, MysqlError> user_connection_ids(
+      MysqlClient &cli, const std::vector<std::string> &usernames);
+
+  static std::vector<std::string> default_usernames() {
+    return {
+        admin_account().username,
+        caching_sha2_empty_password_account().username,
+        caching_sha2_password_account().username,
+        native_empty_password_account().username,
+        native_password_account().username,
+        sha256_empty_password_account().username,
+        sha256_password_account().username,
+        sha256_short_password_account().username,
+    };
+  }
 
   // close all connections.
-  void close_all_connections();
+  stdx::expected<void, MysqlError> close_all_connections() {
+    return close_all_connections(default_usernames());
+  }
 
-  void close_all_connections(MysqlClient &cli);
+  stdx::expected<void, MysqlError> close_all_connections(
+      const std::vector<std::string> &usernames);
+
+  static stdx::expected<void, MysqlError> close_all_connections(
+      MysqlClient &cli) {
+    return close_all_connections(cli, default_usernames());
+  }
+
+  static stdx::expected<void, MysqlError> close_all_connections(
+      MysqlClient &cli, const std::vector<std::string> &usernames);
 
   // set some session-vars back to defaults.
   void reset_to_defaults();
 
-  void reset_to_defaults(MysqlClient &cli);
+  static void reset_to_defaults(MysqlClient &cli);
 
   [[nodiscard]] bool mysqld_failed_to_start() const {
     return mysqld_failed_to_start_;
