@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,10 +26,10 @@
 #ifndef CONNECTION_HANDLER_MANAGER_INCLUDED
 #define CONNECTION_HANDLER_MANAGER_INCLUDED
 
+#include <assert.h>
 #include <stddef.h>
 #include <sys/types.h>
 
-#include "my_dbug.h"
 #include "mysql/psi/mysql_cond.h"  // mysql_cond_t
 #include "mysql/psi/mysql_mutex.h"
 #include "sql/conn_handler/connection_handler.h"  // Connection_handler
@@ -40,7 +41,7 @@ struct mysql_mutex_t;
 
 /**
   Functions to notify interested connection handlers
-  of events like begining of wait and end of wait and post-kill
+  of events like beginning of wait and end of wait and post-kill
   notification events.
 */
 struct THD_event_functions {
@@ -78,7 +79,7 @@ class Connection_handler_manager {
   */
   Connection_handler_manager(Connection_handler *connection_handler)
       : m_connection_handler(connection_handler),
-        m_saved_connection_handler(NULL),
+        m_saved_connection_handler(nullptr),
         m_saved_thread_handling(0),
         m_aborted_connects(0),
         m_connection_errors_max_connection(0) {}
@@ -135,7 +136,7 @@ class Connection_handler_manager {
     Singleton method to return an instance of this class.
   */
   static Connection_handler_manager *get_instance() {
-    DBUG_ASSERT(m_instance != NULL);
+    assert(m_instance != nullptr);
     return m_instance;
   }
 
@@ -163,11 +164,15 @@ class Connection_handler_manager {
   /**
     Increment connection count if max_connections is not exceeded.
 
+    @param ignore_max_connection_count  true if checking for a limit
+                                        specified by the max-connections
+                                        server option should be skipped
+
     @retval
       true   max_connections NOT exceeded
       false  max_connections reached
   */
-  bool check_and_incr_conn_count(bool is_admin_connection);
+  bool check_and_incr_conn_count(bool ignore_max_connection_count);
 
   /**
     Reset the max_used_connections counter to the number of current
@@ -228,5 +233,15 @@ class Connection_handler_manager {
     wat till connection_count to become zero.
   */
   static void wait_till_no_connection();
+
+  /**
+    Return number of connections in thread-safe way.
+  */
+  static uint get_connection_count() {
+    mysql_mutex_lock(&LOCK_connection_count);
+    const uint res = connection_count;
+    mysql_mutex_unlock(&LOCK_connection_count);
+    return res;
+  }
 };
 #endif  // CONNECTION_HANDLER_MANAGER_INCLUDED.
