@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,12 +23,13 @@
 
 #include "sql/dd/impl/cache/storage_adapter.h"
 
+#include <assert.h>
 #include <memory>
 #include <string>
 
 #include "mutex_lock.h"  // Mutex_lock
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_inttypes.h"
 #include "my_sys.h"
 #include "mysql/components/services/log_builtins.h"
@@ -67,8 +69,8 @@
 #include "sql/dd/types/table_stat.h"                // Table_stat
 #include "sql/dd/types/tablespace.h"                // Tablespace
 #include "sql/dd/types/view.h"                      // View
-#include "sql/dd/upgrade_57/upgrade.h"              // allow_sdi_creation
 #include "sql/debug_sync.h"                         // DEBUG_SYNC
+#include "sql/error_handler.h"                      // Internal_error_handler
 #include "sql/log.h"
 #include "sql/sql_class.h"  // THD
 
@@ -103,7 +105,7 @@ Object_id Storage_adapter::core_get_id(const typename T::Name_key &key) {
   MUTEX_LOCK(lock, &m_lock);
   m_core_registry.get(key, &element);
   if (element) {
-    DBUG_ASSERT(element->object());
+    assert(element->object());
     return element->object()->id();
   }
   return INVALID_OBJECT_ID;
@@ -112,7 +114,7 @@ Object_id Storage_adapter::core_get_id(const typename T::Name_key &key) {
 // Get a dictionary object from core storage.
 template <typename K, typename T>
 void Storage_adapter::core_get(const K &key, const T **object) {
-  DBUG_ASSERT(object);
+  assert(object);
   *object = nullptr;
   Cache_element<typename T::Cache_partition> *element = nullptr;
   MUTEX_LOCK(lock, &m_lock);
@@ -134,7 +136,7 @@ void Storage_adapter::core_update(const dd::Tablespace *new_tsp) {
   typename dd::Tablespace::Id_key key(new_tsp->id());
   MUTEX_LOCK(lock, &m_lock);
   m_core_registry.get(key, &element);
-  DBUG_ASSERT(element != nullptr);
+  assert(element != nullptr);
   m_core_registry.remove(element);
   std::unique_ptr<const dd::Tablespace> old{element->object()};
 
@@ -147,7 +149,7 @@ void Storage_adapter::core_update(const dd::Tablespace *new_tsp) {
 template <typename K, typename T>
 bool Storage_adapter::get(THD *thd, const K &key, enum_tx_isolation isolation,
                           bool bypass_core_registry, const T **object) {
-  DBUG_ASSERT(object);
+  assert(object);
   *object = nullptr;
 
   if (!bypass_core_registry) {
@@ -166,7 +168,7 @@ bool Storage_adapter::get(THD *thd, const K &key, enum_tx_isolation isolation,
   trx.otx.register_tables<T>();
 
   if (trx.otx.open_tables()) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -177,15 +179,15 @@ bool Storage_adapter::get(THD *thd, const K &key, enum_tx_isolation isolation,
   // Find record by the object-id.
   std::unique_ptr<Raw_record> r;
   if (t->find_record(key, r)) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
   // Restore the object from the record.
-  Entity_object *new_object = NULL;
+  Entity_object *new_object = nullptr;
   if (r.get() &&
       table.restore_object_from_record(&trx.otx, *r.get(), &new_object)) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -198,7 +200,7 @@ bool Storage_adapter::get(THD *thd, const K &key, enum_tx_isolation isolation,
       /* purecov: begin inspected */
       my_error(ER_INVALID_DD_OBJECT, MYF(0), new_object->name().c_str());
       delete new_object;
-      DBUG_ASSERT(false);
+      assert(false);
       return true;
       /* purecov: end */
     }
@@ -209,11 +211,10 @@ bool Storage_adapter::get(THD *thd, const K &key, enum_tx_isolation isolation,
 
 // Drop a dictionary object from core storage.
 template <typename T>
-void Storage_adapter::core_drop(THD *thd MY_ATTRIBUTE((unused)),
-                                const T *object) {
-  DBUG_ASSERT(s_use_fake_storage || thd->is_dd_system_thread());
-  DBUG_ASSERT(bootstrap::DD_bootstrap_ctx::instance().get_stage() <=
-              bootstrap::Stage::CREATED_TABLES);
+void Storage_adapter::core_drop(THD *thd [[maybe_unused]], const T *object) {
+  assert(s_use_fake_storage || thd->is_dd_system_thread());
+  assert(bootstrap::DD_bootstrap_ctx::instance().get_stage() <=
+         bootstrap::Stage::CREATED_TABLES);
   Cache_element<typename T::Cache_partition> *element = nullptr;
   MUTEX_LOCK(lock, &m_lock);
 
@@ -247,7 +248,7 @@ bool Storage_adapter::drop(THD *thd, const T *object) {
   }
 
   if (object->impl()->validate()) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -261,7 +262,7 @@ bool Storage_adapter::drop(THD *thd, const T *object) {
   ctx.otx.register_tables<T>();
 
   if (ctx.otx.open_tables() || object->impl()->drop(&ctx.otx)) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -271,9 +272,9 @@ bool Storage_adapter::drop(THD *thd, const T *object) {
 // Store a dictionary object to core storage.
 template <typename T>
 void Storage_adapter::core_store(THD *thd, T *object) {
-  DBUG_ASSERT(s_use_fake_storage || thd->is_dd_system_thread());
-  DBUG_ASSERT(bootstrap::DD_bootstrap_ctx::instance().get_stage() <=
-              bootstrap::Stage::CREATED_TABLES);
+  assert(s_use_fake_storage || thd->is_dd_system_thread());
+  assert(bootstrap::DD_bootstrap_ctx::instance().get_stage() <=
+         bootstrap::Stage::CREATED_TABLES);
   Cache_element<typename T::Cache_partition> *element =
       new Cache_element<typename T::Cache_partition>();
 
@@ -281,7 +282,9 @@ void Storage_adapter::core_store(THD *thd, T *object) {
     // For unit tests, drop old object (based on id) to simulate update.
     if (s_use_fake_storage) core_drop(thd, object);
   } else {
-    dynamic_cast<dd::Entity_object_impl *>(object)->set_id(next_oid<T>());
+    dd::Entity_object_impl *object_impl =
+        dynamic_cast<dd::Entity_object_impl *>(object);
+    object_impl->set_id(next_oid<T>());
   }
 
   // Need to clone since core registry takes ownership
@@ -291,6 +294,20 @@ void Storage_adapter::core_store(THD *thd, T *object) {
   m_core_registry.put(element);
 }
 
+// Re-map error messages emitted during DDL with innodb-read-only == 1.
+class Open_dictionary_tables_error_handler : public Internal_error_handler {
+ public:
+  bool handle_condition(THD *, uint sql_errno, const char *,
+                        Sql_condition::enum_severity_level *,
+                        const char *) override {
+    if (sql_errno == ER_OPEN_AS_READONLY) {
+      my_error(ER_READ_ONLY_MODE, MYF(0));
+      return true;
+    }
+    return false;
+  }
+};
+
 // Store a dictionary object to persistent storage.
 template <typename T>
 bool Storage_adapter::store(THD *thd, T *object) {
@@ -302,7 +319,7 @@ bool Storage_adapter::store(THD *thd, T *object) {
   }
 
   if (object->impl()->validate()) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -312,16 +329,20 @@ bool Storage_adapter::store(THD *thd, T *object) {
   ctx.otx.register_tables<T>();
   DEBUG_SYNC(thd, "before_storing_dd_object");
 
+  Open_dictionary_tables_error_handler error_handler;
+  thd->push_internal_handler(&error_handler);
   if (ctx.otx.open_tables() || object->impl()->store(&ctx.otx)) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
+    thd->pop_internal_handler();
     return true;
   }
+  thd->pop_internal_handler();
 
   // Do not create SDIs for tablespaces and tables while creating
   // dictionary entry during upgrade.
   if (bootstrap::DD_bootstrap_ctx::instance().get_stage() >
           bootstrap::Stage::CREATED_TABLES &&
-      dd::upgrade_57::allow_sdi_creation() && sdi::store(thd, object))
+      sdi::store(thd, object))
     return true;
 
   return false;
@@ -331,9 +352,9 @@ bool Storage_adapter::store(THD *thd, T *object) {
 template <typename T>
 bool Storage_adapter::core_sync(THD *thd, const typename T::Name_key &key,
                                 const T *object) {
-  DBUG_ASSERT(thd->is_dd_system_thread());
-  DBUG_ASSERT(bootstrap::DD_bootstrap_ctx::instance().get_stage() <=
-              bootstrap::Stage::CREATED_TABLES);
+  assert(thd->is_dd_system_thread());
+  assert(bootstrap::DD_bootstrap_ctx::instance().get_stage() <=
+         bootstrap::Stage::CREATED_TABLES);
 
   // Copy the name, needed for error output. The object has to be
   // dropped before get().
@@ -398,7 +419,7 @@ void Storage_adapter::erase_all() {
 
 // Dump the contents of the core storage.
 void Storage_adapter::dump() {
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   MUTEX_LOCK(lock, &m_lock);
   fprintf(stderr, "================================\n");
   fprintf(stderr, "Storage adapter\n");
