@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -104,7 +105,9 @@ Routines::Routines() {
   m_target_def.add_field(
       FIELD_DTD_IDENTIFIER, "DTD_IDENTIFIER",
       "IF(rtn.type = 'PROCEDURE', NULL, rtn.result_data_type_utf8)");
-  m_target_def.add_field(FIELD_ROUTINE_BODY, "ROUTINE_BODY", "'SQL'");
+  m_target_def.add_field(
+      FIELD_ROUTINE_BODY, "ROUTINE_BODY",
+      "IF(rtn.external_language = 'SQL', 'SQL', 'EXTERNAL')");
   m_target_def.add_field(
       FIELD_ROUTINE_DEFINITION, "ROUTINE_DEFINITION",
       "IF (CAN_ACCESS_ROUTINE(sch.name, rtn.name, rtn.type, rtn.definer, TRUE),"
