@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,17 +42,17 @@
 
 #include "field_types.h"  // enum_field_types
 #include "lex_string.h"
-#include "m_string.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/plugin.h"
 #include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
 #include "sql/dd/dd_schema.h"                // dd::Schema_MDL_locker
 #include "sql/dd/dd_table.h"                 // dd::get_sql_type_by_field_info
+#include "sql/dd/dd_utility.h"               // check_if_server_ddse_readonly
 #include "sql/dd/impl/bootstrap/bootstrap_ctx.h"  // dd::bootstrap::DD_boot...
 #include "sql/dd/impl/bootstrap/bootstrapper.h"   // dd::Column
 #include "sql/dd/impl/dictionary_impl.h"          // dd::Dictionary_impl
@@ -65,7 +66,6 @@
 #include "sql/dd/types/system_view_definition.h"  // dd::System_view_definition
 #include "sql/dd/types/view.h"
 #include "sql/dd_sql_view.h"  // update_referencing_views_metadata
-#include "sql/handler.h"
 #include "sql/item_create.h"
 #include "sql/mdl.h"
 #include "sql/mysqld.h"     // opt_readonly
@@ -78,6 +78,9 @@
 #include "sql/system_variables.h"
 #include "sql/table.h"
 #include "sql/thd_raii.h"
+#include "string_with_len.h"
+
+struct CHARSET_INFO;
 
 namespace {
 
@@ -88,33 +91,6 @@ const dd::String_type SERVER_I_S_TABLE_STRING("server_i_s_table");
 
 }  // namespace
 
-/**
-  Check if DDSE (Data Dictionary Storage Engine) is in
-  readonly mode.
-
-  @param thd                 Thread
-  @param schema_name_abbrev  Abbreviation of schema (I_S or P_S) for use
-                             in warning message output
-
-  @returns false on success, otherwise true.
-*/
-bool check_if_server_ddse_readonly(THD *thd, const char *schema_name_abbrev) {
-  /*
-    We must check if the DDSE is started in a way that makes the DD
-    read only. For now, we only support InnoDB as SE for the DD. The call
-    to retrieve the handlerton for the DDSE should be replaced by a more
-    generic mechanism.
-  */
-  handlerton *ddse = ha_resolve_by_legacy_type(thd, DB_TYPE_INNODB);
-  if (ddse->is_dict_readonly && ddse->is_dict_readonly()) {
-    LogErr(WARNING_LEVEL, ER_SKIP_UPDATING_METADATA_IN_SE_RO_MODE,
-           schema_name_abbrev);
-    return true;
-  }
-
-  return false;
-}
-
 namespace {
 
 // Hold context during IS metadata update to DD.
@@ -140,7 +116,7 @@ class Update_context {
         m_thd->dd_client()->acquire(INFORMATION_SCHEMA_NAME.str, &m_schema_obj))
       m_schema_obj = nullptr;
 
-    DBUG_ASSERT(m_schema_obj);
+    assert(m_schema_obj);
   }
 
   ~Update_context() {
@@ -290,7 +266,7 @@ bool store_in_dd(THD *thd, Update_context *ctx, ST_SCHEMA_TABLE *schema_table,
 
   // Store the metadata into DD
   if (thd->dd_client()->store(view_obj.get())) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return (true);
   }
 
@@ -304,18 +280,15 @@ bool store_in_dd(THD *thd, Update_context *ctx, ST_SCHEMA_TABLE *schema_table,
 
   @param      thd            Thread ID
   @param      plugin         Reference to a plugin.
-  @param      arg            Pointer to Context for I_S update.
-  @param[out] plugin_exists  Flag is set if plugin metadata already exists in
-                             data-dictionary.
+  @param      ctx            Pointer to Context for I_S update.
 
-  @return
-    false on success
-    true when fails to store the metadata.
+  @retval false on success
+  @retval true when fails to store the metadata.
 */
 
 static bool store_plugin_metadata(THD *thd, plugin_ref plugin,
                                   Update_context *ctx) {
-  DBUG_ASSERT(plugin && ctx);
+  assert(plugin && ctx);
 
   // Store in DD tables.
   st_plugin_int *pi = plugin_ref_to_int(plugin);
@@ -378,7 +351,7 @@ bool store_plugin_and_referencing_views_metadata(THD *thd, plugin_ref plugin,
 */
 bool update_plugins_I_S_metadata(THD *thd) {
   //  Warn if we have DDSE in read only mode and continue server startup.
-  if (check_if_server_ddse_readonly(thd, INFORMATION_SCHEMA_NAME.str))
+  if (dd::check_if_server_ddse_readonly(thd, INFORMATION_SCHEMA_NAME.str))
     return false;
 
   /*
@@ -451,6 +424,68 @@ bool update_plugins_I_S_metadata(THD *thd) {
   return dd::end_transaction(thd, error);
 }
 
+/*
+  Create INFORMATION_SCHEMA system views.
+*/
+bool create_system_views(THD *thd, bool is_non_dd_based) {
+  // Force use of utf8mb3 charset.
+  const CHARSET_INFO *client_cs = thd->variables.character_set_client;
+  const CHARSET_INFO *cs = thd->variables.collation_connection;
+  const CHARSET_INFO *m_client_cs, *m_connection_cl;
+  Disable_binlog_guard binlog_guard(thd);
+  Implicit_substatement_state_guard substatement_guard(thd);
+
+  resolve_charset("utf8mb3", system_charset_info, &m_client_cs);
+  resolve_collation("utf8mb3_general_ci", system_charset_info,
+                    &m_connection_cl);
+
+  thd->variables.character_set_client = m_client_cs;
+  thd->variables.collation_connection = m_connection_cl;
+  thd->update_charset();
+
+  dd::System_views::Types sv_type =
+      is_non_dd_based ? dd::System_views::Types::NON_DD_BASED_INFORMATION_SCHEMA
+                      : dd::System_views::Types::INFORMATION_SCHEMA;
+
+  // Iterate over system view definitions.
+  bool error = false;
+  for (dd::System_views::Const_iterator it =
+           dd::System_views::instance()->begin(sv_type);
+       it != dd::System_views::instance()->end();
+       it = dd::System_views::instance()->next(it, sv_type)) {
+    const dd::system_views::System_view_definition *view_def =
+        (*it)->entity()->view_definition();
+
+    // Build the CREATE VIEW DDL statement and execute it.
+    if (view_def == nullptr ||
+        dd::execute_query(thd, view_def->build_ddl_create_view())) {
+      error = true;
+      break;
+    }
+  }
+
+  // Store the target I_S version.
+  if (!error && !is_non_dd_based) {
+    dd::Dictionary_impl *d = dd::Dictionary_impl::instance();
+    if (!opt_initialize)
+      dd::bootstrap::DD_bootstrap_ctx::instance().set_actual_I_S_version(
+          d->get_actual_I_S_version(thd));
+    error = d->set_I_S_version(thd, d->get_target_I_S_version());
+    dd::bootstrap::DD_bootstrap_ctx::instance().set_I_S_upgrade_done();
+  }
+
+  // Restore the original character set.
+  thd->variables.character_set_client = client_cs;
+  thd->variables.collation_connection = cs;
+  thd->update_charset();
+
+  return error;
+}
+
+bool create_non_dd_based_system_views(THD *thd) {
+  return create_system_views(thd, true);
+}
+
 /**
   Does following during server restart.
 
@@ -483,13 +518,18 @@ bool update_server_I_S_metadata(THD *thd) {
     Stop server restart if I_S version is changed and the server is
     started with DDSE in read-only mode.
   */
-  if (check_if_server_ddse_readonly(thd, INFORMATION_SCHEMA_NAME.str))
+  if (dd::check_if_server_ddse_readonly(thd, INFORMATION_SCHEMA_NAME.str))
     return true;
 
   Update_context ctx(thd, true);
   const dd::Schema *sch_obj = ctx.info_schema();
   if (sch_obj == nullptr) return true;
 
+  LogErr(INFORMATION_LEVEL, ER_INFORMATION_SCHEMA_VERSION_CHANGE,
+         (actual_version > d->get_target_I_S_version() ? "Downgrading"
+                                                       : "Upgrading"),
+         actual_version, d->get_target_I_S_version());
+
   /*
     Stage 1:
     Remove all server I_S metadata from DD tables.
@@ -535,56 +575,39 @@ bool update_server_I_S_metadata(THD *thd) {
   return dd::end_transaction(thd, error);
 }
 
+bool initialize(THD *thd, bool non_dd_based_system_view) {
+  /*
+    Set tx_read_only to false to allow installing system views even
+    if the server is started with --transaction-read-only=true.
+  */
+  thd->variables.transaction_read_only = false;
+  thd->tx_read_only = false;
+
+  Disable_autocommit_guard autocommit_guard(thd);
+
+  dd::Dictionary_impl *d = dd::Dictionary_impl::instance();
+  assert(d);
+
+  if (!non_dd_based_system_view) {
+    if (dd::info_schema::create_system_views(thd) ||
+        dd::info_schema::store_server_I_S_metadata(thd))
+      return true;
+  } else {
+    bool error = create_non_dd_based_system_views(thd);
+    return dd::end_transaction(thd, error);
+  }
+
+  LogErr(INFORMATION_LEVEL, ER_CREATED_SYSTEM_WITH_VERSION,
+         (int)d->get_target_dd_version());
+  return false;
+}
+
 }  // namespace
 
 namespace dd {
 namespace info_schema {
 
-/*
-  Create INFORMATION_SCHEMA system views.
-*/
-bool create_system_views(THD *thd) {
-  // Force use of utf8 charset.
-  const CHARSET_INFO *client_cs = thd->variables.character_set_client;
-  const CHARSET_INFO *cs = thd->variables.collation_connection;
-  const CHARSET_INFO *m_client_cs, *m_connection_cl;
-
-  resolve_charset("utf8", system_charset_info, &m_client_cs);
-  resolve_collation("utf8_general_ci", system_charset_info, &m_connection_cl);
-
-  thd->variables.character_set_client = m_client_cs;
-  thd->variables.collation_connection = m_connection_cl;
-  thd->update_charset();
-
-  // Iterate over system view definitions.
-  dd::System_views::Const_iterator it = dd::System_views::instance()->begin();
-
-  bool error = false;
-  for (; it != dd::System_views::instance()->end(); ++it) {
-    const dd::system_views::System_view_definition *view_def =
-        (*it)->entity()->view_definition();
-
-    // Build the CREATE VIEW DDL statement and execute it.
-    if (view_def == nullptr ||
-        dd::execute_query(thd, view_def->build_ddl_create_view())) {
-      error = true;
-      break;
-    }
-  }
-
-  // Store the target I_S version.
-  if (!error) {
-    dd::Dictionary_impl *d = dd::Dictionary_impl::instance();
-    error = d->set_I_S_version(thd, d->get_target_I_S_version());
-  }
-
-  // Restore the original character set.
-  thd->variables.character_set_client = client_cs;
-  thd->variables.collation_connection = cs;
-  thd->update_charset();
-
-  return error;
-}
+bool create_system_views(THD *thd) { return ::create_system_views(thd, false); }
 
 /*
   Store the server I_S table metadata into dictionary, once during MySQL
@@ -625,7 +648,7 @@ bool update_I_S_metadata(THD *thd) {
 bool store_dynamic_plugin_I_S_metadata(THD *thd, st_plugin_int *plugin_int) {
   plugin_ref plugin = plugin_int_to_ref(plugin_int);
   Update_context ctx(thd, false);
-  DBUG_ASSERT(plugin_int->plugin->type == MYSQL_INFORMATION_SCHEMA_PLUGIN);
+  assert(plugin_int->plugin->type == MYSQL_INFORMATION_SCHEMA_PLUGIN);
 
   return store_plugin_metadata(thd, plugin, &ctx);
 }
@@ -652,37 +675,21 @@ bool remove_I_S_view_metadata(THD *thd, const dd::String_type &view_name) {
                                 &at))
     return (true);
 
-  DBUG_ASSERT(at->type() == dd::enum_table_type::SYSTEM_VIEW);
+  assert(at->type() == dd::enum_table_type::SYSTEM_VIEW);
 
   // Remove view from DD tables.
-  Disable_gtid_state_update_guard disabler(thd);
+  Implicit_substatement_state_guard substatement_guard(thd);
   if (thd->dd_client()->drop(at)) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return (true);
   }
 
   return (false);
 }
 
-bool initialize(THD *thd) {
-  /*
-    Set tx_read_only to false to allow installing system views even
-    if the server is started with --transaction-read-only=true.
-  */
-  thd->variables.transaction_read_only = false;
-  thd->tx_read_only = false;
+bool initialize(THD *thd) { return ::initialize(thd, false); }
 
-  Disable_autocommit_guard autocommit_guard(thd);
-
-  Dictionary_impl *d = dd::Dictionary_impl::instance();
-  DBUG_ASSERT(d);
-
-  if (create_system_views(thd) || store_server_I_S_metadata(thd)) return true;
-
-  LogErr(INFORMATION_LEVEL, ER_CREATED_SYSTEM_WITH_VERSION,
-         (int)d->get_target_dd_version());
-  return false;
-}
+bool init_non_dd_based_system_view(THD *thd) { return ::initialize(thd, true); }
 
 /*
   Get create view definition for the given I_S system view.
