@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,19 +24,19 @@
 #include "sql/event_data_objects.h"
 
 #include <string.h>
+#include <memory>
 
 #include "lex_string.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_sp.h"
 #include "mysql/psi/mysql_statement.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_time.h"
 #include "mysqld.h"
 #include "mysqld_error.h"
@@ -71,6 +72,8 @@
 // calc_time_diff.
 #include "sql/tztime.h"  // my_tz_find, my_tz_OFFSET0
 #include "sql_string.h"
+#include "storage/perfschema/terminology_use_previous_enum.h"
+#include "string_with_len.h"
 
 class Item;
 
@@ -118,23 +121,23 @@ class Event_creation_ctx : public Stored_program_creation_ctx {
                                         Stored_program_creation_ctx **ctx);
 
  public:
-  virtual Stored_program_creation_ctx *clone(MEM_ROOT *mem_root) {
+  Stored_program_creation_ctx *clone(MEM_ROOT *mem_root) override {
     return new (mem_root)
         Event_creation_ctx(m_client_cs, m_connection_cl, m_db_cl);
   }
 
  protected:
-  virtual Object_creation_ctx *create_backup_ctx(THD *) const {
+  Object_creation_ctx *create_backup_ctx(THD *) const override {
     /*
       We can avoid usual backup/restore employed in stored programs since we
       know that this is a top level statement and the worker thread is
       allocated exclusively to execute this event.
     */
 
-    return NULL;
+    return nullptr;
   }
 
-  virtual void delete_backup_ctx() { destroy(this); }
+  void delete_backup_ctx() override { ::destroy_at(this); }
 
  private:
   Event_creation_ctx(const CHARSET_INFO *client_cs,
@@ -149,7 +152,7 @@ bool Event_creation_ctx::create_event_creation_ctx(
   const CHARSET_INFO *client_cs = nullptr;
   const CHARSET_INFO *connection_cl = nullptr;
   const CHARSET_INFO *db_cl = nullptr;
-  bool invalid_creation_ctx = false;
+  const bool invalid_creation_ctx = false;
   auto collation_info = [](uint id) { return get_charset(id, MYF(0)); };
 
   // Set collation or charset attribute of client, connection and database.
@@ -171,7 +174,7 @@ bool Event_creation_ctx::create_event_creation_ctx(
 /*************************************************************************/
 
 /*
-  Initiliazes dbname and name of an Event_queue_element_for_exec
+  Initializes dbname and name of an Event_queue_element_for_exec
   object
 
   SYNOPSIS
@@ -195,9 +198,9 @@ bool Event_queue_element_for_exec::init(LEX_CSTRING db, LEX_CSTRING n) {
   return false;
 }
 
-void Event_queue_element_for_exec::claim_memory_ownership() {
-  my_claim(dbname.str);
-  my_claim(name.str);
+void Event_queue_element_for_exec::claim_memory_ownership(bool claim) {
+  my_claim(dbname.str, claim);
+  my_claim(name.str, claim);
 }
 
 /*
@@ -222,8 +225,6 @@ Event_queue_element_for_exec::~Event_queue_element_for_exec() {
 Event_basic::Event_basic()
     : m_schema_name(NULL_CSTR), m_event_name(NULL_CSTR), m_time_zone(nullptr) {
   DBUG_TRACE;
-  /* init memory root */
-  init_sql_alloc(key_memory_event_basic_root, &mem_root, 256, 512);
 }
 
 /*
@@ -233,10 +234,7 @@ Event_basic::Event_basic()
     Event_basic::Event_basic()
 */
 
-Event_basic::~Event_basic() {
-  DBUG_TRACE;
-  free_root(&mem_root, MYF(0));
-}
+Event_basic::~Event_basic() { DBUG_TRACE; }
 
 /*
   Constructor
@@ -265,7 +263,7 @@ Event_queue_element::Event_queue_element()
   SYNOPSIS
     Event_queue_element::Event_queue_element()
 */
-Event_queue_element::~Event_queue_element() {}
+Event_queue_element::~Event_queue_element() = default;
 
 /*
   Constructor
@@ -286,7 +284,7 @@ Event_timed::Event_timed() : m_created(0), m_modified(0), m_sql_mode(0) {
     Event_timed::~Event_timed()
 */
 
-Event_timed::~Event_timed() {}
+Event_timed::~Event_timed() = default;
 
 /*
   Constructor
@@ -333,7 +331,7 @@ bool Event_job_data::fill_event_info(THD *thd, const dd::Event &event_obj,
 
   m_definition = make_lex_string(&mem_root, event_obj.definition());
 
-  if (m_time_zone == NULL) return true;
+  if (m_time_zone == nullptr) return true;
 
   Event_creation_ctx::create_event_creation_ctx(event_obj, &m_creation_ctx);
   if (m_creation_ctx == nullptr) return true;
@@ -363,7 +361,7 @@ bool Event_queue_element::fill_event_info(THD *thd, const dd::Event &event_obj,
   String str(event_obj.time_zone().c_str(), &my_charset_latin1);
   m_time_zone = my_tz_find(thd, &str);
 
-  if (m_time_zone == NULL) return true;
+  if (m_time_zone == nullptr) return true;
 
   m_starts_null = event_obj.is_starts_null();
   if (!m_starts_null) m_starts = event_obj.starts();
@@ -381,8 +379,7 @@ bool Event_queue_element::fill_event_info(THD *thd, const dd::Event &event_obj,
     If neither STARTS and ENDS is set, then both fields are empty.
     Hence, if execute_at is empty there is an error.
   */
-  DBUG_ASSERT(
-      !(m_starts_null && m_ends_null && !m_expression && m_execute_at_null));
+  assert(!(m_starts_null && m_ends_null && !m_expression && m_execute_at_null));
 
   if (!m_expression && !m_execute_at_null)
     m_execute_at = event_obj.execute_at();
@@ -461,15 +458,14 @@ static my_time_t add_interval(MYSQL_TIME *ltime, const Time_zone *time_zone,
   @param    i_type        type of interval to add (SECOND, MINUTE, HOUR, WEEK
   ...)
 
-  @retval returns 0 on success, 1 on error.
+  @retval 0 on success
+  @retval 1 on error.
 
   @note
-
-  NOTES
-    1) If the interval is conversible to SECOND, like MINUTE, HOUR, DAY, WEEK.
+    1. If the interval is conversible to SECOND, like MINUTE, HOUR, DAY, WEEK.
        Then we use TIMEDIFF()'s implementation as underlying and number of
        seconds as resolution for computation.
-    2) In all other cases - MONTH, QUARTER, YEAR we use MONTH as resolution
+    2. In all other cases - MONTH, QUARTER, YEAR we use MONTH as resolution
        and PERIOD_DIFF()'s implementation
 */
 
@@ -479,7 +475,7 @@ static bool get_next_time(const Time_zone *time_zone, my_time_t *next,
   DBUG_TRACE;
   DBUG_PRINT("enter", ("start: %lu  now: %lu", (long)start, (long)time_now));
 
-  DBUG_ASSERT(start <= time_now);
+  assert(start <= time_now);
 
   longlong months = 0, seconds = 0;
 
@@ -525,7 +521,7 @@ static bool get_next_time(const Time_zone *time_zone, my_time_t *next,
       return true;
       break;
     case INTERVAL_LAST:
-      DBUG_ASSERT(0);
+      assert(0);
   }
   DBUG_PRINT("info",
              ("seconds: %ld  months: %ld", (long)seconds, (long)months));
@@ -546,8 +542,8 @@ static bool get_next_time(const Time_zone *time_zone, my_time_t *next,
   if (seconds) {
     longlong seconds_diff;
     long microsec_diff;
-    bool negative = calc_time_diff(local_now, local_start, 1, &seconds_diff,
-                                   &microsec_diff);
+    const bool negative = calc_time_diff(local_now, local_start, 1,
+                                         &seconds_diff, &microsec_diff);
     if (!negative) {
       /*
         The formula below returns the interval that, when added to
@@ -566,7 +562,7 @@ static bool get_next_time(const Time_zone *time_zone, my_time_t *next,
         then next_time was set, but perhaps to the value that is less
         then time_now.  See below for elaboration.
       */
-      DBUG_ASSERT(negative || next_time > 0);
+      assert(negative || next_time > 0);
 
       /*
         If local_now < local_start, i.e. STARTS time is in the future
@@ -622,8 +618,9 @@ static bool get_next_time(const Time_zone *time_zone, my_time_t *next,
       } while (next_time <= time_now);
     }
   } else {
-    long diff_months = ((long)local_now.year - (long)local_start.year) * 12 +
-                       ((long)local_now.month - (long)local_start.month);
+    const long diff_months =
+        ((long)local_now.year - (long)local_start.year) * 12 +
+        ((long)local_now.month - (long)local_start.month);
 
     /*
       Unlike for seconds above, the formula below returns the interval
@@ -655,7 +652,7 @@ static bool get_next_time(const Time_zone *time_zone, my_time_t *next,
     }
   }
 
-  DBUG_ASSERT(time_now < next_time);
+  assert(time_now < next_time);
 
   *next = next_time;
 
@@ -942,9 +939,16 @@ int Event_timed::get_create_event(const THD *thd, String *buf) {
 
   if (m_status == Event_parse_data::ENABLED)
     buf->append(STRING_WITH_LEN("ENABLE"));
-  else if (m_status == Event_parse_data::SLAVESIDE_DISABLED)
-    buf->append(STRING_WITH_LEN("DISABLE ON SLAVE"));
-  else
+  else if (m_status == Event_parse_data::REPLICA_SIDE_DISABLED) {
+    if (thd->variables.terminology_use_previous !=
+            terminology_use_previous::enum_compatibility_version::NONE &&
+        thd->variables.terminology_use_previous <=
+            (ulong)terminology_use_previous::enum_compatibility_version::
+                BEFORE_8_2_0)
+      buf->append(STRING_WITH_LEN("DISABLE ON SLAVE"));
+    else
+      buf->append(STRING_WITH_LEN("DISABLE ON REPLICA"));
+  } else
     buf->append(STRING_WITH_LEN("DISABLE"));
 
   if (m_comment.length) {
@@ -1009,8 +1013,8 @@ bool Event_job_data::construct_sp_sql(THD *thd, String *sp_sql) {
 
 bool Event_job_data::execute(THD *thd, bool drop) {
   String sp_sql;
-  Security_context event_sctx, *save_sctx = NULL;
-  List<Item> empty_item_list;
+  Security_context event_sctx, *save_sctx = nullptr;
+  mem_root_deque<Item *> empty_item_list(thd->mem_root);
   bool ret = true;
   sql_digest_state *parent_digest = thd->m_digest;
   PSI_statement_locker *parent_locker = thd->m_statement_psi;
@@ -1050,12 +1054,17 @@ bool Event_job_data::execute(THD *thd, bool drop) {
     In case the definer user has SYSTEM_USER privilege then make THD
     non-killable through the users who do not have SYSTEM_USER privilege,
     OR vice-versa.
-    Note - Do not forget to reset the flag after the saved security context is
-           restored.
+    Recalculate the connection_admin flag state as well (CONNECTION_ADMIN
+    privilege).
+    Note - Do not forget to reset the flags after the saved security
+    context is restored.
   */
-  if (save_sctx) set_system_user_flag(thd);
+  if (save_sctx) {
+    set_system_user_flag(thd);
+    set_connection_admin_flag(thd);
+  }
 
-  if (check_access(thd, EVENT_ACL, m_schema_name.str, NULL, NULL, false,
+  if (check_access(thd, EVENT_ACL, m_schema_name.str, nullptr, nullptr, false,
                    false)) {
     /*
       This aspect of behavior is defined in the worklog,
@@ -1068,8 +1077,6 @@ bool Event_job_data::execute(THD *thd, bool drop) {
     goto end;
   }
 
-  if (construct_sp_sql(thd, &sp_sql)) goto end;
-
   /*
     Set up global thread attributes to reflect the properties of
     this Event. We can simply reset these instead of usual
@@ -1081,6 +1088,33 @@ bool Event_job_data::execute(THD *thd, bool drop) {
   thd->variables.sql_mode = m_sql_mode;
   thd->variables.time_zone = m_time_zone;
 
+  if (construct_sp_sql(thd, &sp_sql)) goto end;
+
+  /*
+    If enabled, log the quoted form to performance_schema.error_log.
+    We enclose it in faux guillemets to differentiate the enclosing
+    quotation seen in the log from the SQL-level quotation from
+    construct_sp_sql()'s (which calls append_identifier() in sql_show,
+    and thus ultimately get_quote_char_for_identifier() which evaluates
+    thd->variables.sql_mode & MODE_ANSI_QUOTES).
+
+    We're logging with a priority of SYSTEM_LEVEL so we won't have to
+    worry abot log_error_verbosity. (ERROR_LEVEL would also achieve
+    that, but then mysql-test-run.pl would rightfully complain about
+    the error in the log.)
+  */
+  DBUG_EXECUTE_IF("log_event_query_string", {
+    LEX_STRING sm1;
+    LEX_STRING sm2;
+    sql_mode_string_representation(thd, thd->variables.sql_mode, &sm1);
+    sql_mode_string_representation(thd, m_sql_mode, &sm2);
+    LogEvent()
+        .errcode(ER_CONDITIONAL_DEBUG)
+        .prio(SYSTEM_LEVEL)
+        .message("Query string to be compiled: \"%s\"/\"%s\" >>%s<<\n", sm1.str,
+                 sm2.str, sp_sql.c_ptr_safe());
+  });
+
   thd->set_query(sp_sql.c_ptr_safe(), sp_sql.length());
 
   {
@@ -1088,8 +1122,8 @@ bool Event_job_data::execute(THD *thd, bool drop) {
 
     if (parser_state.init(thd, thd->query().str, thd->query().length)) goto end;
 
-    thd->m_digest = NULL;
-    thd->m_statement_psi = NULL;
+    thd->m_digest = nullptr;
+    thd->m_statement_psi = nullptr;
     if (parse_sql(thd, &parser_state, m_creation_ctx)) {
       LogErr(ERROR_LEVEL, ER_EVENT_ERROR_DURING_COMPILATION,
              thd->is_fatal_error() ? "fatal " : "", m_schema_name.str,
@@ -1105,7 +1139,7 @@ bool Event_job_data::execute(THD *thd, bool drop) {
   {
     sp_head *sphead = thd->lex->sphead;
 
-    DBUG_ASSERT(sphead);
+    assert(sphead);
 
     if (thd->enable_slow_log) sphead->m_flags |= sp_head::LOG_SLOW_STATEMENTS;
     sphead->m_flags |= sp_head::LOG_GENERAL_LOG;
@@ -1154,7 +1188,7 @@ end:
 
       // Prevent InnoDB from automatically committing the InnoDB transaction
       // after updating the data-dictionary table.
-      Disable_autocommit_guard autocommit_guard(thd);
+      const Disable_autocommit_guard autocommit_guard(thd);
 
       /*
         NOTE: even if we run in read-only mode, we should be able to lock
@@ -1168,7 +1202,7 @@ end:
       saved_master_access = thd->security_context()->master_access();
       thd->security_context()->set_master_access(saved_master_access |
                                                  SUPER_ACL);
-      bool save_tx_read_only = thd->tx_read_only;
+      const bool save_tx_read_only = thd->tx_read_only;
       thd->tx_read_only = false;
 
       ret = Events::drop_event(thd, m_schema_name, m_event_name, false);
@@ -1180,11 +1214,12 @@ end:
 
   if (save_sctx) {
     event_sctx.restore_security_context(thd, save_sctx);
-    /* Restore the original value in THD */
+    /* Restore the original values in THD */
     set_system_user_flag(thd);
+    set_connection_admin_flag(thd);
   }
 
-  thd->lex->unit->cleanup(thd, true);
+  thd->lex->cleanup(true);
   thd->end_statement();
   thd->cleanup_after_query();
   /* Avoid races with SHOW PROCESSLIST */
@@ -1204,7 +1239,7 @@ bool construct_drop_event_sql(THD *thd, String *sp_sql, LEX_CSTRING schema_name,
                               LEX_CSTRING event_name) {
   LEX_STRING buffer;
   const uint STATIC_SQL_LENGTH = 14;
-  int ret = 0;
+  bool ret = false;
 
   DBUG_TRACE;
 
