@@ -1,18 +1,19 @@
 #ifndef GSTREAM_INCLUDED
 #define GSTREAM_INCLUDED
 
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,9 +27,9 @@
 #include <stddef.h>
 
 #include "lex_string.h"
-#include "m_ctype.h" /* my_charset_latin1, my_charset_bin */
-#include "m_string.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h" /* my_charset_latin1, my_charset_bin */
+#include "nulls.h"
 
 class THD;
 
@@ -40,7 +41,7 @@ class Gis_read_stream {
                   int size)
       : m_cur(buffer),
         m_limit(buffer + size),
-        m_err_msg(NULL),
+        m_err_msg(nullptr),
         m_charset(charset),
         m_thd(thd) {}
   Gis_read_stream() : m_cur(NullS), m_limit(NullS), m_err_msg(NullS) {}
@@ -59,9 +60,10 @@ class Gis_read_stream {
   /* Skip next character, if match. Return 1 if no match */
   inline bool skip_char(char skip) {
     skip_space();
-    if ((m_cur >= m_limit) || *m_cur != skip) return 1; /* Didn't find char */
+    if ((m_cur >= m_limit) || *m_cur != skip)
+      return true; /* Didn't find char */
     m_cur++;
-    return 0;
+    return false;
   }
   void set_error_msg(const char *msg);
 
