@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,16 +23,18 @@
 
 #include "sql/item_xmlfunc.h"
 
-#include <stdio.h>
-#include <string.h>
+#include <assert.h>
 #include <sys/types.h>
 
-#include "lex_string.h"
-#include "m_ctype.h"
+#include <algorithm>
+#include <cstdio>
+#include <cstring>
+#include <memory>
+
 #include "m_string.h"
-#include "my_dbug.h"
-#include "my_macros.h"
+
 #include "my_sys.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
@@ -49,8 +52,7 @@
 
 /*
   TODO: future development directions:
-  1. add real constants for XPATH_NODESET_CMP and XPATH_NODESET
-     into enum Type in item.h.
+  1. add real constant for XPATH_NODESET_ITEM into enum Type in item.h.
   2. add nodeset_to_nodeset_comparator
   3. add lacking functions:
        - name()
@@ -71,7 +73,7 @@
        - preceding
 */
 
-/* Lexical analizer token */
+/* Lexical analyzer token */
 struct MY_XPATH_LEX {
   int term;        /* token type, see MY_XPATH_LEX_XXXXX below */
   const char *beg; /* beginnign of the token                   */
@@ -156,7 +158,7 @@ class Item_nodeset_func : public Item_str_func {
    @param nodeset   the nodeset to be modified
   */
   virtual void val_nodeset(XPathFilter *nodeset) const = 0;
-  enum Type type() const override { return XPATH_NODESET; }
+  enum Type type() const override { return XPATH_NODESET_ITEM; }
   String *val_str(String *str) override {
     XPathFilter res;
     val_nodeset(&res);
@@ -182,7 +184,7 @@ class Item_nodeset_func : public Item_str_func {
   }
   enum Item_result result_type() const override { return STRING_RESULT; }
   bool resolve_type(THD *) override {
-    max_length = MAX_BLOB_WIDTH;
+    set_data_type_string(uint32{MAX_BLOB_WIDTH});
     // To avoid premature evaluation, mark all nodeset functions as non-const.
     used_tables_cache = RAND_TABLE_BIT;
     return false;
@@ -363,7 +365,7 @@ class Item_xpath_cast_bool final : public Item_int_func {
   const char *func_name() const override { return "xpath_cast_bool"; }
   bool is_bool_func() const override { return true; }
   longlong val_int() override {
-    if (args[0]->type() == XPATH_NODESET) {
+    if (args[0]->type() == XPATH_NODESET_ITEM) {
       auto *nodeset_func = down_cast<const Item_nodeset_func *>(args[0]);
       XPathFilter flt;
       nodeset_func->val_nodeset(&flt);
@@ -389,7 +391,7 @@ class Item_xpath_cast_number : public Item_real_func {
 class Item_nodeset_context_cache : public Item_nodeset_func {
   bool m_is_empty;
   uint32 m_num;
-  uint32 m_pos;
+  uint32 m_position;
   size_t m_size;
 
  public:
@@ -397,20 +399,16 @@ class Item_nodeset_context_cache : public Item_nodeset_func {
       : Item_nodeset_func(pxml_arg, cs),
         m_is_empty(true),
         m_num(0),
-        m_pos(0),
+        m_position(0),
         m_size(0) {}
   void val_nodeset(XPathFilter *nodeset) const override {
     nodeset->clear();
     if (!m_is_empty)
-      nodeset->push_back({m_num, m_pos, static_cast<uint>(m_size)});
-  }
-  bool resolve_type(THD *) override {
-    max_length = MAX_BLOB_WIDTH;
-    return false;
+      nodeset->push_back({m_num, m_position, static_cast<uint>(m_size)});
   }
   void set_element(uint32 num, uint32 pos, size_t size) {
     m_num = num;
-    m_pos = pos;
+    m_position = pos;
     m_size = size;
     m_is_empty = false;
   }
@@ -421,7 +419,7 @@ class Item_func_xpath_position : public Item_int_func {
   explicit Item_func_xpath_position(Item *a) : Item_int_func(a) {}
   const char *func_name() const override { return "xpath_position"; }
   bool resolve_type(THD *) override {
-    max_length = 10;
+    set_data_type_string(10U);
     return false;
   }
   longlong val_int() override {
@@ -438,7 +436,7 @@ class Item_func_xpath_count : public Item_int_func {
   explicit Item_func_xpath_count(Item *a) : Item_int_func(a) {}
   const char *func_name() const override { return "xpath_count"; }
   bool resolve_type(THD *) override {
-    max_length = 10;
+    set_data_type_string(10U);
     return false;
   }
   longlong val_int() override {
@@ -491,7 +489,6 @@ class Item_nodeset_to_const_comparator final : public Item_bool_func {
   Item_nodeset_to_const_comparator(Item_nodeset_func *nodeset,
                                    Item_bool_func *cmpfunc, const ParsedXML *p)
       : Item_bool_func(nodeset, cmpfunc), pxml(p) {}
-  enum Type type() const override { return XPATH_NODESET_CMP; }
   const char *func_name() const override {
     return "xpath_nodeset_to_const_comparator";
   }
@@ -500,7 +497,6 @@ class Item_nodeset_to_const_comparator final : public Item_bool_func {
   longlong val_int() override {
     auto comp = down_cast<Item_bool_func *>(args[1]);
     auto fake = down_cast<Item_string *>(comp->arguments()[0]);
-    fake->collation.collation = collation.collation;
     auto *nodeset_func = down_cast<const Item_nodeset_func *>(args[0]);
     XPathFilter res;
     nodeset_func->val_nodeset(&res);
@@ -700,7 +696,7 @@ void Item_nodeset_func_elementbyindex::val_nodeset(XPathFilter *nodeset) const {
   otherwise returns the item itself.
 */
 static Item *nodeset2bool(Item *item) {
-  if (item->type() == Item::XPATH_NODESET)
+  if (item->type() == Item::XPATH_NODESET_ITEM)
     return new Item_xpath_cast_bool(item);
   return item;
 }
@@ -761,7 +757,7 @@ static Item *nodeset2bool(Item *item) {
 /*
   Create scalar comparator
 
-  SYNOPSYS
+  SYNOPSIS
     Create a comparator function for scalar arguments,
     for the given arguments and operation.
 
@@ -783,13 +779,13 @@ static Item_bool_func *eq_func(int oper, Item *a, Item *b) {
     case MY_XPATH_LEX_LESS:
       return new Item_func_lt(a, b);
   }
-  return 0;
+  return nullptr;
 }
 
 /*
   Create scalar comparator
 
-  SYNOPSYS
+  SYNOPSIS
     Create a comparator function for scalar arguments,
     for the given arguments and reverse operation, e.g.
 
@@ -813,13 +809,13 @@ static Item_bool_func *eq_func_reverse(int oper, Item *a, Item *b) {
     case MY_XPATH_LEX_LESS:
       return new Item_func_gt(a, b);
   }
-  return 0;
+  return nullptr;
 }
 
 /*
   Create a comparator
 
-  SYNOPSYS
+  SYNOPSIS
     Create a comparator for scalar or non-scalar arguments,
     for the given arguments and operation.
 
@@ -828,18 +824,19 @@ static Item_bool_func *eq_func_reverse(int oper, Item *a, Item *b) {
 */
 static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
                                Item *a, Item *b) {
-  if (a->type() != Item::XPATH_NODESET && b->type() != Item::XPATH_NODESET) {
+  if (a->type() != Item::XPATH_NODESET_ITEM &&
+      b->type() != Item::XPATH_NODESET_ITEM) {
     return eq_func(oper, a, b);  // two scalar arguments
-  } else if (a->type() == Item::XPATH_NODESET &&
-             b->type() == Item::XPATH_NODESET) {
+  } else if (a->type() == Item::XPATH_NODESET_ITEM &&
+             b->type() == Item::XPATH_NODESET_ITEM) {
     size_t len = xpath->query.end - context->beg;
-    set_if_smaller(len, 32);
+    len = std::min(len, size_t(32));
     my_printf_error(ER_UNKNOWN_ERROR,
                     "XPATH error: "
                     "comparison of two nodesets is not supported: '%.*s'",
                     MYF(0), static_cast<int>(len), context->beg);
 
-    return 0;  // TODO: Comparison of two nodesets
+    return nullptr;  // TODO: Comparison of two nodesets
   } else {
     /*
      Compare a node set to a scalar value.
@@ -854,7 +851,7 @@ static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
     Item_nodeset_func *nodeset;
     Item *scalar;
     Item_bool_func *comp;
-    if (a->type() == Item::XPATH_NODESET) {
+    if (a->type() == Item::XPATH_NODESET_ITEM) {
       nodeset = down_cast<Item_nodeset_func *>(a);
       scalar = b;
       comp = eq_func(oper, fake, scalar);
@@ -870,7 +867,7 @@ static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
 /*
   Create a step
 
-  SYNOPSYS
+  SYNOPSIS
     Create a step function for the given argument and axis.
 
   RETURN
@@ -878,10 +875,10 @@ static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
 */
 static Item_nodeset_func *nametestfunc(MY_XPATH *xpath, int type, Item *arg,
                                        const char *beg, size_t len) {
-  DBUG_ASSERT(arg != 0);
-  DBUG_ASSERT(arg->type() == Item::XPATH_NODESET);
-  DBUG_ASSERT(beg != 0);
-  DBUG_ASSERT(len > 0);
+  assert(arg != nullptr);
+  assert(arg->type() == Item::XPATH_NODESET_ITEM);
+  assert(beg != nullptr);
+  assert(len > 0);
 
   Item_nodeset_func *res;
   switch (type) {
@@ -921,7 +918,7 @@ static Item_nodeset_func *nametestfunc(MY_XPATH *xpath, int type, Item *arg,
 }
 
 /*
-  Tokens consisting of one character, for faster lexical analizer.
+  Tokens consisting of one character, for faster lexical analyzer.
 */
 static char simpletok[128] = {
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -951,7 +948,7 @@ static struct my_xpath_keyword_names_st my_keyword_names[] = {
     {MY_XPATH_LEX_OR, "or", 2, 0},
     {MY_XPATH_LEX_DIV, "div", 3, 0},
     {MY_XPATH_LEX_MOD, "mod", 3, 0},
-    {0, NULL, 0, 0}};
+    {0, nullptr, 0, 0}};
 
 static struct my_xpath_keyword_names_st my_axis_names[] = {
     {MY_XPATH_LEX_AXIS, "ancestor", 8, MY_XPATH_AXIS_ANCESTOR},
@@ -970,19 +967,19 @@ static struct my_xpath_keyword_names_st my_axis_names[] = {
     {MY_XPATH_LEX_AXIS, "preceding-sibling", 17,
      MY_XPATH_AXIS_PRECEDING_SIBLING},
     {MY_XPATH_LEX_AXIS, "self", 4, MY_XPATH_AXIS_SELF},
-    {0, NULL, 0, 0}};
+    {0, nullptr, 0, 0}};
 
 static struct my_xpath_keyword_names_st my_nodetype_names[] = {
     {MY_XPATH_LEX_NODETYPE, "comment", 7, 0},
     {MY_XPATH_LEX_NODETYPE, "text", 4, 0},
     {MY_XPATH_LEX_NODETYPE, "processing-instruction", 22, 0},
     {MY_XPATH_LEX_NODETYPE, "node", 4, 0},
-    {0, NULL, 0, 0}};
+    {0, nullptr, 0, 0}};
 
 /*
   Lookup a keyword
 
-  SYNOPSYS
+  SYNOPSIS
     Check that the last scanned identifier is a keyword.
 
   RETURN
@@ -993,7 +990,7 @@ static int my_xpath_keyword(MY_XPATH *x,
                             struct my_xpath_keyword_names_st *keyword_names,
                             const char *beg, const char *end) {
   struct my_xpath_keyword_names_st *k;
-  size_t length = end - beg;
+  const size_t length = end - beg;
   for (k = keyword_names; k->name; k++) {
     if (length == k->length && !native_strncasecmp(beg, k->name, length)) {
       x->extra = k->extra;
@@ -1037,7 +1034,7 @@ static Item *create_func_number(MY_XPATH *xpath, Item **args, uint nargs) {
   if (nargs > 0) {
     arg = args[0];
   } else {
-    arg = xpath->context != NULL ? xpath->context : xpath->rootelement;
+    arg = xpath->context != nullptr ? xpath->context : xpath->rootelement;
   }
   return new Item_xpath_cast_number(arg);
 }
@@ -1045,19 +1042,20 @@ static Item *create_func_number(MY_XPATH *xpath, Item **args, uint nargs) {
 static Item *create_func_string_length(MY_XPATH *xpath, Item **args,
                                        uint nargs) {
   Item *arg = nargs ? args[0] : xpath->context;
-  return arg ? new Item_func_char_length(arg) : 0;
+  return arg ? new Item_func_char_length(arg) : nullptr;
 }
 
 static Item *create_func_round(MY_XPATH *, Item **args, uint) {
-  return new Item_func_round(args[0], new Item_int_0(), 0);
+  return new Item_func_round(args[0], new Item_int_0(), false);
 }
 
 static Item *create_func_last(MY_XPATH *xpath, Item **, uint) {
-  return xpath->context ? new Item_func_xpath_count(xpath->context) : NULL;
+  return xpath->context ? new Item_func_xpath_count(xpath->context) : nullptr;
 }
 
 static Item *create_func_position(MY_XPATH *xpath, Item **, uint) {
-  return xpath->context ? new Item_func_xpath_position(xpath->context) : NULL;
+  return xpath->context ? new Item_func_xpath_position(xpath->context)
+                        : nullptr;
 }
 
 static Item *create_func_contains(MY_XPATH *, Item **args, uint) {
@@ -1076,12 +1074,12 @@ static Item *create_func_substr(MY_XPATH *, Item **args, uint nargs) {
 }
 
 static Item *create_func_count(MY_XPATH *, Item **args, uint) {
-  if (args[0]->type() != Item::XPATH_NODESET) return 0;
+  if (args[0]->type() != Item::XPATH_NODESET_ITEM) return nullptr;
   return new Item_func_xpath_count(args[0]);
 }
 
 static Item *create_func_sum(MY_XPATH *xpath, Item **args, uint) {
-  if (args[0]->type() != Item::XPATH_NODESET) return 0;
+  if (args[0]->type() != Item::XPATH_NODESET_ITEM) return nullptr;
   return new Item_func_xpath_sum(args[0], xpath->pxml);
 }
 
@@ -1091,50 +1089,50 @@ static Item *create_func_sum(MY_XPATH *xpath, Item **args, uint) {
 */
 static MY_XPATH_FUNC my_func_names3[] = {{"sum", 3, 1, 1, create_func_sum},
                                          {"not", 3, 1, 1, create_func_not},
-                                         {0, 0, 0, 0, 0}};
+                                         {nullptr, 0, 0, 0, nullptr}};
 
 static MY_XPATH_FUNC my_func_names4[] = {{"last", 4, 0, 0, create_func_last},
                                          {"true", 4, 0, 0, create_func_true},
-                                         {"name", 4, 0, 1, 0},
-                                         {"lang", 4, 1, 1, 0},
-                                         {0, 0, 0, 0, 0}};
+                                         {"name", 4, 0, 1, nullptr},
+                                         {"lang", 4, 1, 1, nullptr},
+                                         {nullptr, 0, 0, 0, nullptr}};
 
 static MY_XPATH_FUNC my_func_names5[] = {{"count", 5, 1, 1, create_func_count},
                                          {"false", 5, 0, 0, create_func_false},
                                          {"floor", 5, 1, 1, create_func_floor},
                                          {"round", 5, 1, 1, create_func_round},
-                                         {0, 0, 0, 0, 0}};
+                                         {nullptr, 0, 0, 0, nullptr}};
 
 static MY_XPATH_FUNC my_func_names6[] = {
     {"concat", 6, 2, 255, create_func_concat},
     {"number", 6, 0, 1, create_func_number},
-    {"string", 6, 0, 1, 0},
-    {0, 0, 0, 0, 0}};
+    {"string", 6, 0, 1, nullptr},
+    {nullptr, 0, 0, 0, nullptr}};
 
 /* Other functions, with name longer than 6, all together */
 static MY_XPATH_FUNC my_func_names[] = {
-    {"id", 2, 1, 1, 0},
+    {"id", 2, 1, 1, nullptr},
     {"boolean", 7, 1, 1, create_func_bool},
     {"ceiling", 7, 1, 1, create_func_ceiling},
     {"position", 8, 0, 0, create_func_position},
     {"contains", 8, 2, 2, create_func_contains},
     {"substring", 9, 2, 3, create_func_substr},
-    {"translate", 9, 3, 3, 0},
+    {"translate", 9, 3, 3, nullptr},
 
-    {"local-name", 10, 0, 1, 0},
-    {"starts-with", 11, 2, 2, 0},
-    {"namespace-uri", 13, 0, 1, 0},
+    {"local-name", 10, 0, 1, nullptr},
+    {"starts-with", 11, 2, 2, nullptr},
+    {"namespace-uri", 13, 0, 1, nullptr},
     {"string-length", 13, 0, 1, create_func_string_length},
-    {"substring-after", 15, 2, 2, 0},
-    {"normalize-space", 15, 0, 1, 0},
-    {"substring-before", 16, 2, 2, 0},
+    {"substring-after", 15, 2, 2, nullptr},
+    {"normalize-space", 15, 0, 1, nullptr},
+    {"substring-before", 16, 2, 2, nullptr},
 
-    {NULL, 0, 0, 0, 0}};
+    {nullptr, 0, 0, 0, nullptr}};
 
 /*
   Lookup a function by name
 
-  SYNOPSYS
+  SYNOPSIS
     Lookup a function by its name.
 
   RETURN
@@ -1144,10 +1142,10 @@ static MY_XPATH_FUNC my_func_names[] = {
 */
 static MY_XPATH_FUNC *my_xpath_function(const char *beg, const char *end) {
   MY_XPATH_FUNC *k, *function_names;
-  size_t length = end - beg;
+  const size_t length = end - beg;
   switch (length) {
     case 1:
-      return 0;
+      return nullptr;
     case 3:
       function_names = my_func_names3;
       break;
@@ -1167,10 +1165,10 @@ static MY_XPATH_FUNC *my_xpath_function(const char *beg, const char *end) {
     if (k->create && length == k->length &&
         !native_strncasecmp(beg, k->name, length))
       return k;
-  return NULL;
+  return nullptr;
 }
 
-/* Initialize a lex analizer token */
+/* Initialize a lex analyzer token */
 static void my_xpath_lex_init(MY_XPATH_LEX *lex, const char *str,
                               const char *strend) {
   lex->beg = str;
@@ -1187,10 +1185,10 @@ static int my_xdigit(int c) { return ((c) >= '0' && (c) <= '9'); }
 /*
   Scan the next token
 
-  SYNOPSYS
+  SYNOPSIS
     Scan the next token from the input.
     lex->term is set to the scanned token type.
-    lex->beg and lex->end are set to the beginnig
+    lex->beg and lex->end are set to the beginning
     and to the end of the token.
   RETURN
     N/A
@@ -1212,14 +1210,14 @@ static void my_xpath_lex_scan(MY_XPATH *xpath, MY_XPATH_LEX *lex,
   if ((length = xpath->cs->cset->ctype(
            xpath->cs, &ctype, reinterpret_cast<const uchar *>(beg),
            reinterpret_cast<const uchar *>(end))) > 0 &&
-      ((ctype & (_MY_L | _MY_U)) || *beg == '_')) {
-    // scan untill the end of the idenfitier
+      ((ctype & (MY_CHAR_L | MY_CHAR_U)) != 0 || *beg == '_')) {
+    // scan until the end of the idenfitier
     for (beg += length;
          (length = xpath->cs->cset->ctype(
               xpath->cs, &ctype, reinterpret_cast<const uchar *>(beg),
               reinterpret_cast<const uchar *>(end))) > 0 &&
-         ((ctype & (_MY_L | _MY_U | _MY_NMR)) || *beg == '_' || *beg == '-' ||
-          *beg == '.');
+         ((ctype & (MY_CHAR_L | MY_CHAR_U | MY_CHAR_NMR)) != 0 || *beg == '_' ||
+          *beg == '-' || *beg == '.');
          beg += length) /* no op */
       ;
     lex->end = beg;
@@ -1289,7 +1287,7 @@ static void my_xpath_lex_scan(MY_XPATH *xpath, MY_XPATH_LEX *lex,
 /*
   Scan the given token
 
-  SYNOPSYS
+  SYNOPSIS
     Scan the given token and rotate lasttok to prevtok on success.
 
   RETURN
@@ -1309,7 +1307,7 @@ static int my_xpath_parse_term(MY_XPATH *xpath, int term) {
 /*
   Scan AxisName
 
-  SYNOPSYS
+  SYNOPSIS
     Scan an axis name and store the scanned axis type into xpath->axis.
 
   RETURN
@@ -1317,7 +1315,7 @@ static int my_xpath_parse_term(MY_XPATH *xpath, int term) {
     0 - failure
 */
 static int my_xpath_parse_AxisName(MY_XPATH *xpath) {
-  int rc = my_xpath_parse_term(xpath, MY_XPATH_LEX_AXIS);
+  const int rc = my_xpath_parse_term(xpath, MY_XPATH_LEX_AXIS);
   xpath->axis = xpath->extra;
   return rc;
 }
@@ -1326,7 +1324,7 @@ static int my_xpath_parse_AxisName(MY_XPATH *xpath) {
 ** Grammar rules, according to http://www.w3.org/TR/xpath
 ** Implemented using recursive descendant method.
 ** All the following grammar processing functions accept
-** a signle "xpath" argument and return 1 on success and 0 on error.
+** a single "xpath" argument and return 1 on success and 0 on error.
 ** They also modify "xpath" argument by creating new items.
 */
 
@@ -1361,7 +1359,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath);
 /*
   Scan LocationPath
 
-  SYNOPSYS
+  SYNOPSIS
 
     [1] LocationPath ::=   RelativeLocationPath
                          | AbsoluteLocationPath
@@ -1375,8 +1373,8 @@ static int my_xpath_parse_LocationPath(MY_XPATH *xpath) {
   Item_nodeset_func *context = xpath->context;
 
   if (!xpath->context) xpath->context = xpath->rootelement;
-  int rc = my_xpath_parse_RelativeLocationPath(xpath) ||
-           my_xpath_parse_AbsoluteLocationPath(xpath);
+  const int rc = my_xpath_parse_RelativeLocationPath(xpath) ||
+                 my_xpath_parse_AbsoluteLocationPath(xpath);
 
   xpath->item = xpath->context;
   xpath->context = context;
@@ -1386,7 +1384,7 @@ static int my_xpath_parse_LocationPath(MY_XPATH *xpath) {
 /*
   Scan Absolute Location Path
 
-  SYNOPSYS
+  SYNOPSIS
 
     [2]     AbsoluteLocationPath ::=   '/' RelativeLocationPath?
                                      | AbbreviatedAbsoluteLocationPath
@@ -1420,7 +1418,7 @@ static int my_xpath_parse_AbsoluteLocationPath(MY_XPATH *xpath) {
 /*
   Scan Relative Location Path
 
-  SYNOPSYS
+  SYNOPSIS
 
     For better performance we combine these two rules
 
@@ -1456,7 +1454,7 @@ static int my_xpath_parse_RelativeLocationPath(MY_XPATH *xpath) {
 /*
   Scan non-abbreviated or abbreviated Step
 
-  SYNOPSYS
+  SYNOPSIS
 
   [4] Step ::=   AxisSpecifier NodeTest Predicate*
                | AbbreviatedStep
@@ -1514,7 +1512,7 @@ static int my_xpath_parse_Step(MY_XPATH *xpath) {
 /*
   Scan Abbreviated Axis Specifier
 
-  SYNOPSYS
+  SYNOPSIS
   [5] AxisSpecifier ::=  AxisName '::'
                          | AbbreviatedAxisSpecifier
 
@@ -1533,7 +1531,7 @@ static int my_xpath_parse_AbbreviatedAxisSpecifier(MY_XPATH *xpath) {
 /*
   Scan non-abbreviated axis specifier
 
-  SYNOPSYS
+  SYNOPSIS
 
   RETURN
     1 - success
@@ -1548,7 +1546,7 @@ static int my_xpath_parse_AxisName_colon_colon(MY_XPATH *xpath) {
 /*
   Scan Abbreviated AxisSpecifier
 
-  SYNOPSYS
+  SYNOPSIS
     [13] AbbreviatedAxisSpecifier  ::=  '@'?
 
   RETURN
@@ -1563,7 +1561,7 @@ static int my_xpath_parse_AxisSpecifier(MY_XPATH *xpath) {
 /*
   Scan NodeType followed by parens
 
-  SYNOPSYS
+  SYNOPSIS
 
   RETURN
     1 - success
@@ -1578,7 +1576,7 @@ static int my_xpath_parse_NodeTest_lp_rp(MY_XPATH *xpath) {
 /*
   Scan NodeTest
 
-  SYNOPSYS
+  SYNOPSIS
 
   [7] NodeTest ::=   NameTest
                    | NodeType '(' ')'
@@ -1594,7 +1592,7 @@ static int my_xpath_parse_NodeTest(MY_XPATH *xpath) {
 /*
   Scan Abbreviated Step
 
-  SYNOPSYS
+  SYNOPSIS
 
   [12] AbbreviatedStep  ::= '.'	| '..'
 
@@ -1613,7 +1611,7 @@ static int my_xpath_parse_AbbreviatedStep(MY_XPATH *xpath) {
 /*
   Scan Primary Expression
 
-  SYNOPSYS
+  SYNOPSIS
 
   [15] PrimaryExpr ::= VariableReference
                        | '(' Expr ')'   (RECURSIVE)
@@ -1652,7 +1650,7 @@ static int my_xpath_parse_PrimaryExpr(MY_XPATH *xpath) {
 /*
   Scan Function Call
 
-  SYNOPSYS
+  SYNOPSIS
     [16] FunctionCall ::= FunctionName '(' ( Argument ( ',' Argument )* )? ')'
     [17] Argument      ::= Expr (RECURSIVE)
     [14] Expr ::= OrExpr
@@ -1700,7 +1698,7 @@ right_paren:
 /*
   Scan Union Expression
 
-  SYNOPSYS
+  SYNOPSIS
     [18] UnionExpr ::=   PathExpr
                        | UnionExpr '|' PathExpr
 
@@ -1713,10 +1711,10 @@ static int my_xpath_parse_UnionExpr(MY_XPATH *xpath) {
 
   while (my_xpath_parse_term(xpath, MY_XPATH_LEX_VLINE)) {
     Item *prev = xpath->item;
-    if (prev->type() != Item::XPATH_NODESET) return 0;
+    if (prev->type() != Item::XPATH_NODESET_ITEM) return 0;
 
     if (!my_xpath_parse_PathExpr(xpath) ||
-        xpath->item->type() != Item::XPATH_NODESET) {
+        xpath->item->type() != Item::XPATH_NODESET_ITEM) {
       xpath->error = 1;
       return 0;
     }
@@ -1729,7 +1727,7 @@ static int my_xpath_parse_UnionExpr(MY_XPATH *xpath) {
 /*
   Scan Path Expression
 
-  SYNOPSYS
+  SYNOPSIS
 
   [19] PathExpr ::=   LocationPath
                     | FilterExpr
@@ -1748,7 +1746,7 @@ static int my_xpath_parse_FilterExpr_opt_slashes_RelativeLocationPath(
 
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_SLASH)) return 1;
 
-  if (xpath->item->type() != Item::XPATH_NODESET) {
+  if (xpath->item->type() != Item::XPATH_NODESET_ITEM) {
     xpath->lasttok = xpath->prevtok;
     xpath->error = 1;
     return 0;
@@ -1779,7 +1777,7 @@ static int my_xpath_parse_PathExpr(MY_XPATH *xpath) {
 /*
   Scan Filter Expression
 
-  SYNOPSYS
+  SYNOPSIS
     [20]  FilterExpr ::=   PrimaryExpr
                          | FilterExpr Predicate
 
@@ -1799,7 +1797,7 @@ static int my_xpath_parse_FilterExpr(MY_XPATH *xpath) {
 /*
   Scan Or Expression
 
-  SYNOPSYS
+  SYNOPSIS
     [21] OrExpr ::=   AndExpr
                     | OrExpr 'or' AndExpr
 
@@ -1830,7 +1828,7 @@ static int my_xpath_parse_OrExpr(MY_XPATH *xpath) {
 /*
   Scan And Expression
 
-  SYNOPSYS
+  SYNOPSIS
     [22] AndExpr ::=   EqualityExpr
                      | AndExpr 'and' EqualityExpr
 
@@ -1857,7 +1855,7 @@ static int my_xpath_parse_AndExpr(MY_XPATH *xpath) {
 /*
   Scan Equality Expression
 
-  SYNOPSYS
+  SYNOPSIS
     [23] EqualityExpr ::=   RelationalExpr
                           | EqualityExpr '=' RelationalExpr
                           | EqualityExpr '!=' RelationalExpr
@@ -1870,7 +1868,7 @@ static int my_xpath_parse_AndExpr(MY_XPATH *xpath) {
     0 - failure
 */
 static int my_xpath_parse_ne(MY_XPATH *xpath) {
-  MY_XPATH_LEX prevtok = xpath->prevtok;
+  const MY_XPATH_LEX prevtok = xpath->prevtok;
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_EXCL)) return 0;
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_EQ)) {
     /* Unget the exclamation mark */
@@ -1898,7 +1896,7 @@ static int my_xpath_parse_EqualityExpr(MY_XPATH *xpath) {
   operator_context = xpath->lasttok;
   while (my_xpath_parse_EqualityOperator(xpath)) {
     Item *prev = xpath->item;
-    int oper = xpath->extra;
+    const int oper = xpath->extra;
     if (!my_xpath_parse_RelationalExpr(xpath)) {
       xpath->error = 1;
       return 0;
@@ -1916,7 +1914,7 @@ static int my_xpath_parse_EqualityExpr(MY_XPATH *xpath) {
 /*
   Scan Relational Expression
 
-  SYNOPSYS
+  SYNOPSIS
 
     [24] RelationalExpr ::=   AdditiveExpr
                             | RelationalExpr '<' AdditiveExpr
@@ -1951,7 +1949,7 @@ static int my_xpath_parse_RelationalExpr(MY_XPATH *xpath) {
   operator_context = xpath->lasttok;
   while (my_xpath_parse_RelationalOperator(xpath)) {
     Item *prev = xpath->item;
-    int oper = xpath->extra;
+    const int oper = xpath->extra;
 
     if (!my_xpath_parse_AdditiveExpr(xpath)) {
       xpath->error = 1;
@@ -1969,7 +1967,7 @@ static int my_xpath_parse_RelationalExpr(MY_XPATH *xpath) {
 /*
   Scan Additive Expression
 
-  SYNOPSYS
+  SYNOPSIS
 
     [25] AdditiveExpr ::=   MultiplicativeExpr
                           | AdditiveExpr '+' MultiplicativeExpr
@@ -1986,7 +1984,7 @@ static int my_xpath_parse_AdditiveExpr(MY_XPATH *xpath) {
   if (!my_xpath_parse_MultiplicativeExpr(xpath)) return 0;
 
   while (my_xpath_parse_AdditiveOperator(xpath)) {
-    int oper = xpath->prevtok.term;
+    const int oper = xpath->prevtok.term;
     Item *prev = xpath->item;
     if (!my_xpath_parse_MultiplicativeExpr(xpath)) {
       xpath->error = 1;
@@ -2004,7 +2002,7 @@ static int my_xpath_parse_AdditiveExpr(MY_XPATH *xpath) {
 /*
   Scan Multiplicative Expression
 
-  SYNOPSYS
+  SYNOPSIS
 
     [26] MultiplicativeExpr ::=   UnaryExpr
                                 | MultiplicativeExpr MultiplyOperator UnaryExpr
@@ -2027,7 +2025,7 @@ static int my_xpath_parse_MultiplicativeExpr(MY_XPATH *xpath) {
   if (!my_xpath_parse_UnaryExpr(xpath)) return 0;
 
   while (my_xpath_parse_MultiplicativeOperator(xpath)) {
-    int oper = xpath->prevtok.term;
+    const int oper = xpath->prevtok.term;
     Item *prev = xpath->item;
     if (!my_xpath_parse_UnaryExpr(xpath)) {
       xpath->error = 1;
@@ -2038,7 +2036,7 @@ static int my_xpath_parse_MultiplicativeExpr(MY_XPATH *xpath) {
         xpath->item = new Item_func_mul(prev, xpath->item);
         break;
       case MY_XPATH_LEX_DIV:
-        xpath->item = new Item_func_int_div(prev, xpath->item);
+        xpath->item = new Item_func_div_int(prev, xpath->item);
         break;
       case MY_XPATH_LEX_MOD:
         xpath->item = new Item_func_mod(prev, xpath->item);
@@ -2051,7 +2049,7 @@ static int my_xpath_parse_MultiplicativeExpr(MY_XPATH *xpath) {
 /*
   Scan Unary Expression
 
-  SYNOPSYS
+  SYNOPSIS
 
     [27] UnaryExpr ::=   UnionExpr
                        | '-' UnaryExpr
@@ -2075,7 +2073,7 @@ static int my_xpath_parse_UnaryExpr(MY_XPATH *xpath) {
 /*
   Scan Number
 
-  SYNOPSYS
+  SYNOPSIS
 
     [30] Number ::= Digits ('.' Digits?)? | '.' Digits)
 
@@ -2090,7 +2088,7 @@ static int my_xpath_parse_UnaryExpr(MY_XPATH *xpath) {
   as it is in conflict with abbreviated step.
   1 + .123    does not work,
   1 + 0.123   does.
-  Perhaps it is better to move this code into lex analizer.
+  Perhaps it is better to move this code into lex analyzer.
 
   RETURN
     1 - success
@@ -2114,7 +2112,7 @@ static int my_xpath_parse_Number(MY_XPATH *xpath) {
 /*
   Scan NCName.
 
-  SYNOPSYS
+  SYNOPSIS
 
     The keywords AND, OR, MOD, DIV are valid identitiers
     when they are in identifier context:
@@ -2196,7 +2194,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
        !my_xpath_parse_term(xpath, MY_XPATH_LEX_IDENT)))
     return 0;
 
-  size_t name_length = xpath->prevtok.end - xpath->prevtok.beg;
+  const size_t name_length = xpath->prevtok.end - xpath->prevtok.beg;
   const char *name_str = xpath->prevtok.beg;
 
   if (user_var)
@@ -2210,15 +2208,15 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
         (spv = spc->find_variable(name_str, name_length, false))) {
       Item_splocal *splocal = new Item_splocal(
           Name_string(name_str, name_length, false), spv->offset, spv->type, 0);
-#ifndef DBUG_OFF
+#ifndef NDEBUG
       if (splocal) splocal->m_sp = lex->sphead;
 #endif
-      xpath->item = down_cast<Item *>(splocal);
+      xpath->item = splocal;
     } else {
-      xpath->item = NULL;
-      DBUG_ASSERT(xpath->query.end > dollar_pos);
+      xpath->item = nullptr;
+      assert(xpath->query.end > dollar_pos);
       size_t len = xpath->query.end - dollar_pos;
-      set_if_smaller(len, 32);
+      len = std::min(len, size_t(32));
       my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.*s'",
                       MYF(0), static_cast<int>(len), dollar_pos);
     }
@@ -2229,7 +2227,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
 /*
   Scan Name Test
 
-  SYNOPSYS
+  SYNOPSIS
 
     [37] NameTest ::=  '*'
                       | NCName ':' '*'
@@ -2240,15 +2238,15 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
 */
 static int my_xpath_parse_NodeTest_QName(MY_XPATH *xpath) {
   if (!my_xpath_parse_QName(xpath)) return 0;
-  DBUG_ASSERT(xpath->context);
-  size_t len = xpath->prevtok.end - xpath->prevtok.beg;
+  assert(xpath->context);
+  const size_t len = xpath->prevtok.end - xpath->prevtok.beg;
   xpath->context =
       nametestfunc(xpath, xpath->axis, xpath->context, xpath->prevtok.beg, len);
   return 1;
 }
 static int my_xpath_parse_NodeTest_asterisk(MY_XPATH *xpath) {
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_ASTERISK)) return 0;
-  DBUG_ASSERT(xpath->context);
+  assert(xpath->context);
   xpath->context = nametestfunc(xpath, xpath->axis, xpath->context, "*", 1);
   return 1;
 }
@@ -2260,7 +2258,7 @@ static int my_xpath_parse_NameTest(MY_XPATH *xpath) {
 /*
   Scan an XPath expression
 
-  SYNOPSYS
+  SYNOPSIS
     Scan xpath expression.
     The expression is returned in xpath->expr.
 
@@ -2281,8 +2279,8 @@ static int my_xpath_parse(MY_XPATH *xpath, const char *str,
          my_xpath_parse_term(xpath, MY_XPATH_LEX_EOF);
 }
 
-bool Item_xml_str_func::resolve_type(THD *) {
-  nodeset_func = NULL;
+bool Item_xml_str_func::resolve_type(THD *thd) {
+  if (param_type_is_default(thd, 0, -1)) return true;
 
   if (agg_arg_charsets_for_comparison(collation, args, arg_count)) return true;
 
@@ -2304,6 +2302,8 @@ bool Item_xml_str_func::resolve_type(THD *) {
 
   if (args[1]->const_item() && parse_xpath(args[1])) return true;
 
+  if (nodeset_func != nullptr) nodeset_func_permanent = true;
+
   return false;
 }
 
@@ -2319,16 +2319,17 @@ bool Item_xml_str_func::parse_xpath(Item *xpath_expr) {
   xpath.debug = 0;
   xpath.pxml = &pxml;
 
-  int rc = my_xpath_parse(&xpath, xp->ptr(), xp->ptr() + xp->length());
+  const int rc = my_xpath_parse(&xpath, xp->ptr(), xp->ptr() + xp->length());
 
   if (!rc) {
     size_t clen = xpath.query.end - xpath.lasttok.beg;
-    set_if_smaller(clen, 32);
+    clen = std::min(clen, size_t(32));
     my_printf_error(ER_UNKNOWN_ERROR, "XPATH syntax error: '%.*s'", MYF(0),
                     static_cast<int>(clen), xpath.lasttok.beg);
     return true;
   }
 
+  assert(nodeset_func == nullptr);
   nodeset_func = xpath.item;
   if (nodeset_func && nodeset_func->fix_fields(current_thd, &nodeset_func))
     return true;
@@ -2347,10 +2348,10 @@ typedef struct {
 /*
   Process tag beginning
 
-  SYNOPSYS
+  SYNOPSIS
 
     A call-back function executed when XML parser
-    is entering a tag or an attribue.
+    is entering a tag or an attribute.
     Appends the new node into data->pxml.
     Increments data->level.
 
@@ -2365,7 +2366,7 @@ int xml_enter(MY_XML_PARSER *st, const char *attr, size_t len) {
 
   node.parent = data->parent;  // Set parent for the new node to old parent
   data->parent = data->pxml->size();  // Remember current node as new parent
-  DBUG_ASSERT(data->level < MAX_LEVEL);
+  assert(data->level < MAX_LEVEL);
   data->pos[data->level] = data->pxml->size();
   if (data->level < MAX_LEVEL - 1)
     node.level = data->level++;
@@ -2381,10 +2382,10 @@ int xml_enter(MY_XML_PARSER *st, const char *attr, size_t len) {
 /*
   Process text node
 
-  SYNOPSYS
+  SYNOPSIS
 
     A call-back function executed when XML parser
-    is entering into a tag or an attribue textual value.
+    is entering into a tag or an attribute textual value.
     The value is appended into data->pxml.
 
   RETURN
@@ -2408,7 +2409,7 @@ int xml_value(MY_XML_PARSER *st, const char *attr, size_t len) {
 /*
   Leave a tag or an attribute
 
-  SYNOPSYS
+  SYNOPSIS
 
     A call-back function executed when XML parser
     is leaving a tag or an attribute.
@@ -2421,7 +2422,7 @@ extern "C" int xml_leave(MY_XML_PARSER *st, const char *attr, size_t len);
 
 int xml_leave(MY_XML_PARSER *st, const char *, size_t) {
   auto *data = reinterpret_cast<MY_XML_USER_DATA *>(st->user_data);
-  DBUG_ASSERT(data->level > 0);
+  assert(data->level > 0);
   data->level--;
 
   data->parent = data->pxml->at(data->parent).parent;
@@ -2433,7 +2434,7 @@ int xml_leave(MY_XML_PARSER *st, const char *, size_t) {
 /*
   Parse raw XML
 
-  SYNOPSYS
+  SYNOPSIS
 
 
   RETURN
@@ -2481,36 +2482,39 @@ String *Item_func_xml_extractvalue::val_str(String *str) {
   String *res;
   null_value = false;
   if (!nodeset_func && parse_xpath(args[1])) {
-    DBUG_ASSERT(maybe_null);
+    assert(is_nullable());
     null_value = true;
-    return NULL;
+    return nullptr;
   }
 
   tmp_value.set("", 0, collation.collation);
   if (!nodeset_func || !(res = args[0]->val_str(str)) ||
       !parse_xml(res, &pxml) || !(res = nodeset_func->val_str(&tmp_value))) {
-    null_value = 1;
-    return 0;
+    null_value = true;
+    return nullptr;
   }
   return res;
 }
 
 String *Item_func_xml_update::val_str(String *str) {
-  String *res, *rep;
-
   null_value = false;
   if (!nodeset_func && parse_xpath(args[1])) {
-    DBUG_ASSERT(maybe_null);
-    null_value = true;
-    return nullptr;
+    assert(is_nullable());
+    return error_str();
   }
 
-  if (!nodeset_func || !(res = args[0]->val_str(str)) ||
-      !(rep = args[2]->val_str(&tmp_value)) || !parse_xml(res, &pxml) ||
-      (nodeset_func->type() != XPATH_NODESET)) {
-    null_value = true;
-    return nullptr;
-  }
+  if (nodeset_func == nullptr) return error_str();
+
+  String *res = eval_string_arg(collation.collation, args[0], str);
+  if (res == nullptr) return error_str();
+
+  StringBuffer<STRING_BUFFER_USUAL_SIZE> rep_buf(nullptr, 0,
+                                                 collation.collation);
+  String *rep = eval_string_arg(collation.collation, args[2], &rep_buf);
+  if (rep == nullptr) return error_str();
+
+  if (!parse_xml(res, &pxml)) return error_str();
+  if (nodeset_func->type() != XPATH_NODESET_ITEM) return error_str();
 
   XPathFilter nodeset;
   down_cast<const Item_nodeset_func *>(nodeset_func)->val_nodeset(&nodeset);
@@ -2523,18 +2527,19 @@ String *Item_func_xml_update::val_str(String *str) {
 
   const MY_XML_NODE *node = &pxml.at(nodeset.at(0).num);
 
-  if (!node->level) {
+  if (node->level == 0) {
     /*
       Root element, without NameTest:
       UpdateXML(xml, '/', 'replacement');
       Just return the replacement string.
     */
-    return rep;
+    tmp_value.copy(*rep);
+    return &tmp_value;
   }
 
   tmp_value.length(0);
   tmp_value.set_charset(collation.collation);
-  uint offs = node->type == MY_XML_NODE_TAG ? 1 : 0;
+  const uint offs = node->type == MY_XML_NODE_TAG ? 1 : 0;
   tmp_value.append(res->ptr(), node->beg - res->ptr() - offs);
   tmp_value.append(rep->ptr(), rep->length());
   const char *end = node->tagend + offs;
