@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +24,7 @@
 #include <algorithm>  // std::find
 #include "binlog.h"   // mysql_bin_log
 #include "mysqld.h"   // key_rwlock_gtid_mode_lock
+#include "nulls.h"
 #include "sql/rpl_gtid.h"
 #include "sql/rpl_msr.h"  // channel_map
 
@@ -47,9 +49,9 @@ Gtid_mode::value_type Gtid_mode::get() const {
   return (value_type)m_atomic_mode.load(std::memory_order_acquire);
 }
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
 const char *Gtid_mode::get_string() const { return to_string(get()); }
-#endif  // ifndef DBUG_OFF
+#endif  // ifndef NDEBUG
 
 std::pair<bool, Gtid_mode::value_type> Gtid_mode::from_string(
     const std::string s) {
@@ -72,7 +74,7 @@ std::ostream &operator<<(std::ostream &oss, Gtid_mode::value_type const &mode) {
   return oss;
 }
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
 std::ostream &operator<<(std::ostream &oss, Gtid_mode const &mode) {
   oss << mode.get_string();
   return oss;
