@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,15 +27,16 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_base.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/dtoa.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql/thread_type.h"
 #include "mysqld_error.h"
 #include "sql/dynamic_ids.h"  // Server_ids
@@ -51,6 +53,7 @@
 #include "sql/system_variables.h"
 #include "sql/table.h"
 #include "sql_string.h"
+#include "strmake.h"
 #include "thr_lock.h"
 
 Rpl_info_table::Rpl_info_table(uint nparam, const char *param_schema,
@@ -145,7 +148,7 @@ int Rpl_info_table::do_init_info(enum_find_method method, uint instance) {
       break;
 
     default:
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
 
@@ -249,12 +252,12 @@ int Rpl_info_table::do_flush_info(const bool force) {
   }
 
 end:
-  DBUG_EXECUTE_IF("mts_debug_concurrent_access", {
+  DBUG_EXECUTE_IF("mta_debug_concurrent_access", {
     while (thd->system_thread == SYSTEM_THREAD_SLAVE_WORKER &&
-           mts_debug_concurrent_access < 2 && mts_debug_concurrent_access > 0) {
+           mta_debug_concurrent_access < 2 && mta_debug_concurrent_access > 0) {
       DBUG_PRINT("mts", ("Waiting while locks are acquired to show "
                          "concurrency in mts: %u %u\n",
-                         mts_debug_concurrent_access, thd->thread_id()));
+                         mta_debug_concurrent_access, thd->thread_id()));
       my_sleep(6000000);
     }
   };);
@@ -366,7 +369,7 @@ int Rpl_info_table::do_reset_info(uint nparam, const char *param_schema,
     goto end;
   }
 
-  if (!(handler_error = table->file->ha_index_init(0, 1))) {
+  if (!(handler_error = table->file->ha_index_init(0, true))) {
     KEY *key_info = table->key_info;
 
     /*
@@ -375,7 +378,7 @@ int Rpl_info_table::do_reset_info(uint nparam, const char *param_schema,
       todo: for another table in future, consider to make use of the
       passed parameter to locate the lookup key.
     */
-    DBUG_ASSERT(strcmp(info->str_table.str, "slave_worker_info") == 0);
+    assert(strcmp(info->str_table.str, "slave_worker_info") == 0);
 
     if (info->verify_table_primary_key_fields(table)) {
       error = 1;
@@ -387,15 +390,17 @@ int Rpl_info_table::do_reset_info(uint nparam, const char *param_schema,
     table->field[fieldnr]->store(channel_name, strlen(channel_name),
                                  &my_charset_bin);
     uint key_len = key_info->key_part[0].store_length;
-    uchar *key_buf = table->field[fieldnr]->ptr;
 
+    uchar key[MAX_KEY_LENGTH];
+    key_copy(key, table->record[0], table->key_info,
+             table->key_info->key_length);
     if (!(handler_error = table->file->ha_index_read_map(
-              table->record[0], key_buf, (key_part_map)1, HA_READ_KEY_EXACT))) {
+              table->record[0], key, (key_part_map)1, HA_READ_KEY_EXACT))) {
       do {
         if ((handler_error = table->file->ha_delete_row(table->record[0])))
           break;
       } while (!(handler_error = table->file->ha_index_next_same(
-                     table->record[0], key_buf, key_len)));
+                     table->record[0], key, key_len)));
       if (handler_error != HA_ERR_END_OF_FILE) error = 1;
     } else {
       /*
@@ -527,7 +532,7 @@ end:
 bool Rpl_info_table::do_count_info(uint nparam, const char *param_schema,
                                    const char *param_table,
                                    MY_BITMAP const *nullable_bitmap,
-                                   uint *counter) {
+                                   ulonglong *counter) {
   int error = 1;
   TABLE *table = nullptr;
   sql_mode_t saved_mode;
@@ -580,6 +585,54 @@ end:
   return error;
 }
 
+std::pair<bool, bool> Rpl_info_table::table_in_use(
+    uint nparam, const char *param_schema, const char *param_table,
+    MY_BITMAP const *nullable_bitmap) {
+  bool error{false};
+  bool empty{false};
+
+  TABLE *table = nullptr;
+  Open_tables_backup backup;
+  Rpl_info_table *info = nullptr;
+
+  DBUG_TRACE;
+
+  if (!(info = new Rpl_info_table(nparam, param_schema, param_table, 0, nullptr,
+                                  nullable_bitmap)))
+    return std::make_pair(true, false);
+
+  THD *thd = info->access->create_thd();
+  sql_mode_t saved_mode = thd->variables.sql_mode;
+
+  /*
+    Opens and locks the rpl_info table before accessing it.
+  */
+  if (!info->access->open_table(thd, to_lex_cstring(info->str_schema),
+                                to_lex_cstring(info->str_table),
+                                info->get_number_info(), TL_READ, &table,
+                                &backup)) {
+    /*
+      Check if table is in use
+    */
+    std::tie(error, empty) = info->access->is_table_in_use(table);
+    if (error) {
+      LogErr(WARNING_LEVEL, ER_RPL_CANT_SCAN_INFO_TABLE, info->str_schema.str,
+             info->str_table.str);
+    }
+  }
+  /* else:
+    We cannot simply print out a warning message at this
+    point because this may represent a bootstrap.
+  */
+
+  // Unlocks and closes the rpl_info table.
+  error = info->access->close_table(thd, table, &backup, error) || error;
+  thd->variables.sql_mode = saved_mode;
+  info->access->drop_thd(thd);
+  delete info;
+  return std::make_pair(error != 0, empty);
+}
+
 void Rpl_info_table::do_end_info() {}
 
 int Rpl_info_table::do_prepare_info_for_read() {
@@ -675,7 +728,7 @@ Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
 
 Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
     const int pos, uchar *value, const size_t size,
-    const uchar *default_value MY_ATTRIBUTE((unused))) {
+    const uchar *default_value [[maybe_unused]]) {
   if (bitmap_is_set(&field_values->is_null, pos)) {
     return Rpl_info_handler::enum_field_get_status::FIELD_VALUE_IS_NULL;
   } else {
@@ -692,7 +745,7 @@ Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
     return Rpl_info_handler::enum_field_get_status::FIELD_VALUE_IS_NULL;
   } else {
     if (field_values->value[pos].length()) {
-      *value = strtoul(field_values->value[pos].c_ptr_safe(), 0, 10);
+      *value = strtoul(field_values->value[pos].c_ptr_safe(), nullptr, 10);
       return Rpl_info_handler::enum_field_get_status::FIELD_VALUE_NOT_NULL;
     } else if (default_value) {
       *value = default_value;
@@ -740,7 +793,7 @@ Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
 
 Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
     const int pos, Server_ids *value,
-    const Server_ids *default_value MY_ATTRIBUTE((unused))) {
+    const Server_ids *default_value [[maybe_unused]]) {
   if (value->unpack_dynamic_ids(field_values->value[pos].c_ptr_safe()))
     return Rpl_info_handler::enum_field_get_status::FAILURE;
 
@@ -758,7 +811,6 @@ bool Rpl_info_table::do_update_is_transactional() {
   Open_tables_backup backup;
 
   DBUG_TRACE;
-  DBUG_EXECUTE_IF("simulate_update_is_transactional_error", { return true; });
 
   THD *thd = access->create_thd();
   saved_mode = thd->variables.sql_mode;
@@ -777,7 +829,7 @@ bool Rpl_info_table::do_update_is_transactional() {
   error = false;
 
 end:
-  error = access->close_table(thd, table, &backup, 0) || error;
+  error = access->close_table(thd, table, &backup, false) || error;
   thd->variables.sql_mode = saved_mode;
   thd->variables.option_bits = saved_options;
   access->drop_thd(thd);
