@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,22 +30,22 @@
 #include <memory>
 #include <string>
 #include <string_view>
+#include <tuple>
 #include <vector>
 
 #include "lex_string.h"
-#include "libbinlogevents/include/binlog_event.h"
-#include "m_string.h"
 #include "map_helpers.h"
 #include "my_bitmap.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
-#include "my_loglevel.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
+#include "mysql/binlog/event/binlog_event.h"
 #include "mysql/components/services/bits/mysql_cond_bits.h"
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/thread_type.h"
 #include "prealloced_array.h"  // Prealloced_array
@@ -62,6 +63,7 @@
 #include "sql/sql_class.h"    // THD
 #include "sql/system_variables.h"
 #include "sql/table.h"
+#include "strmake.h"
 
 class Commit_order_manager;
 class Master_info;
@@ -70,7 +72,7 @@ class Rpl_info_handler;
 class Slave_committed_queue;
 class Slave_worker;
 class String;
-struct LEX_MASTER_INFO;
+struct LEX_SOURCE_INFO;
 struct db_worker_hash_entry;
 
 extern uint sql_replica_skip_counter;
@@ -79,14 +81,14 @@ typedef Prealloced_array<Slave_worker *, 4> Slave_worker_array;
 
 typedef struct slave_job_item {
   Log_event *data;
-  uint relay_number;
   my_off_t relay_pos;
+  char event_relay_log_name[FN_REFLEN + 1];
 } Slave_job_item;
 
 /**
   This class is used to store the type and value for
-  Assign_gtids_to_anonymous_transactions parameter of Change master command on
-  slave.
+  Assign_gtids_to_anonymous_transactions parameter of Change replication source
+  command on slave.
 */
 class Assign_gtids_to_anonymous_transactions_info {
  public:
@@ -98,12 +100,12 @@ class Assign_gtids_to_anonymous_transactions_info {
     of replica which is the server where this transformation of anonymous to
     gtid event happens. UUID: Anonymous gtid events will be converted to Gtid
     event & the UUID used while create GTIDs will be the one specified via
-    Change master command to the parameter
+    Change replication source command to the parameter
     ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS
   */
   enum class enum_type { AGAT_OFF = 1, AGAT_LOCAL, AGAT_UUID };
   /**
-    The default constructor intializes the parameter to their default value
+    The default constructor initializes parameters to their default value
   */
   Assign_gtids_to_anonymous_transactions_info() {
     set_info(enum_type::AGAT_OFF, "");
@@ -138,10 +140,10 @@ Relay_log_info contains:
   - misc information specific to the SQL thread
 
 Relay_log_info is initialized from a repository, i.e. table or file, if there is
-one. Otherwise, data members are intialized with defaults by calling
+one. Otherwise, data members are initialized with defaults by calling
 init_relay_log_info().
 
-The relay.info table/file shall be updated whenever: (i) the relay log file
+The applier metadata shall be updated whenever: (i) the relay log file
 is rotated, (ii) SQL Thread is stopped, (iii) while processing a Xid_log_event,
 (iv) after a Query_log_event (i.e. commit or rollback) and (v) after processing
 any statement written to the binary log without a transaction context.
@@ -227,7 +229,7 @@ class Relay_log_info : public Rpl_info {
      */
     USER_DOES_NOT_EXIST,
     /**
-      Provided user doesn't have the necesary privileges to execute the needed
+      Provided user doesn't have the necessary privileges to execute the needed
       operations.
      */
     USER_DOES_NOT_HAVE_PRIVILEGES,
@@ -263,25 +265,27 @@ class Relay_log_info : public Rpl_info {
   };
 
   /**
-    Identifies what is the slave policy on primary keys in tables.
+    Identifies what is the replica policy on primary keys in tables.
   */
   enum enum_require_table_primary_key {
     /**No policy, used on PFS*/
     PK_CHECK_NONE = 0,
     /**
-      The slave sets the value of sql_require_primary_key according to
+      The replica sets the value of sql_require_primary_key according to
       the source replicated value.
     */
     PK_CHECK_STREAM = 1,
-    /** The slave enforces tables to have primary keys for a given channel*/
+    /** The replica enforces tables to have primary keys for a given channel*/
     PK_CHECK_ON = 2,
-    /** The slave does not enforce any policy around primary keys*/
-    PK_CHECK_OFF = 3
+    /** The replica does not enforce any policy around primary keys*/
+    PK_CHECK_OFF = 3,
+    /** The replica generates GIPKs for incoming keyless tables*/
+    PK_CHECK_GENERATE = 4
   };
 
   /**
     Stores the information related to the ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS
-    parameter of CHANGE MASTER
+    parameter of CHANGE REPLICATION SOURCE
   */
   Assign_gtids_to_anonymous_transactions_info
       m_assign_gtids_to_anonymous_transactions_info;
@@ -336,7 +340,7 @@ class Relay_log_info : public Rpl_info {
   /* The following variables are safe to read any time */
 
   /*
-    When we restart slave thread we need to have access to the previously
+    When we restart replica thread we need to have access to the previously
     created temporary tables. Modified only on init/end and by the SQL
     thread, read only by SQL thread.
   */
@@ -623,7 +627,8 @@ class Relay_log_info : public Rpl_info {
      replicated tables.
 
      @return STREAM if it replicates the source values, ON if it enforces the
-             need on primary keys, OFF if it does no enforce any restrictions.
+             need on primary keys, OFF if it does no enforce any restrictions,
+             GENERATE if a GIPK is added to the table.
    */
   enum_require_table_primary_key get_require_table_primary_key_check() const;
 
@@ -663,7 +668,7 @@ class Relay_log_info : public Rpl_info {
     Let's call a group (of events) :
       - a transaction
       or
-      - an autocommiting query + its associated events (INSERT_ID,
+      - an autocommitting query + its associated events (INSERT_ID,
     TIMESTAMP...)
     We need these rli coordinates :
     - relay log name and position of the beginning of the group we currently are
@@ -673,7 +678,7 @@ class Relay_log_info : public Rpl_info {
     executed. This event is part of the current group.
     Formerly we only had the immediately above coordinates, plus a 'pending'
     variable, but this dealt wrong with the case of a transaction starting on a
-    relay log and finishing (commiting) on another relay log. Case which can
+    relay log and finishing (committing) on another relay log. Case which can
     happen when, for example, the relay log gets rotated because of
     max_binlog_size.
   */
@@ -694,11 +699,39 @@ class Relay_log_info : public Rpl_info {
   char group_relay_log_name[FN_REFLEN];
   ulonglong group_relay_log_pos;
   char event_relay_log_name[FN_REFLEN];
-  /* The suffix number of relay log name */
-  uint event_relay_log_number;
   ulonglong event_relay_log_pos;
   ulonglong future_event_relay_log_pos;
 
+ public:
+  /**
+     Process an event and based on its type () set group beginning and end
+     @param ev - event within a group (including first and last)
+   */
+  void set_group_source_log_start_end_pos(const Log_event *ev);
+  /**
+     Get event group positions in source binary log on a replica which is
+     processed by a worker in MTA or coordinator in STA.
+     @return source event group start and end position in binary log
+   */
+  std::tuple<ulonglong, ulonglong> get_group_source_log_start_end_pos() const;
+
+ private:
+  /**
+   * Event group beginning event has been seen. Event group may begin with two
+   * events marked as beginning.
+   * @see set_group_source_log_start_end_pos
+   */
+  bool group_source_log_seen_start_pos;
+  /**
+   * @see set_group_source_log_start_end_pos, get_group_source_log_start_end_pos
+   */
+  ulonglong group_source_log_start_pos;
+  /**
+   * @see set_group_source_log_start_end_pos, get_group_source_log_start_end_pos
+   */
+  ulonglong group_source_log_end_pos;
+
+ protected:
   /* current event's start position in relay log */
   my_off_t event_start_pos;
   /*
@@ -729,6 +762,9 @@ class Relay_log_info : public Rpl_info {
   /* Flag that ensures the retrieved GTID set is initialized only once. */
   bool gtid_retrieved_initialized;
 
+  /// Flag that ensures the relay log is sanitized only once.
+  bool relay_log_sanitized = false;
+
   /**
     Stores information on the last processed transaction or the transaction
     that is currently being processed.
@@ -784,6 +820,8 @@ class Relay_log_info : public Rpl_info {
     or alter operation that does not have a primary key.
     If set to OFF it does not enforce any policies on the channel for primary
     keys.
+    If set to GENERATE it adds GIPKs to tables that are created without a PK
+    in the replica applier threads.
   */
   enum_require_table_primary_key m_require_table_primary_key_check;
 
@@ -799,13 +837,13 @@ class Relay_log_info : public Rpl_info {
  public:
   bool is_relay_log_truncated() { return m_relay_log_truncated; }
 
-  Sid_map *get_sid_map() { return gtid_set->get_sid_map(); }
+  Tsid_map *get_tsid_map() { return gtid_set->get_tsid_map(); }
 
-  Checkable_rwlock *get_sid_lock() { return get_sid_map()->get_sid_lock(); }
+  Checkable_rwlock *get_tsid_lock() { return get_tsid_map()->get_tsid_lock(); }
 
   void add_logged_gtid(rpl_sidno sidno, rpl_gno gno) {
-    get_sid_lock()->assert_some_lock();
-    assert(sidno <= get_sid_map()->get_max_sidno());
+    get_tsid_lock()->assert_some_lock();
+    assert(sidno <= get_tsid_map()->get_max_sidno());
     gtid_set->ensure_sidno(sidno);
     gtid_set->_add_gtid(sidno, gno);
   }
@@ -853,7 +891,7 @@ class Relay_log_info : public Rpl_info {
 
   /**
     Flag that the group_master_log_pos is invalid. This may occur
-    (for example) after CHANGE MASTER TO RELAY_LOG_POS.  This will
+    (for example) after CHANGE REPLICATION SOURCE TO RELAY_LOG_POS.  This will
     be unset after the first event has been executed and the
     group_master_log_pos is valid again.
 
@@ -863,25 +901,28 @@ class Relay_log_info : public Rpl_info {
 
   /*
     Handling of the relay_log_space_limit optional constraint.
-    ignore_log_space_limit is used to resolve a deadlock between I/O and SQL
-    threads, the SQL thread sets it to unblock the I/O thread and make it
-    temporarily forget about the constraint.
   */
-  ulonglong log_space_limit, log_space_total;
-  std::atomic<bool> ignore_log_space_limit;
+  std::atomic<ulonglong> log_space_limit, log_space_total;
 
-  /*
-    Used by the SQL thread to instructs the IO thread to rotate
-    the logs when the SQL thread needs to purge to release some
-    disk space.
-   */
-  std::atomic<bool> sql_force_rotate_relay;
+  // This flag is used by a coordinator to check if the receiver waits for
+  // a relay log space. If yes, it will enable aggressive relay log
+  // purge.
+  std::atomic_bool is_receiver_waiting_for_rl_space;
+
+  // This is file to which coordinator moved after enforced purge
+  // It is used by the receiver to check if all possible files were purged
+  // before making a decision on whether transaction may fit into the
+  // relay_log_space_limit. It is used to avoid possibly infinite waiting in
+  // case a transaction and required relay log metadata is bigger than
+  // 'relay_log_space_limit'. This filename is protected with the
+  // log_space_lock
+  std::string coordinator_log_after_purge{""};
 
   time_t last_master_timestamp;
 
   /**
     Reset the delay.
-    This is used by RESET SLAVE to clear the delay.
+    This is used by RESET REPLICA to clear the delay.
   */
   void clear_sql_delay() { sql_delay = 0; }
 
@@ -890,13 +931,14 @@ class Relay_log_info : public Rpl_info {
     skipping one or more events in the master log that have caused
     errors, and have been manually applied by DBA already.
   */
-  volatile uint32 slave_skip_counter;
-  volatile ulong abort_pos_wait; /* Incremented on change master */
+  std::atomic<uint32> slave_skip_counter;
+  std::atomic<ulong>
+      abort_pos_wait; /* Incremented on change replication source */
   mysql_mutex_t log_space_lock;
   mysql_cond_t log_space_cond;
 
   /*
-     Condition and its parameters from START SLAVE UNTIL clause.
+     Condition and its parameters from START REPLICA UNTIL clause.
 
      UNTIL condition is tested with is_until_satisfied() method that is
      called by exec_relay_log_event(). is_until_satisfied() caches the result
@@ -939,7 +981,7 @@ class Relay_log_info : public Rpl_info {
   ulonglong ign_master_log_pos_end;
 
   /*
-    Indentifies where the SQL Thread should create temporary files for the
+    Identifies where the SQL Thread should create temporary files for the
     LOAD DATA INFILE. This is used for security reasons.
    */
   char slave_patternload_file[FN_REFLEN];
@@ -1041,20 +1083,20 @@ class Relay_log_info : public Rpl_info {
 
   void close_temporary_tables();
 
-  RPL_TABLE_LIST *tables_to_lock; /* RBR: Tables to lock  */
-  uint tables_to_lock_count;      /* RBR: Count of tables to lock */
-  table_mapping m_table_map;      /* RBR: Mapping table-id to table */
+  RPL_Table_ref *tables_to_lock; /* RBR: Tables to lock  */
+  uint tables_to_lock_count;     /* RBR: Count of tables to lock */
+  table_mapping m_table_map;     /* RBR: Mapping table-id to table */
   /* RBR: Record Rows_query log event */
   Rows_query_log_event *rows_query_ev;
 
   bool get_table_data(TABLE *table_arg, table_def **tabledef_var,
                       TABLE **conv_table_var) const {
     assert(tabledef_var && conv_table_var);
-    for (TABLE_LIST *ptr = tables_to_lock; ptr != nullptr;
+    for (Table_ref *ptr = tables_to_lock; ptr != nullptr;
          ptr = ptr->next_global)
       if (ptr->table == table_arg) {
-        *tabledef_var = &static_cast<RPL_TABLE_LIST *>(ptr)->m_tabledef;
-        *conv_table_var = static_cast<RPL_TABLE_LIST *>(ptr)->m_conv_table;
+        *tabledef_var = &static_cast<RPL_Table_ref *>(ptr)->m_tabledef;
+        *conv_table_var = static_cast<RPL_Table_ref *>(ptr)->m_conv_table;
         DBUG_PRINT("debug", ("Fetching table data for table %s.%s:"
                              " tabledef: %p, conv_table: %p",
                              table_arg->s->db.str, table_arg->s->table_name.str,
@@ -1138,15 +1180,15 @@ class Relay_log_info : public Rpl_info {
   /*
     This flag is turned ON when the workers array is initialized.
     Before destroying the workers array we check this flag to make sure
-    we are not destroying an unitilized array. For the purpose of reporting the
-    worker status in performance schema table, we need to preserve the workers
-    array after worker thread was killed. So, we copy this array into
+    we are not destroying an uninitialized array. For the purpose of reporting
+    the worker status in performance schema table, we need to preserve the
+    workers array after worker thread was killed. So, we copy this array into
     workers_copy_pfs array which is used for reporting until next
     init_workers().
   */
   bool workers_array_initialized;
 
-  volatile ulong pending_jobs;
+  std::atomic<ulong> pending_jobs;
   mysql_mutex_t pending_jobs_lock;
   mysql_cond_t pending_jobs_cond;
   mysql_mutex_t exit_count_lock;  // mutex of worker exit count
@@ -1186,7 +1228,7 @@ class Relay_log_info : public Rpl_info {
      are experiencing and is used as a parameter to compute a nap time for
      Coordinator in order to avoid reaching WQ limits.
   */
-  volatile long mts_wq_excess_cnt;
+  std::atomic<long> mts_wq_excess_cnt;
   long mts_worker_underrun_level;   // % of WQ size at which W is considered
                                     // hungry
   ulong mts_coordinator_basic_nap;  // C sleeps to avoid WQs overrun
@@ -1208,7 +1250,7 @@ class Relay_log_info : public Rpl_info {
   bool mts_recovery_group_seen_begin;
 
   /*
-    While distibuting events basing on their properties MTS
+    While distributing events based on their properties MTS
     Coordinator changes its mts group status.
     Transition normally flowws to follow `=>' arrows on the diagram:
 
@@ -1236,14 +1278,16 @@ class Relay_log_info : public Rpl_info {
   /*
     MTS statistics:
   */
+  long mts_online_stat_curr;      // counter to decide on generating
+                                  // ER_RPL_MTA_STATISTICS message
   ulonglong mts_events_assigned;  // number of events (statements) scheduled
   ulonglong mts_groups_assigned;  // number of groups (transactions) scheduled
-  volatile ulong
+  std::atomic<ulong>
       mts_wq_overrun_cnt;   // counter of all mts_wq_excess_cnt increments
   ulong wq_size_waits_cnt;  // number of times C slept due to WQ:s oversize
   /*
     Counter of how many times Coordinator saw Workers are filled up
-    "enough" with assignements. The enough definition depends on
+    "enough" with assignments. The enough definition depends on
     the scheduler type.
   */
   ulong mts_wq_no_underrun_cnt;
@@ -1266,6 +1310,8 @@ class Relay_log_info : public Rpl_info {
   struct timespec stats_begin;  // applier's bootstrap time
 
   time_t mts_last_online_stat;
+  /// Last moment in time the MTA printed a coordinator waited stats
+  time_t mta_coordinator_has_waited_stat;
   /* end of MTS statistics */
 
   /**
@@ -1504,8 +1550,8 @@ class Relay_log_info : public Rpl_info {
     (if necessary), calculating the received GTID set for the channel and
     storing the updated rli object configuration into the repository.
 
-    When this function is called in a change master process and the change
-    master procedure will purge all the relay log files later, there is no
+    When this function is called in a change replication source process and the
+    change procedure will purge all the relay log files later, there is no
     reason to try to calculate the received GTID set of the channel based on
     existing relay log files (they will be purged). Allowing reads to existing
     relay log files at this point may lead to put the server in a state where
@@ -1513,13 +1559,13 @@ class Relay_log_info : public Rpl_info {
     replication log files was ON and the keyring plugin is not available
     anymore.
 
-    @param skip_received_gtid_set_recovery When true, skips the received GTID
-                                           set recovery.
+    @param skip_received_gtid_set_and_relaylog_recovery When true, skips the
+    received GTID set and relay log recovery.
 
     @retval 0 Success.
     @retval 1 Error.
   */
-  int rli_init_info(bool skip_received_gtid_set_recovery = false);
+  int rli_init_info(bool skip_received_gtid_set_and_relaylog_recovery = false);
   void end_info();
 
   /** No flush options given to relay log flush */
@@ -1602,26 +1648,9 @@ class Relay_log_info : public Rpl_info {
   inline void set_event_relay_log_name(const char *log_file_name) {
     strmake(event_relay_log_name, log_file_name,
             sizeof(event_relay_log_name) - 1);
-    set_event_relay_log_number(relay_log_name_to_number(log_file_name));
     notify_relay_log_change();
   }
 
-  uint get_event_relay_log_number() { return event_relay_log_number; }
-  void set_event_relay_log_number(uint number) {
-    event_relay_log_number = number;
-  }
-
-  /**
-    Given the extension number of the relay log, gets the full
-    relay log path. Currently used in Slave_worker::retry_transaction()
-
-    @param [in]   number      extension number of relay log
-    @param[in, out] name      The full path of the relay log (per-channel)
-                              to be read by the slave worker.
-  */
-  void relay_log_number_to_name(uint number, char name[FN_REFLEN + 1]);
-  uint relay_log_name_to_number(const char *name);
-
   void set_event_start_pos(my_off_t pos) { event_start_pos = pos; }
   my_off_t get_event_start_pos() { return event_start_pos; }
 
@@ -1648,7 +1677,7 @@ class Relay_log_info : public Rpl_info {
 
     This does not actually sleep; it only sets the state of this
     Relay_log_info object to delaying so that the correct state can be
-    reported by SHOW SLAVE STATUS and SHOW PROCESSLIST.
+    reported by SHOW REPLICA STATUS and SHOW PROCESSLIST.
 
     Requires rli->data_lock.
 
@@ -1798,7 +1827,7 @@ class Relay_log_info : public Rpl_info {
  private:
   /*
     Commit order manager to order commits made by its workers. In context of
-    Multi Source Replication each worker will be ordered by the coresponding
+    Multi Source Replication each worker will be ordered by the corresponding
     corrdinator's order manager.
    */
   Commit_order_manager *commit_order_mngr;
@@ -1809,12 +1838,12 @@ class Relay_log_info : public Rpl_info {
     commit time. Exceptionally, if a server in the replication chain does not
     support the commit timestamps in Gtid_log_event, the delay is applied per
     event and is based on the event timestamp.
-    This is set with CHANGE MASTER TO MASTER_DELAY=X.
+    This is set with CHANGE REPLICATION SOURCE TO SOURCE_DELAY=X.
 
     Guarded by data_lock.  Initialized by the client thread executing
-    START SLAVE.  Written by client threads executing CHANGE MASTER TO
-    MASTER_DELAY=X.  Read by SQL thread and by client threads
-    executing SHOW SLAVE STATUS.  Note: must not be written while the
+    START REPLICA.  Written by client threads executing CHANGE REPLICATION
+    SOURCE TO SOURCE_DELAY=X.  Read by SQL thread and by client threads
+    executing SHOW REPLICA STATUS.  Note: must not be written while the
     slave SQL thread is running, since the SQL thread reads it without
     a lock when executing flush_info().
   */
@@ -1823,43 +1852,47 @@ class Relay_log_info : public Rpl_info {
   /**
     During a delay, specifies the point in time when the delay ends.
 
-    This is used for the SQL_Remaining_Delay column in SHOW SLAVE STATUS.
+    This is used for the SQL_Remaining_Delay column in SHOW REPLICA STATUS.
 
     Guarded by data_lock. Written by the sql thread.  Read by client
-    threads executing SHOW SLAVE STATUS.
+    threads executing SHOW REPLICA STATUS.
   */
   time_t sql_delay_end;
 
   uint32 m_flags;
 
   /*
-    Before the MASTER_DELAY parameter was added (WL#344), relay_log.info
+    Historically, the number of entires in applier metadata was the number
+    of lines in applier metadata file. Since WL#13959, applier metadata can
+    be stored only in table, but the notion of number of line is still
+    preserved.
+    Before the SOURCE_DELAY parameter was added (WL#344), applier metadata
     had 4 lines. Now it has 5 lines.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_DELAY = 5;
+  static const int APPLIER_METADATA_LINES_WITH_DELAY = 5;
 
   /*
-    Before the WL#5599, relay_log.info had 5 lines. Now it has 6 lines.
+    Before the WL#5599, applier metadata had 5 lines. Now it has 6 lines.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_WORKERS = 6;
+  static const int APPLIER_METADATA_LINES_WITH_WORKERS = 6;
 
   /*
-    Before the Id was added (BUG#2334346), relay_log.info
+    Before the Id was added (BUG#2334346), applier metadata
     had 6 lines. Now it has 7 lines.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_ID = 7;
+  static const int APPLIER_METADATA_LINES_WITH_ID = 7;
 
   /*
-    Add a channel in the slave relay log info
+    Add a channel in the applier metadata
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_CHANNEL = 8;
+  static const int APPLIER_METADATA_LINES_WITH_CHANNEL = 8;
 
   /*
-    Represents line number in relay_log.info to save PRIVILEGE_CHECKS_USERNAME.
-    It is username part of PRIVILEGES_CHECKS_USER column in
-    performance_schema.replication_applier_configuration.
+    Represents entry id in applier metadata to save
+    PRIVILEGE_CHECKS_USERNAME. It is username part of PRIVILEGES_CHECKS_USER
+    column in performance_schema.replication_applier_configuration.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_PRIV_CHECKS_USERNAME = 9;
+  static const int APPLIER_METADATA_LINES_WITH_PRIV_CHECKS_USERNAME = 9;
 
   /*
     Maximum length of PRIVILEGE_CHECKS_USERNAME.
@@ -1867,11 +1900,11 @@ class Relay_log_info : public Rpl_info {
   static const int PRIV_CHECKS_USERNAME_LENGTH = 32;
 
   /*
-    Represents line number in relay_log.info to save PRIVILEGE_CHECKS_HOSTNAME.
-    It is hostname part of PRIVILEGES_CHECKS_USER column in
-    performance_schema.replication_applier_configuration.
+    Represents entry id in applier metadata to save
+    PRIVILEGE_CHECKS_HOSTNAME. It is hostname part of PRIVILEGES_CHECKS_USER
+    column in performance_schema.replication_applier_configuration.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_PRIV_CHECKS_HOSTNAME = 10;
+  static const int APPLIER_METADATA_LINES_WITH_PRIV_CHECKS_HOSTNAME = 10;
 
   /*
     Maximum length of PRIVILEGE_CHECKS_USERNAME.
@@ -1879,38 +1912,42 @@ class Relay_log_info : public Rpl_info {
   static const int PRIV_CHECKS_HOSTNAME_LENGTH = 255;
 
   /*
-    Represents line number in relay_log.info to save REQUIRE_ROW_FORMAT
+    Represents entry id in applier metadata to save REQUIRE_ROW_FORMAT
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_REQUIRE_ROW_FORMAT = 11;
+  static const int APPLIER_METADATA_LINES_WITH_REQUIRE_ROW_FORMAT = 11;
 
   /*
-    Represents line number in relay_log.info to save
+    Represents entry id in applier metadata to save
     REQUIRE_TABLE_PRIMARY_KEY_CHECK
   */
-  static const int
-      LINES_IN_RELAY_LOG_INFO_WITH_REQUIRE_TABLE_PRIMARY_KEY_CHECK = 12;
+  static const int APPLIER_METADATA_LINES_WITH_REQUIRE_TABLE_PRIMARY_KEY_CHECK =
+      12;
 
   /*
-    Represent line number in relay_log.info to save
+    Represent entry id in applier metadata to save
     ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_TYPE.
   */
   static const int
-      LINES_IN_RELAY_LOG_INFO_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_TYPE =
+      APPLIER_METADATA_LINES_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_TYPE =
           13;
 
   /*
-    Represent line number in relay_log.info to save
+    Represent entry id in applier metadata to save
     ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE.
   */
   static const int
-      LINES_IN_RELAY_LOG_INFO_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE =
+      APPLIER_METADATA_LINES_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE =
           14;
   /*
-    Total lines in relay_log.info.
+    Total lines in applier metadata.
     This has to be updated every time a member is added or removed.
+    Historically, the number of entires in applier metadata was the number
+    of lines in applier metadata file. Since WL#13959, applier metadata can
+    be stored only in table, but the notion of number of line is still
+    preserved.
   */
-  static const int MAXIMUM_LINES_IN_RELAY_LOG_INFO_FILE =
-      LINES_IN_RELAY_LOG_INFO_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE;
+  static const int MAXIMUM_APPLIER_METADATA_LINES =
+      APPLIER_METADATA_LINES_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE;
 
   bool read_info(Rpl_info_handler *from) override;
   bool write_info(Rpl_info_handler *to) override;
@@ -1967,7 +2004,7 @@ class Relay_log_info : public Rpl_info {
   */
   bool m_allow_drop_write_set;
 
-  /* The object stores and handles START SLAVE UNTIL option */
+  /* The object stores and handles START REPLICA UNTIL option */
   Until_option *until_option;
 
  public:
@@ -1994,7 +2031,7 @@ class Relay_log_info : public Rpl_info {
     which is not atomic DDL and has no XID assigned. Checked at commit
     time to decide whether it is safe to update slave info table
     within the same transaction as the write to binary log or this
-    should be deffered. The deffered scenario applies for not XIDed events
+    should be deferred. The deferred scenario applies for not XIDed events
     in which case such update might be lost on recovery.
   */
   bool ddl_not_atomic;
@@ -2031,17 +2068,21 @@ class Relay_log_info : public Rpl_info {
     return until_option != nullptr &&
            until_option->is_satisfied_after_dispatching_event();
   }
+  bool is_until_satisfied_all_transactions_read_from_relay_log() {
+    return until_option != nullptr &&
+           until_option->is_satisfied_all_transactions_read_from_relay_log();
+  }
   /**
-   Intialize until option object when starting slave.
+   Initialize until option object when starting slave.
 
    @param[in] thd The thread object of current session.
-   @param[in] master_param the parameters of START SLAVE.
+   @param[in] master_param the parameters of START REPLICA.
 
    @return int
      @retval 0      Succeeds to initialize until option object.
      @retval <> 0   A defined error number is return if any error happens.
    */
-  int init_until_option(THD *thd, const LEX_MASTER_INFO *master_param);
+  int init_until_option(THD *thd, const LEX_SOURCE_INFO *master_param);
 
   /**
     Detaches the engine ha_data from THD. The fact
@@ -2151,7 +2192,7 @@ bool is_mts_db_partitioned(Relay_log_info *rli);
   @return true  when the event is already committed transactional DDL
 */
 inline bool is_committed_ddl(Log_event *ev) {
-  return ev->get_type_code() == binary_log::QUERY_EVENT &&
+  return ev->get_type_code() == mysql::binlog::event::QUERY_EVENT &&
          /* has been already committed */
          static_cast<Query_log_event *>(ev)->has_ddl_committed;
 }
@@ -2179,7 +2220,7 @@ inline bool is_committed_ddl(Log_event *ev) {
   is always correct.
 
   @param  thd   a pointer to THD describing the transaction context
-  @return true  when a slave applier thread is set to commmit being processed
+  @return true  when a slave applier thread is set to commit being processed
                 DDL query-log-event, otherwise returns false.
 */
 inline bool is_atomic_ddl_commit_on_slave(THD *thd) {
@@ -2196,7 +2237,7 @@ inline bool is_atomic_ddl_commit_on_slave(THD *thd) {
              ? (rli->is_transactional() &&
                 /* has not yet committed */
                 (rli->current_event->get_type_code() ==
-                     binary_log::QUERY_EVENT &&
+                     mysql::binlog::event::QUERY_EVENT &&
                  !static_cast<Query_log_event *>(rli->current_event)
                       ->has_ddl_committed) &&
                 /* unless slave binlogger identified non-atomic */
@@ -2381,9 +2422,9 @@ class Applier_security_context_guard {
   /**
     Checks if the `PRIVILEGE_CHECKS_USER` user has access to the privilieges
     passed on by `extra_privileges` parameter as well as to the privileges
-    passed on at initilization time.
+    passed on at initialization time.
 
-    This particular method checks those privileges agains a given table and
+    This particular method checks those privileges against a given table and
     against that table's columns - the ones that are used or changed in the
     event.
 
@@ -2395,12 +2436,12 @@ class Applier_security_context_guard {
             false, otherwise.
    */
   bool has_access(
-      std::vector<std::tuple<ulong, TABLE const *, Rows_log_event *>>
+      std::vector<std::tuple<Access_bitmask, TABLE const *, Rows_log_event *>>
           &extra_privileges) const;
   /**
     Checks if the `PRIVILEGE_CHECKS_USER` user has access to the privilieges
     passed on by `extra_privileges` parameter as well as to the privileges
-    passed on at initilization time.
+    passed on at initialization time.
 
     @param extra_privileges set of privileges to check, additionally to those
                             passed on at initialization. It's a list of
@@ -2415,7 +2456,7 @@ class Applier_security_context_guard {
   /**
     Checks if the `PRIVILEGE_CHECKS_USER` user has access to the privilieges
     passed on by `extra_privileges` parameter as well as to the privileges
-    passed on at initilization time.
+    passed on at initialization time.
 
     @param extra_privileges set of privileges to check, additionally to those
                             passed on at initialization. It's a list of
@@ -2424,7 +2465,7 @@ class Applier_security_context_guard {
     @return true if the privileges are included in the security context and
             false, otherwise.
    */
-  bool has_access(std::initializer_list<ulong> extra_privileges) const;
+  bool has_access(std::initializer_list<Access_bitmask> extra_privileges) const;
 
   /**
     Returns the username for the user for which the security context was
