@@ -1,15 +1,16 @@
-/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2006, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,24 +26,26 @@
 
 #include <sys/types.h>
 
-#include "handler.h"  // ts_command_type
+#include <optional>
+
 #include "lex_string.h"
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
-#include "sql_cmd.h"  // Sql_cmd
+#include "sql/handler.h"      // ts_command_type
+#include "sql/sql_cmd_ddl.h"  // Sql_cmd_ddl
 
 class THD;
 
 /**
   Structure used by parser to store options for tablespace statements
-  and pass them on to Excution classes.
+  and pass them on to Execution classes.
  */
 struct Tablespace_options {
   ulonglong extent_size = 1024 * 1024;           // Default 1 MByte
   ulonglong undo_buffer_size = 8 * 1024 * 1024;  // Default 8 MByte
   ulonglong redo_buffer_size = 8 * 1024 * 1024;  // Default 8 MByte
   ulonglong initial_size = 128 * 1024 * 1024;    // Default 128 MByte
-  ulonglong autoextend_size = 0;                 // No autoextension as default
+  std::optional<ulonglong> autoextend_size;      // No autoextension as default
   ulonglong max_size = 0;         // Max size == initial size => no extension
   ulonglong file_block_size = 0;  // 0=default or must be a valid Page Size
   uint nodegroup_id = UNDEF_NODEGROUP;
@@ -50,6 +53,8 @@ struct Tablespace_options {
   LEX_STRING ts_comment = {nullptr, 0};  // FIXME: Rename to comment?
   LEX_CSTRING engine_name = {nullptr, 0};
   LEX_STRING encryption = {nullptr, 0};
+
+  LEX_CSTRING engine_attribute = NULL_CSTR;
 };
 
 /**
@@ -91,7 +96,7 @@ bool validate_tablespace_name(ts_command_type ts_cmd,
   Base class for tablespace execution classes including
   CREATE/ALTER/DROP TABLESPACE and LOGFILE GROUP commands.
  */
-class Sql_cmd_tablespace : public Sql_cmd /* purecov: inspected */
+class Sql_cmd_tablespace : public Sql_cmd_ddl /* purecov: inspected */
 {
  protected:
   const LEX_STRING m_tablespace_name;
@@ -110,7 +115,7 @@ class Sql_cmd_tablespace : public Sql_cmd /* purecov: inspected */
     Provide access to the command code enum value.
     @return command code enum value
    */
-  enum_sql_command sql_command_code() const override final;
+  enum_sql_command sql_command_code() const final;
   /**
     Return the Tablespace_options for this object.
    */
