@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,26 +34,28 @@
 
 #include "sql/sql_time.h"
 
+#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
 #include "decimal.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_macros.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
+#include "nulls.h"
+#include "sql-common/my_decimal.h"
 #include "sql/current_thd.h"
 #include "sql/derror.h"
 #include "sql/field.h"
-#include "sql/my_decimal.h"
 #include "sql/sql_class.h"  // THD, MODE_STRICT_ALL_TABLES, MODE_STRICT_TRANS_TABLES
 #include "sql/sql_const.h"
 #include "sql/system_variables.h"
 #include "sql/tztime.h"  // struct Time_zone
+#include "string_with_len.h"
 
 /**
   Name description of interval names used in statements.
@@ -86,6 +89,30 @@ const LEX_CSTRING interval_type_to_name[INTERVAL_LAST] = {
     {STRING_WITH_LEN("MINUTE_MICROSECOND")},
     {STRING_WITH_LEN("SECOND_MICROSECOND")}};
 
+/**
+  Generate flags to use when converting a string to a date or datetime value
+
+  @todo Consider whether to always accept zero date and zero in date.
+        (Add TIME_FUZZY_DATE).
+  Accept invalid dates when ALLOW_INVALID_DATES SQL mode is set.
+  Reject zero date if TIME_NO_ZERO_DATE is set.
+  Reject dates with zero as day or month when TIME_NO_ZERO_IN_DATE is set,
+  even if TIME_FUZZY_DATE is also set.
+  TIME_FRAC_TRUNCATE controls whether to round or truncate if the input
+  value has greater precision than the wanted result.
+
+  @param thd  Thread handle
+
+  @returns    Flags to use for calls to e.g my_str_to_datetime()
+*/
+my_time_flags_t DatetimeConversionFlags(const THD *thd) {
+  const sql_mode_t mode = thd->variables.sql_mode;
+  return (mode & MODE_INVALID_DATES ? TIME_INVALID_DATES : 0) |
+         (mode & MODE_NO_ZERO_DATE ? TIME_NO_ZERO_DATE : 0) |
+         (mode & MODE_NO_ZERO_IN_DATE ? TIME_NO_ZERO_IN_DATE : 0) |
+         (thd->is_fsp_truncate_mode() ? TIME_FRAC_TRUNCATE : 0);
+}
+
 /**
   Convert a string to 8-bit representation,
   for use in str_to_time/str_to_date/str_to_date.
@@ -126,7 +153,7 @@ bool str_to_time(const CHARSET_INFO *cs, const char *str, size_t length,
                  MYSQL_TIME *l_time, my_time_flags_t flags,
                  MYSQL_TIME_STATUS *status) {
   char cnv[MAX_TIME_FULL_WIDTH + 3];  // +3 for nanoseconds (for rounding)
-  if ((cs->state & MY_CS_NONASCII) != 0) {
+  if (!my_charset_is_ascii_based(cs)) {
     length = to_ascii(cs, str, length, cnv, sizeof(cnv));
     str = cnv;
   }
@@ -147,7 +174,7 @@ bool str_to_datetime(const CHARSET_INFO *cs, const char *str, size_t length,
                      MYSQL_TIME *l_time, my_time_flags_t flags,
                      MYSQL_TIME_STATUS *status) {
   char cnv[MAX_DATETIME_FULL_WIDTH + 3];  // +3 for nanoseconds (for rounding)
-  if ((cs->state & MY_CS_NONASCII) != 0) {
+  if (!my_charset_is_ascii_based(cs)) {
     length = to_ascii(cs, str, length, cnv, sizeof(cnv));
     str = cnv;
   }
@@ -182,8 +209,12 @@ bool str_to_datetime_with_warn(String *str, MYSQL_TIME *l_time,
                                      ErrConvString(str), l_time->time_type,
                                      NullS))
       return true;
+    if (ret_val) status.squelch_deprecation();
   }
-  return ret_val;
+  check_deprecated_datetime_format(current_thd, str->charset(), status);
+
+  if (ret_val) return true;
+  return convert_time_zone_displacement(thd->time_zone(), l_time);
 }
 
 /**
@@ -205,7 +236,8 @@ static bool lldiv_t_to_datetime(lldiv_t lld, MYSQL_TIME *ltime,
     if (!*warnings) /* Neither sets warnings in case of ZERO DATE */
       *warnings |= MYSQL_TIME_WARN_TRUNCATED;
     return true;
-  } else if (ltime->time_type == MYSQL_TIMESTAMP_DATE) {
+  }
+  if (ltime->time_type == MYSQL_TIMESTAMP_DATE) {
     /*
       Generate a warning in case of DATE with fractional part:
         20011231.1234 -> '2001-12-31'
@@ -221,6 +253,25 @@ static bool lldiv_t_to_datetime(lldiv_t lld, MYSQL_TIME *ltime,
   return false;
 }
 
+/**
+  Convert decimal value to datetime.
+
+  @param       decimal The value to convert from.
+  @param[out]  ltime   The variable to convert to.
+  @param       flags   Conversion flags.
+
+  @returns false on success, true if not convertible to datetime.
+*/
+bool decimal_to_datetime(const my_decimal *decimal, MYSQL_TIME *ltime,
+                         my_time_flags_t flags) {
+  lldiv_t lld;
+  if (my_decimal2lldiv_t(0, decimal, &lld)) {
+    return true;
+  }
+  int warnings = 0;
+  return lldiv_t_to_datetime(lld, ltime, flags, &warnings);
+}
+
 /**
   Convert decimal value to datetime value with a warning.
   @param       decimal The value to convert from.
@@ -253,6 +304,24 @@ bool my_decimal_to_datetime_with_warn(const my_decimal *decimal,
   return rc;
 }
 
+/**
+  Convert double value to datetime.
+
+  @param       nr      The value to convert from.
+  @param[out]  ltime   The variable to convert to.
+  @param       flags   Conversion flags.
+
+  @returns false on success, true if not convertible to datetime.
+*/
+bool double_to_datetime(double nr, MYSQL_TIME *ltime, my_time_flags_t flags) {
+  lldiv_t lld;
+  if (double2lldiv_t(nr, &lld)) {
+    return true;
+  }
+  int warnings = 0;
+  return lldiv_t_to_datetime(lld, ltime, flags, &warnings);
+}
+
 /**
   Convert double value to datetime value with a warning.
   @param       nr      The value to convert from.
@@ -288,7 +357,7 @@ bool my_double_to_datetime_with_warn(double nr, MYSQL_TIME *ltime,
   Convert longlong value to datetime value with a warning.
   @param       nr      The value to convert from.
   @param[out]  ltime   The variable to convert to.
-  @param       flags
+  @param       flags   Conversion flags
 
   @return False on success, true on error.
 */
@@ -328,6 +397,24 @@ static bool lldiv_t_to_time(lldiv_t lld, MYSQL_TIME *ltime, int *warnings) {
                                           current_thd->is_fsp_truncate_mode());
 }
 
+/**
+  Convert decimal number to TIME
+
+  @param      decimal  The number to convert from.
+  @param[out] ltime          The variable to convert to.
+
+  @returns false on success, true if not convertible to time.
+*/
+bool decimal_to_time(const my_decimal *decimal, MYSQL_TIME *ltime) {
+  lldiv_t lld;
+  int warnings = 0;
+
+  if (my_decimal2lldiv_t(0, decimal, &lld)) {
+    return true;
+  }
+  return lldiv_t_to_time(lld, ltime, &warnings);
+}
+
 /**
   Convert decimal number to TIME
   @param      decimal  The number to convert from.
@@ -357,6 +444,23 @@ bool my_decimal_to_time_with_warn(const my_decimal *decimal,
   return rc;
 }
 
+/**
+  Convert double number to TIME
+
+  @param      nr      The number to convert from.
+  @param[out] ltime   The variable to convert to.
+
+  @returns false on success, true if not convertible to time.
+*/
+bool double_to_time(double nr, MYSQL_TIME *ltime) {
+  lldiv_t lld;
+  if (double2lldiv_t(nr, &lld) != E_DEC_OK) {
+    return true;
+  }
+  int warnings = 0;
+  return lldiv_t_to_time(lld, ltime, &warnings);
+}
+
 /**
   Convert double number to TIME
 
@@ -407,66 +511,37 @@ bool my_longlong_to_time_with_warn(longlong nr, MYSQL_TIME *ltime) {
 }
 
 /**
-  Convert a datetime from broken-down MYSQL_TIME representation
-  to corresponding TIMESTAMP value.
+  Converts a datetime in MYSQL_TIME representation to corresponding `struct
+  timeval` value.
 
-  @param  thd             - current thread
-  @param  t               - datetime in broken-down representation,
-  @param  in_dst_time_gap - pointer to bool which is set to true if t represents
-                            value which doesn't exists (falls into the spring
-                            time-gap) or to false otherwise.
-  @return
-  @retval  Number seconds in UTC since start of Unix Epoch corresponding to t.
-  @retval  0 - t contains datetime value which is out of TIMESTAMP range.
-*/
-my_time_t TIME_to_timestamp(THD *thd, const MYSQL_TIME *t,
-                            bool *in_dst_time_gap) {
-  my_time_t timestamp;
-
-  *in_dst_time_gap = 0;
-
-  timestamp = thd->time_zone()->TIME_to_gmt_sec(t, in_dst_time_gap);
-  if (timestamp) {
-    return timestamp;
-  }
-
-  /* If we are here we have range error. */
-  return (0);
-}
-
-/**
-  Convert a datetime MYSQL_TIME representation
-  to corresponding "struct timeval" value.
-
-  ltime must previously be checked for TIME_NO_ZERO_IN_DATE.
+  `ltime` must be previously checked for `TIME_NO_ZERO_IN_DATE`.
   Things like '0000-01-01', '2000-00-01', '2000-01-00' are not allowed
   and asserted.
 
   Things like '0000-00-00 10:30:30' or '0000-00-00 00:00:00.123456'
   (i.e. empty date with non-empty time) return error.
 
-  Zero datetime '0000-00-00 00:00:00.000000'
-  is allowed and is mapper to {tv_sec=0, tv_usec=0}.
+  Zero datetime '0000-00-00 00:00:00.000000' is allowed and is mapped to
+  {tv_sec=0, tv_usec=0}.
 
-  Note: In case of error, tm value is not initialized.
+  @note In case of error, tm value is not initialized.
 
-  Note: "warnings" is not initialized to zero,
-  so new warnings are added to the old ones.
-  Caller must make sure to initialize "warnings".
+  @note `warnings` is not initialized to zero, so new warnings are added to the
+  old ones. The caller must make sure to initialize `warnings`.
 
-  @param[in]  thd       current thd
-  @param[in]  ltime     datetime value
-  @param[out] tm        timeval value
-  @param[out] warnings  pointer to warnings vector
+  @param[in]  ltime    Datetime value
+  @param[in]  tz       Time zone to convert to.
+  @param[out] tm       Timeval value
+  @param[out] warnings Pointer to warnings.
 
   @return False on success, true on error.
 */
-bool datetime_with_no_zero_in_date_to_timeval(THD *thd, const MYSQL_TIME *ltime,
-                                              struct timeval *tm,
-                                              int *warnings) {
+bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
+                                              const Time_zone &tz,
+                                              my_timeval *tm, int *warnings) {
   if (!ltime->month) /* Zero date */
   {
-    DBUG_ASSERT(!ltime->year && !ltime->day);
+    assert(!ltime->year && !ltime->day);
     if (non_zero_time(*ltime)) {
       /*
         Return error for zero date with non-zero time, e.g.:
@@ -475,19 +550,20 @@ bool datetime_with_no_zero_in_date_to_timeval(THD *thd, const MYSQL_TIME *ltime,
       *warnings |= MYSQL_TIME_WARN_TRUNCATED;
       return true;
     }
-    tm->tv_sec = tm->tv_usec = 0;  // '0000-00-00 00:00:00.000000'
+    tm->m_tv_sec = tm->m_tv_usec = 0;  // '0000-00-00 00:00:00.000000'
     return false;
   }
 
-  bool in_dst_time_gap;
-  if (!(tm->tv_sec = TIME_to_timestamp(thd, ltime, &in_dst_time_gap))) {
+  bool is_in_dst_time_gap = false;
+  if (!(tm->m_tv_sec = tz.TIME_to_gmt_sec(ltime, &is_in_dst_time_gap))) {
     /*
       Date was outside of the supported timestamp range.
       For example: '3001-01-01 00:00:00' or '1000-01-01 00:00:00'
     */
     *warnings |= MYSQL_TIME_WARN_OUT_OF_RANGE;
     return true;
-  } else if (in_dst_time_gap) {
+  }
+  if (is_in_dst_time_gap) {
     /*
       Set MYSQL_TIME_WARN_INVALID_TIMESTAMP warning to indicate
       that date was fine but pointed to winter/summer time switch gap.
@@ -496,7 +572,7 @@ bool datetime_with_no_zero_in_date_to_timeval(THD *thd, const MYSQL_TIME *ltime,
     */
     *warnings |= MYSQL_TIME_WARN_INVALID_TIMESTAMP;
   }
-  tm->tv_usec = ltime->second_part;
+  tm->m_tv_usec = ltime->second_part;
   return false;
 }
 
@@ -519,18 +595,18 @@ bool datetime_with_no_zero_in_date_to_timeval(THD *thd, const MYSQL_TIME *ltime,
   so new warnings are added to the old ones.
   Caller must make sure to initialize "warnings".
 
-  @param[in]  thd       current thd
   @param[in]  ltime     datetime value
+  @param[in]  tz       The time zone.
   @param[out] tm        timeval value
   @param[out] warnings  pointer to warnings vector
 
   @return False on success, true on error.
 */
-bool datetime_to_timeval(THD *thd, const MYSQL_TIME *ltime, struct timeval *tm,
-                         int *warnings) {
+bool datetime_to_timeval(const MYSQL_TIME *ltime, const Time_zone &tz,
+                         my_timeval *tm, int *warnings) {
   return check_date(*ltime, non_zero_date(*ltime), TIME_NO_ZERO_IN_DATE,
                     warnings) ||
-         datetime_with_no_zero_in_date_to_timeval(thd, ltime, tm, warnings);
+         datetime_with_no_zero_in_date_to_timeval(ltime, tz, tm, warnings);
 }
 
 /**
@@ -544,17 +620,22 @@ bool datetime_to_timeval(THD *thd, const MYSQL_TIME *ltime, struct timeval *tm,
 bool str_to_time_with_warn(String *str, MYSQL_TIME *l_time) {
   MYSQL_TIME_STATUS status;
   my_time_flags_t flags = 0;
+  THD *thd = current_thd;
 
   if (current_thd->is_fsp_truncate_mode()) flags = TIME_FRAC_TRUNCATE;
 
   bool ret_val = propagate_datetime_overflow(
       current_thd, &status.warnings, str_to_time(str, l_time, flags, &status));
   if (ret_val || status.warnings) {
-    if (make_truncated_value_warning(current_thd, Sql_condition::SL_WARNING,
+    if (make_truncated_value_warning(thd, Sql_condition::SL_WARNING,
                                      ErrConvString(str), MYSQL_TIMESTAMP_TIME,
                                      NullS))
       return true;
   }
+  check_deprecated_datetime_format(current_thd, str->charset(), status);
+  if (!ret_val)
+    if (convert_time_zone_displacement(thd->time_zone(), l_time)) return true;
+
   return ret_val;
 }
 
@@ -562,8 +643,8 @@ bool str_to_time_with_warn(String *str, MYSQL_TIME *l_time) {
   Convert time to datetime.
 
   The time value is added to the current datetime value.
-  @param thd
-  @param [in] ltime    Time value to convert from.
+  @param thd            Thread context
+  @param [in] ltime     Time value to convert from.
   @param [out] ltime2   Datetime value to convert to.
 */
 void time_to_datetime(THD *thd, const MYSQL_TIME *ltime, MYSQL_TIME *ltime2) {
@@ -578,10 +659,8 @@ void time_to_datetime(THD *thd, const MYSQL_TIME *ltime, MYSQL_TIME *ltime2) {
    Return format string according format name.
    If name is unknown, result is NULL
 
-   @param format
-   @param type
-
-   @return False on success, true on error.
+   @returns format string according format name.
+   @retval NULL if name is unknown.
 */
 const char *get_date_time_format_str(const Known_date_time_format *format,
                                      enum_mysql_timestamp_type type) {
@@ -593,7 +672,7 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
     case MYSQL_TIMESTAMP_TIME:
       return format->time_format;
     default:
-      DBUG_ASSERT(0);  // Impossible
+      assert(0);  // Impossible
       return nullptr;
   }
 }
@@ -603,7 +682,7 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
    @page DEFAULT_TIME_FUNCS Functions to create default time/date/datetime
    strings
    @note
-    For the moment the Date_time_format argument is ignored becasue
+    For the moment the Date_time_format argument is ignored because
     MySQL doesn't support comparing of date/time/datetime strings that
     are not in arbutary order as dates are compared as strings in some
     context)
@@ -621,9 +700,10 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
   @param[out] str      String to convert to
   @param      dec      Number of fractional digits.
 */
-void make_time(const Date_time_format *format MY_ATTRIBUTE((unused)),
+void make_time(const Date_time_format *format [[maybe_unused]],
                const MYSQL_TIME *l_time, String *str, uint dec) {
-  uint length = static_cast<uint>(my_time_to_str(*l_time, str->ptr(), dec));
+  const uint length =
+      static_cast<uint>(my_time_to_str(*l_time, str->ptr(), dec));
   str->length(length);
   str->set_charset(&my_charset_numeric);
 }
@@ -634,9 +714,9 @@ void make_time(const Date_time_format *format MY_ATTRIBUTE((unused)),
   @param      l_time   DATE value
   @param[out] str      String to convert to
 */
-void make_date(const Date_time_format *format MY_ATTRIBUTE((unused)),
+void make_date(const Date_time_format *format [[maybe_unused]],
                const MYSQL_TIME *l_time, String *str) {
-  uint length = static_cast<uint>(my_date_to_str(*l_time, str->ptr()));
+  const uint length = static_cast<uint>(my_date_to_str(*l_time, str->ptr()));
   str->length(length);
   str->set_charset(&my_charset_numeric);
 }
@@ -648,9 +728,10 @@ void make_date(const Date_time_format *format MY_ATTRIBUTE((unused)),
   @param[out] str      String to convert to
   @param      dec      Number of fractional digits.
 */
-void make_datetime(const Date_time_format *format MY_ATTRIBUTE((unused)),
+void make_datetime(const Date_time_format *format [[maybe_unused]],
                    const MYSQL_TIME *l_time, String *str, uint dec) {
-  uint length = static_cast<uint>(my_datetime_to_str(*l_time, str->ptr(), dec));
+  const uint length =
+      static_cast<uint>(my_datetime_to_str(*l_time, str->ptr(), dec));
   str->length(length);
   str->set_charset(&my_charset_numeric);
 }
@@ -770,7 +851,7 @@ my_decimal *my_decimal_from_datetime_packed(my_decimal *dec,
       TIME_from_longlong_datetime_packed(&ltime, packed_value);
       return date2my_decimal(&ltime, dec);
     default:
-      DBUG_ASSERT(0);
+      assert(0);
       ulonglong2decimal(0, dec);
       return dec;
   }
@@ -808,7 +889,7 @@ ulonglong gmt_time_to_local_time(ulonglong gmt_time) {
 MYSQL_TIME my_time_set(uint y, uint m, uint d, uint h, uint mi, uint s,
                        unsigned long ms, bool negative,
                        enum_mysql_timestamp_type type) {
-  return {y, m, d, h, mi, s, ms, negative, type};
+  return {y, m, d, h, mi, s, ms, negative, type, 0};
 }
 
 uint actual_decimals(const MYSQL_TIME *ts) {
