@@ -1,15 +1,16 @@
-/*  Copyright (c) 2015, 2022, Oracle and/or its affiliates.
+/*  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License, version 2.0,
     as published by the Free Software Foundation.
 
-    This program is also distributed with certain software (including
+    This program is designed to work with certain software (including
     but not limited to OpenSSL) that is licensed under separate terms,
     as designated in a particular file or component or in included license
     documentation.  The authors of MySQL hereby grant you an additional
     permission to link the program and your derivative works with the
-    separately licensed software that they have included with MySQL.
+    separately licensed software that they have either included with
+    the program or referenced in the documentation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -92,7 +93,7 @@ int srv_session_info_set_client_port(Srv_session *session, uint16_t port) {
   @param session  Session
 */
 LEX_CSTRING srv_session_info_get_current_db(Srv_session *session) {
-  static LEX_CSTRING empty = {nullptr, 0};
+  static const LEX_CSTRING empty = {nullptr, 0};
   return VALID_SESSION(session) ? session->get_current_database() : empty;
 }
 
