@@ -3,7 +3,7 @@
      -Brian Aker
 
   This file was modified by Oracle on 02-08-2016.
-  Modifications Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+  Modifications Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 */
 
 /* zlib.h -- interface of the 'zlib' general purpose compression library
@@ -195,7 +195,7 @@ extern "C" {
 #define Z_DEFLATED 8
 /* The deflate compression method (the only one supported in this version) */
 
-#define Z_NULL 0 /* for initializing zalloc, zfree, opaque */
+#define MY_Z_NULL 0 /* for initializing zalloc, zfree, opaque */
 #define AZ_BUFSIZE_READ 32768
 #define AZ_BUFSIZE_WRITE 16384
 
