@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1994, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -32,6 +33,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include <sys/types.h>
 #include <new>
+#include <type_traits>
 
 #include "data0data.h"
 #include "ha_prototypes.h"
@@ -62,10 +64,10 @@ bool dtuple_coll_eq(const dtuple_t *tuple1, const dtuple_t *tuple2) {
   ulint i;
   int cmp;
 
-  ut_ad(tuple1 != NULL);
-  ut_ad(tuple2 != NULL);
-  ut_ad(tuple1->magic_n == DATA_TUPLE_MAGIC_N);
-  ut_ad(tuple2->magic_n == DATA_TUPLE_MAGIC_N);
+  ut_ad(tuple1 != nullptr);
+  ut_ad(tuple2 != nullptr);
+  ut_ad(tuple1->magic_n == dtuple_t::MAGIC_N);
+  ut_ad(tuple2->magic_n == dtuple_t::MAGIC_N);
   ut_ad(dtuple_check_typed(tuple1));
   ut_ad(dtuple_check_typed(tuple2));
 
@@ -146,7 +148,7 @@ static bool dtuple_check_typed_no_assert(const dtuple_t *tuple) {
 bool dfield_check_typed(const dfield_t *field) {
   if (dfield_get_type(field)->mtype > DATA_MTYPE_CURRENT_MAX ||
       dfield_get_type(field)->mtype < DATA_MTYPE_CURRENT_MIN) {
-    ib::fatal(ER_IB_MSG_158)
+    ib::fatal(UT_LOCATION_HERE, ER_IB_MSG_158)
         << "Data field type " << dfield_get_type(field)->mtype << ", len "
         << dfield_get_len(field);
   }
@@ -168,7 +170,7 @@ bool dtuple_check_typed(const dtuple_t *tuple) {
 }
 
 bool dtuple_validate(const dtuple_t *tuple) {
-  ut_ad(tuple->magic_n == DATA_TUPLE_MAGIC_N);
+  ut_ad(tuple->magic_n == dtuple_t::MAGIC_N);
 
   auto n_fields = dtuple_get_n_fields(tuple);
 
@@ -180,7 +182,7 @@ bool dtuple_validate(const dtuple_t *tuple) {
     auto len = dfield_get_len(field);
 
     if (!dfield_is_null(field)) {
-      const byte *data;
+      const byte *data [[maybe_unused]];
 
       data = static_cast<const byte *>(dfield_get_data(field));
 #ifndef UNIV_DEBUG_VALGRIND
@@ -330,7 +332,7 @@ void dfield_print_also_hex(const dfield_t *dfield) {
       }
 
       data = static_cast<byte *>(dfield_get_data(dfield));
-      /* fall through */
+      [[fallthrough]];
 
     case DATA_BINARY:
     default:
@@ -353,7 +355,7 @@ void dfield_print_also_hex(const dfield_t *dfield) {
 static void dfield_print_raw(FILE *f, const dfield_t *dfield) {
   ulint len = dfield_get_len(dfield);
   if (!dfield_is_null(dfield)) {
-    ulint print_len = ut_min(len, static_cast<ulint>(1000));
+    ulint print_len = std::min(len, static_cast<ulint>(1000));
     ut_print_buf(f, dfield_get_data(dfield), print_len);
     if (len != print_len) {
       fprintf(f, "(total %lu bytes%s)", (ulong)len,
@@ -419,7 +421,7 @@ void dtuple_print(std::ostream &o, const dtuple_t *tuple) {
 }
 
 big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
-                                  dtuple_t *entry, ulint *n_ext) {
+                                  dtuple_t *entry) {
   DBUG_TRACE;
 
   mem_heap_t *heap;
@@ -427,12 +429,12 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
   dfield_t *dfield;
   dict_field_t *ifield;
   ulint size;
-  ulint n_fields;
+  ulint n_fields [[maybe_unused]];
   ulint local_len;
   ulint local_prefix_len;
 
   if (!index->is_clustered()) {
-    return NULL;
+    return nullptr;
   }
 
   if (!dict_table_has_atomic_blobs(index->table)) {
@@ -445,7 +447,7 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
 
   ut_a(dtuple_check_typed_no_assert(entry));
 
-  size = rec_get_converted_size(index, entry, *n_ext);
+  size = rec_get_converted_size(index, entry);
 
   if (size > 1000000000) {
     ib::warn(ER_IB_MSG_159) << "Tuple size is very big: " << size;
@@ -455,7 +457,8 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
   }
 
   heap = mem_heap_create(
-      size + dtuple_get_n_fields(entry) * sizeof(big_rec_field_t) + 1000);
+      size + dtuple_get_n_fields(entry) * sizeof(big_rec_field_t) + 1000,
+      UT_LOCATION_HERE);
 
   vector = big_rec_t::alloc(heap, dtuple_get_n_fields(entry));
 
@@ -465,14 +468,13 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
 
   n_fields = 0;
 
-  while (page_zip_rec_needs_ext(rec_get_converted_size(index, entry, *n_ext),
-                                dict_table_is_comp(index->table),
-                                dict_index_get_n_fields(index),
-                                dict_table_page_size(index->table))) {
+  while (page_zip_rec_needs_ext(
+      rec_get_converted_size(index, entry), dict_table_is_comp(index->table),
+      dict_index_get_n_fields(index), dict_table_page_size(index->table))) {
     byte *data;
     ulint longest = 0;
     ulint longest_i = ULINT_MAX;
-    upd_field_t *uf = nullptr;
+    const upd_field_t *uf = nullptr;
 
     for (ulint i = dict_index_get_n_unique_in_tree(index);
          i < dtuple_get_n_fields(entry); i++) {
@@ -481,6 +483,8 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
       dfield = dtuple_get_nth_field(entry, i);
       ifield = index->get_field(i);
 
+      ut_ad(dfield_get_len(dfield) != UNIV_SQL_INSTANT_DROP_COL);
+
       /* Skip fixed-length, NULL, externally stored,
       or short columns */
 
@@ -523,7 +527,7 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
 
       mem_heap_free(heap);
 
-      return NULL;
+      return nullptr;
     }
 
     /* Move data from field longest_i to big rec vector.
@@ -565,19 +569,18 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
     }
 
 #if 0
-		/* The following would fail the Valgrind checks in
-		page_cur_insert_rec_low() and page_cur_insert_rec_zip().
-		The BLOB pointers in the record will be initialized after
-		the record and the BLOBs have been written. */
-		UNIV_MEM_ALLOC(data + local_prefix_len,
-			       BTR_EXTERN_FIELD_REF_SIZE);
+                /* The following would fail the Valgrind checks in
+                page_cur_insert_rec_low() and page_cur_insert_rec_zip().
+                The BLOB pointers in the record will be initialized after
+                the record and the BLOBs have been written. */
+                UNIV_MEM_ALLOC(data + local_prefix_len,
+                               BTR_EXTERN_FIELD_REF_SIZE);
 #endif
 
     dfield_set_data(dfield, data, local_len);
     dfield_set_ext(dfield);
 
     n_fields++;
-    (*n_ext)++;
     ut_ad(n_fields < dtuple_get_n_fields(entry));
 
     if (upd && !upd->is_modified(longest_i)) {
@@ -585,9 +588,11 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
 
       upd_field_t upd_field;
       upd_field.field_no = longest_i;
+      IF_DEBUG(upd_field.field_phy_pos =
+                   index->get_field(longest_i)->col->get_col_phy_pos();)
       upd_field.orig_len = 0;
-      upd_field.exp = NULL;
-      upd_field.old_v_val = NULL;
+      upd_field.exp = nullptr;
+      upd_field.old_v_val = nullptr;
       upd_field.ext_in_old = dfield_is_ext(dfield);
       dfield_copy(&upd_field.new_val, dfield->clone(upd->heap));
       upd->append(upd_field);
@@ -601,7 +606,8 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
     if (upd == nullptr) {
       big_rec.ext_in_old = false;
     } else {
-      upd_field_t *uf = upd->get_field_by_field_no(longest_i, index);
+      const upd_field_t *const uf =
+          upd->get_field_by_field_no(longest_i, index);
       ut_ad(uf != nullptr);
       big_rec.ext_in_old = uf->ext_in_old;
     }
@@ -652,7 +658,7 @@ big_rec_t *big_rec_t::alloc(mem_heap_t *heap, ulint n_fld) {
       mem_heap_alloc(heap, n_fld * sizeof(big_rec_field_t)));
 
   rec->n_fields = 0;
-  return (rec);
+  return rec;
 }
 
 dfield_t *dfield_t::clone(mem_heap_t *heap) {
@@ -669,16 +675,16 @@ dfield_t *dfield_t::clone(mem_heap_t *heap) {
     obj->data = obj + 1;
     memcpy(obj->data, data, len);
   } else {
-    obj->data = 0;
+    obj->data = nullptr;
   }
 
-  return (obj);
+  return obj;
 }
 
 byte *dfield_t::blobref() const {
   ut_ad(ext);
 
-  return (static_cast<byte *>(data) + len - BTR_EXTERN_FIELD_REF_SIZE);
+  return static_cast<byte *>(data) + len - BTR_EXTERN_FIELD_REF_SIZE;
 }
 
 uint32_t dfield_t::lob_version() const {
@@ -773,6 +779,32 @@ std::ostream &big_rec_t::print(std::ostream &out) const {
   out << "]";
   return (out);
 }
+
+void dtuple_t::validate() const {
+  for (uint16_t i = 0; i < n_fields; i++) {
+    const dfield_t &field = fields[i];
+    /* All fields in the fields vector represent actual fields in the
+    record, so their DATA_VIRTUAL flag is cleared regardless of
+    whether they are defined for virtual columns. */
+    ut_a_eq((field.type.prtype & DATA_VIRTUAL), 0U);
+  }
+  for (uint16_t i = 0; i < n_v_fields; i++) {
+    const dfield_t &field = v_fields[i];
+    /* All fields in the v_fields vector represent fields not present
+    in the record, and as such have their DATA_VIRTUAL flag set. */
+    ut_a_eq((field.type.prtype & DATA_VIRTUAL), DATA_VIRTUAL);
+  }
+}
+
+void dtuple_t::validate_for_index(const dict_index_t *index) const {
+  validate();
+  if (!index->is_clustered()) {
+    /* Only tuples for clustered index may have virtual fields.
+    Secondary index fields based on virtual columns are not
+    considered virtual. */
+    ut_a_eq(n_v_fields, 0);
+  }
+}
 #endif /* UNIV_DEBUG */
 
 trx_id_t dtuple_t::get_trx_id() const {
@@ -790,9 +822,8 @@ trx_id_t dtuple_t::get_trx_id() const {
 }
 
 void dtuple_t::ignore_trailing_default(const dict_index_t *index) {
-  if (!index->has_instant_cols()) {
-    return;
-  }
+  ut_a(index->has_instant_cols());
+  ut_a(!index->has_row_versions());
 
   /* It's necessary to check all the fields that could be default.
   If it's from normal update, it should be OK to keep original
@@ -802,6 +833,11 @@ void dtuple_t::ignore_trailing_default(const dict_index_t *index) {
   it has to check all possible default values. */
   for (; n_fields > index->get_instant_fields(); --n_fields) {
     const dict_col_t *col = index->get_field(n_fields - 1)->col;
+
+    /* We shall never come here if INSTANT ADD/DROP is done in a version. */
+    ut_a(!col->is_instant_added());
+    ut_a(!col->is_instant_dropped());
+
     const dfield_t *dfield = dtuple_get_nth_field(this, n_fields - 1);
     ulint len = dfield_get_len(dfield);
 
@@ -826,7 +862,7 @@ bool is_multi_value_clust_and_sec_equal(const byte *clust_field,
     return (false);
   }
 
-  ut_ad(clust_len == 0);
+  ut_ad(clust_len == UNIV_MULTI_VALUE_ARRAY_MARKER);
 
   const multi_value_data *multi_value =
       reinterpret_cast<const multi_value_data *>(clust_field);
@@ -852,6 +888,9 @@ bool multi_value_data::has(ulint mtype, ulint prtype, const byte *data,
   return (false);
 }
 
+void dtuple_t::set_min_rec_flag() { info_bits |= REC_INFO_MIN_REC_FLAG; }
+void dtuple_t::unset_min_rec_flag() { info_bits &= ~REC_INFO_MIN_REC_FLAG; }
+
 #endif /* !UNIV_HOTBACKUP */
 
 void multi_value_data::alloc(uint32_t num, bool alc_bitset, mem_heap_t *heap) {
@@ -874,12 +913,21 @@ void multi_value_data::alloc(uint32_t num, bool alc_bitset, mem_heap_t *heap) {
 void multi_value_data::alloc_bitset(mem_heap_t *heap, uint32_t size) {
   ut_ad(bitset == nullptr);
 
-  bitset = static_cast<Bitset *>(mem_heap_zalloc(heap, sizeof(Bitset)));
-  uint32_t alloc_size = (size == 0 ? num_v : size);
-  byte *bitmap =
-      static_cast<byte *>(mem_heap_zalloc(heap, UT_BITS_IN_BYTES(alloc_size)));
-  bitset->init(bitmap, UT_BITS_IN_BYTES(alloc_size));
+  bitset = static_cast<Bitset<> *>(mem_heap_zalloc(heap, sizeof(Bitset<>)));
+  /*TODO: why is it num_v? Shouldn't it be num_alc? Looks like copy_low assumes
+  that this->bitset points to a bitmap which is at least as large as other's
+  multi_value->bitset->size(), as it is copying bytes from it, yet the only
+  assert protection is that num_alc is larger than that. So, what's preventing
+  num_alc > multi_value->bitset->size() > num_v = bitmap_bytes? */
+  uint32_t alloc_size = UT_BITS_IN_BYTES(size == 0 ? num_v : size);
+  byte *bitmap = static_cast<byte *>(mem_heap_zalloc(heap, alloc_size));
+  ut_d(bitmap_bytes = alloc_size);
+  new (bitset) Bitset(bitmap, alloc_size);
   bitset->set();
+  static_assert(
+      std::is_trivially_destructible_v<Bitset<>>,
+      "Bitset<> must be trivially destructible in order to be able to "
+      "(de)allocate it via mem_heap_t");
 }
 
 uint32_t Multi_value_logger::get_log_len(bool precise) const {
@@ -944,12 +992,17 @@ byte *Multi_value_logger::log(byte **ptr) {
     ut_memcpy(*ptr, m_mv_data->datap[i], m_mv_data->data_len[i]);
     *ptr += m_mv_data->data_len[i];
   }
-
+  /* This fragment is optional, which will be detected by reader by comparing
+  the amount of data read, to the length we will write to *old_ptr later. */
   if (m_mv_data->bitset != nullptr) {
     /* Always just write out the bitset of enough size for all data,
     rather than the size of bitset. */
     uint32_t bitset_len = UT_BITS_IN_BYTES(m_mv_data->num_v);
-    ut_memcpy(*ptr, m_mv_data->bitset->bitset(), bitset_len);
+    /* make sure we will not read past the buffer */
+    ut_ad_le(bitset_len, m_mv_data->bitmap_bytes);
+    /* make sure we will not miss actual data */
+    ut_ad_le(m_mv_data->bitset->size_bytes(), bitset_len);
+    ut_memcpy(*ptr, m_mv_data->bitset->data(), bitset_len);
     *ptr += bitset_len;
   }
 
@@ -1009,7 +1062,10 @@ const byte *Multi_value_logger::read(const byte *ptr, dfield_t *field,
 
   if (ptr < old_ptr + total_len) {
     multi_val->alloc_bitset(heap);
-    multi_val->bitset->copy(ptr, UT_BITS_IN_BYTES(num));
+    ut_ad_eq(num, multi_val->num_v);
+    ut_ad_eq(UT_BITS_IN_BYTES(num), multi_val->bitmap_bytes);
+    ut_a_eq(UT_BITS_IN_BYTES(num), multi_val->bitset->size_bytes());
+    multi_val->bitset->copy_from(ptr);
     ptr += UT_BITS_IN_BYTES(num);
   }
 
@@ -1043,3 +1099,11 @@ uint32_t Multi_value_logger::get_keys_capacity(uint32_t log_size,
 
   return (keys_length);
 }
+
+dtuple_t *dtuple_t::deep_copy(mem_heap_t *heap) const {
+  dtuple_t *copy = dtuple_copy(this, heap);
+  for (uint32_t i = 0; i < n_fields; ++i) {
+    dfield_dup(&copy->fields[i], heap);
+  }
+  return copy;
+}
