@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2022, Oracle and/or its affiliates.
+Copyright (c) 1995, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -192,8 +193,10 @@ then writes the page to the datafile.
 
 /** Obtain the encrypted frame and store it in bpage->m_io_frame
 @param[in,out]  bpage  the buffer page containing the unencrypted frame.
+@param[in,out]  type   i/o operation type.
 @return the memory block containing the compressed + encrypted frame. */
-[[nodiscard]] file::Block *get_encrypted_frame(buf_page_t *bpage) noexcept;
+[[nodiscard]] file::Block *get_encrypted_frame(buf_page_t *bpage,
+                                               IORequest &type) noexcept;
 
 /** Updates the double write buffer when a write request is completed.
 @param[in] bpage               Block that has just been writtent to disk.
@@ -224,7 +227,8 @@ namespace v1 {
 
 namespace dblwr {
 
-/** Number of pages per doublewrite thread/segment */
+/** Number of pages per doublewrite thread/segment of the dblwr file.  Refer
+to the system variable --innodb-doublewrite-pages for more details. */
 extern ulong n_pages;
 
 const uint32_t REDUCED_BATCH_PAGE_SIZE = 8192;
@@ -406,8 +410,8 @@ bool Mode::is_disabled_to_enabled(ulong new_value) {
 @param[in]     mode    dblwr ENUM */
 const char *to_string(ulong mode);
 
-/** Number of files to use for the double write buffer. It must be <= than
-the number of buffer pool instances. */
+/** Number of files to use for the double write buffer. Refer to the system
+variable --innodb-doublewrite-files for more details. */
 extern ulong n_files;
 
 /** Maximum number of pages to write in one batch. */
@@ -439,9 +443,8 @@ void create(Pages *&pages) noexcept;
 /** Restore pages from the double write buffer to the tablespace.
 @param[in,out]  pages  Pages from the doublewrite buffer
 @param[in]   space  Tablespace pages to restore, if set to nullptr then try and
-                    restore all.
-@return DB_SUCCESS on success, error code on failure. */
-dberr_t recover(Pages *pages, fil_space_t *space) noexcept;
+                    restore all. */
+void recover(Pages *pages, fil_space_t *space) noexcept;
 
 /** Find a doublewrite copy of a page.
 @param[in]      pages           Pages read from the doublewrite buffer
@@ -495,8 +498,8 @@ class DBLWR {
   @param[in]    space           Tablespace pages to restore,
                                   if set to nullptr then try
                                   and restore all. */
-  dberr_t recover(fil_space_t *space = nullptr) noexcept {
-    return dblwr::recv::recover(m_pages, space);
+  void recover(fil_space_t *space = nullptr) noexcept {
+    dblwr::recv::recover(m_pages, space);
   }
 
   /** Find a doublewrite copy of a page.
