@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2006, 2022, Oracle and/or its affiliates.
+Copyright (c) 2006, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -50,8 +51,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "ut0vec.h"
 #include "ut0wqueue.h"
 
-/** "NULL" value of a document id. */
-#define FTS_NULL_DOC_ID 0
+struct CHARSET_INFO;
 
 /** FTS hidden column that is used to map to and from the row */
 #define FTS_DOC_ID_COL_NAME "FTS_DOC_ID"
@@ -101,6 +101,9 @@ those defined in mysql file ft_global.h */
 #define FTS_INDEX_TABLE_IND_NAME "FTS_INDEX_TABLE_IND"
 #define FTS_COMMON_TABLE_IND_NAME "FTS_COMMON_TABLE_IND"
 
+/** "NULL" value of a document id. */
+constexpr doc_id_t FTS_NULL_DOC_ID = 0;
+
 /** The number of FTS index partitions for a fulltext index. */
 constexpr size_t FTS_NUM_AUX_INDEX = 6;
 
@@ -859,9 +862,11 @@ void fts_add_index(dict_index_t *index,  /*!< FTS index to be added */
 @param[in]      index           Index to be dropped
 @param[in]      trx             Transaction for the drop
 @param[in,out]  aux_vec         Aux table name vector
+@param[in]      adding_another  Another FTS index is to be added as part
+                                of the same transaction
 @return DB_SUCCESS or error number */
 dberr_t fts_drop_index(dict_table_t *table, dict_index_t *index, trx_t *trx,
-                       aux_name_vec_t *aux_vec);
+                       aux_name_vec_t *aux_vec, bool adding_another = false);
 
 /** Rename auxiliary tables for all fts index for a table
  @return DB_SUCCESS or error code */
