@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -57,13 +58,13 @@ will wait for its execution to complete if it is already called by another
 thread or will do nothing if the function has already completed its execution
 earlier.
 
-This mimics pthread_once(3), but unfortunatelly pthread_once(3) does not
+This mimics pthread_once(3), but unfortunately pthread_once(3) does not
 support passing arguments to the init_routine() function. We should use
 std::call_once() when we start compiling with C++11 enabled. */
 class os_once {
  public:
   /** Control variables' state type */
-  typedef ib_uint32_t state_t;
+  typedef uint32_t state_t;
 
   /** Not yet executed. */
   static const state_t NEVER_DONE = 0;
@@ -76,23 +77,24 @@ class os_once {
 
   /** Call a given function or wait its execution to complete if it is
   already called by another thread.
-  @param[in,out]	state		control variable
-  @param[in]	do_func		function to call
-  @param[in,out]	do_func_arg	an argument to pass to do_func(). */
-  static void do_or_wait_for_done(volatile state_t *state,
+  @param[in,out]        state           control variable
+  @param[in]    do_func         function to call
+  @param[in,out]        do_func_arg     an argument to pass to do_func(). */
+  static void do_or_wait_for_done(std::atomic<state_t> *state,
                                   void (*do_func)(void *), void *do_func_arg) {
-    /* Avoid calling os_compare_and_swap_uint32() in the most
-    common case. */
+    /* Avoid calling compare_exchange_strong() in the most common case. */
     if (*state == DONE) {
       return;
     }
 
-    if (os_compare_and_swap_uint32(state, NEVER_DONE, IN_PROGRESS)) {
+    state_t never_done = NEVER_DONE;
+    if (state->compare_exchange_strong(never_done, IN_PROGRESS)) {
       /* We are the first. Call the function. */
 
       do_func(do_func_arg);
 
-      const bool swapped = os_compare_and_swap_uint32(state, IN_PROGRESS, DONE);
+      state_t in_progress = IN_PROGRESS;
+      const bool swapped = state->compare_exchange_strong(in_progress, DONE);
 
       ut_a(swapped);
     } else {
@@ -109,7 +111,7 @@ class os_once {
           case IN_PROGRESS:
             break;
           case NEVER_DONE:
-            /* fall through */
+            [[fallthrough]];
           default:
             ut_error;
         }
