@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2020, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 1995, 2025, Oracle and/or its affiliates.
 Copyright (c) 2008, 2009, Google Inc.
 Copyright (c) 2009, Percona Inc.
 
@@ -21,12 +21,13 @@ This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -50,7 +51,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "buf0checksum.h"
 #include "fil0fil.h"
-#include "log0types.h"
 #include "mysql/psi/mysql_stage.h"
 #include "univ.i"
 
@@ -64,6 +64,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "trx0types.h"
 #include "ut0counter.h"
 
+#include <cstdint>
 #include <future>
 
 /* Global counters used inside InnoDB. */
@@ -168,6 +169,9 @@ struct Srv_threads {
   /** Error monitor thread. */
   IB_thread m_error_monitor;
 
+  /** Redo files governor thread. */
+  IB_thread m_log_files_governor;
+
   /** Redo checkpointer thread. */
   IB_thread m_log_checkpointer;
 
@@ -327,7 +331,7 @@ extern bool srv_buffer_pool_load_at_startup;
 extern bool srv_disable_sort_file_cache;
 
 /** Enable or disable writing of NULLs while extending a tablespace.
-If this is FALSE, then the server will just allocate the space without
+If this is false, then the server will just allocate the space without
 actually initializing it with NULLs. If the variable is true, the
 server will allocate and initialize the space by writing NULLs in it. */
 extern bool tbsp_extend_and_initialize;
@@ -347,7 +351,7 @@ extern FILE *srv_monitor_file;
 This mutex has a very low rank; threads reserving it should not
 acquire any further latches or sleep before releasing this one. */
 extern ib_mutex_t srv_misc_tmpfile_mutex;
-/* Temporary file for miscellanous diagnostic output */
+/* Temporary file for miscellaneous diagnostic output */
 extern FILE *srv_misc_tmpfile;
 #endif /* !UNIV_HOTBACKUP */
 
@@ -355,6 +359,9 @@ extern FILE *srv_misc_tmpfile;
 
 extern char *srv_data_home;
 
+/* Number of threads used for initializing rollback segments */
+extern uint32_t srv_rseg_init_threads;
+
 /** Number of pages per doublewrite thread/segment */
 extern ulong srv_dblwr_pages;
 
@@ -382,7 +389,7 @@ extern unsigned long long srv_online_max_size;
 /** Number of threads to use for parallel reads. */
 extern ulong srv_parallel_read_threads;
 
-/* If this flag is TRUE, then we will use the native aio of the
+/* If this flag is true, then we will use the native aio of the
 OS (provided we compiled Innobase with it in), otherwise we will
 use simulated aio we build below with threads.
 Currently we support native aio on windows and linux */
@@ -414,9 +421,6 @@ extern bool srv_undo_log_truncate;
 /** Enable or disable Encrypt of UNDO tablespace. */
 extern bool srv_undo_log_encrypt;
 
-/** Default size of UNDO tablespace while it is created new. */
-extern const page_no_t SRV_UNDO_TABLESPACE_SIZE_IN_PAGES;
-
 /** Maximum number of recently truncated undo tablespace IDs for
 the same undo number. */
 extern const size_t CONCURRENT_UNDO_TRUNCATE_LIMIT;
@@ -427,29 +431,29 @@ extern char *srv_log_group_home_dir;
 extern bool srv_redo_log_encrypt;
 
 /* Maximum number of redo files of a cloned DB. */
-#define SRV_N_LOG_FILES_CLONE_MAX 1000
+constexpr size_t SRV_N_LOG_FILES_CLONE_MAX = 1000;
 
-/** Maximum number of srv_n_log_files, or innodb_log_files_in_group */
-#define SRV_N_LOG_FILES_MAX 100
-extern ulong srv_n_log_files;
+/** Value of innodb_log_files_in_group. This is deprecated. */
+extern ulong srv_log_n_files;
+
+/** Value of innodb_log_file_size. Expressed in bytes. This is deprecated. */
+extern ulonglong srv_log_file_size;
+
+/** Value of innodb_redo_log_capacity. Expressed in bytes. Might be set
+during startup automatically when started in "dedicated server mode". */
+extern ulonglong srv_redo_log_capacity;
+
+/** Assumed value of innodb_redo_log_capacity's - value which is used.
+Expressed in bytes. Might be set during startup automatically when
+started in "dedicated server mode". Might also be set during startup
+when old sysvar (innodb_log_file_size or innodb_log_files_in_group)
+are configured and the new sysvar (innodb_redo_log_capacity) is not. */
+extern ulonglong srv_redo_log_capacity_used;
 
 #ifdef UNIV_DEBUG_DEDICATED
 extern ulong srv_debug_system_mem_size;
 #endif /* UNIV_DEBUG_DEDICATED */
 
-/** At startup, this is the current redo log file size.
-During startup, if this is different from srv_log_file_size_requested
-(innodb_log_file_size), the redo log will be rebuilt and this size
-will be initialized to srv_log_file_size_requested.
-When upgrading from a previous redo log format, this will be set to 0,
-and writing to the redo log is not allowed.
-
-During startup, this is in bytes, and later converted to pages. */
-extern ulonglong srv_log_file_size;
-
-/** The value of the startup parameter innodb_log_file_size. */
-extern ulonglong srv_log_file_size_requested;
-
 /** Space for log buffer, expressed in bytes. Note, that log buffer
 will use only the largest power of two, which is not greater than
 the assigned space. */
@@ -493,8 +497,8 @@ written up to given LSN, before we fallback to loop with sleeps.
 This is not used when user thread has to wait for log flushed to disk. */
 extern ulong srv_log_wait_for_write_spin_delay;
 
-/** Timeout used when waiting for redo write (microseconds). */
-extern ulong srv_log_wait_for_write_timeout;
+/** Timeout used when waiting for redo write. */
+std::chrono::microseconds get_srv_log_wait_for_write_timeout();
 
 /** Number of spin iterations, when spinning and waiting for log flushed. */
 extern ulong srv_log_wait_for_flush_spin_delay;
@@ -504,41 +508,41 @@ When flushing takes longer, user threads no longer spin when waiting for
 flushed redo. Expressed in microseconds. */
 extern ulong srv_log_wait_for_flush_spin_hwm;
 
-/** Timeout used when waiting for redo flush (microseconds). */
-extern ulong srv_log_wait_for_flush_timeout;
+/** Timeout used when waiting for redo flush. */
+std::chrono::microseconds get_srv_log_wait_for_flush_timeout();
 
 /** Number of spin iterations, for which log writer thread is waiting
 for new data to write or flush without sleeping. */
 extern ulong srv_log_writer_spin_delay;
 
 /** Initial timeout used to wait on writer_event. */
-extern ulong srv_log_writer_timeout;
+std::chrono::microseconds get_srv_log_writer_timeout();
 
-/** Number of milliseconds every which a periodical checkpoint is written
-by the log checkpointer thread (unless periodical checkpoints are disabled,
+/** Period every which a periodical checkpoint is written by the log
+checkpointer thread (unless periodical checkpoints are disabled,
 which is a case during initial phase of startup). */
-extern ulong srv_log_checkpoint_every;
+std::chrono::milliseconds get_srv_log_checkpoint_every();
 
 /** Number of spin iterations, for which log flusher thread is waiting
 for new data to flush, without sleeping. */
 extern ulong srv_log_flusher_spin_delay;
 
 /** Initial timeout used to wait on flusher_event. */
-extern ulong srv_log_flusher_timeout;
+std::chrono::microseconds get_srv_log_flusher_timeout();
 
 /** Number of spin iterations, for which log write notifier thread is waiting
 for advanced writeed_to_disk_lsn without sleeping. */
 extern ulong srv_log_write_notifier_spin_delay;
 
 /** Initial timeout used to wait on write_notifier_event. */
-extern ulong srv_log_write_notifier_timeout;
+std::chrono::microseconds get_srv_log_write_notifier_timeout();
 
 /** Number of spin iterations, for which log flush notifier thread is waiting
 for advanced flushed_to_disk_lsn without sleeping. */
 extern ulong srv_log_flush_notifier_spin_delay;
 
 /** Initial timeout used to wait on flush_notifier_event. */
-extern ulong srv_log_flush_notifier_timeout;
+std::chrono::microseconds get_srv_log_flush_notifier_timeout();
 
 /** Whether to generate and require checksums on the redo log pages. */
 extern bool srv_log_checksums;
@@ -554,12 +558,12 @@ extern bool srv_inject_too_many_concurrent_trxs;
 #endif /* UNIV_DEBUG */
 
 extern ulong srv_flush_log_at_trx_commit;
-extern uint srv_flush_log_at_timeout;
 extern ulong srv_log_write_ahead_size;
+std::chrono::seconds get_srv_flush_log_at_timeout();
 extern bool srv_adaptive_flushing;
 extern bool srv_flush_sync;
 
-/* If this flag is TRUE, then we will load the indexes' (and tables') metadata
+/* If this flag is true, then we will load the indexes' (and tables') metadata
 even if they are marked as "corrupted". Mostly it is for DBA to process
 corrupted index and table */
 extern bool srv_load_corrupted;
@@ -592,6 +596,8 @@ extern const ulong srv_buf_pool_instances_default;
 extern ulong srv_n_page_hash_locks;
 /** Whether to validate InnoDB tablespace paths on startup */
 extern bool srv_validate_tablespace_paths;
+/** Use fdatasync() instead of fsync(). */
+extern bool srv_use_fdatasync;
 /** Scan depth for LRU flush batch i.e.: number of blocks scanned*/
 extern ulong srv_LRU_scan_depth;
 /** Whether or not to flush neighbors of a block */
@@ -607,7 +613,6 @@ extern ulong srv_buf_pool_dump_pct;
 /** Lock table size in bytes */
 extern ulint srv_lock_table_size;
 
-extern ulint srv_n_file_io_threads;
 extern bool srv_random_read_ahead;
 extern ulong srv_read_ahead_threshold;
 extern ulong srv_n_read_io_threads;
@@ -625,23 +630,45 @@ extern ulong srv_io_capacity;
 
 /* We use this dummy default value at startup for max_io_capacity.
 The real value is set based on the value of io_capacity. */
-#define SRV_MAX_IO_CAPACITY_DUMMY_DEFAULT (~0UL)
-#define SRV_MAX_IO_CAPACITY_LIMIT (~0UL)
+constexpr uint32_t SRV_MAX_IO_CAPACITY_DUMMY_DEFAULT = UINT32_MAX;
+constexpr uint32_t SRV_MAX_IO_CAPACITY_LIMIT = UINT32_MAX;
 extern ulong srv_max_io_capacity;
 /* Returns the number of IO operations that is X percent of the
 capacity. PCT_IO(5) -> returns the number of IO operations that
 is 5% of the max where max is srv_io_capacity.  */
-#define PCT_IO(p) ((ulong)(srv_io_capacity * ((double)(p) / 100.0)))
+static inline ulong PCT_IO(ulong p) {
+  return (ulong)(srv_io_capacity * ((double)(p) / 100.0));
+}
 
 /** Maximum number of purge threads, including the purge coordinator */
-#define MAX_PURGE_THREADS 32
+constexpr uint32_t MAX_PURGE_THREADS = 32;
 
 /* The "innodb_stats_method" setting, decides how InnoDB is going
 to treat NULL value when collecting statistics. It is not defined
 as enum type because the configure option takes unsigned integer type. */
 extern ulong srv_innodb_stats_method;
 
-extern ulint srv_max_n_open_files;
+/** Returns current value of the "innodb_open_files" configuration variable. */
+long innobase_get_open_files_limit();
+/** Sets new value of the "innodb_open_files" configuration variable to present
+to users.
+@param[in] new_limit new limit to be set. */
+void innobase_set_open_files_limit(long new_limit);
+
+extern ulong srv_fatal_semaphore_wait_threshold;
+extern uint srv_flush_log_at_timeout;
+extern bool srv_btr_search_enabled;
+extern ulong srv_replication_delay;
+
+extern ulong srv_log_wait_for_write_timeout;
+extern ulong srv_log_wait_for_flush_timeout;
+extern ulong srv_log_writer_timeout;
+extern ulong srv_log_checkpoint_every;
+extern ulong srv_log_flusher_timeout;
+extern ulong srv_log_write_notifier_timeout;
+extern ulong srv_log_flush_notifier_timeout;
+
+extern uint buf_LRU_old_threshold;
 
 extern ulong srv_n_page_cleaners;
 
@@ -669,7 +696,7 @@ pool to data files, cleanly shutting down the redo log.
 If innodb_fast_shutdown=2, shutdown will effectively 'crash' InnoDB
 (but lose no committed transactions). */
 extern ulong srv_fast_shutdown;
-extern ibool srv_innodb_status;
+extern bool srv_innodb_status;
 
 extern unsigned long long srv_stats_transient_sample_pages;
 extern bool srv_stats_persistent;
@@ -683,16 +710,21 @@ extern double srv_max_buf_pool_modified_pct;
 extern ulong srv_max_purge_lag;
 extern ulong srv_max_purge_lag_delay;
 
-extern ulong srv_replication_delay;
+std::chrono::milliseconds get_srv_replication_delay();
 /*-------------------------------------------*/
 
 extern bool srv_print_innodb_monitor;
+/** In contrast to srv_print_innodb_monitor which is controlled by the user,
+this variable is controlled by InnoDB itself: if some module of InnoDB decides
+it would be good to print the monitoring information it increments this value,
+and decrements it when it no longer needs it. */
+extern std::atomic_uint32_t srv_innodb_needs_monitoring;
 extern bool srv_print_innodb_lock_monitor;
 
 extern ulong srv_n_spin_wait_rounds;
 extern ulong srv_n_free_tickets_to_enter;
 extern ulong srv_spin_wait_delay;
-extern ibool srv_priority_boost;
+extern bool srv_priority_boost;
 
 extern ulint srv_truncated_status_writes;
 
@@ -709,15 +741,16 @@ extern bool srv_purge_view_update_only_debug;
 extern bool srv_master_thread_disabled_debug;
 #endif /* UNIV_DEBUG */
 
-extern ulong srv_fatal_semaphore_wait_threshold;
-#define SRV_SEMAPHORE_WAIT_EXTENSION 7200
+std::chrono::seconds get_srv_fatal_semaphore_wait_threshold();
+extern std::atomic<int> srv_fatal_semaphore_wait_extend;
+
 extern ulint srv_dml_needed_delay;
 
 #ifdef UNIV_HOTBACKUP
 // MAHI: changed from 130 to 1 assuming the apply-log is single threaded
-#define SRV_MAX_N_IO_THREADS 1
-#else /* UNIV_HOTBACKUP */
-#define SRV_MAX_N_IO_THREADS 130
+constexpr uint32_t SRV_MAX_N_IO_THREADS = 1;
+#else  /* UNIV_HOTBACKUP */
+constexpr uint32_t SRV_MAX_N_IO_THREADS = 130;
 #endif /* UNIV_HOTBACKUP */
 
 /* Array of English strings describing the current state of an
@@ -756,20 +789,21 @@ extern srv_stats_t srv_stats;
 #ifdef UNIV_PFS_THREAD
 extern mysql_pfs_key_t log_archiver_thread_key;
 extern mysql_pfs_key_t page_archiver_thread_key;
+extern mysql_pfs_key_t buf_pool_create_thread_key;
 extern mysql_pfs_key_t buf_dump_thread_key;
 extern mysql_pfs_key_t buf_resize_thread_key;
 extern mysql_pfs_key_t clone_ddl_thread_key;
 extern mysql_pfs_key_t clone_gtid_thread_key;
+extern mysql_pfs_key_t ddl_thread_key;
 extern mysql_pfs_key_t dict_stats_thread_key;
 extern mysql_pfs_key_t fts_optimize_thread_key;
 extern mysql_pfs_key_t fts_parallel_merge_thread_key;
 extern mysql_pfs_key_t fts_parallel_tokenization_thread_key;
-extern mysql_pfs_key_t io_handler_thread_key;
 extern mysql_pfs_key_t io_ibuf_thread_key;
-extern mysql_pfs_key_t io_log_thread_key;
 extern mysql_pfs_key_t io_read_thread_key;
 extern mysql_pfs_key_t io_write_thread_key;
 extern mysql_pfs_key_t log_writer_thread_key;
+extern mysql_pfs_key_t log_files_governor_thread_key;
 extern mysql_pfs_key_t log_checkpointer_thread_key;
 extern mysql_pfs_key_t log_flusher_thread_key;
 extern mysql_pfs_key_t log_write_notifier_thread_key;
@@ -786,6 +820,9 @@ extern mysql_pfs_key_t srv_worker_thread_key;
 extern mysql_pfs_key_t trx_recovery_rollback_thread_key;
 extern mysql_pfs_key_t srv_ts_alter_encrypt_thread_key;
 extern mysql_pfs_key_t parallel_read_thread_key;
+extern mysql_pfs_key_t parallel_rseg_init_thread_key;
+extern mysql_pfs_key_t bulk_flusher_thread_key;
+extern mysql_pfs_key_t bulk_alloc_thread_key;
 #endif /* UNIV_PFS_THREAD */
 #endif /* !UNIV_HOTBACKUP */
 
@@ -839,25 +876,23 @@ extern PSI_stage_info srv_stage_clone_page_copy;
 /** Alternatives for the file flush option in Unix.
 @see innodb_flush_method_names */
 enum srv_unix_flush_t {
-  SRV_UNIX_FSYNC = 0,  /*!< fsync, the default */
-  SRV_UNIX_O_DSYNC,    /*!< open log files in O_SYNC mode */
-  SRV_UNIX_LITTLESYNC, /*!< do not call os_file_flush()
-                       when writing data files, but do flush
-                       after writing to log files */
-  SRV_UNIX_NOSYNC,     /*!< do not flush after writing */
-  SRV_UNIX_O_DIRECT,   /*!< invoke os_file_set_nocache() on
-                       data files. This implies using
-                       non-buffered IO but still using fsync,
-                       the reason for which is that some FS
-                       do not flush meta-data when
-                       unbuffered IO happens */
+  /** fsync, the default if O_DIRECT is not supported */
+  SRV_UNIX_FSYNC = 0,
+  /** open log files in O_SYNC mode */
+  SRV_UNIX_O_DSYNC,
+  /** do not call os_file_flush() when writing data files, but do flush after
+     writing to log files */
+  SRV_UNIX_LITTLESYNC,
+  /** do not flush after writing */
+  SRV_UNIX_NOSYNC,
+  /** invoke os_file_set_nocache() on data files. This implies using
+     non-buffered IO but still using fsync, the reason for which is that some FS
+     do not flush meta-data when unbuffered IO happens. Default if supported */
+  SRV_UNIX_O_DIRECT,
+  /** do not use fsync() when using direct IO i.e.: it can be set to avoid the
+     fsync() call that we make when using SRV_UNIX_O_DIRECT. However, in this
+     case user/DBA should be sure about the integrity of the meta-data */
   SRV_UNIX_O_DIRECT_NO_FSYNC
-  /*!< do not use fsync() when using
-  direct IO i.e.: it can be set to avoid
-  the fsync() call that we make when
-  using SRV_UNIX_O_DIRECT. However, in
-  this case user/DBA should be sure about
-  the integrity of the meta-data */
 };
 extern enum srv_unix_flush_t srv_unix_file_flush_method;
 
@@ -957,14 +992,13 @@ void srv_wake_purge_thread_if_not_active(void);
  and wakes up the master thread if it is suspended (not sleeping). Used
  in the MySQL interface. Note that there is a small chance that the master
  thread stays suspended (we do not protect our operation with the kernel
- mutex, for performace reasons). */
+ mutex, for performance reasons). */
 void srv_active_wake_master_thread_low(void);
-#define srv_active_wake_master_thread()    \
-  do {                                     \
-    if (!srv_read_only_mode) {             \
-      srv_active_wake_master_thread_low(); \
-    }                                      \
-  } while (0)
+static inline void srv_active_wake_master_thread() {
+  if (!srv_read_only_mode) {
+    srv_active_wake_master_thread_low();
+  }
+}
 /** Wakes up the master thread if it is suspended or being suspended. */
 void srv_wake_master_thread(void);
 #ifndef UNIV_HOTBACKUP
@@ -988,12 +1022,12 @@ void srv_export_innodb_status(void);
 ulint srv_get_activity_count(void);
 /** Check if there has been any activity.
  @return false if no change in activity counter. */
-ibool srv_check_activity(ulint old_activity_count); /*!< old activity count */
+bool srv_check_activity(ulint old_activity_count); /*!< old activity count */
 /** Increment the server activity counter. */
 void srv_inc_activity_count(void);
 
 /** Enqueues a task to server task queue and releases a worker thread, if there
- is a suspended one. */
+is a suspended one. */
 void srv_que_task_enqueue_low(que_thr_t *thr); /*!< in: query thread */
 
 /** A thread which prints the info output by various InnoDB monitors. */
@@ -1012,30 +1046,19 @@ void srv_purge_coordinator_thread();
 /** Worker thread that reads tasks from the work queue and executes them. */
 void srv_worker_thread();
 
-/** Rotate default master key for UNDO tablespace. */
-void undo_rotate_default_master_key();
-
 /** Set encryption for UNDO tablespace with given space id.
 @param[in] space_id     Undo tablespace id
 @param[in] mtr          Mini-transaction
-@param[in] is_boot	true if it is called during server start up.
 @return false for success, true otherwise */
-bool set_undo_tablespace_encryption(space_id_t space_id, mtr_t *mtr,
-                                    bool is_boot);
+bool set_undo_tablespace_encryption(space_id_t space_id, mtr_t *mtr);
 
 /** Enable UNDO tablespaces encryption.
-@param[in] is_boot	true if it is called during server start up. In this
-                        case, default master key will be used which will be
-                        rotated later with actual master key from kyering.
 @return false for success, true otherwise. */
-bool srv_enable_undo_encryption(bool is_boot);
+bool srv_enable_undo_encryption();
 
 /** Enable REDO log encryption.
-@param[in] is_boot	true if it is called during server start up. In this
-                        case, default master key will be used which will be
-                        rotated later with actual master key from kyering.
 @return false for success, true otherwise. */
-bool srv_enable_redo_encryption(bool is_boot);
+bool srv_enable_redo_encryption();
 
 /** Get count of tasks in the queue.
  @return number of tasks in queue */
@@ -1082,7 +1105,7 @@ called once during thread de-initialization. */
 void undo_spaces_deinit();
 
 /** Set redo log variable for performance schema global status.
-@param[in]	enable	true => redo log enabled, false => redo log disabled */
+@param[in]      enable  true => redo log enabled, false => redo log disabled */
 void set_srv_redo_log(bool enable);
 
 #ifdef UNIV_DEBUG
@@ -1090,10 +1113,10 @@ struct SYS_VAR;
 
 /** Disables master thread. It's used by:
         SET GLOBAL innodb_master_thread_disabled_debug = 1 (0).
-@param[in]	thd		thread handle
-@param[in]	var		pointer to system variable
-@param[out]	var_ptr		where the formal string goes
-@param[in]	save		immediate result from check function */
+@param[in]      thd             thread handle
+@param[in]      var             pointer to system variable
+@param[out]     var_ptr         where the formal string goes
+@param[in]      save            immediate result from check function */
 void srv_master_thread_disabled_debug_update(THD *thd, SYS_VAR *var,
                                              void *var_ptr, const void *save);
 #endif /* UNIV_DEBUG */
@@ -1114,13 +1137,17 @@ struct export_var_t {
   char innodb_buffer_pool_load_status[OS_FILE_MAX_PATH +
                                       128];   /*!< Buf pool load status */
   char innodb_buffer_pool_resize_status[512]; /*!< Buf pool resize status */
-  ulint innodb_buffer_pool_pages_total;       /*!< Buffer pool size */
-  ulint innodb_buffer_pool_pages_data;        /*!< Data pages */
-  ulint innodb_buffer_pool_bytes_data;        /*!< File bytes used */
-  ulint innodb_buffer_pool_pages_dirty;       /*!< Dirty data pages */
-  ulint innodb_buffer_pool_bytes_dirty;       /*!< File bytes modified */
-  ulint innodb_buffer_pool_pages_misc;        /*!< Miscellanous pages */
-  ulint innodb_buffer_pool_pages_free;        /*!< Free pages */
+  uint32_t
+      innodb_buffer_pool_resize_status_code; /*!< Buf pool resize status code */
+  uint32_t innodb_buffer_pool_resize_status_progress; /*!< Buf pool resize
+                                                     status progess */
+  ulint innodb_buffer_pool_pages_total;               /*!< Buffer pool size */
+  ulint innodb_buffer_pool_pages_data;                /*!< Data pages */
+  ulint innodb_buffer_pool_bytes_data;                /*!< File bytes used */
+  ulint innodb_buffer_pool_pages_dirty;               /*!< Dirty data pages */
+  ulint innodb_buffer_pool_bytes_dirty; /*!< File bytes modified */
+  ulint innodb_buffer_pool_pages_misc;  /*!< Miscellaneous pages */
+  ulint innodb_buffer_pool_pages_free;  /*!< Free pages */
 #ifdef UNIV_DEBUG
   ulint innodb_buffer_pool_pages_latched;  /*!< Latched pages */
 #endif                                     /* UNIV_DEBUG */
@@ -1134,14 +1161,25 @@ struct export_var_t {
   ulint innodb_buffer_pool_read_ahead_evicted; /*!< srv_read_ahead evicted*/
   ulint innodb_dblwr_pages_written;            /*!< srv_dblwr_pages_written */
   ulint innodb_dblwr_writes;                   /*!< srv_dblwr_writes */
-  ulint innodb_log_waits;                      /*!< srv_log_waits */
-  ulint innodb_log_write_requests;             /*!< srv_log_write_requests */
-  ulint innodb_log_writes;                     /*!< srv_log_writes */
-  lsn_t innodb_os_log_written;                 /*!< srv_os_log_written */
-  ulint innodb_os_log_fsyncs;                  /*!< fil_n_log_flushes */
-  ulint innodb_os_log_pending_writes;          /*!< srv_os_log_pending_writes */
-  ulint innodb_os_log_pending_fsyncs;          /*!< fil_n_pending_log_flushes */
-  ulint innodb_page_size;                      /*!< UNIV_PAGE_SIZE */
+  char innodb_redo_log_resize_status[512];     /*!< Redo log resize status */
+  bool innodb_redo_log_read_only;              /*!< Is redo log read-only ? */
+  ulonglong innodb_redo_log_uuid;              /*!< Redo log UUID */
+  ulonglong innodb_redo_log_checkpoint_lsn;    /*!< Redo log checkpoint LSN */
+  ulonglong innodb_redo_log_current_lsn;       /*!< Redo log current LSN */
+  ulonglong
+      innodb_redo_log_flushed_to_disk_lsn; /*!< Redo log flushed to disk LSN */
+  ulonglong innodb_redo_log_logical_size;  /*!< Redo log logical size */
+  ulonglong innodb_redo_log_physical_size; /*!< Redo log physical size */
+  ulonglong innodb_redo_log_capacity_resized; /*!< Redo log capacity after
+                                              the last finished redo resize */
+  ulint innodb_log_waits;                     /*!< srv_log_waits */
+  ulint innodb_log_write_requests;            /*!< srv_log_write_requests */
+  ulint innodb_log_writes;                    /*!< srv_log_writes */
+  lsn_t innodb_os_log_written;                /*!< srv_os_log_written */
+  ulint innodb_os_log_fsyncs;                 /*!< log_total_flushes() */
+  ulint innodb_os_log_pending_writes;         /*!< srv_os_log_pending_writes */
+  ulint innodb_os_log_pending_fsyncs;         /*!< log_pending_flushes() */
+  ulint innodb_page_size;                     /*!< UNIV_PAGE_SIZE */
   ulint innodb_pages_created;          /*!< buf_pool->stat.n_pages_created */
   ulint innodb_pages_read;             /*!< buf_pool->stat.n_pages_read */
   ulint innodb_pages_written;          /*!< buf_pool->stat.n_pages_written */
@@ -1165,7 +1203,7 @@ struct export_var_t {
   ulint innodb_system_rows_deleted;    /*!< srv_n_system_rows_deleted*/
   ulint innodb_sampled_pages_read;
   ulint innodb_sampled_pages_skipped;
-  ulint innodb_num_open_files;            /*!< fil_n_file_opened */
+  ulint innodb_num_open_files;            /*!< fil_n_files_open */
   ulint innodb_truncated_status_writes;   /*!< srv_truncated_status_writes */
   ulint innodb_undo_tablespaces_total;    /*!< total number of undo tablespaces
                                           innoDB is tracking. */
@@ -1191,15 +1229,15 @@ struct srv_slot_t {
   /** Thread type: user, utility etc. */
   srv_thread_type type;
 
-  /** TRUE if this slot is in use. */
+  /** true if this slot is in use. */
   bool in_use;
 
-  /** TRUE if the thread is waiting for the event of this slot. */
+  /** true if the thread is waiting for the event of this slot. */
   bool suspended;
 
   /** Time when the thread was suspended. Initialized by
   lock_wait_table_reserve_slot() for lock wait. */
-  ib_time_monotonic_t suspend_time;
+  std::chrono::steady_clock::time_point suspend_time;
 
   /** Stores the current value of lock_wait_table_reservations, when
   lock_wait_table_reserve_slot is called.
@@ -1213,7 +1251,7 @@ struct srv_slot_t {
 
   /** Wait time that if exceeded the thread will be timed out.
   Initialized by lock_wait_table_reserve_slot() for lock wait. */
-  ulong wait_timeout;
+  std::chrono::steady_clock::duration wait_timeout;
 
   /** Event used in suspending the thread when it has nothing to do. */
   os_event_t event;
