@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -31,6 +32,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
  *******************************************************/
 
 #include "read0read.h"
+#include "clone0clone.h"
 
 #include "srv0srv.h"
 #include "trx0sys.h"
@@ -187,7 +189,7 @@ struct ViewCheck {
   ViewCheck() : m_prev_view() {}
 
   void operator()(const ReadView *view) {
-    ut_a(m_prev_view == NULL || view->is_closed() || view->le(m_prev_view));
+    ut_a(m_prev_view == nullptr || view->is_closed() || view->le(m_prev_view));
 
     m_prev_view = view;
   }
@@ -201,7 +203,7 @@ Validates a read view list. */
 bool MVCC::validate() const {
   ViewCheck check;
 
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   ut_list_map(m_views, check);
 
@@ -209,11 +211,9 @@ bool MVCC::validate() const {
 }
 #endif /* UNIV_DEBUG */
 
-/**
-Try and increase the size of the array. Old elements are
-copied across.
-@param n 		Make space for n elements */
-
+/** Try and increase the size of the array. Old elements are copied across. It
+is a no-op if n is < current size.
+@param  n  Make space for n elements */
 void ReadView::ids_t::reserve(ulint n) {
   if (n <= capacity()) {
     return;
@@ -226,23 +226,24 @@ void ReadView::ids_t::reserve(ulint n) {
 
   value_type *p = m_ptr;
 
-  m_ptr = UT_NEW_ARRAY_NOKEY(value_type, n);
+  m_ptr =
+      ut::new_arr_withkey<value_type>(UT_NEW_THIS_FILE_PSI_KEY, ut::Count{n});
 
   m_reserved = n;
 
   ut_ad(size() < capacity());
 
-  if (p != NULL) {
+  if (p != nullptr) {
     ::memmove(m_ptr, p, size() * sizeof(value_type));
 
-    UT_DELETE_ARRAY(p);
+    ut::delete_arr(p);
   }
 }
 
 /**
 Copy and overwrite this array contents
-@param start		Source array
-@param end		Pointer to end of array */
+@param start            Source array
+@param end              Pointer to end of array */
 
 void ReadView::ids_t::assign(const value_type *start, const value_type *end) {
   ut_ad(end >= start);
@@ -264,7 +265,7 @@ void ReadView::ids_t::assign(const value_type *start, const value_type *end) {
 
 /**
 Append a value to the array.
-@param value		the value to append */
+@param value            the value to append */
 
 void ReadView::ids_t::push_back(value_type value) {
   if (capacity() <= size()) {
@@ -318,6 +319,7 @@ ReadView::ReadView()
       m_ids(),
       m_low_limit_no() {
   ut_d(::memset(&m_view_list, 0x0, sizeof(m_view_list)));
+  ut_d(m_view_low_limit_no = 0);
 }
 
 /**
@@ -327,24 +329,20 @@ ReadView::~ReadView() {
 }
 
 /** Constructor
-@param size		Number of views to pre-allocate */
-MVCC::MVCC(ulint size) {
-  UT_LIST_INIT(m_free, &ReadView::m_view_list);
-  UT_LIST_INIT(m_views, &ReadView::m_view_list);
-
+@param size             Number of views to pre-allocate */
+MVCC::MVCC(ulint size) : m_free(), m_views() {
   for (ulint i = 0; i < size; ++i) {
-    ReadView *view = UT_NEW_NOKEY(ReadView());
+    ReadView *view = ut::new_withkey<ReadView>(UT_NEW_THIS_FILE_PSI_KEY);
 
     UT_LIST_ADD_FIRST(m_free, view);
   }
 }
 
 MVCC::~MVCC() {
-  for (ReadView *view = UT_LIST_GET_FIRST(m_free); view != NULL;
-       view = UT_LIST_GET_FIRST(m_free)) {
+  while (ReadView *view = UT_LIST_GET_FIRST(m_free)) {
     UT_LIST_REMOVE(m_free, view);
 
-    UT_DELETE(view);
+    ut::delete_(view);
   }
 
   ut_a(UT_LIST_GET_LEN(m_views) == 0);
@@ -354,6 +352,8 @@ MVCC::~MVCC() {
 Copy the transaction ids from the source vector */
 
 void ReadView::copy_trx_ids(const trx_ids_t &trx_ids) {
+  ut_ad(trx_sys_mutex_own());
+
   ulint size = trx_ids.size();
 
   if (m_creator_trx_id > 0) {
@@ -408,12 +408,33 @@ void ReadView::copy_trx_ids(const trx_ids_t &trx_ids) {
   m_up_limit_id = m_ids.front();
 
 #ifdef UNIV_DEBUG
-  /* Assert that all transaction ids in list are active. */
-  for (trx_ids_t::const_iterator it = trx_ids.begin(); it != trx_ids.end();
-       ++it) {
-    trx_t *trx = trx_get_rw_trx_by_id(*it);
-    ut_ad(trx != NULL);
-    ut_ad(trx->state == TRX_STATE_ACTIVE || trx->state == TRX_STATE_PREPARED);
+  /* The check is done randomly from time to time, because the check adds
+  a kind of extra synchronization which itself could hide existing bugs. */
+  if (ut::random_from_interval_fast(0, 99) == 0) {
+    /* Assert that all transaction ids in list are active. */
+    for (auto trx_id : trx_ids) {
+      while (trx_sys->latch_and_execute_with_active_trx(
+          trx_id,
+          [](trx_t *trx) {
+            if (trx != nullptr) {
+              const auto trx_state = trx->state.load(std::memory_order_relaxed);
+              /* Transaction in active_rw_trxs might only be ACTIVE or
+              PREPARED, before it becomes COMMITTED it is removed from
+              active_rw_trxs. */
+              ut_ad(trx_state == TRX_STATE_ACTIVE ||
+                    trx_state == TRX_STATE_PREPARED);
+              return false;
+            }
+            return true;
+          },
+          UT_LOCATION_HERE)) {
+        /* It might happen that transaction became added to rw_trx_ids,
+        then trx_sys mutex has been released and thread become scheduled
+        out before the call to trx_sys_rw_trx_add(trx). We need to wait,
+        it will come (if we hang forever in this loop - it's a bug). */
+        std::this_thread::sleep_for(std::chrono::microseconds(10));
+      }
+    }
   }
 #endif /* UNIV_DEBUG */
 }
@@ -421,14 +442,18 @@ void ReadView::copy_trx_ids(const trx_ids_t &trx_ids) {
 /**
 Opens a read view where exactly the transactions serialized before this
 point in time are seen in the view.
-@param id		Creator transaction id */
+@param id               Creator transaction id */
 
 void ReadView::prepare(trx_id_t id) {
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   m_creator_trx_id = id;
 
-  m_low_limit_no = m_low_limit_id = m_up_limit_id = trx_sys->max_trx_id;
+  m_low_limit_no = trx_get_serialisation_min_trx_no();
+
+  m_low_limit_id = trx_sys_get_next_trx_id_or_no();
+
+  ut_a(m_low_limit_no <= m_low_limit_id);
 
   if (!trx_sys->rw_trx_ids.empty()) {
     copy_trx_ids(trx_sys->rw_trx_ids);
@@ -436,18 +461,12 @@ void ReadView::prepare(trx_id_t id) {
     m_ids.clear();
   }
 
-  ut_ad(m_up_limit_id <= m_low_limit_id);
+  /* The first active transaction has the smallest id. */
+  m_up_limit_id = !m_ids.empty() ? m_ids.front() : m_low_limit_id;
 
-  if (UT_LIST_GET_LEN(trx_sys->serialisation_list) > 0) {
-    const trx_t *trx;
-
-    trx = UT_LIST_GET_FIRST(trx_sys->serialisation_list);
-
-    if (trx->no < m_low_limit_no) {
-      m_low_limit_no = trx->no;
-    }
-  }
+  ut_a(m_up_limit_id <= m_low_limit_id);
 
+  ut_d(m_view_low_limit_no = m_low_limit_no);
   m_closed = false;
 }
 
@@ -457,7 +476,7 @@ a new view.
 @return a view to use */
 
 ReadView *MVCC::get_view() {
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   ReadView *view;
 
@@ -465,9 +484,9 @@ ReadView *MVCC::get_view() {
     view = UT_LIST_GET_FIRST(m_free);
     UT_LIST_REMOVE(m_free, view);
   } else {
-    view = UT_NEW_NOKEY(ReadView());
+    view = ut::new_withkey<ReadView>(UT_NEW_THIS_FILE_PSI_KEY);
 
-    if (view == NULL) {
+    if (view == nullptr) {
       ib::error(ER_IB_MSG_918) << "Failed to allocate MVCC view";
     }
   }
@@ -475,45 +494,16 @@ ReadView *MVCC::get_view() {
   return (view);
 }
 
-/**
-Release a view that is inactive but not closed. Caller must own
-the trx_sys_t::mutex.
-@param view		View to release */
-void MVCC::view_release(ReadView *&view) {
-  ut_ad(!srv_read_only_mode);
-  ut_ad(trx_sys_mutex_own());
-
-  uintptr_t p = reinterpret_cast<uintptr_t>(view);
-
-  ut_a(p & 0x1);
-
-  view = reinterpret_cast<ReadView *>(p & ~1);
-
-  ut_ad(view->m_closed);
-
-  /** RW transactions should not free their views here. Their views
-  should freed using view_close_view() */
-
-  ut_ad(view->m_creator_trx_id == 0);
-
-  UT_LIST_REMOVE(m_views, view);
-
-  UT_LIST_ADD_LAST(m_free, view);
-
-  view = NULL;
-}
-
-/**
-Allocate and create a view.
-@param view		view owned by this class created for the
-                        caller. Must be freed by calling view_close()
-@param trx		transaction instance of caller */
+/** Allocate and create a view.
+@param view     View owned by this class created for the caller. Must be
+freed by calling view_close()
+@param trx      Transaction instance of caller */
 void MVCC::view_open(ReadView *&view, trx_t *trx) {
   ut_ad(!srv_read_only_mode);
 
   /** If no new RW transaction has been started since the last view
   was created then reuse the the existing view. */
-  if (view != NULL) {
+  if (view != nullptr) {
     uintptr_t p = reinterpret_cast<uintptr_t>(view);
 
     view = reinterpret_cast<ReadView *>(p & ~1);
@@ -521,7 +511,7 @@ void MVCC::view_open(ReadView *&view, trx_t *trx) {
     ut_ad(view->m_closed);
 
     /* NOTE: This can be optimised further, for now we only
-    resuse the view iff there are no active RW transactions.
+    reuse the view if there are no active RW transactions.
 
     There is an inherent race here between purge and this
     thread. Purge will skip views that are marked as closed.
@@ -531,24 +521,24 @@ void MVCC::view_open(ReadView *&view, trx_t *trx) {
     if (trx_is_autocommit_non_locking(trx) && view->empty()) {
       view->m_closed = false;
 
-      if (view->m_low_limit_id == trx_sys_get_max_trx_id()) {
+      if (view->m_low_limit_id == trx_sys_get_next_trx_id_or_no()) {
         return;
       } else {
         view->m_closed = true;
       }
     }
+  }
 
-    mutex_enter(&trx_sys->mutex);
+  trx_sys_mutex_enter();
 
+  if (view != nullptr) {
     UT_LIST_REMOVE(m_views, view);
 
   } else {
-    mutex_enter(&trx_sys->mutex);
-
     view = get_view();
   }
 
-  if (view != NULL) {
+  if (view != nullptr) {
     view->prepare(trx->id);
 
     UT_LIST_ADD_FIRST(m_views, view);
@@ -561,25 +551,6 @@ void MVCC::view_open(ReadView *&view, trx_t *trx) {
   trx_sys_mutex_exit();
 }
 
-ReadView *MVCC::get_view_created_by_trx_id(trx_id_t trx_id) const {
-  ReadView *view;
-
-  ut_ad(mutex_own(&trx_sys->mutex));
-
-  for (view = UT_LIST_GET_LAST(m_views); view != NULL;
-       view = UT_LIST_GET_PREV(m_view_list, view)) {
-    if (view->is_closed()) {
-      continue;
-    }
-
-    if (view->m_creator_trx_id == trx_id) {
-      break;
-    }
-  }
-
-  return (view);
-}
-
 /**
 Get the oldest (active) view in the system.
 @return oldest view if found or NULL */
@@ -587,9 +558,9 @@ Get the oldest (active) view in the system.
 ReadView *MVCC::get_oldest_view() const {
   ReadView *view;
 
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
-  for (view = UT_LIST_GET_LAST(m_views); view != NULL;
+  for (view = UT_LIST_GET_LAST(m_views); view != nullptr;
        view = UT_LIST_GET_PREV(m_view_list, view)) {
     if (!view->is_closed()) {
       break;
@@ -601,7 +572,7 @@ ReadView *MVCC::get_oldest_view() const {
 
 /**
 Copy state from another view. Must call copy_complete() to finish.
-@param other		view to copy from */
+@param other            view to copy from */
 
 void ReadView::copy_prepare(const ReadView &other) {
   ut_ad(&other != this);
@@ -618,6 +589,8 @@ void ReadView::copy_prepare(const ReadView &other) {
 
   m_low_limit_no = other.m_low_limit_no;
 
+  ut_d(m_view_low_limit_no = other.m_view_low_limit_no);
+
   m_low_limit_id = other.m_low_limit_id;
 
   m_creator_trx_id = other.m_creator_trx_id;
@@ -647,16 +620,16 @@ void ReadView::copy_complete() {
 
 /** Clones the oldest view and stores it in view. No need to
 call view_close(). The caller owns the view that is passed in.
-This function is called by Purge to determine whether it should
+It will also move the closed views from the m_views list to the
+m_free list. This function is called by Purge to determine whether it should
 purge the delete marked record or not.
-@param view		Preallocated view, owned by the caller */
-
+@param view             Preallocated view, owned by the caller */
 void MVCC::clone_oldest_view(ReadView *view) {
-  mutex_enter(&trx_sys->mutex);
+  trx_sys_mutex_enter();
 
   ReadView *oldest_view = get_oldest_view();
 
-  if (oldest_view == NULL) {
+  if (oldest_view == nullptr) {
     view->prepare(0);
 
     trx_sys_mutex_exit();
@@ -668,6 +641,10 @@ void MVCC::clone_oldest_view(ReadView *view) {
 
     view->copy_complete();
   }
+  /* Update view to block purging transaction till GTID is persisted. */
+  auto &gtid_persistor = clone_sys->get_gtid_persistor();
+  auto gtid_oldest_trxno = gtid_persistor.get_oldest_trx_no();
+  view->reduce_low_limit(gtid_oldest_trxno);
 }
 
 /**
@@ -678,8 +655,7 @@ ulint MVCC::size() const {
 
   ulint size = 0;
 
-  for (const ReadView *view = UT_LIST_GET_FIRST(m_views); view != NULL;
-       view = UT_LIST_GET_NEXT(m_view_list, view)) {
+  for (const ReadView *view : m_views) {
     if (!view->is_closed()) {
       ++size;
     }
@@ -692,8 +668,8 @@ ulint MVCC::size() const {
 
 /**
 Close a view created by the above function.
-@param view		view allocated by trx_open.
-@param own_mutex	true if caller owns trx_sys_t::mutex */
+@param view             view allocated by trx_open.
+@param own_mutex        true if caller owns trx_sys_t::mutex */
 
 void MVCC::view_close(ReadView *&view, bool own_mutex) {
   uintptr_t p = reinterpret_cast<uintptr_t>(view);
@@ -720,19 +696,6 @@ void MVCC::view_close(ReadView *&view, bool own_mutex) {
 
     ut_ad(validate());
 
-    view = NULL;
+    view = nullptr;
   }
 }
-
-/**
-Set the view creator transaction id. Note: This shouldbe set only
-for views created by RW transactions.
-@param view		Set the creator trx id for this view
-@param id		Transaction id to set */
-
-void MVCC::set_view_creator_trx_id(ReadView *view, trx_id_t id) {
-  ut_ad(id > 0);
-  ut_ad(mutex_own(&trx_sys->mutex));
-
-  view->creator_trx_id(id);
-}
