@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +28,9 @@
 #include <sys/types.h>
 #include <time.h>
 
-#include "my_bit.h"
+#include <algorithm>
+#include <bit>
+
 #include "my_byteorder.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -43,7 +46,7 @@
 #include <fcntl.h>
 #include <process.h>
 #endif
-#include "m_ctype.h"
+#include "mysql/strings/m_ctype.h"
 
 /*
   Old options is used when recreating database, from myisamchk
@@ -60,7 +63,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
       key_length, info_length, key_segs, options, min_key_length_skip, base_pos,
       long_varchar_count, varchar_length, max_key_block_length,
       unique_key_parts, fulltext_keys, offset;
-  uint aligned_key_start, block_length;
+  uint aligned_key_start;
   uint internal_table = flags & HA_CREATE_INTERNAL_TABLE;
   ulong reclength, real_reclength, min_pack_length;
   char filename[FN_REFLEN], linkname[FN_REFLEN], *linkname_ptr;
@@ -200,9 +203,9 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     else
       ci->data_file_length = (ulonglong)ci->max_rows * pack_reclength;
   } else if (!ci->max_rows)
-    ci->max_rows = (ha_rows)(
-        ci->data_file_length /
-        (min_pack_length + ((options & HA_OPTION_PACK_RECORD) ? 3 : 0)));
+    ci->max_rows = (ha_rows)(ci->data_file_length /
+                             (min_pack_length +
+                              ((options & HA_OPTION_PACK_RECORD) ? 3 : 0)));
 
   if (options & (HA_OPTION_COMPRESS_RECORD | HA_OPTION_PACK_RECORD))
     pointer =
@@ -307,7 +310,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
           keydef->flag &= ~HA_PACK_KEY;
           keydef->flag |= HA_BINARY_PACK_KEY | HA_VAR_LENGTH_KEY;
         } else {
-          keydef->seg[0].flag |= HA_PACK_KEY; /* for easyer intern test */
+          keydef->seg[0].flag |= HA_PACK_KEY; /* for easier intern test */
           keydef->flag |= HA_VAR_LENGTH_KEY;
           options |= HA_OPTION_PACK_KEYS; /* Using packed keys */
         }
@@ -351,7 +354,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
             break;
         }
         if (keyseg->flag & HA_SPACE_PACK) {
-          DBUG_ASSERT(!(keyseg->flag & HA_VAR_LENGTH_PART));
+          assert(!(keyseg->flag & HA_VAR_LENGTH_PART));
           keydef->flag |= HA_SPACE_PACK_USED | HA_VAR_LENGTH_KEY;
           options |= HA_OPTION_PACK_KEYS; /* Using packed keys */
           length++;                       /* At least one length byte */
@@ -362,8 +365,8 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
           }
         }
         if (keyseg->flag & (HA_VAR_LENGTH_PART | HA_BLOB_PART)) {
-          DBUG_ASSERT(!test_all_bits(keyseg->flag,
-                                     (HA_VAR_LENGTH_PART | HA_BLOB_PART)));
+          assert(!test_all_bits(keyseg->flag,
+                                (HA_VAR_LENGTH_PART | HA_BLOB_PART)));
           keydef->flag |= HA_VAR_LENGTH_KEY;
           length++;                       /* At least one length byte */
           options |= HA_OPTION_PACK_KEYS; /* Using packed keys */
@@ -397,11 +400,10 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
       share.state.rec_per_key_part[key_segs - 1] = 1L;
     length += key_length;
     /* Get block length for key, if defined by user */
-    block_length =
-        (keydef->block_length ? my_round_up_to_next_power(keydef->block_length)
-                              : myisam_block_size);
-    block_length = MY_MAX(block_length, MI_MIN_KEY_BLOCK_LENGTH);
-    block_length = MY_MIN(block_length, MI_MAX_KEY_BLOCK_LENGTH);
+    uint block_length =
+        keydef->block_length != 0 ? keydef->block_length : myisam_block_size;
+    block_length = std::clamp(std::bit_ceil(block_length),
+                              MI_MIN_KEY_BLOCK_LENGTH, MI_MAX_KEY_BLOCK_LENGTH);
 
     keydef->block_length = (uint16)MI_BLOCK_SIZE(
         length - real_length_diff, pointer, MI_MAX_KEYPTR_SIZE, block_length);
@@ -410,7 +412,8 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
       set_my_errno(HA_WRONG_CREATE_OPTION);
       goto err_no_lock;
     }
-    set_if_bigger(max_key_block_length, keydef->block_length);
+    max_key_block_length =
+        std::max(max_key_block_length, uint(keydef->block_length));
     keydef->keylength = (uint16)key_length;
     keydef->minlength = (uint16)(length - min_key_length_skip);
     keydef->maxlength = (uint16)length;
@@ -475,7 +478,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   share.state.process = (ulong)getpid();
   share.state.unique = (ulong)0;
   share.state.update_count = (ulong)0;
-  share.state.version = (ulong)time((time_t *)0);
+  share.state.version = (ulong)time((time_t *)nullptr);
   share.state.sortkey = (ushort)~0;
   share.state.auto_increment = ci->auto_increment;
   share.options = options;
@@ -488,7 +491,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     got from MYI file header (see also myisampack.c:save_state)
   */
   share.base.key_reflength =
-      mi_get_pointer_length(MY_MAX(ci->key_file_length, tmp), 3);
+      mi_get_pointer_length(std::max(ci->key_file_length, tmp), 3);
   share.base.keys = share.state.header.keys = keys;
   share.state.header.uniques = uniques;
   share.state.header.fulltext_keys = fulltext_keys;
@@ -496,8 +499,8 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   mi_int2store(share.state.header.unique_key_parts, unique_key_parts);
 
   mi_set_all_keys_active(share.state.key_map, keys);
-  aligned_key_start = my_round_up_to_next_power(
-      max_key_block_length ? max_key_block_length : myisam_block_size);
+  aligned_key_start = std::bit_ceil(max_key_block_length ? max_key_block_length
+                                                         : myisam_block_size);
 
   share.base.keystart = share.state.state.key_file_length =
       MY_ALIGN(info_length, aligned_key_start);
@@ -521,10 +524,10 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   share.base.min_block_length =
       (share.base.pack_reclength + 3 < MI_EXTEND_BLOCK_LENGTH &&
        !share.base.blobs)
-          ? MY_MAX(share.base.pack_reclength, MI_MIN_BLOCK_LENGTH)
+          ? std::max(share.base.pack_reclength, ulong{MI_MIN_BLOCK_LENGTH})
           : MI_EXTEND_BLOCK_LENGTH;
   if (!(flags & HA_DONT_TOUCH_DATA))
-    share.state.create_time = (long)time((time_t *)0);
+    share.state.create_time = (long)time((time_t *)nullptr);
 
   if (!internal_table) mysql_mutex_lock(&THR_LOCK_myisam);
 
@@ -537,7 +540,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     int have_iext = iext && !strcmp(iext, MI_NAME_IEXT);
     if (options & HA_OPTION_TMP_TABLE) {
       char *path;
-      /* chop off the table name, tempory tables use generated name */
+      /* chop off the table name, temporary tables use generated name */
       if ((path = strrchr(const_cast<char *>(ci->index_file_name), FN_LIBCHAR)))
         *path = '\0';
       fn_format(filename, name, ci->index_file_name, MI_NAME_IEXT,
@@ -553,7 +556,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     linkname_ptr = linkname;
     /*
       Don't create the table if the link or file exists to ensure that one
-      doesn't accidently destroy another table.
+      doesn't accidentally destroy another table.
     */
     create_flag = 0;
   } else {
@@ -562,7 +565,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     fn_format(filename, name, "", MI_NAME_IEXT,
               MY_UNPACK_FILENAME | MY_RETURN_REAL_PATH |
                   (have_iext ? MY_REPLACE_EXT : MY_APPEND_EXT));
-    linkname_ptr = 0;
+    linkname_ptr = nullptr;
     /* Replace the current file */
     create_flag = (flags & HA_CREATE_KEEP_FILES) ? 0 : MY_DELETE_OLD;
   }
@@ -600,7 +603,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
 
         if (options & HA_OPTION_TMP_TABLE) {
           char *path;
-          /* chop off the table name, tempory tables use generated name */
+          /* chop off the table name, temporary tables use generated name */
           if ((path =
                    strrchr(const_cast<char *>(ci->data_file_name), FN_LIBCHAR)))
             *path = '\0';
@@ -619,7 +622,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
       } else {
         fn_format(filename, name, "", MI_NAME_DEXT,
                   MY_UNPACK_FILENAME | MY_APPEND_EXT);
-        linkname_ptr = 0;
+        linkname_ptr = nullptr;
         create_flag = (flags & HA_CREATE_KEEP_FILES) ? 0 : MY_DELETE_OLD;
       }
       if ((dfile = mysql_file_create_with_symlink(
@@ -634,7 +637,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   if (mi_state_info_write(file, &share.state, 2) ||
       mi_base_info_write(file, &share.base))
     goto err;
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   if ((uint)mysql_file_tell(file, MYF(0)) != base_pos + MI_BASE_INFO_SIZE) {
     uint pos = (uint)mysql_file_tell(file, MYF(0));
     DBUG_PRINT("warning", ("base_length: %d  != used_length: %d",
@@ -717,7 +720,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   for (i = 0; i < share.base.fields; i++)
     if (mi_recinfo_write(file, &recinfo[i])) goto err;
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   if ((uint)mysql_file_tell(file, MYF(0)) != info_length) {
     uint pos = (uint)mysql_file_tell(file, MYF(0));
     DBUG_PRINT("warning",
@@ -747,7 +750,7 @@ err_no_lock:
   switch (errpos) {
     case 3:
       (void)mysql_file_close(dfile, MYF(0));
-      /* fall through */
+      [[fallthrough]];
     case 2:
       if (!(flags & HA_DONT_TOUCH_DATA))
         mysql_file_delete_with_symlink(
@@ -755,7 +758,7 @@ err_no_lock:
             fn_format(filename, name, "", MI_NAME_DEXT,
                       MY_UNPACK_FILENAME | MY_APPEND_EXT),
             MYF(0));
-      /* fall through */
+      [[fallthrough]];
     case 1:
       (void)mysql_file_close(file, MYF(0));
       if (!(flags & HA_DONT_TOUCH_DATA))
@@ -771,7 +774,7 @@ err_no_lock:
 }
 
 uint mi_get_pointer_length(ulonglong file_length, uint def) {
-  DBUG_ASSERT(def >= 2 && def <= 7);
+  assert(def >= 2 && def <= 7);
   if (file_length) /* If not default */
   {
     if (file_length >= 1ULL << 48)
