@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2010, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -78,6 +79,9 @@ public class DomainTypeHandlerImpl<T> extends AbstractDomainTypeHandlerImpl<T> {
     private Map<String, Method> unmatchedGetMethods = new HashMap<String, Method>();
     private Map<String, Method> unmatchedSetMethods = new HashMap<String, Method>();
 
+    /** The Proxy class for the Domain Class. */
+    private Class<T> proxyClass;
+
     /** The proxy interfaces implemented by the domain object */
     Class<?>[] proxyInterfaces = null;
 
@@ -105,7 +109,7 @@ public class DomainTypeHandlerImpl<T> extends AbstractDomainTypeHandlerImpl<T> {
         this(cls, dictionary, null);
     }
 
-    @SuppressWarnings( "unchecked" )
+    @SuppressWarnings( {"unchecked","deprecation"} )
     public DomainTypeHandlerImpl(Class<T> cls, Dictionary dictionary,
             ValueHandlerFactory smartValueHandlerFactory) {
         this.valueHandlerFactory = smartValueHandlerFactory!=null?
@@ -125,6 +129,8 @@ public class DomainTypeHandlerImpl<T> extends AbstractDomainTypeHandlerImpl<T> {
                         "ERR_Not_Persistence_Capable_Type", name));
             }
             proxyInterfaces = new Class<?>[] {cls, Finalizable.class};
+            proxyClass = (Class<T>)
+                Proxy.getProxyClass(cls.getClassLoader(), proxyInterfaces);
             // Get the table name from Persistence Capable annotation
             persistenceCapable = cls.getAnnotation(PersistenceCapable.class);
             if (persistenceCapable == null) {
@@ -408,8 +414,8 @@ public class DomainTypeHandlerImpl<T> extends AbstractDomainTypeHandlerImpl<T> {
         handler.markModified(fieldNumber);
     }
 
-    public Class<?>[] getProxyInterfaces() {
-        return proxyInterfaces;
+    public Class<?> getProxyClass() {
+        return proxyClass;
     }
 
     public Class<T> getDomainClass() {
@@ -481,8 +487,6 @@ public class DomainTypeHandlerImpl<T> extends AbstractDomainTypeHandlerImpl<T> {
                 ((DynamicObject)instance).delegate((DynamicObjectDelegate)valueHandler);
             } else {
                 instance = (T)Proxy.newProxyInstance(cls.getClassLoader(), proxyInterfaces, valueHandler);
-                // TODO is setProxy really needed?
-                valueHandler.setProxy(instance);
             }
             return instance;
         } catch (InstantiationException ex) {
@@ -526,12 +530,6 @@ public class DomainTypeHandlerImpl<T> extends AbstractDomainTypeHandlerImpl<T> {
         return head + tail;
     }
 
-    @SuppressWarnings( "unchecked" )
-    public T getInstance(ValueHandler valueHandler) {
-        T instance = (T)valueHandler.getProxy();
-        return instance;
-    }
-
     private Class<?> getType(Method method) {
         Class<?> result = null;
         if (method.getName().startsWith("get")) {
