@@ -1,16 +1,17 @@
 /*
- *  Copyright (c) 2010, 2020, Oracle and/or its affiliates.
+ *  Copyright (c) 2010, 2025, Oracle and/or its affiliates.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2.0,
  *  as published by the Free Software Foundation.
  *
- *  This program is also distributed with certain software (including
+ *  This program is designed to work with certain software (including
  *  but not limited to OpenSSL) that is licensed under separate terms,
  *  as designated in a particular file or component or in included license
  *  documentation.  The authors of MySQL hereby grant you an additional
  *  permission to link the program and your derivative works with the
- *  separately licensed software that they have included with MySQL.
+ *  separately licensed software that they have either included with
+ *  the program or referenced in the documentation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,7 +72,7 @@ public class ClusterConnectionImpl
     final String connectString;
 
     /** The node id requested for this connection; 0 for default */
-    final int nodeId;
+    int nodeId;
 
     /** The timeout value to connect to mgm */
     final int connectTimeoutMgm;
@@ -97,12 +98,6 @@ public class ClusterConnectionImpl
     /** The byte buffer pool for DbImpl error buffers */
     protected FixedByteBufferPoolImpl byteBufferPoolForDBImplError;
 
-    /** The size of the coordinated transaction identifier buffer */
-    private final static int COORDINATED_TRANSACTION_ID_SIZE = 44;
-
-    /** The byte buffer pool for coordinated transaction id */
-    protected FixedByteBufferPoolImpl byteBufferPoolForCoordinatedTransactionId;
-
     /** The size of the partition key scratch buffer */
     private final static int PARTITION_KEY_BUFFER_SIZE = 10000;
 
@@ -134,8 +129,6 @@ public class ClusterConnectionImpl
         this.connectTimeoutMgm = connectTimeoutMgm;
         byteBufferPoolForDBImplError =
                 new FixedByteBufferPoolImpl(errorBufferSize, "DBImplErrorBufferPool");
-        byteBufferPoolForCoordinatedTransactionId =
-                new FixedByteBufferPoolImpl(COORDINATED_TRANSACTION_ID_SIZE, "CoordinatedTransactionIdBufferPool");
         byteBufferPoolForPartitionKey =
                 new FixedByteBufferPoolImpl(PARTITION_KEY_BUFFER_SIZE, "PartitionKeyBufferPool");
         clusterConnection = Ndb_cluster_connection.create(connectString, nodeId);
@@ -150,6 +143,7 @@ public class ClusterConnectionImpl
         checkConnection();
         int returnCode = clusterConnection.connect(connectRetries, connectDelay, verbose?1:0);
         handleError(returnCode, clusterConnection, connectString, nodeId);
+        nodeId = clusterConnection.node_id();
     }
 
     public Db createDb(String database, int maxTransactions) {
@@ -175,6 +169,11 @@ public class ClusterConnectionImpl
         return result;
     }
 
+    public void configureTls(String tlsSearchPath, int tlsRequirement) {
+        if(tlsSearchPath == null) tlsSearchPath = "";
+        clusterConnection.configure_tls(tlsSearchPath, tlsRequirement);
+    }
+
     public void waitUntilReady(int connectTimeoutBefore, int connectTimeoutAfter) {
         checkConnection();
         int returnCode = clusterConnection.wait_until_ready(connectTimeoutBefore, connectTimeoutAfter);
