@@ -1,17 +1,25 @@
-/* Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is designed to work with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #ifndef BACKUP_LOCK_TAB
 #define BACKUP_LOCK_TAB
@@ -20,7 +28,6 @@
 
 #define JAM_FILE_ID 82
 
-
 /* This class is used for both REQ, CONF, and REF. */
 
 class BackupLockTab {
@@ -30,22 +37,15 @@ class BackupLockTab {
   /* Receiver(s). */
   friend class Dbdict;
 
-public:
-  STATIC_CONST( SignalLength = 7 );
+ public:
+  static constexpr Uint32 SignalLength = 7;
 
-private:
+ private:
   /* Values for m_lock_unlock. */
-  enum {
-    UNLOCK_TABLE = 0,
-    LOCK_TABLE = 1
-  };
+  enum { UNLOCK_TABLE = 0, LOCK_TABLE = 1 };
 
   /* Values for m_backup_state. */
-  enum {
-    BACKUP_FRAGMENT_INFO = 0,
-    GET_TABINFO_CONF = 1,
-    CLEANUP = 2
-  };
+  enum { BACKUP_FRAGMENT_INFO = 0, GET_TABINFO_CONF = 1, CLEANUP = 2 };
 
   Uint32 m_senderRef;
   Uint32 m_tableId;
@@ -57,7 +57,6 @@ private:
   Uint32 m_tablePtr_I;
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
