@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +28,6 @@
 
 #define JAM_FILE_ID 158
 
-
 struct DropNodegroupImplReq {
   /**
    * Sender(s) / Reciver(s)
@@ -39,15 +39,16 @@ struct DropNodegroupImplReq {
   /**
    * For printing
    */
-  friend bool printDROP_NODEGROUP_IMPL_REQ(FILE*, const Uint32*, Uint32, Uint16);
+  friend bool printDROP_NODEGROUP_IMPL_REQ(FILE *, const Uint32 *, Uint32,
+                                           Uint16);
 
-  STATIC_CONST( SignalLength = 6 );
+  static constexpr Uint32 SignalLength = 6;
 
   enum {
-    RT_PARSE    = 0x1,
-    RT_PREPARE  = 0x2,
-    RT_ABORT    = 0x3,
-    RT_COMMIT   = 0x4,
+    RT_PARSE = 0x1,
+    RT_PREPARE = 0x2,
+    RT_ABORT = 0x3,
+    RT_COMMIT = 0x4,
     RT_COMPLETE = 0x5
   };
 
@@ -72,9 +73,10 @@ struct DropNodegroupImplRef {
   /**
    * For printing
    */
-  friend bool printDROP_NODEGROUP_IMPL_REF(FILE*, const Uint32*, Uint32, Uint16);
+  friend bool printDROP_NODEGROUP_IMPL_REF(FILE *, const Uint32 *, Uint32,
+                                           Uint16);
 
-  STATIC_CONST( SignalLength = 3 );
+  static constexpr Uint32 SignalLength = 3;
 
   enum ErrorCode {
     NoError = 0,
@@ -99,9 +101,10 @@ struct DropNodegroupImplConf {
   /**
    * For printing
    */
-  friend bool printDROP_NODEGROUP_IMPL_CONF(FILE*, const Uint32*, Uint32, Uint16);
+  friend bool printDROP_NODEGROUP_IMPL_CONF(FILE *, const Uint32 *, Uint32,
+                                            Uint16);
 
-  STATIC_CONST( SignalLength = 4 );
+  static constexpr Uint32 SignalLength = 4;
 
   Uint32 senderData;
   Uint32 senderRef;
@@ -109,7 +112,6 @@ struct DropNodegroupImplConf {
   Uint32 gci_lo;
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
