@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2019, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +26,7 @@
 // Implements the interface defined in
 #include "storage/ndb/plugin/ndb_ddl_transaction_ctx.h"
 
+#include "my_dbug.h"
 #include "sql/handler.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
@@ -70,6 +72,12 @@ bool Ndb_DDL_transaction_ctx::rollback_create_table(
   DBUG_PRINT("info",
              ("Rollback : Dropping table '%s.%s'", db_name, table_name));
 
+  DBUG_EXECUTE_IF("ndb_simulate_failure_during_rollback", {
+    DBUG_SET("-d,ndb_simulate_failure_during_rollback");
+    thd_ndb->push_warning("Failed to rollback after CREATE TABLE failure.");
+    return false;
+  });
+
   /* Drop the table created during this DDL execution */
   Ndb *ndb = thd_ndb->ndb;
   if (drop_table_impl(m_thd, ndb,
@@ -114,8 +122,8 @@ bool Ndb_DDL_transaction_ctx::rollback_rename_table(
   Thd_ndb *thd_ndb = get_thd_ndb(m_thd);
   Ndb *ndb = thd_ndb->ndb;
   Ndb_table_guard ndbtab_g(ndb, new_db_name, new_table_name);
-  const NdbDictionary::Table *renamed_table;
-  if (!(renamed_table = ndbtab_g.get_table())) {
+  const NdbDictionary::Table *renamed_table = ndbtab_g.get_table();
+  if (renamed_table == nullptr) {
     thd_ndb->push_ndb_error_warning(ndbtab_g.getNdbError());
     thd_ndb->push_warning("Failed to rename table during rollback.");
     return false;
@@ -272,8 +280,8 @@ bool Ndb_DDL_transaction_ctx::post_ddl_hook_rename_table(
 
   /* Load the table from NDB */
   Ndb_table_guard ndbtab_g(ndb, db_name, table_name);
-  const NdbDictionary::Table *ndb_table;
-  if (!(ndb_table = ndbtab_g.get_table())) {
+  const NdbDictionary::Table *ndb_table = ndbtab_g.get_table();
+  if (ndb_table == nullptr) {
     thd_ndb->push_ndb_error_warning(ndbtab_g.getNdbError());
     thd_ndb->push_warning("Unable to load table during rollback");
     return false;
@@ -350,9 +358,9 @@ bool Ndb_DDL_transaction_ctx::post_ddl_hook_drop_temp_table(
     DBUG_PRINT("info", ("ALTER to different engine = '%s' detected",
                         ha_resolve_storage_engine_name(create_info->db_type)));
 
-    const char *orig_db_name = m_thd->lex->query_block->table_list.first->db;
+    const char *orig_db_name = m_thd->lex->query_block->get_table_list()->db;
     const char *orig_table_name =
-        m_thd->lex->query_block->table_list.first->table_name;
+        m_thd->lex->query_block->get_table_list()->table_name;
     DBUG_PRINT("info",
                ("original table name: '%s.%s'", orig_db_name, orig_table_name));
 
