@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2004, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,18 +23,18 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include <NdbLockCpuUtil.h>
+#include <NdbMutex.h>
+#include <NdbSpin.h>
 #include <ndb_global.h>
 #include "my_sys.h"
-#include <NdbMutex.h>
-#include <NdbLockCpuUtil.h>
-#include <NdbSpin.h>
 
 class EventLogger *g_eventLogger = nullptr;
 
 NdbMutex *g_ndb_connection_mutex = nullptr;
 
-extern class EventLogger * create_event_logger();
-extern void destroy_event_logger(class EventLogger ** g_eventLogger);
+extern class EventLogger *create_event_logger();
+extern void destroy_event_logger(class EventLogger **g_eventLogger);
 
 static int ndb_init_called = 0;
 
@@ -52,18 +53,14 @@ extern void NdbSpin_Init();
 extern int NdbHW_Init();
 extern void NdbHW_End();
 
-extern "C"
-{
+extern "C" {
 
 #define NORMAL_USER 0
 #define MYSQLD_USER 1
 #define THREAD_REGISTER_USER 2
-void
-ndb_init_internal(Uint32 caller)
-{
+void ndb_init_internal(Uint32 caller) {
   bool init_all = true;
-  if (caller != NORMAL_USER)
-  {
+  if (caller != NORMAL_USER) {
     /**
      * This is called from MySQL Server, normally called
      * from ndbcluster_init, but can also be called from
@@ -78,29 +75,21 @@ ndb_init_internal(Uint32 caller)
      */
     Uint32 init_called = ndb_init_called;
     ndb_init_called++;
-    if (init_called > 0)
-    {
-      if (caller == THREAD_REGISTER_USER)
-      {
+    if (init_called > 0) {
+      if (caller == THREAD_REGISTER_USER) {
         return;
       }
       init_all = false;
     }
   }
-  if (caller != THREAD_REGISTER_USER)
-    NdbOut_Init();
-  if (init_all)
-    NdbMutex_SysInit();
-  if (caller != THREAD_REGISTER_USER)
-  {
-    if (!g_ndb_connection_mutex)
-      g_ndb_connection_mutex = NdbMutex_Create();
-    if (!g_eventLogger)
-      g_eventLogger = create_event_logger();
-    if ((g_ndb_connection_mutex == nullptr) || (g_eventLogger == nullptr))
-    {
+  if (caller != THREAD_REGISTER_USER) NdbOut_Init();
+  if (init_all) NdbMutex_SysInit();
+  if (caller != THREAD_REGISTER_USER) {
+    if (!g_ndb_connection_mutex) g_ndb_connection_mutex = NdbMutex_Create();
+    if (!g_eventLogger) g_eventLogger = create_event_logger();
+    if ((g_ndb_connection_mutex == nullptr) || (g_eventLogger == nullptr)) {
       {
-        const char* err = "ndb_init() failed - exit\n";
+        const char *err = "ndb_init() failed - exit\n";
         int res = (int)write(2, err, (unsigned)strlen(err));
         (void)res;
         exit(1);
@@ -111,30 +100,23 @@ ndb_init_internal(Uint32 caller)
     NdbGetRUsage_Init();
     NdbSpin_Init();
   }
-  if (init_all)
-  {
+  if (init_all) {
     NdbThread_Init();
-    if (NdbLockCpu_Init() != 0)
-    {
-      const char* err = "ndbLockCpu_Init() failed - exit\n";
+    if (NdbLockCpu_Init() != 0) {
+      const char *err = "ndbLockCpu_Init() failed - exit\n";
       int res = (int)write(2, err, (unsigned)strlen(err));
       (void)res;
       exit(1);
     }
-    if (caller != MYSQLD_USER)
-      NdbHW_Init();
+    if (caller != MYSQLD_USER) NdbHW_Init();
   }
 }
 
-int
-ndb_init()
-{
-  if (ndb_init_called == 0)
-  {
+int ndb_init() {
+  if (ndb_init_called == 0) {
     ndb_init_called = 1;
-    if (my_init())
-    {
-      const char* err = "my_init() failed - exit\n";
+    if (my_init()) {
+      const char *err = "my_init() failed - exit\n";
       int res = (int)write(2, err, (unsigned)strlen(err));
       (void)res;
       exit(1);
@@ -152,48 +134,35 @@ ndb_init()
   return 0;
 }
 
-void
-ndb_end_internal(Uint32 caller)
-{
+void ndb_end_internal(Uint32 caller) {
   bool end_all = true;
-  if (caller != NORMAL_USER)
-  {
+  if (caller != NORMAL_USER) {
     ndb_init_called--;
-    if (ndb_init_called > 0)
-    {
-      if (caller == THREAD_REGISTER_USER)
-      {
+    if (ndb_init_called > 0) {
+      if (caller == THREAD_REGISTER_USER) {
         return;
       }
       end_all = false;
     }
   }
-  if (caller != THREAD_REGISTER_USER)
-  {
-    if (g_ndb_connection_mutex) 
-    {
+  if (caller != THREAD_REGISTER_USER) {
+    if (g_ndb_connection_mutex) {
       NdbMutex_Destroy(g_ndb_connection_mutex);
-      g_ndb_connection_mutex=nullptr;
+      g_ndb_connection_mutex = nullptr;
     }
-    if (g_eventLogger)
-      destroy_event_logger(&g_eventLogger);
+    if (g_eventLogger) destroy_event_logger(&g_eventLogger);
     NdbGetRUsage_End();
   }
-  if (end_all)
-  {
+  if (end_all) {
     NdbLockCpu_End();
     NdbThread_End();
     NdbMutex_SysEnd();
-    if (caller != MYSQLD_USER)
-      NdbHW_End();
+    if (caller != MYSQLD_USER) NdbHW_End();
   }
 }
 
-void
-ndb_end(int flags)
-{
-  if (ndb_init_called == 1)
-  {
+void ndb_end(int flags) {
+  if (ndb_init_called == 1) {
     my_end(flags);
     ndb_end_internal(0);
     ndb_init_called = 0;
