@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -126,15 +127,18 @@ const ParserRow<CPCDAPISession> commands[] = {
     CPCD_ARG("version", Int, Mandatory, "Protocol version to use"),
 
     CPCD_END()};
-CPCDAPISession::CPCDAPISession(NDB_SOCKET_TYPE sock, CPCD &cpcd)
-    : SocketServer::Session(sock), m_cpcd(cpcd), m_protocol_version(1) {
-  m_input = new SocketInputStream(sock, 7 * 24 * 60 * 60000);
-  m_output = new SocketOutputStream(sock);
+CPCDAPISession::CPCDAPISession(NdbSocket &&sock, CPCD &cpcd)
+    : SocketServer::Session(m_secure_socket),
+      m_cpcd(cpcd),
+      m_secure_socket(std::move(sock)),
+      m_protocol_version(1) {
+  m_input = new SocketInputStream(m_secure_socket, 7 * 24 * 60 * 60000);
+  m_output = new SocketOutputStream(m_secure_socket);
   m_parser = new Parser<CPCDAPISession>(commands, *m_input);
 }
 
 CPCDAPISession::CPCDAPISession(FILE *f, CPCD &cpcd)
-    : SocketServer::Session(ndb_socket_create_invalid()),
+    : SocketServer::Session(m_secure_socket),
       m_cpcd(cpcd),
       m_protocol_version(1) {
   m_input = new FileInputStream(f);
@@ -170,8 +174,7 @@ void CPCDAPISession::runSession() {
         break;
     }
   }
-  ndb_socket_close(m_socket);
-  ndb_socket_invalidate(&m_socket);
+  m_secure_socket.close();
 }
 
 void CPCDAPISession::stopSession() {
@@ -395,12 +398,11 @@ void CPCDAPISession::listProcesses(Parser_t::Context & /* unused */,
 }
 
 void CPCDAPISession::showVersion(Parser_t::Context & /* unused */,
-                                 const class Properties &args) {
+                                 const class Properties & /*args*/) {
   CPCD::RequestStatus rs;
 
   m_output->println("show version");
 
-  m_output->println("compile time: %s %s", __DATE__, __TIME__);
   m_output->println("supported protocol: %u", CPCD::CPC_PROTOCOL_VERSION);
   m_output->println("effective protocol: %u", m_protocol_version);
   m_output->println("%s", "");
