@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,17 +26,16 @@
 #ifndef __UNDO_BUFFER_HPP
 #define __UNDO_BUFFER_HPP
 
-#include <ndb_global.h>
 #include <kernel_types.h>
+#include <ndb_global.h>
 
 #define JAM_FILE_ID 404
 
 struct UndoPage;
 
-struct Undo_buffer 
-{
-  Undo_buffer(class Ndbd_mem_manager*);
-  
+struct Undo_buffer {
+  Undo_buffer(class Ndbd_mem_manager *);
+
   /**
    * Alloc space for a copy tuple of size <em>words</em>
    *   store address to copy in dst
@@ -43,33 +43,32 @@ struct Undo_buffer
    *
    * @return 0 if unable to alloc space
    */
-  Uint32 * alloc_copy_tuple(Local_key* dst, Uint32 words);
+  Uint32 *alloc_copy_tuple(Local_key *dst, Uint32 words);
   bool reuse_page_for_copy_tuple(Uint32 reuse_page);
 
   /**
    * Shrink size of copy tuple
    *   note: Only shrink latest allocated tuple
    */
-  void shrink_copy_tuple(Local_key* dst, Uint32 words);
-  
+  void shrink_copy_tuple(Local_key *dst, Uint32 words);
+
   /**
    * Free space for copy tuple at key
    */
-  void free_copy_tuple(Local_key* key);
-  
+  void free_copy_tuple(Local_key *key);
+
   /**
    * Get pointer to copy tuple
    */
-  Uint32 * get_ptr(const Local_key* key);
-  
-private:
-  void init_copy_tuple_page(UndoPage* page);
+  Uint32 *get_ptr(const Local_key *key);
 
-  class Ndbd_mem_manager* m_mm;
+ private:
+  void init_copy_tuple_page(UndoPage *page);
+
+  class Ndbd_mem_manager *m_mm;
   Uint32 m_first_free;
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
