@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +29,7 @@
 /*************************************************************
  *
  * SafeCounter "automates" three way to node-fais safe protocols
- * for "slave" failures.  This is done by registing "fake" signals
+ * for "slave" failures.  This is done by registering "fake" signals
  * to be sent in case of nodefailure.
  *
  * init<SignalClass>(..., GSN, senderData);
@@ -56,12 +57,12 @@
  */
 
 #include <NodeBitmask.hpp>
+#include "ArrayPool.hpp"
 #include "IntrusiveList.hpp"
 #include "VMSignal.hpp"
 
 #define JAM_FILE_ID 286
 
-
 class SimulatedBlock;
 
 /**
@@ -71,28 +72,29 @@ class SafeCounterManager {
   friend class SafeCounter;
   friend class SafeCounterHandle;
   friend class SimulatedBlock;
-public:
+
+ public:
   SafeCounterManager(class SimulatedBlock &);
-  
+
   bool setSize(Uint32 maxNoOfActiveMutexes, bool exit_on_error = true);
-  Uint32 getSize() const ;
+  Uint32 getSize() const;
   Uint32 getNoOfFree() const;
 
-  void execNODE_FAILREP(Signal*); 
-  void printNODE_FAILREP(); 
+  void execNODE_FAILREP(Signal *, const NdbNodeBitmask &failed_nodes);
+  void printNODE_FAILREP();
 
 #ifdef ERROR_INSERT
   void setFakeEmpty(bool val);
 #endif
 
-private:
-  struct ActiveCounter { /** sizeof = 7words = 28bytes */ 
-  public:
+ private:
+  struct ActiveCounter { /** sizeof = 7words = 28bytes */
+   public:
     Uint32 m_senderData;
     NdbNodeBitmask m_nodes;
     struct SignalDesc {
-    public:
-      Uint16 m_gsn; 
+     public:
+      Uint16 m_gsn;
       Uint16 m_block;
       Uint8 m_senderRefOffset;
       Uint8 m_senderDataOffset;
@@ -109,12 +111,12 @@ private:
   typedef Ptr<ActiveCounter> ActiveCounterPtr;
   typedef ArrayPool<ActiveCounter> ActiveCounter_pool;
   typedef DLList<ActiveCounter_pool> ActiveCounter_list;
-  
-  bool seize(ActiveCounterPtr& ptr);
-  void release(ActiveCounterPtr& ptr);
-  void getPtr(ActiveCounterPtr& ptr, Uint32 ptrI) const;
 
-  SimulatedBlock & m_block;
+  bool seize(ActiveCounterPtr &ptr);
+  void release(ActiveCounterPtr &ptr);
+  void getPtr(ActiveCounterPtr &ptr, Uint32 ptrI) const;
+
+  SimulatedBlock &m_block;
   ActiveCounter_pool m_counterPool;
   ActiveCounter_list m_activeCounters;
 #ifdef ERROR_INSERT
@@ -122,87 +124,80 @@ private:
 #endif
 
   BlockReference reference() const;
-  [[noreturn]] void progError(int line,
-                              int err_code,
-                              const char* extra = 0,
-                              const char* check="");
+  [[noreturn]] void progError(int line, int err_code, const char *extra = 0,
+                              const char *check = "");
 };
 
-
 class SafeCounterHandle {
   friend class SafeCounter;
-public:
+
+ public:
   SafeCounterHandle();
 
   /**
    * Return if done (no nodes set in bitmask)
    */
-  bool clearWaitingFor(SafeCounterManager& mgr, Uint32 nodeId);
-  
+  bool clearWaitingFor(SafeCounterManager &mgr, Uint32 nodeId);
+
   bool done() const;
-  
-private:
+
+ private:
   Uint32 m_activeCounterPtrI;
 };
 
 class SafeCounter {
   friend class SafeCounterManager;
-public:
-  SafeCounter(SafeCounterManager&, SafeCounterHandle&);
-  
-  template<typename SignalClass>
-    bool init(Uint16 block, Uint16 GSN, Uint32 senderData);
-  
-  template<typename SignalClass>
-    bool init(NodeReceiverGroup rg, Uint16 GSN, Uint32 senderData);
 
-  template<typename SignalClass>
-    bool init(NodeReceiverGroup rg, Uint32 senderData);
-  
+ public:
+  SafeCounter(SafeCounterManager &, SafeCounterHandle &);
+
+  template <typename SignalClass>
+  bool init(Uint16 block, Uint16 GSN, Uint32 senderData);
+
+  template <typename SignalClass>
+  bool init(NodeReceiverGroup rg, Uint16 GSN, Uint32 senderData);
+
+  template <typename SignalClass>
+  bool init(NodeReceiverGroup rg, Uint32 senderData);
+
   ~SafeCounter();
-  
+
   void clearWaitingFor();
-  
+
   /**
    * When sending to different node
    */
   void setWaitingFor(Uint32 nodeId);
   bool clearWaitingFor(Uint32 nodeId);
   bool forceClearWaitingFor(Uint32 nodeId);
-  
+
   bool isWaitingFor(Uint32 nodeId) const;
   bool done() const;
 
-  const char * getText() const; /* ? needed for, some portability issues */
+  const char *getText() const; /* ? needed for, some portability issues */
 
-  SafeCounter& operator=(const NdbNodeBitmask&);
-  SafeCounter& operator=(const NodeReceiverGroup&);
-private:
+  SafeCounter &operator=(const NdbNodeBitmask &);
+  SafeCounter &operator=(const NodeReceiverGroup &);
+
+ private:
   Uint32 m_count;
   NdbNodeBitmask m_nodes;
-  
-  SafeCounterManager & m_mgr;
+
+  SafeCounterManager &m_mgr;
   SafeCounterManager::ActiveCounterPtr m_ptr;
-  
-  Uint32 & m_activeCounterPtrI;
+
+  Uint32 &m_activeCounterPtrI;
 };
 
-inline
-SafeCounterHandle::SafeCounterHandle(){
-  m_activeCounterPtrI = RNIL;
-}
+inline SafeCounterHandle::SafeCounterHandle() { m_activeCounterPtrI = RNIL; }
 
-inline
-bool
-SafeCounterHandle::done() const {
+inline bool SafeCounterHandle::done() const {
   return m_activeCounterPtrI == RNIL;
 }
 
-inline
-SafeCounter::SafeCounter(SafeCounterManager& mgr, SafeCounterHandle& handle)
-  : m_mgr(mgr),
-    m_activeCounterPtrI(handle.m_activeCounterPtrI)
-{
+inline SafeCounter::SafeCounter(SafeCounterManager &mgr,
+                                SafeCounterHandle &handle)
+    : m_mgr(mgr), m_activeCounterPtrI(handle.m_activeCounterPtrI) {
   m_ptr.i = handle.m_activeCounterPtrI;
   if (m_ptr.i == RNIL) {
     m_nodes.clear();
@@ -214,11 +209,8 @@ SafeCounter::SafeCounter(SafeCounterManager& mgr, SafeCounterHandle& handle)
   }
 }
 
-template<typename Ref>
-inline
-bool
-SafeCounter::init(Uint16 block, Uint16 GSN, Uint32 senderData){
-  
+template <typename Ref>
+inline bool SafeCounter::init(Uint16 block, Uint16 GSN, Uint32 senderData) {
   SafeCounterManager::ActiveCounter::SignalDesc signalDesc;
   signalDesc.m_gsn = GSN;
   signalDesc.m_block = block;
@@ -227,10 +219,10 @@ SafeCounter::init(Uint16 block, Uint16 GSN, Uint32 senderData){
   signalDesc.m_senderDataOffset = offsetof(Ref, senderData) >> 2;
   signalDesc.m_nodeFailErrorCode = Ref::NF_FakeErrorREF;
   assert(((Uint32)Ref::NF_FakeErrorREF) < 256);
-  
-  if(m_ptr.i == RNIL){
+
+  if (m_ptr.i == RNIL) {
     SafeCounterManager::ActiveCounterPtr ptr;
-    if(m_mgr.seize(ptr)){
+    if (m_mgr.seize(ptr)) {
       ptr.p->m_senderData = senderData;
       ptr.p->m_signalDesc = signalDesc;
       m_ptr = ptr;
@@ -239,60 +231,49 @@ SafeCounter::init(Uint16 block, Uint16 GSN, Uint32 senderData){
     return false;
   }
 
-  if(m_count == 0){
+  if (m_count == 0) {
     m_ptr.p->m_senderData = senderData;
     m_ptr.p->m_signalDesc = signalDesc;
     return true;
-  } 
+  }
 
-  ErrorReporter::handleAssert("SafeCounter::init twice", __FILE__, __LINE__);  
+  ErrorReporter::handleAssert("SafeCounter::init twice", __FILE__, __LINE__);
   return false;
 }
 
-template<typename Ref>
-inline
-bool
-SafeCounter::init(NodeReceiverGroup rg, Uint16 GSN, Uint32 senderData){
-  
-  if (init<Ref>(rg.m_block, GSN, senderData))
-  {
+template <typename Ref>
+inline bool SafeCounter::init(NodeReceiverGroup rg, Uint16 GSN,
+                              Uint32 senderData) {
+  if (init<Ref>(rg.m_block, GSN, senderData)) {
     m_nodes = rg.m_nodes;
     m_count = m_nodes.count();
 
-    if (unlikely(m_count == 0))
-    {
-      ErrorReporter::handleAssert("SafeCounter::empty node list",
-                                  __FILE__, __LINE__);
+    if (unlikely(m_count == 0)) {
+      ErrorReporter::handleAssert("SafeCounter::empty node list", __FILE__,
+                                  __LINE__);
     }
     return true;
   }
   return false;
 }
 
-template<typename Ref>
-inline
-bool
-SafeCounter::init(NodeReceiverGroup rg, Uint32 senderData)
-{
-  if (init<Ref>(rg.m_block, Ref::GSN, senderData))
-  {
+template <typename Ref>
+inline bool SafeCounter::init(NodeReceiverGroup rg, Uint32 senderData) {
+  if (init<Ref>(rg.m_block, Ref::GSN, senderData)) {
     m_nodes = rg.m_nodes;
     m_count = m_nodes.count();
 
-    if (unlikely(m_count == 0))
-    {
-      ErrorReporter::handleAssert("SafeCounter::empty node list",
-                                  __FILE__, __LINE__);
+    if (unlikely(m_count == 0)) {
+      ErrorReporter::handleAssert("SafeCounter::empty node list", __FILE__,
+                                  __LINE__);
     }
     return true;
   }
   return false;
 }
 
-inline
-void 
-SafeCounter::setWaitingFor(Uint32 nodeId) {
-  if(!m_nodes.get(nodeId)){
+inline void SafeCounter::setWaitingFor(Uint32 nodeId) {
+  if (!m_nodes.get(nodeId)) {
     m_nodes.set(nodeId);
     m_count++;
     return;
@@ -300,22 +281,14 @@ SafeCounter::setWaitingFor(Uint32 nodeId) {
   ErrorReporter::handleAssert("SafeCounter::set", __FILE__, __LINE__);
 }
 
-inline
-bool
-SafeCounter::isWaitingFor(Uint32 nodeId) const {
+inline bool SafeCounter::isWaitingFor(Uint32 nodeId) const {
   return m_nodes.get(nodeId);
 }
 
-inline
-bool
-SafeCounter::done() const {
-  return m_count == 0;
-}
+inline bool SafeCounter::done() const { return m_count == 0; }
 
-inline
-bool
-SafeCounter::clearWaitingFor(Uint32 nodeId) {
-  if(m_count > 0 && m_nodes.get(nodeId)){
+inline bool SafeCounter::clearWaitingFor(Uint32 nodeId) {
+  if (m_count > 0 && nodeId <= MAX_DATA_NODE_ID && m_nodes.get(nodeId)) {
     m_count--;
     m_nodes.clear(nodeId);
     return (m_count == 0);
@@ -324,23 +297,18 @@ SafeCounter::clearWaitingFor(Uint32 nodeId) {
   return false;
 }
 
-inline
-void
-SafeCounter::clearWaitingFor(){
+inline void SafeCounter::clearWaitingFor() {
   m_count = 0;
   m_nodes.clear();
 }
 
-inline
-bool
-SafeCounter::forceClearWaitingFor(Uint32 nodeId){
-  if(isWaitingFor(nodeId)){
+inline bool SafeCounter::forceClearWaitingFor(Uint32 nodeId) {
+  if (isWaitingFor(nodeId)) {
     return clearWaitingFor(nodeId);
   }
   return (m_count == 0);
 }
 
-
 #undef JAM_FILE_ID
 
 #endif
