@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,47 +32,42 @@
 
 #define JAM_FILE_ID 247
 
-
 #define MAX_NO_OF_ZERO_TQ 128
 #define MAX_NO_OF_SHORT_TQ 512
 #define MAX_NO_OF_LONG_TQ 512
-#define MAX_NO_OF_TQ (MAX_NO_OF_ZERO_TQ + MAX_NO_OF_SHORT_TQ + \
-                      MAX_NO_OF_LONG_TQ)
+#define MAX_NO_OF_TQ \
+  (MAX_NO_OF_ZERO_TQ + MAX_NO_OF_SHORT_TQ + MAX_NO_OF_LONG_TQ)
 #define NULL_TQ_ENTRY 65535
 
-struct TimeStruct
-{
+struct TimeStruct {
   Uint16 delay_time;
   Uint16 job_index;
 };
 
-union TimerEntry
-{
+union TimerEntry {
   struct TimeStruct time_struct;
   Uint32 copy_struct;
 };
 
-class TimeQueue
-{
-public:
+class TimeQueue {
+ public:
   TimeQueue();
   ~TimeQueue();
 
-  void   insert(Signal25* signal, Uint32 delayTime);
-  void   clear();
-  void   scanTable(); // Called once per millisecond
-  void   scanZeroTimeQueue(); // Called after each doJob call
+  void insert(Signal25 *signal, Uint32 delayTime);
+  void clear();
+  void scanTable();          // Called once per millisecond
+  void scanZeroTimeQueue();  // Called after each doJob call
   Uint32 getIndex();
-  void   releaseIndex(Uint32 aIndex);
-  void   recount_timers();
-  
-private:
-  TimerEntry  theZeroQueue[MAX_NO_OF_ZERO_TQ];
-  TimerEntry  theShortQueue[MAX_NO_OF_SHORT_TQ];
-  TimerEntry  theLongQueue[MAX_NO_OF_LONG_TQ];
-  Uint16     theFreeIndex[MAX_NO_OF_TQ];
-};
+  void releaseIndex(Uint32 aIndex);
+  void recount_timers();
 
+ private:
+  TimerEntry theZeroQueue[MAX_NO_OF_ZERO_TQ];
+  TimerEntry theShortQueue[MAX_NO_OF_SHORT_TQ];
+  TimerEntry theLongQueue[MAX_NO_OF_LONG_TQ];
+  Uint16 theFreeIndex[MAX_NO_OF_TQ];
+};
 
 #undef JAM_FILE_ID
 
