@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,11 +36,14 @@
 #include "my_alloc.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
+#include "my_inttypes.h"
 #include "my_sys.h"
 #include "mysql/components/services/pfs_plugin_table_service.h"
 #include "mysql/plugin.h"
 #include "mysql/status_var.h"
+#include "mysql/strings/int2str.h"
 #include "mysqld_error.h"
+#include "nulls.h"
 #include "sql/hostname_cache.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
@@ -107,7 +111,7 @@ handlerton *pfs_hton = nullptr;
 #define PFS_ENABLED() \
   (pfs_initialized && (pfs_enabled || m_table_share->m_perpetual))
 
-#define IS_NATIVE_TABLE(X) ((X)->m_st_table.open_table == NULL) ? true : false
+#define IS_NATIVE_TABLE(X) ((X)->m_st_table.open_table == NULL)
 
 static void lock_pfs_external_table_shares() {
   if (!opt_initialize) {
@@ -127,7 +131,7 @@ static handler *pfs_create_handler(handlerton *hton, TABLE_SHARE *table, bool,
 }
 
 static size_t size_of_global_error_stat_buffer() {
-  size_t size = sizeof(PFS_error_single_stat) * max_global_server_errors;
+  const size_t size = sizeof(PFS_error_single_stat) * max_global_server_errors;
   return size;
 }
 
@@ -1295,11 +1299,38 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
         name = "(max_session_server_errors).count";
         size = max_session_server_errors;
         break;
+      case 247:
+        name = "(pfs_meter_class).size";
+        size = sizeof(PFS_meter_class);
+        break;
+      case 248:
+        name = "(pfs_meter_class).count";
+        size = meter_class_max;
+        break;
+      case 249:
+        name = "(pfs_meter_class).memory";
+        size = meter_class_max * sizeof(PFS_meter_class);
+        total_memory += size;
+        break;
+      case 250:
+        name = "(pfs_metric_class).size";
+        size = sizeof(PFS_metric_class);
+        break;
+      case 251:
+        name = "(pfs_metric_class).count";
+        size = metric_class_max;
+        break;
+      case 252:
+        name = "(pfs_metric_class).memory";
+        size = metric_class_max * sizeof(PFS_metric_class);
+        total_memory += size;
+        break;
+
       /*
         This case must be last,
         for aggregation in total_memory.
       */
-      case 247:
+      case 253:
         name = "performance_schema.memory";
         size = total_memory;
         break;
@@ -1552,6 +1583,10 @@ static SHOW_VAR pfs_status_vars[] = {
      SHOW_SCOPE_GLOBAL},
     {"Performance_schema_metadata_lock_lost",
      (char *)&global_mdl_container.m_lost, SHOW_LONG, SHOW_SCOPE_GLOBAL},
+    {"Performance_schema_meter_lost", (char *)&meter_class_lost, SHOW_LONG,
+     SHOW_SCOPE_GLOBAL},
+    {"Performance_schema_metric_lost", (char *)&metric_class_lost, SHOW_LONG,
+     SHOW_SCOPE_GLOBAL},
     {NullS, NullS, SHOW_LONG, SHOW_SCOPE_GLOBAL}};
 
 struct st_mysql_storage_engine pfs_storage_engine = {
@@ -1609,7 +1644,7 @@ int ha_perfschema::open(const char *, int, uint, const dd::Table *) {
   return 0;
 }
 
-int ha_perfschema::close(void) {
+int ha_perfschema::close() {
   DBUG_TRACE;
 
   /* Only for table added by plugin/components */
@@ -1641,7 +1676,7 @@ int ha_perfschema::write_row(uchar *buf) {
   return result;
 }
 
-void ha_perfschema::use_hidden_primary_key(void) {
+void ha_perfschema::use_hidden_primary_key() {
   /*
     This is also called in case of row based replication,
     see TABLE::mark_columns_needed_for_update().
@@ -1663,7 +1698,8 @@ int ha_perfschema::update_row(const uchar *old_data, uchar *new_data) {
 
   assert(m_table);
   ha_statistic_increment(&System_status_var::ha_update_count);
-  int result = m_table->update_row(table, old_data, new_data, table->field);
+  const int result =
+      m_table->update_row(table, old_data, new_data, table->field);
   return result;
 }
 
@@ -1675,7 +1711,7 @@ int ha_perfschema::delete_row(const uchar *buf) {
 
   assert(m_table);
   ha_statistic_increment(&System_status_var::ha_delete_count);
-  int result = m_table->delete_row(table, buf, table->field);
+  const int result = m_table->delete_row(table, buf, table->field);
   return result;
 }
 
@@ -1701,7 +1737,7 @@ int ha_perfschema::rnd_init(bool scan) {
   return result;
 }
 
-int ha_perfschema::rnd_end(void) {
+int ha_perfschema::rnd_end() {
   DBUG_TRACE;
   assert(m_table);
   delete m_table;
@@ -1762,7 +1798,7 @@ int ha_perfschema::info(uint flag) {
   return 0;
 }
 
-int ha_perfschema::delete_all_rows(void) {
+int ha_perfschema::delete_all_rows() {
   int result;
 
   DBUG_TRACE;
@@ -1873,7 +1909,7 @@ ulong ha_perfschema::index_flags(uint, uint, bool) const {
     /* ha_perfschema::index_flags is const, can not save in m_table_share. */
   }
 
-  ulong flags = HA_KEY_SCAN_NOT_ROR;
+  const ulong flags = HA_KEY_SCAN_NOT_ROR;
 
   if (!tmp) {
     unlock_pfs_external_table_shares();
