@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,6 +41,7 @@
 #include "my_sys.h"
 #include "my_systime.h"
 #include "mysql/psi/mysql_thread.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/mysqld.h"  // lower_case_table_names
 #include "sql/table.h"
 #include "storage/perfschema/pfs_buffer_container.h"
@@ -97,6 +99,10 @@ static std::atomic<uint32> rwlock_class_dirty_count{0};
 static std::atomic<uint32> rwlock_class_allocated_count{0};
 static std::atomic<uint32> cond_class_dirty_count{0};
 static std::atomic<uint32> cond_class_allocated_count{0};
+static std::atomic<uint32> meter_class_dirty_count{0};
+static std::atomic<uint32> meter_class_allocated_count{0};
+static std::atomic<uint32> metric_class_dirty_count{0};
+static std::atomic<uint32> metric_class_allocated_count{0};
 
 /** Size of the mutex class array. @sa mutex_class_array */
 ulong mutex_class_max = 0;
@@ -134,6 +140,14 @@ ulong socket_class_lost = 0;
 ulong memory_class_max = 0;
 /** Number of memory class lost. @sa memory_class_array */
 ulong memory_class_lost = 0;
+/** Size of the meter class array. @sa meter_class_array */
+ulong meter_class_max = 0;
+/** Number of meter class lost. @sa meter_class_array */
+ulong meter_class_lost = 0;
+/** Size of the metric class array. @sa metric_class_array */
+ulong metric_class_max = 0;
+/** Number of metric class lost. @sa metric_class_array */
+ulong metric_class_lost = 0;
 
 /**
   Number of transaction classes. Although there is only one transaction class,
@@ -152,6 +166,8 @@ ulong error_class_max = 0;
 PFS_mutex_class *mutex_class_array = nullptr;
 PFS_rwlock_class *rwlock_class_array = nullptr;
 PFS_cond_class *cond_class_array = nullptr;
+PFS_meter_class *meter_class_array = nullptr;
+PFS_metric_class *metric_class_array = nullptr;
 
 /**
   Current number or elements in thread_class_array.
@@ -176,6 +192,8 @@ PFS_ALIGNED PFS_instr_class global_idle_class;
 PFS_ALIGNED PFS_instr_class global_metadata_class;
 PFS_ALIGNED PFS_error_class global_error_class;
 PFS_ALIGNED PFS_transaction_class global_transaction_class;
+PFS_ALIGNED PFS_meter_class global_meter_class;
+PFS_ALIGNED PFS_metric_class global_metric_class;
 
 /**
   Hash index for instrumented table shares.
@@ -227,16 +245,14 @@ const char *PFS_instr_name::str() const {
   if (m_private_old_name != nullptr &&
       terminology_use_previous::is_older_required(m_private_version))
     return m_private_old_name;
-  else
-    return m_private_name;
+  return m_private_name;
 }
 
 uint PFS_instr_name::length() const {
   if (m_private_old_name != nullptr &&
       terminology_use_previous::is_older_required(m_private_version))
     return m_private_old_name_length;
-  else
-    return m_private_name_length;
+  return m_private_name_length;
 }
 
 /**
@@ -260,7 +276,7 @@ static uint safe_strlen(const char *s, uint max_len) {
 void PFS_instr_name::set(PFS_class_type class_type, const char *name,
                          uint max_length_arg) {
   // Copy the given name to the member.
-  uint length = safe_strlen(name, std::min(max_length, max_length_arg));
+  const uint length = safe_strlen(name, std::min(max_length, max_length_arg));
   memcpy(m_private_name, name, length);
   m_private_name[length] = '\0';
   m_private_name_length = length;
@@ -360,6 +376,7 @@ int init_sync_class(uint mutex_class_sizing, uint rwlock_class_sizing,
         &builtin_memory_mutex_class, mutex_class_max, sizeof(PFS_mutex_class),
         PFS_mutex_class, MYF(MY_ZEROFILL));
     if (unlikely(mutex_class_array == nullptr)) {
+      mutex_class_max = 0;
       return 1;
     }
   }
@@ -369,6 +386,7 @@ int init_sync_class(uint mutex_class_sizing, uint rwlock_class_sizing,
         &builtin_memory_rwlock_class, rwlock_class_max,
         sizeof(PFS_rwlock_class), PFS_rwlock_class, MYF(MY_ZEROFILL));
     if (unlikely(rwlock_class_array == nullptr)) {
+      rwlock_class_max = 0;
       return 1;
     }
   }
@@ -378,6 +396,7 @@ int init_sync_class(uint mutex_class_sizing, uint rwlock_class_sizing,
                                         cond_class_max, sizeof(PFS_cond_class),
                                         PFS_cond_class, MYF(MY_ZEROFILL));
     if (unlikely(cond_class_array == nullptr)) {
+      cond_class_max = 0;
       return 1;
     }
   }
@@ -386,7 +405,7 @@ int init_sync_class(uint mutex_class_sizing, uint rwlock_class_sizing,
 }
 
 /** Cleanup the instrument synch class buffers. */
-void cleanup_sync_class(void) {
+void cleanup_sync_class() {
   unsigned int i;
 
   if (mutex_class_array != nullptr) {
@@ -440,6 +459,7 @@ int init_thread_class(uint thread_class_sizing) {
         &builtin_memory_thread_class, thread_class_max,
         sizeof(PFS_thread_class), PFS_thread_class, MYF(MY_ZEROFILL));
     if (unlikely(thread_class_array == nullptr)) {
+      thread_class_max = 0;
       result = 1;
     }
   } else {
@@ -450,7 +470,7 @@ int init_thread_class(uint thread_class_sizing) {
 }
 
 /** Cleanup the thread class buffers. */
-void cleanup_thread_class(void) {
+void cleanup_thread_class() {
   unsigned int i;
 
   if (thread_class_array != nullptr) {
@@ -479,8 +499,98 @@ int init_table_share(uint table_share_sizing) {
   return 0;
 }
 
+/**
+  Initialize the meter class buffer.
+  @param meter_class_sizing          max number of meter class
+  @return 0 on success
+*/
+int init_meter_class(uint meter_class_sizing) {
+  int result = 0;
+  meter_class_dirty_count = meter_class_allocated_count = 0;
+  meter_class_max = meter_class_sizing;
+  meter_class_lost = 0;
+
+  if (meter_class_max > 0) {
+    meter_class_array = PFS_MALLOC_ARRAY(
+        &builtin_memory_meter_class, meter_class_max, sizeof(PFS_meter_class),
+        PFS_meter_class, MYF(MY_ZEROFILL));
+    if (unlikely(meter_class_array == nullptr)) {
+      meter_class_max = 0;
+      result = 1;
+    }
+  } else {
+    meter_class_array = nullptr;
+  }
+
+  return result;
+}
+
+/** Cleanup the meter class buffers. */
+void cleanup_meter_class() {
+  unsigned int i;
+
+  if (meter_class_array != nullptr) {
+    for (i = 0; i < meter_class_max; i++) {
+      my_free(meter_class_array[i].m_documentation);
+      PFS_FREE_ARRAY(&builtin_memory_meter_class, metric_class_max,
+                     sizeof(PFS_metric_key), meter_class_array[i].m_metrics);
+      meter_class_array[i].m_metrics = nullptr;
+      meter_class_array[i].m_metrics_size = 0;
+    }
+  }
+
+  PFS_FREE_ARRAY(&builtin_memory_meter_class, meter_class_max,
+                 sizeof(PFS_meter_class), meter_class_array);
+  meter_class_array = nullptr;
+  meter_class_dirty_count = meter_class_allocated_count = 0;
+  meter_class_max = 0;
+}
+
+/**
+  Initialize the metric class buffer.
+  @param metric_class_sizing          max number of metric class
+  @return 0 on success
+*/
+int init_metric_class(uint metric_class_sizing) {
+  int result = 0;
+  metric_class_dirty_count = metric_class_allocated_count = 0;
+  metric_class_max = metric_class_sizing;
+  metric_class_lost = 0;
+
+  if (metric_class_max > 0) {
+    metric_class_array = PFS_MALLOC_ARRAY(
+        &builtin_memory_metric_class, metric_class_max,
+        sizeof(PFS_metric_class), PFS_metric_class, MYF(MY_ZEROFILL));
+    if (unlikely(metric_class_array == nullptr)) {
+      metric_class_max = 0;
+      result = 1;
+    }
+  } else {
+    metric_class_array = nullptr;
+  }
+
+  return result;
+}
+
+/** Cleanup the metric class buffers. */
+void cleanup_metric_class() {
+  unsigned int i;
+
+  if (metric_class_array != nullptr) {
+    for (i = 0; i < metric_class_max; i++) {
+      my_free(metric_class_array[i].m_documentation);
+    }
+  }
+
+  PFS_FREE_ARRAY(&builtin_memory_metric_class, metric_class_max,
+                 sizeof(PFS_metric_class), metric_class_array);
+  metric_class_array = nullptr;
+  metric_class_dirty_count = metric_class_allocated_count = 0;
+  metric_class_max = 0;
+}
+
 /** Cleanup the table share buffers. */
-void cleanup_table_share(void) { global_table_share_container.cleanup(); }
+void cleanup_table_share() { global_table_share_container.cleanup(); }
 
 /** get_key function for @c table_share_hash. */
 static const uchar *table_share_hash_get_key(const uchar *entry,
@@ -561,7 +671,7 @@ int init_table_share_hash(const PFS_global_param *param) {
 }
 
 /** Cleanup the table share hash table. */
-void cleanup_table_share_hash(void) {
+void cleanup_table_share_hash() {
   if (table_share_hash_inited) {
     lf_hash_destroy(&table_share_hash);
     table_share_hash_inited = false;
@@ -612,7 +722,7 @@ static void set_table_share_key(PFS_table_share_key *key, bool temporary,
   @return a table share lock.
 */
 PFS_table_share_lock *PFS_table_share::find_lock_stat() const {
-  PFS_table_share *that = const_cast<PFS_table_share *>(this);
+  auto *that = const_cast<PFS_table_share *>(this);
   return that->m_race_lock_stat.load();
 }
 
@@ -756,11 +866,10 @@ PFS_table_share_lock *create_table_share_lock_stat() {
 void release_table_share_lock_stat(PFS_table_share_lock *pfs) {
   pfs->m_owner = nullptr;
   global_table_share_lock_container.deallocate(pfs);
-  return;
 }
 
 /** Cleanup the table stat buffers. */
-void cleanup_table_share_lock_stat(void) {
+void cleanup_table_share_lock_stat() {
   global_table_share_lock_container.cleanup();
 }
 
@@ -795,7 +904,7 @@ PFS_table_share_index *create_table_share_index_stat(
       pfs->m_key.m_name_length = 0;
     } else {
       KEY *key_info = server_share->key_info + server_index;
-      size_t len = strlen(key_info->name);
+      const size_t len = strlen(key_info->name);
 
       memcpy(pfs->m_key.m_name, key_info->name, len);
       pfs->m_key.m_name_length = len;
@@ -815,11 +924,10 @@ PFS_table_share_index *create_table_share_index_stat(
 void release_table_share_index_stat(PFS_table_share_index *pfs) {
   pfs->m_owner = nullptr;
   global_table_share_index_container.deallocate(pfs);
-  return;
 }
 
 /** Cleanup the table stat buffers. */
-void cleanup_table_share_index_stat(void) {
+void cleanup_table_share_index_stat() {
   global_table_share_index_container.cleanup();
 }
 
@@ -829,7 +937,6 @@ void cleanup_table_share_index_stat(void) {
   @return 0 on success
 */
 int init_file_class(uint file_class_sizing) {
-  int result = 0;
   file_class_dirty_count = file_class_allocated_count = 0;
   file_class_max = file_class_sizing;
   file_class_lost = 0;
@@ -839,17 +946,18 @@ int init_file_class(uint file_class_sizing) {
                                         file_class_max, sizeof(PFS_file_class),
                                         PFS_file_class, MYF(MY_ZEROFILL));
     if (unlikely(file_class_array == nullptr)) {
+      file_class_max = 0;
       return 1;
     }
   } else {
     file_class_array = nullptr;
   }
 
-  return result;
+  return 0;
 }
 
 /** Cleanup the file class buffers. */
-void cleanup_file_class(void) {
+void cleanup_file_class() {
   unsigned int i;
 
   if (file_class_array != nullptr) {
@@ -871,7 +979,6 @@ void cleanup_file_class(void) {
   @return 0 on success
 */
 int init_stage_class(uint stage_class_sizing) {
-  int result = 0;
   stage_class_dirty_count = stage_class_allocated_count = 0;
   stage_class_max = stage_class_sizing;
   stage_class_lost = 0;
@@ -881,17 +988,18 @@ int init_stage_class(uint stage_class_sizing) {
         &builtin_memory_stage_class, stage_class_max, sizeof(PFS_stage_class),
         PFS_stage_class, MYF(MY_ZEROFILL));
     if (unlikely(stage_class_array == nullptr)) {
+      stage_class_max = 0;
       return 1;
     }
   } else {
     stage_class_array = nullptr;
   }
 
-  return result;
+  return 0;
 }
 
 /** Cleanup the stage class buffers. */
-void cleanup_stage_class(void) {
+void cleanup_stage_class() {
   unsigned int i;
 
   if (stage_class_array != nullptr) {
@@ -913,7 +1021,6 @@ void cleanup_stage_class(void) {
   @return 0 on success
 */
 int init_statement_class(uint statement_class_sizing) {
-  int result = 0;
   statement_class_dirty_count = statement_class_allocated_count = 0;
   statement_class_max = statement_class_sizing;
   statement_class_lost = 0;
@@ -923,17 +1030,18 @@ int init_statement_class(uint statement_class_sizing) {
         &builtin_memory_statement_class, statement_class_max,
         sizeof(PFS_statement_class), PFS_statement_class, MYF(MY_ZEROFILL));
     if (unlikely(statement_class_array == nullptr)) {
+      statement_class_max = 0;
       return 1;
     }
   } else {
     statement_class_array = nullptr;
   }
 
-  return result;
+  return 0;
 }
 
 /** Cleanup the statement class buffers. */
-void cleanup_statement_class(void) {
+void cleanup_statement_class() {
   unsigned int i;
 
   if (statement_class_array != nullptr) {
@@ -955,7 +1063,6 @@ void cleanup_statement_class(void) {
   @return 0 on success
 */
 int init_socket_class(uint socket_class_sizing) {
-  int result = 0;
   socket_class_dirty_count = socket_class_allocated_count = 0;
   socket_class_max = socket_class_sizing;
   socket_class_lost = 0;
@@ -965,17 +1072,18 @@ int init_socket_class(uint socket_class_sizing) {
         &builtin_memory_socket_class, socket_class_max,
         sizeof(PFS_socket_class), PFS_socket_class, MYF(MY_ZEROFILL));
     if (unlikely(socket_class_array == nullptr)) {
+      socket_class_max = 0;
       return 1;
     }
   } else {
     socket_class_array = nullptr;
   }
 
-  return result;
+  return 0;
 }
 
 /** Cleanup the socket class buffers. */
-void cleanup_socket_class(void) {
+void cleanup_socket_class() {
   unsigned int i;
 
   if (socket_class_array != nullptr) {
@@ -997,7 +1105,6 @@ void cleanup_socket_class(void) {
   @return 0 on success
 */
 int init_memory_class(uint memory_class_sizing) {
-  int result = 0;
   memory_class_dirty_count = memory_class_allocated_count = 0;
   memory_class_max = memory_class_sizing;
   memory_class_lost = 0;
@@ -1007,17 +1114,18 @@ int init_memory_class(uint memory_class_sizing) {
         &builtin_memory_memory_class, memory_class_max,
         sizeof(PFS_memory_class), PFS_memory_class, MYF(MY_ZEROFILL));
     if (unlikely(memory_class_array.load() == nullptr)) {
+      memory_class_max = 0;
       return 1;
     }
   } else {
     memory_class_array = nullptr;
   }
 
-  return result;
+  return 0;
 }
 
 /** Cleanup the memory class buffers. */
-void cleanup_memory_class(void) {
+void cleanup_memory_class() {
   unsigned int i;
 
   if (memory_class_array.load() != nullptr) {
@@ -1112,6 +1220,19 @@ static void configure_instr_class(PFS_instr_class *entry) {
     }                                                                  \
   }
 
+// Changes compared to above version:
+// - additionally checks for valid key
+// - return UINT_MAX key on error
+#define REGISTER_CLASS_BODY_PART_V2(INDEX, ARRAY, MAX, NAME, NAME_LENGTH) \
+  for (INDEX = 0; INDEX < MAX; ++INDEX) {                                 \
+    entry = &ARRAY[INDEX];                                                \
+    if ((entry->m_key > 0) && (entry->m_name.length() == NAME_LENGTH) &&  \
+        (strncmp(entry->m_name.str(), NAME, NAME_LENGTH) == 0)) {         \
+      assert(entry->m_flags == info->m_flags);                            \
+      return UINT_MAX;                                                    \
+    }                                                                     \
+  }
+
 /**
   Register a mutex instrumentation metadata.
   @param name                         the instrumented name
@@ -1304,6 +1425,13 @@ PFS_sync_key register_cond_class(const char *name, uint name_length,
   }                                        \
   return &ARRAY[KEY - 1]
 
+// Additionally check if this slot has valid key
+#define FIND_CLASS_BODY_V2(KEY, COUNT, ARRAY)                       \
+  if ((KEY == 0) || (KEY > COUNT) || (ARRAY[KEY - 1].m_key == 0)) { \
+    return NULL;                                                    \
+  }                                                                 \
+  return &ARRAY[KEY - 1]
+
 /**
   Find a mutex instrumentation class by key.
   @param key                          the instrument key
@@ -1345,6 +1473,34 @@ PFS_cond_class *sanitize_cond_class(PFS_cond_class *unsafe) {
   SANITIZE_ARRAY_BODY(PFS_cond_class, cond_class_array, cond_class_max, unsafe);
 }
 
+/**
+  Find a meter instrumentation class by key.
+  @param key                          the instrument key
+  @return the instrument class, or NULL
+*/
+PFS_meter_class *find_meter_class(PSI_meter_key key) {
+  FIND_CLASS_BODY_V2(key, meter_class_max, meter_class_array);
+}
+
+PFS_meter_class *sanitize_meter_class(PFS_meter_class *unsafe) {
+  SANITIZE_ARRAY_BODY(PFS_meter_class, meter_class_array, meter_class_max,
+                      unsafe);
+}
+
+/**
+  Find a metric instrumentation class by key.
+  @param key                          the instrument key
+  @return the instrument class, or NULL
+*/
+PFS_metric_class *find_metric_class(PSI_metric_key key) {
+  FIND_CLASS_BODY_V2(key, metric_class_max, metric_class_array);
+}
+
+PFS_metric_class *sanitize_metric_class(PFS_metric_class *unsafe) {
+  SANITIZE_ARRAY_BODY(PFS_metric_class, metric_class_array, metric_class_max,
+                      unsafe);
+}
+
 /**
   Register a thread instrumentation metadata.
   @param name                         the instrumented name
@@ -1393,8 +1549,8 @@ PFS_thread_key register_thread_class(const char *name, uint name_length,
       */
       assert(strlen(info->m_os_name) < PFS_MAX_OS_NAME_LENGTH - 3);
 
-      snprintf(entry->m_os_name, PFS_MAX_OS_NAME_LENGTH, "%s-%%d",
-               info->m_os_name);
+      (void)snprintf(entry->m_os_name, PFS_MAX_OS_NAME_LENGTH, "%s-%%d",
+                     info->m_os_name);
     } else {
       assert(strlen(info->m_os_name) < PFS_MAX_OS_NAME_LENGTH);
       strncpy(entry->m_os_name, info->m_os_name, PFS_MAX_OS_NAME_LENGTH);
@@ -1542,10 +1698,27 @@ PFS_statement_key register_statement_class(const char *name, uint name_length,
                      0, /* statements have no volatility */
                      info->m_documentation, PFS_CLASS_STATEMENT);
     entry->m_event_name_index = index;
-    entry->m_enabled = true; /* enabled by default */
-    entry->m_timed = true;
 
-    entry->enforce_valid_flags(PSI_FLAG_MUTABLE);
+    entry->enforce_valid_flags(PSI_FLAG_MUTABLE | PSI_FLAG_DISABLED |
+                               PSI_FLAG_UNTIMED);
+
+    if (entry->is_disabled()) {
+      /*
+        The instrumentation is PSI_FLAG_DISABLED.
+      */
+      entry->m_enabled = false;
+    } else {
+      entry->m_enabled = true;
+    }
+
+    if (entry->is_untimed()) {
+      /*
+        The instrumentation is PSI_FLAG_UNTIMED.
+      */
+      entry->m_timed = false;
+    } else {
+      entry->m_timed = true;
+    }
 
     /* Set user-defined configuration options for this instrument */
     configure_instr_class(entry);
@@ -1712,6 +1885,230 @@ PFS_memory_class *sanitize_memory_class(PFS_memory_class *unsafe) {
                       memory_class_max, unsafe);
 }
 
+/**
+  Register a meter instrumentation metadata.
+  @param name                         the instrumented name
+  @param name_length                  length in bytes of name
+  @param info                         the instrumentation properties
+  @return a meter instrumentation key, 0 on error, UINT_MAX on duplicate
+*/
+PFS_meter_key register_meter_class(const char *name, uint name_length,
+                                   PSI_meter_info_v1 *info) {
+  /* See comments in register_mutex_class */
+  uint32 index;
+  PFS_meter_class *entry;
+
+  REGISTER_CLASS_BODY_PART_V2(index, meter_class_array, meter_class_max, name,
+                              name_length)
+
+  // search free slot
+  if (meter_class_dirty_count < meter_class_max &&
+      meter_class_array[meter_class_dirty_count].m_key == 0) {
+    // fast path
+    index = meter_class_dirty_count;
+  } else {
+    index = meter_class_max;
+    for (uint32 i = 0; i < meter_class_max; i++) {
+      if (meter_class_array[i].m_key == 0) {
+        index = i;
+        break;
+      }
+    }
+  }
+
+  if (index < meter_class_max) {
+    const PFS_meter_key key = index + 1;
+
+    entry = &meter_class_array[index];
+
+    // init entry lock
+    pfs_dirty_state dirty_state;
+    if (!entry->m_lock.free_to_dirty(&dirty_state)) {
+      if (pfs_enabled) {
+        meter_class_lost++;
+      }
+      return 0;
+    }
+    entry->m_lock.dirty_to_allocated(&dirty_state);
+
+    init_instr_class(entry, name, name_length, 0,
+                     0, /* statements have no volatility */
+                     PSI_DOCUMENT_ME, PFS_CLASS_METRIC);
+    entry->m_event_name_index = index;
+    entry->m_enabled = true; /* enabled by default */
+    entry->m_timed = true;
+
+    // storage for metric keys allocated once per slot
+    // on a when-needed bases (instead of doing it in init_meter_class)
+    if (metric_class_max > 0 && entry->m_metrics == nullptr) {
+      entry->m_metrics = PFS_MALLOC_ARRAY(
+          &builtin_memory_meter_class, metric_class_max, sizeof(PFS_metric_key),
+          PFS_metric_key, MYF(MY_ZEROFILL));
+    }
+    entry->m_metrics_size = 0;
+
+    // copy metric source info
+    entry->m_meter = info->m_meter;
+    entry->m_meter_length =
+        (info->m_meter == nullptr) ? 0 : strlen(info->m_meter);
+    entry->m_description = info->m_description;
+    entry->m_description_length =
+        (info->m_description == nullptr) ? 0 : strlen(info->m_description);
+    entry->m_frequency = info->m_frequency;
+    entry->m_key = key;
+
+    // update input entry (same structure used on unregister)
+    info->m_key = key;
+
+    entry->enforce_valid_flags(PSI_FLAG_MUTABLE);
+
+    /* Set user-defined configuration options for this instrument */
+    configure_instr_class(entry);
+    ++meter_class_allocated_count;
+
+    if (index == meter_class_dirty_count) meter_class_dirty_count++;
+
+    return key;
+  }
+
+  if (pfs_enabled) {
+    meter_class_lost++;
+  }
+  return 0;
+}
+
+void unregister_meter_class(PSI_meter_info_v1 *info) {
+  assert(info != nullptr);
+
+  if (info->m_key == 0) return;
+
+  const uint32 index = info->m_key - 1;
+  PFS_meter_class *entry = &meter_class_array[index];
+
+  // unregister
+  entry->m_key = 0;
+  info->m_key = 0;
+
+  entry->m_lock.allocated_to_free();
+
+  --meter_class_allocated_count;
+  meter_class_dirty_count = 0;
+}
+
+uint32 meter_class_count() { return meter_class_allocated_count; }
+
+/**
+  Register a metric instrumentation metadata.
+  @param name                         the instrumented name
+  @param name_length                  length in bytes of name
+  @param info                         the instrumentation properties
+  @param meter                        group name for this metric
+  @return a metric instrumentation key, 0 on error, UINT_MAX on duplicate
+*/
+PFS_metric_key register_metric_class(const char *name, uint name_length,
+                                     PSI_metric_info_v1 *info,
+                                     const char *meter) {
+  /* See comments in register_mutex_class */
+  uint32 index;
+  PFS_metric_class *entry;
+
+  REGISTER_CLASS_BODY_PART_V2(index, metric_class_array, metric_class_max, name,
+                              name_length)
+
+  // search free slot
+  if (metric_class_dirty_count < metric_class_max &&
+      metric_class_array[metric_class_dirty_count].m_key == 0) {
+    // fast path
+    index = metric_class_dirty_count;
+  } else {
+    index = metric_class_max;
+    for (uint32 i = 0; i < metric_class_max; i++) {
+      if (metric_class_array[i].m_key == 0) {
+        index = i;
+        break;
+      }
+    }
+  }
+
+  if (index < metric_class_max) {
+    const PFS_metric_key key = index + 1;
+
+    entry = &metric_class_array[index];
+
+    // init entry lock
+    pfs_dirty_state dirty_state;
+    if (!entry->m_lock.free_to_dirty(&dirty_state)) {
+      if (pfs_enabled) {
+        meter_class_lost++;
+      }
+      return 0;
+    }
+    entry->m_lock.dirty_to_allocated(&dirty_state);
+
+    init_instr_class(entry, name, name_length, 0,
+                     0, /* statements have no volatility */
+                     PSI_DOCUMENT_ME, PFS_CLASS_METRIC);
+    entry->m_event_name_index = index;
+    entry->m_enabled = true; /* enabled by default */
+    entry->m_timed = true;
+
+    // copy metric source info
+    entry->m_metric = info->m_metric;
+    entry->m_metric_length =
+        (info->m_metric == nullptr) ? 0 : strlen(info->m_metric);
+    entry->m_group = meter;
+    entry->m_group_length = (meter == nullptr) ? 0 : strlen(meter);
+    entry->m_unit = info->m_unit;
+    entry->m_unit_length = (info->m_unit == nullptr) ? 0 : strlen(info->m_unit);
+    entry->m_description = info->m_description;
+    entry->m_description_length =
+        (info->m_description == nullptr) ? 0 : strlen(info->m_description);
+    entry->m_num_type = info->m_num_type;
+    entry->m_metric_type = info->m_metric_type;
+    entry->m_key = key;
+    entry->m_measurement_callback = info->m_measurement_callback;
+    entry->m_measurement_context = info->m_measurement_context;
+
+    // update input entry (same structure used on unregister)
+    info->m_key = key;
+
+    entry->enforce_valid_flags(PSI_FLAG_MUTABLE);
+
+    /* Set user-defined configuration options for this instrument */
+    configure_instr_class(entry);
+    ++metric_class_allocated_count;
+
+    if (index == metric_class_dirty_count) metric_class_dirty_count++;
+
+    return key;
+  }
+
+  if (pfs_enabled) {
+    metric_class_lost++;
+  }
+  return 0;
+}
+
+void unregister_metric_class(PSI_metric_info_v1 *info) {
+  assert(info != nullptr);
+
+  if (info->m_key == 0) return;
+
+  const uint32 index = info->m_key - 1;
+  PFS_metric_class *entry = &metric_class_array[index];
+
+  // unregister
+  entry->m_key = 0;
+  info->m_key = 0;
+
+  entry->m_lock.allocated_to_free();
+
+  --metric_class_allocated_count;
+  metric_class_dirty_count = 0;
+}
+
+uint32 metric_class_count() { return metric_class_allocated_count; }
+
 PFS_instr_class *find_table_class(uint index) {
   if (index == 1) {
     return &global_table_io_class;
@@ -1794,7 +2191,7 @@ static int compare_keys(PFS_table_share *pfs, const TABLE_SHARE *share) {
 
   size_t len;
   uint index = 0;
-  uint key_count = share->keys;
+  const uint key_count = share->keys;
   KEY *key_info = share->key_info;
   PFS_table_share_index *index_stat;
 
@@ -1835,9 +2232,9 @@ PFS_table_share *find_or_create_table_share(PFS_thread *thread, bool temporary,
   }
 
   const char *schema_name = share->db.str;
-  size_t schema_name_length = share->db.length;
+  const size_t schema_name_length = share->db.length;
   const char *table_name = share->table_name.str;
-  size_t table_name_length = share->table_name.length;
+  const size_t table_name_length = share->table_name.length;
 
   set_table_share_key(&key, temporary, schema_name, schema_name_length,
                       table_name, table_name_length);
@@ -1898,7 +2295,6 @@ search:
     pfs->m_key_count = share->keys;
 
     int res;
-    pfs->m_lock.dirty_to_allocated(&dirty_state);
     res = lf_hash_insert(&table_share_hash, pins, &pfs);
 
     if (likely(res == 0)) {
@@ -1906,10 +2302,11 @@ search:
       for (uint index = 0; index < pfs->m_key_count; index++) {
         (void)pfs->find_or_create_index_stat(share, index);
       }
+      pfs->m_lock.dirty_to_allocated(&dirty_state);
       return pfs;
     }
 
-    global_table_share_container.deallocate(pfs);
+    global_table_share_container.dirty_to_free(&dirty_state, pfs);
 
     if (res > 0) {
       /* Duplicate insert by another thread */
@@ -1929,9 +2326,9 @@ search:
   return nullptr;
 }
 
-void PFS_table_share::aggregate_io(void) {
+void PFS_table_share::aggregate_io() {
   uint index;
-  uint safe_key_count = sanitize_index_count(m_key_count);
+  const uint safe_key_count = sanitize_index_count(m_key_count);
   PFS_table_share_index *from_stat;
   PFS_table_io_stat sum_io;
 
@@ -1989,7 +2386,7 @@ void PFS_table_share::sum(PFS_single_stat *result, uint key_count) {
   sum_lock(result);
 }
 
-void PFS_table_share::aggregate_lock(void) {
+void PFS_table_share::aggregate_lock() {
   PFS_table_share_lock *lock_stat;
   lock_stat = find_lock_stat();
   if (lock_stat != nullptr) {
@@ -2058,7 +2455,7 @@ void reset_events_waits_by_class() {
 }
 
 /** Reset the I/O statistics per file class. */
-void reset_file_class_io(void) {
+void reset_file_class_io() {
   PFS_file_class *pfs = file_class_array;
   PFS_file_class *pfs_last = file_class_array + file_class_max;
 
@@ -2068,7 +2465,7 @@ void reset_file_class_io(void) {
 }
 
 /** Reset the I/O statistics per socket class. */
-void reset_socket_class_io(void) {
+void reset_socket_class_io() {
   PFS_socket_class *pfs = socket_class_array;
   PFS_socket_class *pfs_last = socket_class_array + socket_class_max;
 
@@ -2117,7 +2514,8 @@ void update_program_share_derived_flags(PFS_thread *thread) {
 
 ulonglong gtid_monitoring_getsystime() {
   if (pfs_enabled) {
-    return my_getsystime();
+    // Divide by 10 to get the time in microseconds
+    return my_getsystime() / 10;
   }
   return 0;
 }
