@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2013, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2013, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -53,8 +54,7 @@ static void set_channel_name(void *const, const char &, size_t) {}
 
 static void set_group_name(void *const context, const char &value,
                            size_t length) {
-  struct st_row_connect_status *row =
-      static_cast<struct st_row_connect_status *>(context);
+  auto *row = static_cast<struct st_row_connect_status *>(context);
   const size_t max = UUID_LENGTH;
   length = std::min(length, max);
 
@@ -64,8 +64,7 @@ static void set_group_name(void *const context, const char &value,
 
 static void set_source_uuid(void *const context, const char &value,
                             size_t length) {
-  struct st_row_connect_status *row =
-      static_cast<struct st_row_connect_status *>(context);
+  auto *row = static_cast<struct st_row_connect_status *>(context);
   const size_t max = UUID_LENGTH;
   length = std::min(length, max);
 
@@ -74,8 +73,7 @@ static void set_source_uuid(void *const context, const char &value,
 }
 
 static void set_service_state(void *const context, bool value) {
-  struct st_row_connect_status *row =
-      static_cast<struct st_row_connect_status *>(context);
+  auto *row = static_cast<struct st_row_connect_status *>(context);
 
   row->service_state = value ? PS_RPL_CONNECT_SERVICE_STATE_YES
                              : PS_RPL_CONNECT_SERVICE_STATE_NO;
@@ -103,14 +101,14 @@ Plugin_table table_replication_connection_status::m_table_def(
     "  LAST_ERROR_TIMESTAMP TIMESTAMP(6) not null,\n"
     "  PRIMARY KEY (CHANNEL_NAME) USING HASH,\n"
     "  KEY (THREAD_ID) USING HASH,\n"
-    "  LAST_QUEUED_TRANSACTION CHAR(57),\n"
+    "  LAST_QUEUED_TRANSACTION CHAR(90),\n"
     "  LAST_QUEUED_TRANSACTION_ORIGINAL_COMMIT_TIMESTAMP TIMESTAMP(6)\n"
     "                                                    not null,\n"
     "  LAST_QUEUED_TRANSACTION_IMMEDIATE_COMMIT_TIMESTAMP TIMESTAMP(6)\n"
     "                                                     not null,\n"
     "  LAST_QUEUED_TRANSACTION_START_QUEUE_TIMESTAMP TIMESTAMP(6) not null,\n"
     "  LAST_QUEUED_TRANSACTION_END_QUEUE_TIMESTAMP TIMESTAMP(6) not null,\n"
-    "  QUEUEING_TRANSACTION CHAR(57),\n"
+    "  QUEUEING_TRANSACTION CHAR(90),\n"
     "  QUEUEING_TRANSACTION_ORIGINAL_COMMIT_TIMESTAMP TIMESTAMP(6) not null,\n"
     "  QUEUEING_TRANSACTION_IMMEDIATE_COMMIT_TIMESTAMP TIMESTAMP(6)\n"
     "                                                  not null,\n"
@@ -186,7 +184,7 @@ table_replication_connection_status::table_replication_connection_status()
 table_replication_connection_status::~table_replication_connection_status() =
     default;
 
-void table_replication_connection_status::reset_position(void) {
+void table_replication_connection_status::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
@@ -196,14 +194,14 @@ ha_rows table_replication_connection_status::get_row_count() {
   return channel_map.get_max_channels();
 }
 
-int table_replication_connection_status::rnd_next(void) {
+int table_replication_connection_status::rnd_next() {
   Master_info *mi = nullptr;
   channel_map.rdlock();
 
   for (m_pos.set_at(&m_next_pos);
        m_pos.m_index < channel_map.get_max_channels(); m_pos.next()) {
     mi = channel_map.get_mi_at_pos(m_pos.m_index);
-    if (mi && mi->host[0]) {
+    if (channel_map.is_channel_configured(mi)) {
       make_row(mi);
       m_next_pos.set_after(&m_pos);
       channel_map.unlock();
@@ -224,7 +222,8 @@ int table_replication_connection_status::rnd_pos(const void *pos) {
 
   channel_map.rdlock();
 
-  if ((mi = channel_map.get_mi_at_pos(m_pos.m_index))) {
+  mi = channel_map.get_mi_at_pos(m_pos.m_index);
+  if (mi) {
     res = make_row(mi);
   }
 
@@ -252,7 +251,7 @@ int table_replication_connection_status::index_init(uint idx, bool) {
   return 0;
 }
 
-int table_replication_connection_status::index_next(void) {
+int table_replication_connection_status::index_next() {
   Master_info *mi = nullptr;
 
   channel_map.rdlock();
@@ -261,7 +260,7 @@ int table_replication_connection_status::index_next(void) {
        m_pos.m_index < channel_map.get_max_channels(); m_pos.next()) {
     mi = channel_map.get_mi_at_pos(m_pos.m_index);
 
-    if (mi && mi->host[0]) {
+    if (channel_map.is_channel_configured(mi)) {
       if (m_opened_index->match(mi)) {
         if (!make_row(mi)) {
           m_next_pos.set_after(&m_pos);
@@ -298,7 +297,8 @@ int table_replication_connection_status::make_row(Master_info *mi) {
   memcpy(m_row.channel_name, mi->get_channel(), m_row.channel_name_length);
 
   if (is_group_replication_plugin_loaded() &&
-      channel_map.is_group_replication_channel_name(mi->get_channel(), true)) {
+      channel_map.is_group_replication_applier_channel_name(
+          mi->get_channel())) {
     /*
       Group Replication applier channel.
       Set callbacks on GROUP_REPLICATION_GROUP_MEMBER_STATS_CALLBACKS.
@@ -346,12 +346,12 @@ int table_replication_connection_status::make_row(Master_info *mi) {
 
   {
     const Gtid_set *io_gtid_set = mi->rli->get_gtid_set();
-    Checkable_rwlock *sid_lock = mi->rli->get_sid_lock();
+    Checkable_rwlock *tsid_lock = mi->rli->get_tsid_lock();
 
-    sid_lock->wrlock();
+    tsid_lock->wrlock();
     m_row.received_transaction_set_length =
         io_gtid_set->to_string(&m_row.received_transaction_set);
-    sid_lock->unlock();
+    tsid_lock->unlock();
 
     if (m_row.received_transaction_set_length < 0) {
       my_free(m_row.received_transaction_set);
@@ -384,14 +384,14 @@ int table_replication_connection_status::make_row(Master_info *mi) {
 
   mysql_mutex_unlock(&mi->data_lock);
 
-  queueing_trx.copy_to_ps_table(mi->rli->get_sid_map(), m_row.queueing_trx,
+  queueing_trx.copy_to_ps_table(mi->rli->get_tsid_map(), m_row.queueing_trx,
                                 &m_row.queueing_trx_length,
                                 &m_row.queueing_trx_original_commit_timestamp,
                                 &m_row.queueing_trx_immediate_commit_timestamp,
                                 &m_row.queueing_trx_start_queue_timestamp);
 
   last_queued_trx.copy_to_ps_table(
-      mi->rli->get_sid_map(), m_row.last_queued_trx,
+      mi->rli->get_tsid_map(), m_row.last_queued_trx,
       &m_row.last_queued_trx_length,
       &m_row.last_queued_trx_original_commit_timestamp,
       &m_row.last_queued_trx_immediate_commit_timestamp,
@@ -419,20 +419,21 @@ int table_replication_connection_status::read_row_values(TABLE *table,
     if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
       switch (f->field_index()) {
         case 0: /** channel_name*/
-          set_field_char_utf8(f, m_row.channel_name, m_row.channel_name_length);
+          set_field_char_utf8mb4(f, m_row.channel_name,
+                                 m_row.channel_name_length);
           break;
         case 1: /** group_name */
           if (m_row.group_name_is_null) {
             f->set_null();
           } else {
-            set_field_char_utf8(f, m_row.group_name, UUID_LENGTH);
+            set_field_char_utf8mb4(f, m_row.group_name, UUID_LENGTH);
           }
           break;
         case 2: /** source_uuid */
           if (m_row.source_uuid_is_null) {
             f->set_null();
           } else {
-            set_field_char_utf8(f, m_row.source_uuid, UUID_LENGTH);
+            set_field_char_utf8mb4(f, m_row.source_uuid, UUID_LENGTH);
           }
           break;
         case 3: /** thread_id */
@@ -459,15 +460,15 @@ int table_replication_connection_status::read_row_values(TABLE *table,
           set_field_ulong(f, m_row.last_error_number);
           break;
         case 9: /*last_error_message*/
-          set_field_varchar_utf8(f, m_row.last_error_message,
-                                 m_row.last_error_message_length);
+          set_field_varchar_utf8mb4(f, m_row.last_error_message,
+                                    m_row.last_error_message_length);
           break;
         case 10: /*last_error_timestamp*/
           set_field_timestamp(f, m_row.last_error_timestamp);
           break;
         case 11: /*last_queued_trx*/
-          set_field_char_utf8(f, m_row.last_queued_trx,
-                              m_row.last_queued_trx_length);
+          set_field_char_utf8mb4(f, m_row.last_queued_trx,
+                                 m_row.last_queued_trx_length);
           break;
         case 12: /*last_queued_trx_original_commit_timestamp*/
           set_field_timestamp(f,
@@ -484,7 +485,8 @@ int table_replication_connection_status::read_row_values(TABLE *table,
           set_field_timestamp(f, m_row.last_queued_trx_end_queue_timestamp);
           break;
         case 16: /*queueing_trx*/
-          set_field_char_utf8(f, m_row.queueing_trx, m_row.queueing_trx_length);
+          set_field_char_utf8mb4(f, m_row.queueing_trx,
+                                 m_row.queueing_trx_length);
           break;
         case 17: /*queueing_trx_original_commit_timestamp*/
           set_field_timestamp(f, m_row.queueing_trx_original_commit_timestamp);
