@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -40,6 +41,7 @@ enum class Result {
   RECORD_FILE_FULL = HA_ERR_RECORD_FILE_FULL,
   TABLE_CORRUPT = HA_ERR_TABLE_CORRUPT,
   TABLE_EXIST = HA_ERR_TABLE_EXIST,
+  TOO_BIG_ROW = HA_ERR_TOO_BIG_ROW,
   UNSUPPORTED = HA_ERR_UNSUPPORTED,
   WRONG_COMMAND = HA_ERR_WRONG_COMMAND,
   WRONG_INDEX = HA_ERR_WRONG_INDEX,
