@@ -26,18 +26,16 @@
 #include <string>
 
 #include <gtest/gtest.h>
-#include "libbinlogevents/include/binary_log.h"
-#include "libbinlogevents/include/compression/none_comp.h"
-#include "libbinlogevents/include/compression/none_dec.h"
-#include "libbinlogevents/include/compression/zstd_comp.h"
-#include "libbinlogevents/include/compression/zstd_dec.h"
-#include "libbinlogevents/include/string/concat.h"
+#include "mysql/binlog/event/binary_log.h"
+#include "mysql/binlog/event/compression/none_comp.h"
+#include "mysql/binlog/event/compression/none_dec.h"
+#include "mysql/binlog/event/compression/zstd_comp.h"
+#include "mysql/binlog/event/compression/zstd_dec.h"
+#include "mysql/binlog/event/string/concat.h"
 
-using mysqlns::string::concat;
+using mysql::binlog::event::string::concat;
 
-namespace binary_log {
-namespace transaction {
-namespace unittests {
+namespace mysql::binlog::event::compression::unittests {
 
 static std::size_t MAX_BUFFER_SIZE = 1024 * 1024 * 512;
 
@@ -46,16 +44,15 @@ static std::array<std::size_t, 10> buffer_sizes{
 
 class TransactionPayloadCompressionTest : public ::testing::Test {
  public:
-  using Compressor_t = binary_log::transaction::compression::Compressor;
+  using Compressor_t = mysql::binlog::event::compression::Compressor;
   using Managed_buffer_sequence_t = Compressor_t::Managed_buffer_sequence_t;
-  using Decompressor_t = binary_log::transaction::compression::Decompressor;
+  using Decompressor_t = mysql::binlog::event::compression::Decompressor;
   using Managed_buffer_t = Decompressor_t::Managed_buffer_t;
   using Size_t = Decompressor_t::Size_t;
   using Char_t = Decompressor_t::Char_t;
   using Decompress_status_t =
-      binary_log::transaction::compression::Decompress_status;
-  using Compress_status_t =
-      binary_log::transaction::compression::Compress_status;
+      mysql::binlog::event::compression::Decompress_status;
+  using Compress_status_t = mysql::binlog::event::compression::Compress_status;
 
   static std::string constant_data(Size_t size) {
     return std::string(size, (Char_t)'a');
@@ -71,7 +68,7 @@ class TransactionPayloadCompressionTest : public ::testing::Test {
   static void compression_idempotency_test(Compressor_t &c, Decompressor_t &d,
                                            const std::string &data) {
     auto debug_string = concat(
-        binary_log::transaction::compression::type_to_string(c.get_type_code()),
+        mysql::binlog::event::compression::type_to_string(c.get_type_code()),
         " ", data.size());
     Managed_buffer_sequence_t managed_buffer_sequence;
     c.feed(data.data(), data.size());
@@ -80,7 +77,7 @@ class TransactionPayloadCompressionTest : public ::testing::Test {
 
     // assert that it is smaller than the uncompressed size
     auto compressed_size = managed_buffer_sequence.read_part().size();
-    if (c.get_type_code() != binary_log::transaction::compression::NONE) {
+    if (c.get_type_code() != mysql::binlog::event::compression::NONE) {
       // For very small data the constant overhead results in size inflation
       if (data.size() > 100) {
         ASSERT_LT(compressed_size, data.size()) << debug_string;
@@ -98,7 +95,7 @@ class TransactionPayloadCompressionTest : public ::testing::Test {
     auto ret = d.decompress(managed_buffer, data.size());
     auto expected_status = Decompress_status_t::success;
     if (data.empty() &&
-        c.get_type_code() == binary_log::transaction::compression::NONE)
+        c.get_type_code() == mysql::binlog::event::compression::NONE)
       expected_status = Decompress_status_t::end;
     ASSERT_EQ(ret, expected_status) << debug_string;
 
@@ -116,8 +113,8 @@ class TransactionPayloadCompressionTest : public ::testing::Test {
 
 TEST_F(TransactionPayloadCompressionTest, CompressDecompressZstdTest) {
   for (auto size : buffer_sizes) {
-    binary_log::transaction::compression::Zstd_dec d;
-    binary_log::transaction::compression::Zstd_comp c;
+    mysql::binlog::event::compression::Zstd_dec d;
+    mysql::binlog::event::compression::Zstd_comp c;
     std::string data{TransactionPayloadCompressionTest::constant_data(size)};
     TransactionPayloadCompressionTest::compression_idempotency_test(c, d, data);
     c.set_compression_level(22);
@@ -127,13 +124,11 @@ TEST_F(TransactionPayloadCompressionTest, CompressDecompressZstdTest) {
 
 TEST_F(TransactionPayloadCompressionTest, CompressDecompressNoneTest) {
   for (auto size : buffer_sizes) {
-    binary_log::transaction::compression::None_dec d;
-    binary_log::transaction::compression::None_comp c;
+    mysql::binlog::event::compression::None_dec d;
+    mysql::binlog::event::compression::None_comp c;
     std::string data{TransactionPayloadCompressionTest::constant_data(size)};
     TransactionPayloadCompressionTest::compression_idempotency_test(c, d, data);
   }
 }
 
-}  // namespace unittests
-}  // namespace transaction
-}  // namespace binary_log
+}  // namespace mysql::binlog::event::compression::unittests
