@@ -1,15 +1,16 @@
-/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,8 +40,8 @@ using ::testing::StrictMock;
 
 class HandlerTest : public ::testing::Test {
  protected:
-  virtual void SetUp() { initializer.SetUp(); }
-  virtual void TearDown() { initializer.TearDown(); }
+  void SetUp() override { initializer.SetUp(); }
+  void TearDown() override { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -56,25 +57,25 @@ class HandlerTest : public ::testing::Test {
  */
 TEST_F(HandlerTest, ReportErrorHandler) {
   Mock_field_datetime field_datetime;
-  Fake_TABLE table(&field_datetime);
-  handlerton *hton = NULL;
-  StrictMock<Mock_HANDLER> mock_handler(hton, table.get_share());
-  table.set_handler(&mock_handler);
+  Fake_TABLE *table = static_cast<Fake_TABLE *>(field_datetime.table);
+  handlerton *hton = nullptr;
+  StrictMock<Mock_HANDLER> mock_handler(hton, table->get_share());
+  table->set_handler(&mock_handler);
 
   // This error should be ignored.
-  EXPECT_EQ(-1, report_handler_error(&table, HA_ERR_END_OF_FILE));
+  EXPECT_EQ(-1, report_handler_error(table, HA_ERR_END_OF_FILE));
 
   // This one should not be printed to stderr, but passed on to the handler.
   EXPECT_CALL(mock_handler, print_error(HA_ERR_TABLE_DEF_CHANGED, 0)).Times(1);
-  EXPECT_EQ(1, report_handler_error(&table, HA_ERR_TABLE_DEF_CHANGED));
+  EXPECT_EQ(1, report_handler_error(table, HA_ERR_TABLE_DEF_CHANGED));
 }
 
 TEST_F(HandlerTest, TableInMemoryEstimate) {
   Mock_field_datetime field_datetime;
-  Fake_TABLE table(&field_datetime);
-  handlerton *hton = NULL;
-  StrictMock<Mock_HANDLER> mock_handler(hton, table.get_share());
-  table.set_handler(&mock_handler);
+  Fake_TABLE *table = static_cast<Fake_TABLE *>(field_datetime.table);
+  handlerton *hton = nullptr;
+  StrictMock<Mock_HANDLER> mock_handler(hton, table->get_share());
+  table->set_handler(&mock_handler);
 
   // Verify that the handler does not know the buffer size
   EXPECT_EQ(mock_handler.get_memory_buffer_size(), -1);
@@ -150,11 +151,11 @@ TEST_F(HandlerTest, TableInMemoryEstimate) {
 
 TEST_F(HandlerTest, IndexInMemoryEstimate) {
   Mock_field_datetime field_datetime;
-  Fake_TABLE table(&field_datetime);
-  handlerton *hton = NULL;
-  StrictMock<Mock_HANDLER> mock_handler(hton, table.get_share());
-  table.set_handler(&mock_handler);
-  mock_handler.change_table_ptr(&table, table.get_share());
+  Fake_TABLE *table = static_cast<Fake_TABLE *>(field_datetime.table);
+  handlerton *hton = nullptr;
+  NiceMock<Mock_HANDLER> mock_handler(hton, table->get_share());
+  table->set_handler(&mock_handler);
+  mock_handler.change_table_ptr(table, table->get_share());
   const uint key_no = 0;
 
   // Verify that the handler does not know the buffer size
@@ -177,7 +178,7 @@ TEST_F(HandlerTest, IndexInMemoryEstimate) {
   const uint index_size_large = mem_buf_size * 2;
 
   // Initialize the estimate for how much of the index that is in memory
-  table.key_info[key_no].set_in_memory_estimate(IN_MEMORY_ESTIMATE_UNKNOWN);
+  table->key_info[key_no].set_in_memory_estimate(IN_MEMORY_ESTIMATE_UNKNOWN);
 
   /*
     Test with an index that is less than 20% of memory buffer. This should
@@ -204,7 +205,7 @@ TEST_F(HandlerTest, IndexInMemoryEstimate) {
     Simulate that the storage engine has reported that 50 percent of the
     index is in a memory buffer.
   */
-  table.key_info[key_no].set_in_memory_estimate(0.5);
+  table->key_info[key_no].set_in_memory_estimate(0.5);
 
   /*
     Set the index size to be less than 20 percent but larger than 10K.
@@ -227,16 +228,18 @@ TEST_F(HandlerTest, IndexInMemoryEstimate) {
 
 TEST_F(HandlerTest, SamplingInterfaceAllRows) {
   Mock_field_datetime field_datetime;
-  Fake_TABLE table(&field_datetime);
-  StrictMock<Mock_SAMPLING_HANDLER> mock_handler(nullptr, &table,
-                                                 table.get_share());
-  table.set_handler(&mock_handler);
+  Fake_TABLE *table = static_cast<Fake_TABLE *>(field_datetime.table);
+  StrictMock<Mock_SAMPLING_HANDLER> mock_handler(nullptr, table,
+                                                 table->get_share());
+  table->set_handler(&mock_handler);
 
   uchar buffer[8];
+  void *scan_ctx = nullptr;
 
   // rnd_init should be called exactly one time by ha_sample_init.
   EXPECT_CALL(mock_handler, rnd_init(true)).Times(1);
-  EXPECT_EQ(mock_handler.ha_sample_init(100.0, 0, enum_sampling_method::SYSTEM),
+  EXPECT_EQ(mock_handler.ha_sample_init(scan_ctx, 100.0, 0,
+                                        enum_sampling_method::SYSTEM, false),
             0);
   EXPECT_EQ(mock_handler.inited, handler::SAMPLING);
 
@@ -247,26 +250,29 @@ TEST_F(HandlerTest, SamplingInterfaceAllRows) {
   const int num_iterations = 100;
   EXPECT_CALL(mock_handler, rnd_next(buffer)).Times(num_iterations);
 
-  for (int i = 0; i < num_iterations; ++i) mock_handler.ha_sample_next(buffer);
+  for (int i = 0; i < num_iterations; ++i)
+    mock_handler.ha_sample_next(scan_ctx, buffer);
 
   // rnd_end should be called exactly one time by ha_sample_end.
   EXPECT_CALL(mock_handler, rnd_end()).Times(1);
-  EXPECT_EQ(mock_handler.ha_sample_end(), 0);
+  EXPECT_EQ(mock_handler.ha_sample_end(scan_ctx), 0);
   EXPECT_EQ(mock_handler.inited, handler::NONE);
 }
 
 TEST_F(HandlerTest, SamplingInterfaceNoRows) {
   Mock_field_datetime field_datetime;
-  Fake_TABLE table(&field_datetime);
-  StrictMock<Mock_SAMPLING_HANDLER> mock_handler(nullptr, &table,
-                                                 table.get_share());
-  table.set_handler(&mock_handler);
+  Fake_TABLE *table = reinterpret_cast<Fake_TABLE *>(field_datetime.table);
+  StrictMock<Mock_SAMPLING_HANDLER> mock_handler(nullptr, table,
+                                                 table->get_share());
+  table->set_handler(&mock_handler);
 
   uchar buffer[8];
+  void *scan_ctx = nullptr;
 
   // rnd_init should be called exactly one time by ha_sample_init.
   EXPECT_CALL(mock_handler, rnd_init(true)).Times(1);
-  EXPECT_EQ(mock_handler.ha_sample_init(0.0, 0, enum_sampling_method::SYSTEM),
+  EXPECT_EQ(mock_handler.ha_sample_init(scan_ctx, 0.0, 0,
+                                        enum_sampling_method::SYSTEM, false),
             0);
   EXPECT_EQ(mock_handler.inited, handler::SAMPLING);
 
@@ -276,11 +282,11 @@ TEST_F(HandlerTest, SamplingInterfaceNoRows) {
   */
   EXPECT_CALL(mock_handler, rnd_next(buffer)).Times(0);
 
-  for (int i = 0; i < 100; ++i) mock_handler.ha_sample_next(buffer);
+  for (int i = 0; i < 100; ++i) mock_handler.ha_sample_next(scan_ctx, buffer);
 
   // rnd_end should be called exactly one time by ha_sample_end.
   EXPECT_CALL(mock_handler, rnd_end()).Times(1);
-  EXPECT_EQ(mock_handler.ha_sample_end(), 0);
+  EXPECT_EQ(mock_handler.ha_sample_end(scan_ctx), 0);
   EXPECT_EQ(mock_handler.inited, handler::NONE);
 }
 
