@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,17 +27,17 @@
 #include <string>  // std::string
 
 #include "lex_string.h"
-#include "m_ctype.h"  // my_charset_latin1, my_charset_bin
 #include "my_inttypes.h"
 #include "my_systime.h"                  // my_micro_time()
 #include "my_time.h"                     // MYSQL_TIME
+#include "mysql/strings/m_ctype.h"       // my_charset_latin1, my_charset_bin
 #include "sql-common/json_dom.h"         // Json_object
+#include "sql-common/my_decimal.h"       // my_decimal
 #include "sql/field.h"                   // my_charset_numeric
 #include "sql/histograms/equi_height.h"  // Equi_height
 #include "sql/histograms/histogram.h"    // Histogram, Histogram_comparator
 #include "sql/histograms/singleton.h"    // Singleton
 #include "sql/histograms/value_map.h"    // Value_map<T>
-#include "sql/my_decimal.h"              // my_decimal
 #include "sql/sql_time.h"                // my_time_compare
 #include "sql/tztime.h"                  // my_tz_UTC
 #include "template_utils.h"              // down_cast
@@ -2149,9 +2150,7 @@ TEST_F(HistogramsTest, MultiByteStrings) {
     Declare the strings to have UCS2 character set, which is fixed 2 byte per
     character.
   */
-  MY_CHARSET_LOADER loader;
-  CHARSET_INFO *cs =
-      my_collation_get_by_name(&loader, "ucs2_general_ci", MYF(0));
+  CHARSET_INFO *cs = get_charset_by_name("ucs2_general_ci", MYF(0));
 
   Value_map<String> long_strings(cs, Value_map_type::STRING);
 
