@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,8 +40,8 @@ using my_testing::Server_initializer;
 class JoinSyntaxTest : public ParserTest {};
 
 void check_name_resolution_tables(std::initializer_list<const char *> aliases,
-                                  SQL_I_List<TABLE_LIST> tables) {
-  TABLE_LIST *table_list = tables.first;
+                                  SQL_I_List<Table_ref> tables) {
+  Table_ref *table_list = tables.first;
   for (auto alias : aliases) {
     ASSERT_FALSE(table_list == nullptr);
     EXPECT_STREQ(alias, table_list->alias)
@@ -52,12 +53,12 @@ void check_name_resolution_tables(std::initializer_list<const char *> aliases,
 
 TEST_F(JoinSyntaxTest, CrossJoin) {
   Query_block *query_block = parse("SELECT * FROM t1 JOIN t2 JOIN t3");
-  check_name_resolution_tables({"t1", "t2", "t3"}, query_block->table_list);
+  check_name_resolution_tables({"t1", "t2", "t3"}, query_block->m_table_list);
 }
 
 TEST_F(JoinSyntaxTest, CrossJoinOn) {
   Query_block *query_block = parse("SELECT * FROM t1 JOIN t2 JOIN t3 ON 1");
-  check_name_resolution_tables({"t1", "t2", "t3"}, query_block->table_list);
+  check_name_resolution_tables({"t1", "t2", "t3"}, query_block->m_table_list);
 }
 
 }  // namespace join_syntax_unittest
