@@ -1,15 +1,16 @@
-/* Copyright (c) 2013, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -74,7 +75,7 @@ class OptRefTest : public ::testing::Test {
     t1 = new Fake_TABLE(field_t1_a, field_t1_b);
     t2 = new Fake_TABLE(field_t2_a, field_t2_b);
 
-    index_over_t1ab_id = t1->create_index(field_t1_a, field_t1_b);
+    index_over_t1ab_id = t1->create_index({field_t1_a, field_t1_b});
     indexes.set_bit(index_over_t1ab_id);
 
     t1_join_tab.set_qs(&t1_qep_shared);
@@ -147,7 +148,7 @@ TEST_F(OptRefTest, addKeyFieldsFromInOneRow) {
     an expression in to (a, b) = (0, 0), which gets rewritten into a =
     0 AND b = 0 before the ref optimizer runs.
    */
-  PT_item_list *all_args = new (current_thd->mem_root) PT_item_list;
+  PT_item_list *all_args = new (current_thd->mem_root) PT_item_list(POS());
   all_args->push_front(make_item_row(item_zero, item_zero));
   all_args->push_front(make_item_row(item_field_t1_a, item_field_t1_b));
   Item *cond = new Item_func_in(POS(), all_args, false);
@@ -167,7 +168,7 @@ TEST_F(OptRefTest, addKeyFieldsFromInOneRow) {
 
 TEST_F(OptRefTest, addKeyFieldsFromInTwoRows) {
   // We simulate the where condition (col_a, col_b) IN ((0, 0), (1, 1))
-  PT_item_list *all_args = new (current_thd->mem_root) PT_item_list;
+  PT_item_list *all_args = new (current_thd->mem_root) PT_item_list(POS());
   all_args->push_front(make_item_row(item_one, item_one));
   all_args->push_front(make_item_row(item_zero, item_zero));
   all_args->push_front(make_item_row(item_field_t1_a, item_field_t1_b));
@@ -186,7 +187,7 @@ TEST_F(OptRefTest, addKeyFieldsFromInTwoRows) {
 
 TEST_F(OptRefTest, addKeyFieldsFromInOneRowWithCols) {
   // We simulate the where condition (t1.a, t1.b) IN ((t2.a, t2.b))
-  PT_item_list *all_args = new (current_thd->mem_root) PT_item_list;
+  PT_item_list *all_args = new (current_thd->mem_root) PT_item_list(POS());
   all_args->push_front(make_item_row(item_field_t2_a, item_field_t2_b));
   all_args->push_front(make_item_row(item_field_t1_a, item_field_t1_b));
   Item *cond = new Item_func_in(POS(), all_args, false);
