@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2025, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,14 +32,19 @@
 
 namespace xcl {
 namespace test {
+namespace mock {
 
-class Mock_query_instances : public Query_instances {
+class Query_instances : public xcl::Query_instances {
  public:
-  MOCK_METHOD0(instances_fetch_begin, Instance_id());
-  MOCK_METHOD0(instances_fetch_end, void());
-  MOCK_METHOD1(is_instance_active, bool(const Instance_id id));
+  Query_instances();
+  virtual ~Query_instances() override;
+
+  MOCK_METHOD(Instance_id, instances_fetch_begin, (), (override));
+  MOCK_METHOD(void, instances_fetch_end, (), (override));
+  MOCK_METHOD(bool, is_instance_active, (const Instance_id id), (override));
 };
 
+}  // namespace mock
 }  // namespace test
 }  // namespace xcl
 
