@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,41 +23,53 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
-#include "my_config.h"
-
 #include <errno.h>
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
-#include "my_inttypes.h"
-#include "my_io.h"
+#include <cstdint>
+
+#include "my_config.h"  // NOLINT(build/include_subdir)
+#include "my_io.h"      // NOLINT(build/include_subdir)
+
 #include "plugin/x/src/io/xpl_listener_unix_socket.h"
-#include "unittest/gunit/xplugin/xpl/mock/ngs_general.h"
+#include "unittest/gunit/xplugin/xpl/mock/file.h"
+#include "unittest/gunit/xplugin/xpl/mock/operations_factory.h"
+#include "unittest/gunit/xplugin/xpl/mock/socket.h"
+#include "unittest/gunit/xplugin/xpl/mock/socket_events.h"
+#include "unittest/gunit/xplugin/xpl/mock/system.h"
 
 namespace xpl {
+namespace test {
 
-namespace tests {
+using namespace ::testing;  // NOLINT(build/namespaces)
 
-using namespace ::testing;
-
-const uint32 BACKLOG = 122;
+const uint32_t BACKLOG = 122;
 const my_socket SOCKET_OK = 10;
 const int BIND_OK = 0;
+#if defined(HAVE_SYS_UN_H)
+const int CLOSE_ERR = -1;
+const int FSYNC_ERR = -1;
 const int LISTEN_ERR = -1;
-const int LISTEN_OK = 0;
 const int OPEN_ERR = -1;
 const int READ_ERR = -1;
-const int WRITE_ERR = -1;
 const int UNLINK_ERR = -1;
-const int UNLINK_OK = 0;
-const int FSYNC_ERR = -1;
-const int FSYNC_OK = 0;
-const int CLOSE_ERR = -1;
+const int WRITE_ERR = -1;
+#endif
 const int CLOSE_OK = 0;
 const int CURRENT_PID = 6;
-const std::string UNIX_SOCKET_FILE_CONTENT = "X6\n";  // "X%d" % CURRENT_PID
-const std::string UNIX_SOCKET_FILE = "/tmp/xplugin_test.sock";
-const std::string UNIX_SOCKET_LOCK_FILE = "/tmp/xplugin_test.sock.lock";
+const int CURRENT_PPID = 2;
+const int FSYNC_OK = 0;
+const int LISTEN_OK = 0;
+const int UNLINK_OK = 0;
+
+const char *const UNIX_SOCKET_FILE_CONTENT = "6\n";  // "%d" % CURRENT_PID
+const char *const UNIX_SOCKET_FILE = "/tmp/xplugin_test.sock";
+const char *const UNIX_SOCKET_LOCK_FILE = "/tmp/xplugin_test.sock.lock";
+
+ACTION_P(SetArg0ToPChar, value) { strcpy(static_cast<char *>(arg0), value); }
+ACTION_P(SetArg0ToChar, value) { *static_cast<char *>(arg0) = value; }
+ACTION_P(SetArg0ToChar2, value) { (static_cast<char *>(arg0)[1]) = value; }
 
 MATCHER(EqInvalidSocket, "") {
   return INVALID_SOCKET == mysql_socket_getfd(arg);
@@ -64,19 +77,18 @@ MATCHER(EqInvalidSocket, "") {
 
 MATCHER_P(EqCastToCStr, expected, "") {
   std::string force_string = expected;
-  return force_string == (char *)arg;
+  return force_string == static_cast<char *>(arg);
 }
 
 class Listener_unix_socket_testsuite : public Test {
  public:
-  void SetUp() {
-    m_mock_factory = std::make_shared<StrictMock<ngs::test::Mock_factory>>();
-    m_mock_socket = std::make_shared<StrictMock<ngs::test::Mock_socket>>();
-    m_mock_system = std::make_shared<StrictMock<ngs::test::Mock_system>>();
-    m_mock_file = std::make_shared<StrictMock<ngs::test::Mock_file>>();
-    m_mock_socket_invalid =
-        std::make_shared<StrictMock<ngs::test::Mock_socket>>();
-    m_mock_file_invalid = std::make_shared<StrictMock<ngs::test::Mock_file>>();
+  void SetUp() override {
+    m_mock_factory = std::make_shared<StrictMock<mock::Operations_factory>>();
+    m_mock_socket = std::make_shared<StrictMock<mock::Socket>>();
+    m_mock_system = std::make_shared<StrictMock<mock::System>>();
+    m_mock_file = std::make_shared<StrictMock<mock::File>>();
+    m_mock_socket_invalid = std::make_shared<StrictMock<mock::Socket>>();
+    m_mock_file_invalid = std::make_shared<StrictMock<mock::File>>();
 
     EXPECT_CALL(*m_mock_factory, create_system_interface())
         .WillRepeatedly(Return(m_mock_system));
@@ -94,20 +106,41 @@ class Listener_unix_socket_testsuite : public Test {
         BACKLOG);
   }
 
+  void assert_valid_lock_file_but_kills_old() {
+    EXPECT_CALL(*m_mock_system, get_pid()).WillOnce(Return(CURRENT_PID));
+    EXPECT_CALL(*m_mock_factory, open_file(StrEq(UNIX_SOCKET_LOCK_FILE), _, _))
+        .WillOnce(Return(m_mock_file_invalid))
+        .WillOnce(Return(m_mock_file))
+        .WillOnce(Return(m_mock_file));
+    EXPECT_CALL(*m_mock_system, get_errno()).WillOnce(Return(EEXIST));
+    EXPECT_CALL(*m_mock_file, read(_, _))
+        .WillOnce(DoAll(SetArg0ToPChar("X101"), Return(4)))
+        .WillOnce(Return(0));
+    EXPECT_CALL(*m_mock_system, get_ppid()).WillOnce(Return(CURRENT_PPID));
+
+    EXPECT_CALL(*m_mock_system, kill(101, _)).WillOnce(Return(1));
+    EXPECT_CALL(*m_mock_file, write(EqCastToCStr(UNIX_SOCKET_FILE_CONTENT),
+                                    strlen(UNIX_SOCKET_FILE_CONTENT)))
+        .WillOnce(Return(strlen(UNIX_SOCKET_FILE_CONTENT)));
+    EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_LOCK_FILE)));
+    EXPECT_CALL(*m_mock_file, fsync()).WillOnce(Return(FSYNC_OK));
+    EXPECT_CALL(*m_mock_file, close())
+        .WillOnce(Return(CLOSE_OK))
+        .WillOnce(Return(CLOSE_OK));
+  }
+
   void assert_valid_lock_file() {
     EXPECT_CALL(*m_mock_system, get_pid()).WillOnce(Return(CURRENT_PID));
     EXPECT_CALL(*m_mock_factory, open_file(StrEq(UNIX_SOCKET_LOCK_FILE), _, _))
         .WillOnce(Return(m_mock_file));
     EXPECT_CALL(*m_mock_file, write(EqCastToCStr(UNIX_SOCKET_FILE_CONTENT),
-                                    UNIX_SOCKET_FILE_CONTENT.length()))
-        .WillOnce(Return(UNIX_SOCKET_FILE_CONTENT.length()));
+                                    strlen(UNIX_SOCKET_FILE_CONTENT)))
+        .WillOnce(Return(strlen(UNIX_SOCKET_FILE_CONTENT)));
     EXPECT_CALL(*m_mock_file, fsync()).WillOnce(Return(FSYNC_OK));
     EXPECT_CALL(*m_mock_file, close()).WillOnce(Return(CLOSE_OK));
   }
 
-  void assert_setup_listener_successful() {
-    ASSERT_NO_FATAL_FAILURE(assert_valid_lock_file());
-
+  void assert_unix_socket_listen() {
     EXPECT_CALL(*m_mock_factory, create_socket(_, AF_UNIX, SOCK_STREAM, 0))
         .WillOnce(Return(m_mock_socket));
     EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_FILE)));
@@ -118,11 +151,16 @@ class Listener_unix_socket_testsuite : public Test {
         .WillRepeatedly(Return(SOCKET_OK));  // back in setup_listener()
     EXPECT_CALL(*m_mock_socket, set_socket_thread_owner());
 
-    ngs::Socket_interface::Shared_ptr socket = m_mock_socket;
+    std::shared_ptr<iface::Socket> socket = m_mock_socket;
     EXPECT_CALL(m_mock_socket_events, listen(socket, _)).WillOnce(Return(true));
+  }
+
+  void assert_setup_listener_successful() {
+    ASSERT_NO_FATAL_FAILURE(assert_valid_lock_file());
+    ASSERT_NO_FATAL_FAILURE(assert_unix_socket_listen());
 
     ASSERT_TRUE(sut->setup_listener(nullptr));
-    ASSERT_TRUE(sut->get_state().is(ngs::State_listener_prepared));
+    ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_prepared));
 
     ASSERT_NO_FATAL_FAILURE(assert_and_clear_mocks());
   }
@@ -135,20 +173,32 @@ class Listener_unix_socket_testsuite : public Test {
     ASSERT_TRUE(Mock::VerifyAndClearExpectations(m_mock_factory.get()));
   }
 
-  std::shared_ptr<ngs::test::Mock_socket> m_mock_socket;
-  std::shared_ptr<ngs::test::Mock_socket> m_mock_socket_invalid;
-  std::shared_ptr<ngs::test::Mock_system> m_mock_system;
-  std::shared_ptr<ngs::test::Mock_file> m_mock_file_invalid;
-  std::shared_ptr<ngs::test::Mock_file> m_mock_file;
-  StrictMock<ngs::test::Mock_socket_events> m_mock_socket_events;
-  std::shared_ptr<ngs::test::Mock_factory> m_mock_factory;
+  void assert_close_listener() {
+    EXPECT_CALL(*m_mock_factory, create_system_interface())
+        .WillRepeatedly(Return(m_mock_system));
+    EXPECT_CALL(*m_mock_socket, get_socket_fd()).WillOnce(Return(SOCKET_OK));
+    EXPECT_CALL(*m_mock_socket, close());
+    EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_LOCK_FILE)))
+        .WillOnce(Return(UNLINK_OK));
+    EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_FILE)))
+        .WillOnce(Return(UNLINK_OK));
+
+    sut->close_listener();
+
+    ASSERT_NO_FATAL_FAILURE(assert_and_clear_mocks());
+  }
+
+  std::shared_ptr<mock::Socket> m_mock_socket;
+  std::shared_ptr<mock::Socket> m_mock_socket_invalid;
+  std::shared_ptr<mock::System> m_mock_system;
+  std::shared_ptr<mock::File> m_mock_file_invalid;
+  std::shared_ptr<mock::File> m_mock_file;
+  StrictMock<mock::Socket_events> m_mock_socket_events;
+  std::shared_ptr<mock::Operations_factory> m_mock_factory;
 
   std::shared_ptr<Listener_unix_socket> sut;
 };
 
-ACTION_P(SetArg0ToChar, value) { *static_cast<char *>(arg0) = value; }
-ACTION_P(SetArg0ToChar2, value) { (static_cast<char *>(arg0)[1]) = value; }
-
 TEST_F(Listener_unix_socket_testsuite,
        unixsocket_try_to_create_empty_unixsocket_filename) {
 #if defined(HAVE_SYS_UN_H)
@@ -158,7 +208,7 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -172,7 +222,7 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -189,7 +239,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_cant_create_a_lockfile) {
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -207,7 +257,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_cant_open_existing_lockfile) {
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -226,7 +276,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_cant_read_existing_lockfile) {
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -246,7 +296,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_read_empty_lockfile) {
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -256,6 +306,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_read_not_x_plugin_lockfile) {
                                                std::ref(m_mock_socket_events),
                                                BACKLOG);
 
+  EXPECT_CALL(*m_mock_system, get_ppid()).WillOnce(Return(CURRENT_PPID));
   EXPECT_CALL(*m_mock_system, get_pid()).WillOnce(Return(CURRENT_PID));
   EXPECT_CALL(*m_mock_factory, open_file(StrEq(UNIX_SOCKET_LOCK_FILE), _, _))
       .WillOnce(Return(m_mock_file_invalid))
@@ -268,7 +319,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_read_not_x_plugin_lockfile) {
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -295,7 +346,7 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -323,7 +374,7 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -341,7 +392,7 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -355,15 +406,14 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_system, get_pid()).WillOnce(Return(CURRENT_PID));
   EXPECT_CALL(*m_mock_factory, open_file(StrEq(UNIX_SOCKET_LOCK_FILE), _, _))
       .WillOnce(Return(m_mock_file));
-  EXPECT_CALL(*m_mock_file,
-              write(EqCastToCStr(UNIX_SOCKET_FILE_CONTENT.c_str()),
-                    UNIX_SOCKET_FILE_CONTENT.length()))
-      .WillOnce(Return(UNIX_SOCKET_FILE_CONTENT.length()));
+  EXPECT_CALL(*m_mock_file, write(EqCastToCStr(UNIX_SOCKET_FILE_CONTENT),
+                                  strlen(UNIX_SOCKET_FILE_CONTENT)))
+      .WillOnce(Return(strlen(UNIX_SOCKET_FILE_CONTENT)));
   EXPECT_CALL(*m_mock_file, fsync()).WillOnce(Return(FSYNC_ERR));
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -377,16 +427,15 @@ TEST_F(Listener_unix_socket_testsuite,
   EXPECT_CALL(*m_mock_system, get_pid()).WillOnce(Return(CURRENT_PID));
   EXPECT_CALL(*m_mock_factory, open_file(StrEq(UNIX_SOCKET_LOCK_FILE), _, _))
       .WillOnce(Return(m_mock_file));
-  EXPECT_CALL(*m_mock_file,
-              write(EqCastToCStr(UNIX_SOCKET_FILE_CONTENT.c_str()),
-                    UNIX_SOCKET_FILE_CONTENT.length()))
-      .WillOnce(Return(UNIX_SOCKET_FILE_CONTENT.length()));
+  EXPECT_CALL(*m_mock_file, write(EqCastToCStr(UNIX_SOCKET_FILE_CONTENT),
+                                  strlen(UNIX_SOCKET_FILE_CONTENT)))
+      .WillOnce(Return(strlen(UNIX_SOCKET_FILE_CONTENT)));
   EXPECT_CALL(*m_mock_file, fsync()).WillOnce(Return(FSYNC_OK));
   EXPECT_CALL(*m_mock_file, close()).WillOnce(Return(CLOSE_ERR));
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -397,11 +446,11 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_create_socket_failed) {
   EXPECT_CALL(*m_mock_factory, create_socket(_, AF_UNIX, SOCK_STREAM, 0))
       .WillOnce(Return(m_mock_socket_invalid));
   EXPECT_CALL(*m_mock_system, get_socket_error_and_message(_, _))
-      .WillOnce(DoAll(SetArgReferee<0>(0), SetArgReferee<1>("")));
+      .WillOnce(DoAll(SetArgPointee<0>(0), SetArgPointee<1>("")));
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -415,7 +464,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_listen_failed) {
   EXPECT_CALL(*m_mock_socket, bind(_, _)).WillOnce(Return(BIND_OK));
   EXPECT_CALL(*m_mock_socket, listen(_)).WillOnce(Return(LISTEN_ERR));
   EXPECT_CALL(*m_mock_system, get_socket_error_and_message(_, _))
-      .WillOnce(DoAll(SetArgReferee<0>(0), SetArgReferee<1>("")));
+      .WillOnce(DoAll(SetArgPointee<0>(0), SetArgPointee<1>("")));
   EXPECT_CALL(*m_mock_socket, get_socket_fd())
       .WillOnce(Return(SOCKET_OK))              // before m_mock_socket.close()
       .WillRepeatedly(Return(INVALID_SOCKET));  // after  m_mock_socket.close()
@@ -424,7 +473,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_listen_failed) {
                   // second call in SUT destructor
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -444,7 +493,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_event_regiester_failure) {
       .WillRepeatedly(Return(SOCKET_OK));  // back in setup_listener()
   EXPECT_CALL(*m_mock_socket, set_socket_thread_owner());
 
-  ngs::Socket_interface::Shared_ptr socket = m_mock_socket;
+  std::shared_ptr<iface::Socket> socket = m_mock_socket;
   EXPECT_CALL(m_mock_socket_events, listen(socket, _)).WillOnce(Return(false));
 
   EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_LOCK_FILE)))
@@ -453,7 +502,7 @@ TEST_F(Listener_unix_socket_testsuite, unixsocket_event_regiester_failure) {
   EXPECT_CALL(*m_mock_socket, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 
 #endif
 }
@@ -484,7 +533,7 @@ TEST_F(Listener_unix_socket_testsuite, unix_socket_unsupported) {
   EXPECT_CALL(*m_mock_socket_invalid, close());
 
   ASSERT_FALSE(sut->setup_listener(nullptr));
-  ASSERT_TRUE(sut->get_state().is(ngs::State_listener_stopped));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_stopped));
 #endif
 }
 
@@ -501,20 +550,27 @@ TEST_F(Listener_unix_socket_testsuite, close_listener_closes_valid_socket) {
 #if defined(HAVE_SYS_UN_H)
   ASSERT_NO_FATAL_FAILURE(assert_setup_listener_successful());
 
-  EXPECT_CALL(*m_mock_factory, create_system_interface())
-      .WillRepeatedly(Return(m_mock_system));
-  EXPECT_CALL(*m_mock_socket, get_socket_fd()).WillOnce(Return(SOCKET_OK));
-  EXPECT_CALL(*m_mock_socket, close());
-  EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_LOCK_FILE)))
-      .WillOnce(Return(UNLINK_OK));
-  EXPECT_CALL(*m_mock_system, unlink(StrEq(UNIX_SOCKET_FILE)))
-      .WillOnce(Return(UNLINK_OK));
-  sut->close_listener();
-
-  ASSERT_NO_FATAL_FAILURE(assert_and_clear_mocks());
+  ASSERT_NO_FATAL_FAILURE(assert_close_listener());
 #endif
 }
 
-}  // namespace tests
+TEST_F(Listener_unix_socket_testsuite, unixsocket_read_old_x_plugin_lockfile) {
+#if defined(HAVE_SYS_UN_H)
+  sut = std::make_shared<Listener_unix_socket>(m_mock_factory, UNIX_SOCKET_FILE,
+                                               std::ref(m_mock_socket_events),
+                                               BACKLOG);
 
+  ASSERT_NO_FATAL_FAILURE(assert_valid_lock_file_but_kills_old());
+  ASSERT_NO_FATAL_FAILURE(assert_unix_socket_listen());
+
+  ASSERT_TRUE(sut->setup_listener(nullptr));
+  ASSERT_TRUE(sut->get_state().is(iface::Listener::State::k_prepared));
+
+  ASSERT_NO_FATAL_FAILURE(assert_and_clear_mocks());
+
+  ASSERT_NO_FATAL_FAILURE(assert_close_listener());
+#endif
+}
+
+}  // namespace test
 }  // namespace xpl
